-- MySQL dump 10.11
--
-- Host: localhost    Database: teleahor_mgn01
-- ------------------------------------------------------
-- Server version	5.0.75-community-log

/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;
/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;
/*!40101 SET NAMES utf8 */;
/*!40103 SET @OLD_TIME_ZONE=@@TIME_ZONE */;
/*!40103 SET TIME_ZONE='+00:00' */;
/*!40014 SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;
/*!40111 SET @OLD_SQL_NOTES=@@SQL_NOTES, SQL_NOTES=0 */;

--
-- Table structure for table `admin_assert`
--

DROP TABLE IF EXISTS `admin_assert`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `admin_assert` (
  `assert_id` int(10) unsigned NOT NULL auto_increment,
  `assert_type` varchar(20) NOT NULL default '',
  `assert_data` text,
  PRIMARY KEY  (`assert_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='ACL Asserts';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `admin_assert`
--

LOCK TABLES `admin_assert` WRITE;
/*!40000 ALTER TABLE `admin_assert` DISABLE KEYS */;
/*!40000 ALTER TABLE `admin_assert` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `admin_role`
--

DROP TABLE IF EXISTS `admin_role`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `admin_role` (
  `role_id` int(10) unsigned NOT NULL auto_increment,
  `parent_id` int(10) unsigned NOT NULL default '0',
  `tree_level` tinyint(3) unsigned NOT NULL default '0',
  `sort_order` tinyint(3) unsigned NOT NULL default '0',
  `role_type` char(1) NOT NULL default '0',
  `user_id` int(11) unsigned NOT NULL default '0',
  `role_name` varchar(50) NOT NULL default '',
  PRIMARY KEY  (`role_id`),
  KEY `parent_id` (`parent_id`,`sort_order`),
  KEY `tree_level` (`tree_level`)
) ENGINE=InnoDB AUTO_INCREMENT=5 DEFAULT CHARSET=utf8 COMMENT='ACL Roles';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `admin_role`
--

LOCK TABLES `admin_role` WRITE;
/*!40000 ALTER TABLE `admin_role` DISABLE KEYS */;
INSERT INTO `admin_role` VALUES (1,0,1,1,'G',0,'Administrators'),(3,1,2,0,'U',2,'Your'),(4,1,2,0,'U',3,'Sonia');
/*!40000 ALTER TABLE `admin_role` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `admin_rule`
--

DROP TABLE IF EXISTS `admin_rule`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `admin_rule` (
  `rule_id` int(10) unsigned NOT NULL auto_increment,
  `role_id` int(10) unsigned NOT NULL default '0',
  `resource_id` varchar(255) NOT NULL default '',
  `privileges` varchar(20) NOT NULL default '',
  `assert_id` int(10) unsigned NOT NULL default '0',
  `role_type` char(1) default NULL,
  `permission` varchar(10) default NULL,
  PRIMARY KEY  (`rule_id`),
  KEY `resource` (`resource_id`,`role_id`),
  KEY `role_id` (`role_id`,`resource_id`),
  CONSTRAINT `FK_admin_rule` FOREIGN KEY (`role_id`) REFERENCES `admin_role` (`role_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=utf8 COMMENT='ACL Rules';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `admin_rule`
--

LOCK TABLES `admin_rule` WRITE;
/*!40000 ALTER TABLE `admin_rule` DISABLE KEYS */;
INSERT INTO `admin_rule` VALUES (1,1,'all','',0,'G','allow');
/*!40000 ALTER TABLE `admin_rule` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `admin_user`
--

DROP TABLE IF EXISTS `admin_user`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `admin_user` (
  `user_id` mediumint(9) unsigned NOT NULL auto_increment,
  `firstname` varchar(32) NOT NULL default '',
  `lastname` varchar(32) NOT NULL default '',
  `email` varchar(128) NOT NULL default '',
  `username` varchar(40) NOT NULL default '',
  `password` varchar(40) NOT NULL default '',
  `created` datetime NOT NULL default '0000-00-00 00:00:00',
  `modified` datetime default NULL,
  `logdate` datetime default NULL,
  `lognum` smallint(5) unsigned NOT NULL default '0',
  `reload_acl_flag` tinyint(1) NOT NULL default '0',
  `is_active` tinyint(1) NOT NULL default '1',
  `extra` text,
  PRIMARY KEY  (`user_id`)
) ENGINE=InnoDB AUTO_INCREMENT=4 DEFAULT CHARSET=utf8 COMMENT='Users';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `admin_user`
--

LOCK TABLES `admin_user` WRITE;
/*!40000 ALTER TABLE `admin_user` DISABLE KEYS */;
INSERT INTO `admin_user` VALUES (2,'Leopoldo','Martinez-Cubells','polo@lasbolas.es','admin','1278590f3737367e8ec5b0da8a151c3c:1I','2009-06-22 06:40:44','2009-06-22 12:45:15','2009-07-02 22:35:33',17,0,1,'a:1:{s:11:\"configState\";a:65:{s:15:\"general_country\";s:1:\"1\";s:14:\"general_locale\";s:1:\"1\";s:7:\"web_url\";s:1:\"1\";s:7:\"web_seo\";s:1:\"1\";s:12:\"web_unsecure\";s:1:\"1\";s:10:\"web_secure\";s:1:\"1\";s:11:\"web_default\";s:1:\"1\";s:9:\"web_polls\";s:1:\"1\";s:10:\"web_cookie\";s:1:\"1\";s:11:\"web_session\";s:1:\"1\";s:24:\"web_browser_capabilities\";s:1:\"1\";s:14:\"design_package\";s:1:\"1\";s:12:\"design_theme\";s:1:\"1\";s:11:\"design_head\";s:1:\"1\";s:13:\"design_header\";s:1:\"1\";s:13:\"design_footer\";s:1:\"1\";s:16:\"design_watermark\";s:1:\"0\";s:16:\"currency_options\";s:1:\"1\";s:20:\"currency_webservicex\";s:1:\"1\";s:15:\"currency_import\";s:1:\"1\";s:25:\"trans_email_ident_general\";s:1:\"1\";s:23:\"trans_email_ident_sales\";s:1:\"1\";s:25:\"trans_email_ident_support\";s:1:\"1\";s:25:\"trans_email_ident_custom1\";s:1:\"1\";s:25:\"trans_email_ident_custom2\";s:1:\"1\";s:17:\"contacts_contacts\";s:1:\"1\";s:14:\"contacts_email\";s:1:\"1\";s:11:\"system_cron\";s:1:\"1\";s:11:\"system_smtp\";s:1:\"1\";s:15:\"system_currency\";s:1:\"1\";s:10:\"system_log\";s:1:\"1\";s:24:\"system_adminnotification\";s:1:\"1\";s:14:\"payment_ccsave\";s:1:\"0\";s:12:\"payment_free\";s:1:\"0\";s:15:\"payment_checkmo\";s:1:\"0\";s:21:\"payment_purchaseorder\";s:1:\"0\";s:20:\"payment_authorizenet\";s:1:\"0\";s:16:\"payment_verisign\";s:1:\"0\";s:22:\"payment_paypal_express\";s:1:\"0\";s:21:\"payment_paypal_direct\";s:1:\"0\";s:23:\"payment_paypal_standard\";s:1:\"0\";s:24:\"payment_paypaluk_express\";s:1:\"0\";s:23:\"payment_paypaluk_direct\";s:1:\"0\";s:26:\"payment_amazonpayments_asp\";s:1:\"0\";s:26:\"payment_amazonpayments_cba\";s:1:\"0\";s:22:\"payment_cashondelivery\";s:1:\"1\";s:15:\"shipping_origin\";s:1:\"1\";s:15:\"shipping_option\";s:1:\"1\";s:17:\"carriers_flatrate\";s:1:\"1\";s:18:\"carriers_tablerate\";s:1:\"0\";s:21:\"carriers_freeshipping\";s:1:\"1\";s:12:\"carriers_ups\";s:1:\"1\";s:13:\"carriers_usps\";s:1:\"1\";s:14:\"carriers_fedex\";s:1:\"1\";s:12:\"carriers_dhl\";s:1:\"1\";s:12:\"admin_emails\";s:1:\"0\";s:13:\"admin_startup\";s:1:\"0\";s:9:\"admin_url\";s:1:\"0\";s:14:\"admin_security\";s:1:\"0\";s:28:\"admin_bouncingorange_tinymce\";s:1:\"1\";s:12:\"dev_restrict\";s:1:\"1\";s:9:\"dev_debug\";s:1:\"1\";s:20:\"dev_translate_inline\";s:1:\"1\";s:7:\"dev_log\";s:1:\"0\";s:6:\"dev_js\";s:1:\"0\";}}'),(3,'Sonia','Alonso','salonso@mcaarquitectura.com','Sonia','7984df8c1dd5054cd897bf326c3af829:SW','2009-06-23 14:42:17','2009-06-23 14:42:29','2009-07-02 18:48:19',4,0,1,'a:1:{s:11:\"configState\";a:65:{s:15:\"general_country\";s:1:\"1\";s:14:\"general_locale\";s:1:\"1\";s:7:\"web_url\";s:1:\"1\";s:7:\"web_seo\";s:1:\"1\";s:12:\"web_unsecure\";s:1:\"1\";s:10:\"web_secure\";s:1:\"1\";s:11:\"web_default\";s:1:\"1\";s:9:\"web_polls\";s:1:\"1\";s:10:\"web_cookie\";s:1:\"1\";s:11:\"web_session\";s:1:\"1\";s:24:\"web_browser_capabilities\";s:1:\"1\";s:14:\"design_package\";s:1:\"1\";s:12:\"design_theme\";s:1:\"1\";s:11:\"design_head\";s:1:\"1\";s:13:\"design_header\";s:1:\"1\";s:13:\"design_footer\";s:1:\"1\";s:16:\"design_watermark\";s:1:\"0\";s:16:\"currency_options\";s:1:\"1\";s:20:\"currency_webservicex\";s:1:\"1\";s:15:\"currency_import\";s:1:\"1\";s:25:\"trans_email_ident_general\";s:1:\"1\";s:23:\"trans_email_ident_sales\";s:1:\"1\";s:25:\"trans_email_ident_support\";s:1:\"1\";s:25:\"trans_email_ident_custom1\";s:1:\"1\";s:25:\"trans_email_ident_custom2\";s:1:\"1\";s:17:\"contacts_contacts\";s:1:\"1\";s:14:\"contacts_email\";s:1:\"1\";s:11:\"system_cron\";s:1:\"1\";s:11:\"system_smtp\";s:1:\"1\";s:15:\"system_currency\";s:1:\"1\";s:10:\"system_log\";s:1:\"1\";s:24:\"system_adminnotification\";s:1:\"1\";s:14:\"payment_ccsave\";s:1:\"0\";s:12:\"payment_free\";s:1:\"0\";s:15:\"payment_checkmo\";s:1:\"0\";s:21:\"payment_purchaseorder\";s:1:\"0\";s:20:\"payment_authorizenet\";s:1:\"0\";s:16:\"payment_verisign\";s:1:\"0\";s:22:\"payment_paypal_express\";s:1:\"0\";s:21:\"payment_paypal_direct\";s:1:\"0\";s:23:\"payment_paypal_standard\";s:1:\"0\";s:24:\"payment_paypaluk_express\";s:1:\"0\";s:23:\"payment_paypaluk_direct\";s:1:\"0\";s:26:\"payment_amazonpayments_asp\";s:1:\"0\";s:26:\"payment_amazonpayments_cba\";s:1:\"0\";s:22:\"payment_cashondelivery\";s:1:\"1\";s:15:\"shipping_origin\";s:1:\"1\";s:15:\"shipping_option\";s:1:\"1\";s:17:\"carriers_flatrate\";s:1:\"1\";s:18:\"carriers_tablerate\";s:1:\"0\";s:21:\"carriers_freeshipping\";s:1:\"1\";s:12:\"carriers_ups\";s:1:\"1\";s:13:\"carriers_usps\";s:1:\"1\";s:14:\"carriers_fedex\";s:1:\"1\";s:12:\"carriers_dhl\";s:1:\"1\";s:12:\"admin_emails\";s:1:\"0\";s:13:\"admin_startup\";s:1:\"0\";s:9:\"admin_url\";s:1:\"0\";s:14:\"admin_security\";s:1:\"0\";s:28:\"admin_bouncingorange_tinymce\";s:1:\"1\";s:12:\"dev_restrict\";s:1:\"1\";s:9:\"dev_debug\";s:1:\"1\";s:20:\"dev_translate_inline\";s:1:\"1\";s:7:\"dev_log\";s:1:\"0\";s:6:\"dev_js\";s:1:\"0\";}}');
/*!40000 ALTER TABLE `admin_user` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `adminnotification_inbox`
--

DROP TABLE IF EXISTS `adminnotification_inbox`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `adminnotification_inbox` (
  `notification_id` int(10) unsigned NOT NULL auto_increment,
  `severity` tinyint(3) unsigned NOT NULL default '0',
  `date_added` datetime NOT NULL,
  `title` varchar(255) NOT NULL,
  `description` text,
  `url` varchar(255) NOT NULL,
  `is_read` tinyint(1) unsigned NOT NULL default '0',
  `is_remove` tinyint(1) unsigned NOT NULL default '0',
  PRIMARY KEY  (`notification_id`),
  KEY `IDX_SEVERITY` (`severity`),
  KEY `IDX_IS_READ` (`is_read`),
  KEY `IDX_IS_REMOVE` (`is_remove`)
) ENGINE=InnoDB AUTO_INCREMENT=25 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `adminnotification_inbox`
--

LOCK TABLES `adminnotification_inbox` WRITE;
/*!40000 ALTER TABLE `adminnotification_inbox` DISABLE KEYS */;
INSERT INTO `adminnotification_inbox` VALUES (1,4,'2008-07-25 01:24:40','Magento 1.1 Production Version Now Available','We are thrilled to announce the availability of the production release of Magento 1.1. Read more about the release in the Magento Blog.','http://www.magentocommerce.com/blog/comments/magento-11-is-here-1/',0,1),(2,4,'2008-08-02 01:30:16','Updated iPhone Theme is now available','Updated iPhone theme for Magento 1.1 is now available on Magento Connect and for upgrade through your Magento Connect Manager.','http://www.magentocommerce.com/blog/comments/updated-iphone-theme-for-magento-11-is-now-available/',0,1),(3,3,'2008-08-02 01:40:27','Magento version 1.1.2 is now available','Magento version 1.1.2 is now available for download and upgrade.','http://www.magentocommerce.com/blog/comments/magento-version-112-is-now-available/',0,1),(4,3,'2008-08-13 17:51:46','Magento version 1.1.3 is now available','Magento version 1.1.3 is now available','http://www.magentocommerce.com/blog/comments/magento-version-113-is-now-available/',0,1),(5,1,'2008-09-02 21:10:31','Magento Version 1.1.4 Security Update Now Available','Magento 1.1.4 Security Update Now Available. If you are using Magento version 1.1.x, we highly recommend upgrading to this version as soon as possible.','http://www.magentocommerce.com/blog/comments/magento-version-114-security-update/',0,1),(6,3,'2008-09-15 22:09:54','Magento version 1.1.5 Now Available','Magento version 1.1.5 Now Available.\n\nThis release includes many bug fixes, a new category manager and a new skin for the default Magento theme.','http://www.magentocommerce.com/blog/comments/magento-version-115-now-available/',0,1),(7,3,'2008-09-17 20:18:35','Magento version 1.1.6 Now Available','Magento version 1.1.6 Now Available.\n\nThis version includes bug fixes for Magento 1.1.x that are listed in the release notes section.','http://www.magentocommerce.com/blog/comments/magento-version-116-now-available/',0,1),(8,4,'2008-11-07 23:46:42','Reminder: Change Magento`s default phone numbers and callouts before site launch','Before launching your Magento store, please remember to change Magento`s default phone numbers that appear in email templates, callouts, templates, etc.','',0,1),(9,3,'2008-11-20 01:31:12','Magento version 1.1.7 Now Available','Magento version 1.1.7 Now Available.\n\nThis version includes over 350 issue resolutions for Magento 1.1.x that are listed in the release notes section, and new functionality that includes:\n\n-Google Website Optimizer integration\n-Google Base integration\n-Scheduled DB logs cleaning option','http://www.magentocommerce.com/blog/comments/magento-version-117-now-available/',0,1),(10,3,'2008-11-26 21:24:50','Magento Version 1.1.8 Now Available','Magento version 1.1.8 now available.\n\nThis version includes some issue resolutions for Magento 1.1.x that are listed in the release notes section.','http://www.magentocommerce.com/blog/comments/magento-version-118-now-available/',0,1),(11,3,'2008-12-30 07:45:59','Magento version 1.2.0 is now available for download and upgrade','We are extremely happy to announce the availability of Magento version 1.2.0 for download and upgrade.\n\nThis version includes numerous issue resolutions for Magento version 1.1.x and some highly requested new features such as:\n\n    * Support for Downloadable/Digital Products. \n    * Added Layered Navigation to site search result page.\n    * Improved site search to utilize MySQL fulltext search\n    * Added support for fixed-taxes on product level.\n    * Upgraded Zend Framework to the latest stable version 1.7.2','http://www.magentocommerce.com/blog/comments/magento-version-120-is-now-available/',0,1),(12,2,'2008-12-30 21:59:22','Magento version 1.2.0.1 now available','Magento version 1.2.0.1 now available.This version includes some issue resolutions for Magento 1.2.x that are listed in the release notes section.','http://www.magentocommerce.com/blog/comments/magento-version-1201-available/',0,1),(13,2,'2009-01-12 20:41:49','Magento version 1.2.0.2 now available','Magento version 1.2.0.2 is now available for download and upgrade. This version includes an issue resolutions for Magento version 1.2.0.x as listed in the release notes.','http://www.magentocommerce.com/blog/comments/magento-version-1202-now-available/',0,1),(14,3,'2009-01-24 00:25:56','Magento version 1.2.0.3 now available','Magento version 1.2.0.3 is now available for download and upgrade. This version includes issue resolutions for Magento version 1.2.0.x as listed in the release notes.','http://www.magentocommerce.com/blog/comments/magento-version-1203-now-available/',0,1),(15,3,'2009-02-02 21:57:00','Magento version 1.2.1 is now available for download and upgrade','We are happy to announce the availability of Magento version 1.2.1 for download and upgrade.\n\nThis version includes some issue resolutions for Magento version 1.2.x. A full list of items included in this release can be found on the release notes page.','http://www.magentocommerce.com/blog/comments/magento-version-121-now-available/',0,1),(16,3,'2009-02-24 00:45:47','Magento version 1.2.1.1 now available','Magento version 1.2.1.1 now available.This version includes some issue resolutions for Magento 1.2.x that are listed in the release notes section.','http://www.magentocommerce.com/blog/comments/magento-version-1211-now-available/',0,1),(17,3,'2009-02-27 01:39:24','CSRF Attack Prevention','We have just posted a blog entry about a hypothetical CSRF attack on a Magento admin panel. Please read the post to find out if your Magento installation is at risk at http://www.magentocommerce.com/blog/comments/csrf-vulnerabilities-in-web-application-and-how-to-avoid-them-in-magento/','http://www.magentocommerce.com/blog/comments/csrf-vulnerabilities-in-web-application-and-how-to-avoid-them-in-magento/',0,1),(18,2,'2009-03-03 23:03:58','Magento version 1.2.1.2 now available','Magento version 1.2.1.2 is now available for download and upgrade.\nThis version includes some updates to improve admin security as described in the release notes page.','http://www.magentocommerce.com/blog/comments/magento-version-1212-now-available/',0,1),(19,3,'2009-03-31 02:22:40','Magento version 1.3.0 now available','Magento version 1.3.0 is now available for download and upgrade. This version includes numerous issue resolutions for Magento version 1.2.x and new features as described on the release notes page.','http://www.magentocommerce.com/blog/comments/magento-version-130-is-now-available/',0,1),(20,3,'2009-04-18 04:06:02','Magento version 1.3.1 now available','Magento version 1.3.1 is now available for download and upgrade. This version includes some issue resolutions for Magento version 1.3.x and new features such as Checkout By Amazon and Amazon Flexible Payment. To see a full list of updates please check the release notes page.','http://www.magentocommerce.com/blog/comments/magento-version-131-now-available/',0,1),(21,3,'2009-05-19 22:31:21','Magento version 1.3.1.1 now available','Magento version 1.3.1.1 is now available for download and upgrade. This version includes some issue resolutions for Magento version 1.3.x and a security update for Magento installations that run on multiple domains or sub-domains. If you are running Magento with multiple domains or sub-domains we highly recommend upgrading to this version.','http://www.magentocommerce.com/blog/comments/magento-version-1311-now-available/',0,1),(22,3,'2009-05-29 22:54:06','Magento version 1.3.2 now available','This version includes some improvements and issue resolutions for version 1.3.x that are listed on the release notes page. also included is a Beta version of the Compile module.','http://www.magentocommerce.com/blog/comments/magento-version-132-now-available/',0,1),(23,3,'2009-06-01 19:32:52','Magento version 1.3.2.1 now available','Magento version 1.3.2.1 now available for download and upgrade.\n\nThis release solves an issue for users running Magento with PHP 5.2.0, and changes to index.php to support the new Compiler Module.','http://www.magentocommerce.com/blog/comments/magento-version-1321-now-available/',0,1),(24,3,'2009-07-02 01:21:44','Magento version 1.3.2.2 now available','Magento version 1.3.2.2 is now available for download and upgrade.\n\nThis release includes issue resolution for Magento version 1.3.x. To see a full list of changes please visit the release notes page http://www.magentocommerce.com/download/release_notes.','http://www.magentocommerce.com/blog/comments/magento-version-1322-now-available/',0,0);
/*!40000 ALTER TABLE `adminnotification_inbox` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `amazonpayments_api_debug`
--

DROP TABLE IF EXISTS `amazonpayments_api_debug`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `amazonpayments_api_debug` (
  `debug_id` int(10) unsigned NOT NULL auto_increment,
  `transaction_id` varchar(255) NOT NULL default '',
  `debug_at` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `request_body` text,
  `response_body` text,
  PRIMARY KEY  (`debug_id`),
  KEY `debug_at` (`debug_at`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `amazonpayments_api_debug`
--

LOCK TABLES `amazonpayments_api_debug` WRITE;
/*!40000 ALTER TABLE `amazonpayments_api_debug` DISABLE KEYS */;
/*!40000 ALTER TABLE `amazonpayments_api_debug` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `api_assert`
--

DROP TABLE IF EXISTS `api_assert`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `api_assert` (
  `assert_id` int(10) unsigned NOT NULL auto_increment,
  `assert_type` varchar(20) NOT NULL default '',
  `assert_data` text,
  PRIMARY KEY  (`assert_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Api ACL Asserts';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `api_assert`
--

LOCK TABLES `api_assert` WRITE;
/*!40000 ALTER TABLE `api_assert` DISABLE KEYS */;
/*!40000 ALTER TABLE `api_assert` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `api_role`
--

DROP TABLE IF EXISTS `api_role`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `api_role` (
  `role_id` int(10) unsigned NOT NULL auto_increment,
  `parent_id` int(10) unsigned NOT NULL default '0',
  `tree_level` tinyint(3) unsigned NOT NULL default '0',
  `sort_order` tinyint(3) unsigned NOT NULL default '0',
  `role_type` char(1) NOT NULL default '0',
  `user_id` int(11) unsigned NOT NULL default '0',
  `role_name` varchar(50) NOT NULL default '',
  PRIMARY KEY  (`role_id`),
  KEY `parent_id` (`parent_id`,`sort_order`),
  KEY `tree_level` (`tree_level`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Api ACL Roles';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `api_role`
--

LOCK TABLES `api_role` WRITE;
/*!40000 ALTER TABLE `api_role` DISABLE KEYS */;
/*!40000 ALTER TABLE `api_role` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `api_rule`
--

DROP TABLE IF EXISTS `api_rule`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `api_rule` (
  `rule_id` int(10) unsigned NOT NULL auto_increment,
  `role_id` int(10) unsigned NOT NULL default '0',
  `resource_id` varchar(255) NOT NULL default '',
  `privileges` varchar(20) NOT NULL default '',
  `assert_id` int(10) unsigned NOT NULL default '0',
  `role_type` char(1) default NULL,
  `permission` varchar(10) default NULL,
  PRIMARY KEY  (`rule_id`),
  KEY `resource` (`resource_id`,`role_id`),
  KEY `role_id` (`role_id`,`resource_id`),
  CONSTRAINT `FK_api_rule` FOREIGN KEY (`role_id`) REFERENCES `api_role` (`role_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Api ACL Rules';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `api_rule`
--

LOCK TABLES `api_rule` WRITE;
/*!40000 ALTER TABLE `api_rule` DISABLE KEYS */;
/*!40000 ALTER TABLE `api_rule` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `api_session`
--

DROP TABLE IF EXISTS `api_session`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `api_session` (
  `user_id` mediumint(9) unsigned NOT NULL,
  `logdate` datetime NOT NULL default '0000-00-00 00:00:00',
  `sessid` varchar(40) NOT NULL default '',
  KEY `API_SESSION_USER` (`user_id`),
  KEY `API_SESSION_SESSID` (`sessid`),
  CONSTRAINT `FK_API_SESSION_USER` FOREIGN KEY (`user_id`) REFERENCES `api_user` (`user_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Api Sessions';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `api_session`
--

LOCK TABLES `api_session` WRITE;
/*!40000 ALTER TABLE `api_session` DISABLE KEYS */;
/*!40000 ALTER TABLE `api_session` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `api_user`
--

DROP TABLE IF EXISTS `api_user`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `api_user` (
  `user_id` mediumint(9) unsigned NOT NULL auto_increment,
  `firstname` varchar(32) NOT NULL default '',
  `lastname` varchar(32) NOT NULL default '',
  `email` varchar(128) NOT NULL default '',
  `username` varchar(40) NOT NULL default '',
  `api_key` varchar(40) NOT NULL default '',
  `created` datetime NOT NULL default '0000-00-00 00:00:00',
  `modified` datetime default NULL,
  `lognum` smallint(5) unsigned NOT NULL default '0',
  `reload_acl_flag` tinyint(1) NOT NULL default '0',
  `is_active` tinyint(1) NOT NULL default '1',
  PRIMARY KEY  (`user_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Api Users';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `api_user`
--

LOCK TABLES `api_user` WRITE;
/*!40000 ALTER TABLE `api_user` DISABLE KEYS */;
/*!40000 ALTER TABLE `api_user` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_category_entity`
--

DROP TABLE IF EXISTS `catalog_category_entity`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_category_entity` (
  `entity_id` int(10) unsigned NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_set_id` smallint(5) unsigned NOT NULL default '0',
  `parent_id` int(10) unsigned NOT NULL default '0',
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `path` varchar(255) NOT NULL,
  `position` int(11) NOT NULL,
  `level` int(11) NOT NULL,
  `children_count` int(11) NOT NULL,
  PRIMARY KEY  (`entity_id`),
  KEY `IDX_LEVEL` (`level`)
) ENGINE=InnoDB AUTO_INCREMENT=14 DEFAULT CHARSET=utf8 COMMENT='Category Entities';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_category_entity`
--

LOCK TABLES `catalog_category_entity` WRITE;
/*!40000 ALTER TABLE `catalog_category_entity` DISABLE KEYS */;
INSERT INTO `catalog_category_entity` VALUES (1,3,0,0,'0000-00-00 00:00:00','2009-07-02 18:52:58','1',3,0,11),(2,3,3,1,'2009-06-03 19:52:24','2009-07-02 18:52:58','1/2',3,1,10),(4,3,3,2,'2009-07-02 16:10:39','2009-07-02 16:37:30','1/2/4',1,2,2),(5,3,3,4,'2009-07-02 16:45:13','2009-07-02 16:45:13','1/2/4/5',1,3,0),(6,3,3,4,'2009-07-02 17:07:24','2009-07-02 17:07:24','1/2/4/6',3,3,0),(7,3,3,2,'2009-07-02 18:52:44','2009-07-02 18:52:44','1/2/7',2,2,3),(8,3,3,7,'2009-07-02 18:55:52','2009-07-02 18:55:52','1/2/7/8',1,3,0),(9,3,3,7,'2009-07-02 19:05:54','2009-07-02 19:05:54','1/2/7/9',2,3,0),(10,3,3,7,'2009-07-02 19:10:11','2009-07-02 19:10:11','1/2/7/10',3,3,0),(11,3,3,2,'2009-07-02 19:29:57','2009-07-02 19:29:57','1/2/11',3,2,2),(12,3,3,11,'2009-07-02 19:35:09','2009-07-02 19:35:09','1/2/11/12',1,3,0),(13,3,3,11,'2009-07-02 19:38:18','2009-07-02 19:38:18','1/2/11/13',2,3,0);
/*!40000 ALTER TABLE `catalog_category_entity` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_category_entity_datetime`
--

DROP TABLE IF EXISTS `catalog_category_entity_datetime`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_category_entity_datetime` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_BASE` (`entity_type_id`,`entity_id`,`attribute_id`,`store_id`),
  KEY `FK_ATTRIBUTE_DATETIME_ENTITY` (`entity_id`),
  KEY `FK_CATALOG_CATEGORY_ENTITY_DATETIME_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CATALOG_CATEGORY_ENTITY_DATETIME_STORE` (`store_id`),
  CONSTRAINT `FK_CATALOG_CATEGORY_ENTITY_DATETIME_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_CATEGORY_ENTITY_DATETIME_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_category_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_CATEGORY_ENTITY_DATETIME_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_category_entity_datetime`
--

LOCK TABLES `catalog_category_entity_datetime` WRITE;
/*!40000 ALTER TABLE `catalog_category_entity_datetime` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_category_entity_datetime` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_category_entity_decimal`
--

DROP TABLE IF EXISTS `catalog_category_entity_decimal`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_category_entity_decimal` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` decimal(12,4) NOT NULL default '0.0000',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_BASE` (`entity_type_id`,`entity_id`,`attribute_id`,`store_id`),
  KEY `FK_ATTRIBUTE_DECIMAL_ENTITY` (`entity_id`),
  KEY `FK_CATALOG_CATEGORY_ENTITY_DECIMAL_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CATALOG_CATEGORY_ENTITY_DECIMAL_STORE` (`store_id`),
  CONSTRAINT `FK_CATALOG_CATEGORY_ENTITY_DECIMAL_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_CATEGORY_ENTITY_DECIMAL_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_category_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_CATEGORY_ENTITY_DECIMAL_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_category_entity_decimal`
--

LOCK TABLES `catalog_category_entity_decimal` WRITE;
/*!40000 ALTER TABLE `catalog_category_entity_decimal` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_category_entity_decimal` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_category_entity_int`
--

DROP TABLE IF EXISTS `catalog_category_entity_int`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_category_entity_int` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` int(11) NOT NULL default '0',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_BASE` (`entity_type_id`,`entity_id`,`attribute_id`,`store_id`),
  KEY `FK_ATTRIBUTE_INT_ENTITY` (`entity_id`),
  KEY `FK_CATALOG_CATEGORY_EMTITY_INT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CATALOG_CATEGORY_EMTITY_INT_STORE` (`store_id`),
  CONSTRAINT `FK_CATALOG_CATEGORY_EMTITY_INT_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_CATEGORY_EMTITY_INT_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_category_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_CATEGORY_EMTITY_INT_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=36 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_category_entity_int`
--

LOCK TABLES `catalog_category_entity_int` WRITE;
/*!40000 ALTER TABLE `catalog_category_entity_int` DISABLE KEYS */;
INSERT INTO `catalog_category_entity_int` VALUES (1,3,32,0,2,1),(2,3,32,1,2,1),(6,3,32,0,4,1),(7,3,41,0,4,0),(8,3,49,0,4,1),(9,3,32,0,5,1),(10,3,41,0,5,0),(11,3,49,0,5,1),(12,3,32,0,6,1),(13,3,41,0,6,0),(14,3,49,0,6,1),(15,3,32,0,7,1),(16,3,41,0,7,0),(17,3,49,0,7,1),(18,3,32,0,8,1),(19,3,41,0,8,0),(20,3,49,0,8,1),(21,3,32,0,9,1),(22,3,41,0,9,0),(23,3,49,0,9,1),(24,3,32,0,10,1),(25,3,41,0,10,0),(26,3,49,0,10,1),(27,3,32,0,11,1),(28,3,41,0,11,0),(29,3,49,0,11,1),(30,3,32,0,12,1),(31,3,41,0,12,0),(32,3,49,0,12,1),(33,3,32,0,13,1),(34,3,41,0,13,0),(35,3,49,0,13,1);
/*!40000 ALTER TABLE `catalog_category_entity_int` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_category_entity_text`
--

DROP TABLE IF EXISTS `catalog_category_entity_text`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_category_entity_text` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` text NOT NULL,
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_BASE` (`entity_type_id`,`entity_id`,`attribute_id`,`store_id`),
  KEY `FK_ATTRIBUTE_TEXT_ENTITY` (`entity_id`),
  KEY `FK_CATALOG_CATEGORY_ENTITY_TEXT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CATALOG_CATEGORY_ENTITY_TEXT_STORE` (`store_id`),
  CONSTRAINT `FK_CATALOG_CATEGORY_ENTITY_TEXT_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_CATEGORY_ENTITY_TEXT_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_category_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_CATEGORY_ENTITY_TEXT_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=63 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_category_entity_text`
--

LOCK TABLES `catalog_category_entity_text` WRITE;
/*!40000 ALTER TABLE `catalog_category_entity_text` DISABLE KEYS */;
INSERT INTO `catalog_category_entity_text` VALUES (6,3,98,0,1,''),(7,3,98,0,2,''),(13,3,34,0,4,'<p>Manganeso y rojo...........</p>'),(14,3,37,0,4,''),(15,3,38,0,4,''),(16,3,53,0,4,''),(17,3,98,0,4,''),(18,3,34,0,5,'<p>Piedras manganeso y rojo.....largas, cortas.....</p>'),(19,3,37,0,5,''),(20,3,38,0,5,''),(21,3,53,0,5,''),(22,3,98,0,5,''),(23,3,34,0,6,''),(24,3,37,0,6,''),(25,3,38,0,6,''),(26,3,53,0,6,''),(27,3,98,0,6,''),(28,3,34,0,7,'<p>Rojo sobre blanco......</p>'),(29,3,37,0,7,''),(30,3,38,0,7,''),(31,3,53,0,7,''),(32,3,98,0,7,''),(33,3,34,0,8,'<p>Piedras...rojo sobre blanco....largas y cortas....</p>'),(34,3,37,0,8,''),(35,3,38,0,8,''),(36,3,53,0,8,''),(37,3,98,0,8,''),(38,3,34,0,9,'<p>Placas...rojo sobre blanco....lisas y grabadas....largas y cortas.....</p>'),(39,3,37,0,9,''),(40,3,38,0,9,''),(41,3,53,0,9,''),(42,3,98,0,9,''),(43,3,34,0,10,'<p>Medallones rojo sobre blanco...lisos y grabados....sencillos y dobles...y largos muy largos....</p>'),(44,3,37,0,10,''),(45,3,38,0,10,''),(46,3,53,0,10,''),(47,3,98,0,10,''),(48,3,34,0,11,'<p>Barro y esmalte...blanco y negro....contrastes...</p>'),(49,3,37,0,11,''),(50,3,38,0,11,''),(51,3,53,0,11,''),(52,3,98,0,11,''),(53,3,34,0,12,'<p>Medallones negros esmaltados en blanco....contrastes, manchas, formas...todos distintos...y largos,muy largos.....</p>'),(54,3,37,0,12,''),(55,3,38,0,12,''),(56,3,53,0,12,''),(57,3,98,0,12,''),(58,3,34,0,13,'<p>Plumas negras esmaltadas...largas, cortas...todas diferentes....</p>'),(59,3,37,0,13,''),(60,3,38,0,13,''),(61,3,53,0,13,''),(62,3,98,0,13,'');
/*!40000 ALTER TABLE `catalog_category_entity_text` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_category_entity_varchar`
--

DROP TABLE IF EXISTS `catalog_category_entity_varchar`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_category_entity_varchar` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_BASE` USING BTREE (`entity_type_id`,`entity_id`,`attribute_id`,`store_id`),
  KEY `FK_ATTRIBUTE_VARCHAR_ENTITY` (`entity_id`),
  KEY `FK_CATALOG_CATEGORY_ENTITY_VARCHAR_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CATALOG_CATEGORY_ENTITY_VARCHAR_STORE` (`store_id`),
  CONSTRAINT `FK_CATALOG_CATEGORY_ENTITY_VARCHAR_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_CATEGORY_ENTITY_VARCHAR_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_category_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_CATEGORY_ENTITY_VARCHAR_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=124 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_category_entity_varchar`
--

LOCK TABLES `catalog_category_entity_varchar` WRITE;
/*!40000 ALTER TABLE `catalog_category_entity_varchar` DISABLE KEYS */;
INSERT INTO `catalog_category_entity_varchar` VALUES (1,3,31,0,1,'Root Catalog'),(2,3,31,1,1,'Root Catalog'),(3,3,33,0,1,'root-catalog'),(4,3,31,0,2,'Default Category'),(5,3,39,0,2,'PRODUCTS'),(6,3,33,0,2,'default-category'),(32,3,31,0,4,'Vulcano'),(33,3,33,0,4,'vulcano'),(34,3,36,0,4,''),(35,3,39,0,4,'PRODUCTS'),(36,3,48,0,4,''),(37,3,52,0,4,''),(38,3,47,1,4,'vulcano.html'),(39,3,47,0,4,'vulcano.html'),(40,3,35,0,4,'SNC00347_1.jpg'),(43,3,31,0,5,'Colgante piedra'),(44,3,33,0,5,'colgante-piedra'),(45,3,36,0,5,''),(46,3,39,0,5,'PRODUCTS'),(47,3,48,0,5,''),(48,3,52,0,5,''),(49,3,35,0,5,'file.jpg'),(50,3,47,1,5,'vulcano/colgante-piedra.html'),(51,3,47,0,5,'vulcano/colgante-piedra.html'),(52,3,31,0,6,'medallones vulcano'),(53,3,33,0,6,'medallones-vulcano'),(54,3,36,0,6,''),(55,3,39,0,6,'PRODUCTS'),(56,3,48,0,6,''),(57,3,52,0,6,''),(58,3,35,0,6,'file_1.jpg'),(59,3,47,1,6,'vulcano/medallones-vulcano.html'),(60,3,47,0,6,'vulcano/medallones-vulcano.html'),(61,3,31,0,7,'Dominica'),(62,3,33,0,7,'dominica'),(63,3,36,0,7,''),(64,3,39,0,7,'PRODUCTS'),(65,3,48,0,7,''),(66,3,52,0,7,''),(67,3,35,0,7,'file_2.jpg'),(68,3,47,1,7,'dominica.html'),(69,3,47,0,7,'dominica.html'),(70,3,31,0,8,'Piedras dominica'),(71,3,33,0,8,'piedras-dominica'),(72,3,36,0,8,''),(73,3,39,0,8,'PRODUCTS'),(74,3,48,0,8,''),(75,3,52,0,8,''),(76,3,35,0,8,'file_3.jpg'),(77,3,47,1,8,'dominica/piedras-dominica.html'),(78,3,47,0,8,'dominica/piedras-dominica.html'),(79,3,31,0,9,'Placas dominica'),(80,3,33,0,9,'placas-dominica'),(81,3,36,0,9,''),(82,3,39,0,9,'PRODUCTS'),(83,3,48,0,9,''),(84,3,52,0,9,''),(85,3,35,0,9,'file_4.jpg'),(86,3,47,1,9,'dominica/placas-dominica.html'),(87,3,47,0,9,'dominica/placas-dominica.html'),(88,3,31,0,10,'Medallones dominica'),(89,3,33,0,10,'medallones-dominica'),(90,3,36,0,10,''),(91,3,39,0,10,'PRODUCTS'),(92,3,48,0,10,''),(93,3,52,0,10,''),(94,3,35,0,10,'file_5.jpg'),(95,3,47,1,10,'dominica/medallones-dominica.html'),(96,3,47,0,10,'dominica/medallones-dominica.html'),(97,3,31,0,11,'Hindú'),(98,3,33,0,11,'hindu'),(99,3,36,0,11,''),(100,3,39,0,11,'PRODUCTS'),(101,3,48,0,11,''),(102,3,52,0,11,''),(103,3,35,0,11,'file.JPG'),(104,3,47,1,11,'hindu.html'),(105,3,47,0,11,'hindu.html'),(106,3,31,0,12,'Medallones hindu'),(107,3,33,0,12,'medallones-hindu'),(108,3,36,0,12,''),(109,3,39,0,12,'PRODUCTS'),(110,3,48,0,12,''),(111,3,52,0,12,''),(112,3,35,0,12,'file_1.JPG'),(113,3,47,1,12,'hindu/medallones-hindu.html'),(114,3,47,0,12,'hindu/medallones-hindu.html'),(115,3,31,0,13,'Plumas hindu'),(116,3,33,0,13,'plumas-hindu'),(117,3,36,0,13,''),(118,3,39,0,13,'PRODUCTS'),(119,3,48,0,13,''),(120,3,52,0,13,''),(121,3,35,0,13,'file_2.JPG'),(122,3,47,1,13,'hindu/plumas-hindu.html'),(123,3,47,0,13,'hindu/plumas-hindu.html');
/*!40000 ALTER TABLE `catalog_category_entity_varchar` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_category_flat`
--

DROP TABLE IF EXISTS `catalog_category_flat`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_category_flat` (
  `entity_id` int(10) unsigned NOT NULL,
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `parent_id` int(10) unsigned NOT NULL default '0',
  `path` varchar(255) NOT NULL default '',
  `level` int(11) NOT NULL default '0',
  `position` int(11) NOT NULL default '0',
  `children_count` int(11) NOT NULL,
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  KEY `CATEGORY_FLAT_CATEGORY_ID` (`entity_id`),
  KEY `CATEGORY_FLAT_STORE_ID` (`store_id`),
  KEY `path` (`path`),
  KEY `IDX_LEVEL` (`level`),
  CONSTRAINT `FK_CATEGORY_FLAT_CATEGORY_ID` FOREIGN KEY (`entity_id`) REFERENCES `catalog_category_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATEGORY_FLAT_STORE_ID` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Flat Category';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_category_flat`
--

LOCK TABLES `catalog_category_flat` WRITE;
/*!40000 ALTER TABLE `catalog_category_flat` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_category_flat` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_category_product`
--

DROP TABLE IF EXISTS `catalog_category_product`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_category_product` (
  `category_id` int(10) unsigned NOT NULL default '0',
  `product_id` int(10) unsigned NOT NULL default '0',
  `position` int(10) unsigned NOT NULL default '0',
  UNIQUE KEY `UNQ_CATEGORY_PRODUCT` (`category_id`,`product_id`),
  KEY `CATALOG_CATEGORY_PRODUCT_CATEGORY` (`category_id`),
  KEY `CATALOG_CATEGORY_PRODUCT_PRODUCT` (`product_id`),
  CONSTRAINT `CATALOG_CATEGORY_PRODUCT_CATEGORY` FOREIGN KEY (`category_id`) REFERENCES `catalog_category_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `CATALOG_CATEGORY_PRODUCT_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_category_product`
--

LOCK TABLES `catalog_category_product` WRITE;
/*!40000 ALTER TABLE `catalog_category_product` DISABLE KEYS */;
INSERT INTO `catalog_category_product` VALUES (6,2,0);
/*!40000 ALTER TABLE `catalog_category_product` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_category_product_index`
--

DROP TABLE IF EXISTS `catalog_category_product_index`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_category_product_index` (
  `category_id` int(10) unsigned NOT NULL default '0',
  `product_id` int(10) unsigned NOT NULL default '0',
  `position` int(10) unsigned NOT NULL default '0',
  `is_parent` tinyint(1) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `visibility` tinyint(3) unsigned NOT NULL,
  UNIQUE KEY `UNQ_CATEGORY_PRODUCT` (`category_id`,`product_id`,`is_parent`,`store_id`),
  KEY `FK_CATALOG_CATEGORY_PRODUCT_INDEX_CATEGORY_ENTITY` (`category_id`),
  KEY `IDX_JOIN` (`product_id`,`store_id`,`category_id`,`visibility`),
  KEY `IDX_BASE` (`store_id`,`category_id`,`visibility`,`is_parent`,`position`),
  CONSTRAINT `FK_CATALOG_CATEGORY_PRODUCT_INDEX_CATEGORY_ENTITY` FOREIGN KEY (`category_id`) REFERENCES `catalog_category_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_CATEGORY_PRODUCT_INDEX_PRODUCT_ENTITY` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATEGORY_PRODUCT_INDEX_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_category_product_index`
--

LOCK TABLES `catalog_category_product_index` WRITE;
/*!40000 ALTER TABLE `catalog_category_product_index` DISABLE KEYS */;
INSERT INTO `catalog_category_product_index` VALUES (2,2,0,0,1,4),(4,2,0,0,1,4),(6,2,0,1,1,4);
/*!40000 ALTER TABLE `catalog_category_product_index` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_compare_item`
--

DROP TABLE IF EXISTS `catalog_compare_item`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_compare_item` (
  `catalog_compare_item_id` int(11) unsigned NOT NULL auto_increment,
  `visitor_id` int(11) unsigned NOT NULL default '0',
  `customer_id` int(11) unsigned default NULL,
  `product_id` int(11) unsigned NOT NULL default '0',
  PRIMARY KEY  (`catalog_compare_item_id`),
  KEY `FK_CATALOG_COMPARE_ITEM_CUSTOMER` (`customer_id`),
  KEY `FK_CATALOG_COMPARE_ITEM_PRODUCT` (`product_id`),
  KEY `IDX_VISITOR_PRODUCTS` (`visitor_id`,`product_id`),
  KEY `IDX_CUSTOMER_PRODUCTS` (`customer_id`,`product_id`),
  CONSTRAINT `FK_CATALOG_COMPARE_ITEM_CUSTOMER` FOREIGN KEY (`customer_id`) REFERENCES `customer_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_COMPARE_ITEM_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_compare_item`
--

LOCK TABLES `catalog_compare_item` WRITE;
/*!40000 ALTER TABLE `catalog_compare_item` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_compare_item` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_bundle_option`
--

DROP TABLE IF EXISTS `catalog_product_bundle_option`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_bundle_option` (
  `option_id` int(10) unsigned NOT NULL auto_increment,
  `parent_id` int(10) unsigned NOT NULL,
  `required` tinyint(1) unsigned NOT NULL default '0',
  `position` int(10) unsigned NOT NULL default '0',
  `type` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`option_id`),
  KEY `FK_CATALOG_PRODUCT_BUNDLE_OPTION_PARENT` (`parent_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_BUNDLE_OPTION_PARENT` FOREIGN KEY (`parent_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Bundle Options';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_bundle_option`
--

LOCK TABLES `catalog_product_bundle_option` WRITE;
/*!40000 ALTER TABLE `catalog_product_bundle_option` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_bundle_option` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_bundle_option_value`
--

DROP TABLE IF EXISTS `catalog_product_bundle_option_value`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_bundle_option_value` (
  `value_id` int(10) unsigned NOT NULL auto_increment,
  `option_id` int(10) unsigned NOT NULL,
  `store_id` smallint(5) unsigned NOT NULL,
  `title` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`value_id`),
  KEY `FK_CATALOG_PRODUCT_BUNDLE_OPTION_VALUE_OPTION` (`option_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_BUNDLE_OPTION_VALUE_OPTION` FOREIGN KEY (`option_id`) REFERENCES `catalog_product_bundle_option` (`option_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Bundle Selections';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_bundle_option_value`
--

LOCK TABLES `catalog_product_bundle_option_value` WRITE;
/*!40000 ALTER TABLE `catalog_product_bundle_option_value` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_bundle_option_value` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_bundle_price_index`
--

DROP TABLE IF EXISTS `catalog_product_bundle_price_index`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_bundle_price_index` (
  `entity_id` int(10) unsigned NOT NULL,
  `website_id` smallint(5) unsigned NOT NULL,
  `customer_group_id` smallint(3) unsigned NOT NULL,
  `min_price` decimal(12,4) NOT NULL,
  `max_price` decimal(12,4) NOT NULL,
  PRIMARY KEY  (`entity_id`,`website_id`,`customer_group_id`),
  KEY `IDX_WEBSITE` (`website_id`),
  KEY `IDX_CUSTOMER_GROUP` (`customer_group_id`),
  CONSTRAINT `CATALOG_PRODUCT_BUNDLE_PRICE_INDEX_CUSTOMER_GROUP` FOREIGN KEY (`customer_group_id`) REFERENCES `customer_group` (`customer_group_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `CATALOG_PRODUCT_BUNDLE_PRICE_INDEX_PRODUCT_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `CATALOG_PRODUCT_BUNDLE_PRICE_INDEX_WEBSITE` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_bundle_price_index`
--

LOCK TABLES `catalog_product_bundle_price_index` WRITE;
/*!40000 ALTER TABLE `catalog_product_bundle_price_index` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_bundle_price_index` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_bundle_selection`
--

DROP TABLE IF EXISTS `catalog_product_bundle_selection`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_bundle_selection` (
  `selection_id` int(10) unsigned NOT NULL auto_increment,
  `option_id` int(10) unsigned NOT NULL,
  `parent_product_id` int(10) unsigned NOT NULL,
  `product_id` int(10) unsigned NOT NULL,
  `position` int(10) unsigned NOT NULL default '0',
  `is_default` tinyint(1) unsigned NOT NULL default '0',
  `selection_price_type` tinyint(1) unsigned NOT NULL default '0',
  `selection_price_value` decimal(12,4) NOT NULL default '0.0000',
  `selection_qty` decimal(12,4) NOT NULL default '0.0000',
  `selection_can_change_qty` tinyint(1) NOT NULL default '0',
  PRIMARY KEY  (`selection_id`),
  KEY `FK_CATALOG_PRODUCT_BUNDLE_SELECTION_OPTION` (`option_id`),
  KEY `FK_CATALOG_PRODUCT_BUNDLE_SELECTION_PRODUCT` (`product_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_BUNDLE_SELECTION_OPTION` FOREIGN KEY (`option_id`) REFERENCES `catalog_product_bundle_option` (`option_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_BUNDLE_SELECTION_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Bundle Selections';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_bundle_selection`
--

LOCK TABLES `catalog_product_bundle_selection` WRITE;
/*!40000 ALTER TABLE `catalog_product_bundle_selection` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_bundle_selection` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_enabled_index`
--

DROP TABLE IF EXISTS `catalog_product_enabled_index`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_enabled_index` (
  `product_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `visibility` smallint(5) unsigned NOT NULL default '0',
  UNIQUE KEY `UNQ_PRODUCT_STORE` (`product_id`,`store_id`),
  KEY `IDX_PRODUCT_VISIBILITY_IN_STORE` (`product_id`,`store_id`,`visibility`),
  KEY `FK_CATALOG_PRODUCT_ENABLED_INDEX_STORE` (`store_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_ENABLED_INDEX_PRODUCT_ENTITY` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENABLED_INDEX_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_enabled_index`
--

LOCK TABLES `catalog_product_enabled_index` WRITE;
/*!40000 ALTER TABLE `catalog_product_enabled_index` DISABLE KEYS */;
INSERT INTO `catalog_product_enabled_index` VALUES (1,1,4),(2,1,4);
/*!40000 ALTER TABLE `catalog_product_enabled_index` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_entity`
--

DROP TABLE IF EXISTS `catalog_product_entity`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_entity` (
  `entity_id` int(10) unsigned NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_set_id` smallint(5) unsigned NOT NULL default '0',
  `type_id` varchar(32) NOT NULL default 'simple',
  `sku` varchar(64) default NULL,
  `category_ids` text,
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `has_options` smallint(1) NOT NULL default '0',
  `required_options` tinyint(1) unsigned NOT NULL default '0',
  PRIMARY KEY  (`entity_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_ATTRIBUTE_SET_ID` (`attribute_set_id`),
  KEY `sku` (`sku`),
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_ATTRIBUTE_SET_ID` FOREIGN KEY (`attribute_set_id`) REFERENCES `eav_attribute_set` (`attribute_set_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_ENTITY_TYPE` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=3 DEFAULT CHARSET=utf8 COMMENT='Product Entities';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_entity`
--

LOCK TABLES `catalog_product_entity` WRITE;
/*!40000 ALTER TABLE `catalog_product_entity` DISABLE KEYS */;
INSERT INTO `catalog_product_entity` VALUES (1,4,4,'simple','Codigo','4','2009-06-24 12:54:31','2009-06-24 17:12:42',0,0),(2,4,4,'simple','3-CHD7-RE-L','6','2009-07-02 15:54:24','2009-07-02 17:12:41',0,0);
/*!40000 ALTER TABLE `catalog_product_entity` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_entity_datetime`
--

DROP TABLE IF EXISTS `catalog_product_entity_datetime`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_entity_datetime` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`,`store_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_DATETIME_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_DATETIME_STORE` (`store_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_DATETIME_PRODUCT_ENTITY` (`entity_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_DATETIME_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_DATETIME_PRODUCT_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_DATETIME_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_entity_datetime`
--

LOCK TABLES `catalog_product_entity_datetime` WRITE;
/*!40000 ALTER TABLE `catalog_product_entity_datetime` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_entity_datetime` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_entity_decimal`
--

DROP TABLE IF EXISTS `catalog_product_entity_decimal`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_entity_decimal` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` decimal(12,4) NOT NULL default '0.0000',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`,`store_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_DECIMAL_STORE` (`store_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_DECIMAL_PRODUCT_ENTITY` (`entity_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_DECIMAL_ATTRIBUTE` (`attribute_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_DECIMAL_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_DECIMAL_PRODUCT_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_DECIMAL_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=5 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_entity_decimal`
--

LOCK TABLES `catalog_product_entity_decimal` WRITE;
/*!40000 ALTER TABLE `catalog_product_entity_decimal` DISABLE KEYS */;
INSERT INTO `catalog_product_entity_decimal` VALUES (1,4,65,0,1,'0.0000'),(2,4,60,0,1,'1.2500'),(3,4,65,0,2,'0.0000'),(4,4,60,0,2,'35.0000');
/*!40000 ALTER TABLE `catalog_product_entity_decimal` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_entity_gallery`
--

DROP TABLE IF EXISTS `catalog_product_entity_gallery`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_entity_gallery` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `position` int(11) NOT NULL default '0',
  `value` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_BASE` (`entity_type_id`,`entity_id`,`attribute_id`,`store_id`),
  KEY `FK_ATTRIBUTE_GALLERY_ENTITY` (`entity_id`),
  KEY `FK_CATALOG_CATEGORY_ENTITY_GALLERY_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CATALOG_CATEGORY_ENTITY_GALLERY_STORE` (`store_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_GALLERY_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_GALLERY_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_GALLERY_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_entity_gallery`
--

LOCK TABLES `catalog_product_entity_gallery` WRITE;
/*!40000 ALTER TABLE `catalog_product_entity_gallery` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_entity_gallery` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_entity_int`
--

DROP TABLE IF EXISTS `catalog_product_entity_int`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_entity_int` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` mediumint(8) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` int(11) NOT NULL default '0',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`,`store_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_INT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_INT_STORE` (`store_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_INT_PRODUCT_ENTITY` (`entity_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_INT_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_INT_PRODUCT_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_INT_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=9 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_entity_int`
--

LOCK TABLES `catalog_product_entity_int` WRITE;
/*!40000 ALTER TABLE `catalog_product_entity_int` DISABLE KEYS */;
INSERT INTO `catalog_product_entity_int` VALUES (1,4,80,0,1,1),(2,4,85,0,1,4),(3,4,81,0,1,2),(4,4,467,0,1,0),(5,4,80,0,2,1),(6,4,85,0,2,4),(7,4,81,0,2,2),(8,4,467,0,2,1);
/*!40000 ALTER TABLE `catalog_product_entity_int` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_entity_media_gallery`
--

DROP TABLE IF EXISTS `catalog_product_entity_media_gallery`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_entity_media_gallery` (
  `value_id` int(11) unsigned NOT NULL auto_increment,
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` varchar(255) default NULL,
  PRIMARY KEY  (`value_id`),
  KEY `FK_CATALOG_PRODUCT_MEDIA_GALLERY_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CATALOG_PRODUCT_MEDIA_GALLERY_ENTITY` (`entity_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_MEDIA_GALLERY_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_MEDIA_GALLERY_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=4 DEFAULT CHARSET=utf8 COMMENT='Catalog product media gallery';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_entity_media_gallery`
--

LOCK TABLES `catalog_product_entity_media_gallery` WRITE;
/*!40000 ALTER TABLE `catalog_product_entity_media_gallery` DISABLE KEYS */;
INSERT INTO `catalog_product_entity_media_gallery` VALUES (1,73,1,'/l/e/legumbres.jpg'),(2,73,2,'/s/n/snc00351_1.jpg'),(3,73,2,'/s/n/snc00350.jpg');
/*!40000 ALTER TABLE `catalog_product_entity_media_gallery` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_entity_media_gallery_value`
--

DROP TABLE IF EXISTS `catalog_product_entity_media_gallery_value`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_entity_media_gallery_value` (
  `value_id` int(11) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `label` varchar(255) default NULL,
  `position` int(11) unsigned default NULL,
  `disabled` tinyint(1) unsigned NOT NULL default '0',
  PRIMARY KEY  (`value_id`,`store_id`),
  KEY `FK_CATALOG_PRODUCT_MEDIA_GALLERY_VALUE_STORE` (`store_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_MEDIA_GALLERY_VALUE_GALLERY` FOREIGN KEY (`value_id`) REFERENCES `catalog_product_entity_media_gallery` (`value_id`) ON DELETE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_MEDIA_GALLERY_VALUE_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Catalog product media gallery values';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_entity_media_gallery_value`
--

LOCK TABLES `catalog_product_entity_media_gallery_value` WRITE;
/*!40000 ALTER TABLE `catalog_product_entity_media_gallery_value` DISABLE KEYS */;
INSERT INTO `catalog_product_entity_media_gallery_value` VALUES (1,0,'',1,0),(2,0,'',1,0),(3,0,'',2,0);
/*!40000 ALTER TABLE `catalog_product_entity_media_gallery_value` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_entity_text`
--

DROP TABLE IF EXISTS `catalog_product_entity_text`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_entity_text` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` mediumint(8) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` text NOT NULL,
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`,`store_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_TEXT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_TEXT_STORE` (`store_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_TEXT_PRODUCT_ENTITY` (`entity_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_TEXT_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_TEXT_PRODUCT_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_TEXT_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=9 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_entity_text`
--

LOCK TABLES `catalog_product_entity_text` WRITE;
/*!40000 ALTER TABLE `catalog_product_entity_text` DISABLE KEYS */;
INSERT INTO `catalog_product_entity_text` VALUES (1,4,57,0,1,'Decripcion'),(2,4,58,0,1,'Descripcion corta'),(3,4,68,0,1,''),(4,4,89,0,1,''),(5,4,57,0,2,'Medallón doble 7/5 redondo espiral largo'),(6,4,58,0,2,'Medallón doble 7/5 redondo espiral largo'),(7,4,68,0,2,''),(8,4,89,0,2,'');
/*!40000 ALTER TABLE `catalog_product_entity_text` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_entity_tier_price`
--

DROP TABLE IF EXISTS `catalog_product_entity_tier_price`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_entity_tier_price` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_id` int(10) unsigned NOT NULL default '0',
  `all_groups` tinyint(1) unsigned NOT NULL default '1',
  `customer_group_id` smallint(5) unsigned NOT NULL default '0',
  `qty` decimal(12,4) NOT NULL default '1.0000',
  `value` decimal(12,4) NOT NULL default '0.0000',
  `website_id` smallint(5) unsigned NOT NULL,
  PRIMARY KEY  (`value_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_TIER_PRICE_PRODUCT_ENTITY` (`entity_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_TIER_PRICE_GROUP` (`customer_group_id`),
  KEY `FK_CATALOG_PRODUCT_TIER_WEBSITE` (`website_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_TIER_PRICE_GROUP` FOREIGN KEY (`customer_group_id`) REFERENCES `customer_group` (`customer_group_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_TIER_PRICE_PRODUCT_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_TIER_WEBSITE` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_entity_tier_price`
--

LOCK TABLES `catalog_product_entity_tier_price` WRITE;
/*!40000 ALTER TABLE `catalog_product_entity_tier_price` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_entity_tier_price` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_entity_varchar`
--

DROP TABLE IF EXISTS `catalog_product_entity_varchar`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_entity_varchar` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` mediumint(8) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`,`store_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_VARCHAR_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_VARCHAR_STORE` (`store_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_VARCHAR_PRODUCT_ENTITY` (`entity_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_VARCHAR_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_VARCHAR_PRODUCT_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_ENTITY_VARCHAR_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=31 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_entity_varchar`
--

LOCK TABLES `catalog_product_entity_varchar` WRITE;
/*!40000 ALTER TABLE `catalog_product_entity_varchar` DISABLE KEYS */;
INSERT INTO `catalog_product_entity_varchar` VALUES (1,4,56,0,1,'PR1'),(2,4,82,0,1,'pr1'),(3,4,469,0,1,'2'),(4,4,67,0,1,''),(5,4,69,0,1,''),(6,4,70,0,1,'/l/e/legumbres.jpg'),(7,4,71,0,1,'/l/e/legumbres.jpg'),(8,4,72,0,1,'/l/e/legumbres.jpg'),(9,4,86,0,1,''),(10,4,90,0,1,''),(11,4,92,0,1,'container2'),(12,4,83,0,1,'pr1.html'),(13,4,95,0,1,''),(14,4,96,0,1,''),(15,4,97,0,1,''),(16,4,56,0,2,'Medallón doble vulcano espiral '),(17,4,82,0,2,'medallon-doble-vulcano-espiral'),(18,4,469,0,2,'2'),(19,4,67,0,2,''),(20,4,69,0,2,''),(21,4,70,0,2,'/s/n/snc00351_1.jpg'),(22,4,71,0,2,'/s/n/snc00351_1.jpg'),(23,4,72,0,2,'/s/n/snc00350.jpg'),(24,4,86,0,2,''),(25,4,90,0,2,''),(26,4,92,0,2,'container2'),(27,4,95,0,2,''),(28,4,96,0,2,''),(29,4,97,0,2,''),(30,4,83,0,2,'medallon-doble-vulcano-espiral.html');
/*!40000 ALTER TABLE `catalog_product_entity_varchar` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_link`
--

DROP TABLE IF EXISTS `catalog_product_link`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_link` (
  `link_id` int(11) unsigned NOT NULL auto_increment,
  `product_id` int(10) unsigned NOT NULL default '0',
  `linked_product_id` int(10) unsigned NOT NULL default '0',
  `link_type_id` tinyint(3) unsigned NOT NULL default '0',
  PRIMARY KEY  (`link_id`),
  KEY `FK_LINK_PRODUCT` (`product_id`),
  KEY `FK_LINKED_PRODUCT` (`linked_product_id`),
  KEY `FK_PRODUCT_LINK_TYPE` (`link_type_id`),
  CONSTRAINT `FK_PRODUCT_LINK_LINKED_PRODUCT` FOREIGN KEY (`linked_product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_PRODUCT_LINK_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_PRODUCT_LINK_TYPE` FOREIGN KEY (`link_type_id`) REFERENCES `catalog_product_link_type` (`link_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Related products';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_link`
--

LOCK TABLES `catalog_product_link` WRITE;
/*!40000 ALTER TABLE `catalog_product_link` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_link` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_link_attribute`
--

DROP TABLE IF EXISTS `catalog_product_link_attribute`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_link_attribute` (
  `product_link_attribute_id` smallint(6) unsigned NOT NULL auto_increment,
  `link_type_id` tinyint(3) unsigned NOT NULL default '0',
  `product_link_attribute_code` varchar(32) NOT NULL default '',
  `data_type` varchar(32) NOT NULL default '',
  PRIMARY KEY  (`product_link_attribute_id`),
  KEY `FK_ATTRIBUTE_PRODUCT_LINK_TYPE` (`link_type_id`),
  CONSTRAINT `FK_ATTRIBUTE_PRODUCT_LINK_TYPE` FOREIGN KEY (`link_type_id`) REFERENCES `catalog_product_link_type` (`link_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=9 DEFAULT CHARSET=utf8 COMMENT='Attributes for product link';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_link_attribute`
--

LOCK TABLES `catalog_product_link_attribute` WRITE;
/*!40000 ALTER TABLE `catalog_product_link_attribute` DISABLE KEYS */;
INSERT INTO `catalog_product_link_attribute` VALUES (1,2,'qty','decimal'),(2,1,'position','int'),(3,4,'position','int'),(4,5,'position','int'),(6,1,'qty','decimal'),(7,3,'position','int'),(8,3,'qty','decimal');
/*!40000 ALTER TABLE `catalog_product_link_attribute` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_link_attribute_decimal`
--

DROP TABLE IF EXISTS `catalog_product_link_attribute_decimal`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_link_attribute_decimal` (
  `value_id` int(11) unsigned NOT NULL auto_increment,
  `product_link_attribute_id` smallint(6) unsigned default NULL,
  `link_id` int(11) unsigned default NULL,
  `value` decimal(12,4) NOT NULL default '0.0000',
  PRIMARY KEY  (`value_id`),
  KEY `FK_DECIMAL_PRODUCT_LINK_ATTRIBUTE` (`product_link_attribute_id`),
  KEY `FK_DECIMAL_LINK` (`link_id`),
  CONSTRAINT `FK_DECIMAL_LINK` FOREIGN KEY (`link_id`) REFERENCES `catalog_product_link` (`link_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_DECIMAL_PRODUCT_LINK_ATTRIBUTE` FOREIGN KEY (`product_link_attribute_id`) REFERENCES `catalog_product_link_attribute` (`product_link_attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Decimal attributes values';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_link_attribute_decimal`
--

LOCK TABLES `catalog_product_link_attribute_decimal` WRITE;
/*!40000 ALTER TABLE `catalog_product_link_attribute_decimal` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_link_attribute_decimal` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_link_attribute_int`
--

DROP TABLE IF EXISTS `catalog_product_link_attribute_int`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_link_attribute_int` (
  `value_id` int(11) unsigned NOT NULL auto_increment,
  `product_link_attribute_id` smallint(6) unsigned default NULL,
  `link_id` int(11) unsigned default NULL,
  `value` int(11) NOT NULL default '0',
  PRIMARY KEY  (`value_id`),
  KEY `FK_INT_PRODUCT_LINK_ATTRIBUTE` (`product_link_attribute_id`),
  KEY `FK_INT_PRODUCT_LINK` (`link_id`),
  CONSTRAINT `FK_INT_PRODUCT_LINK` FOREIGN KEY (`link_id`) REFERENCES `catalog_product_link` (`link_id`) ON DELETE CASCADE,
  CONSTRAINT `FK_INT_PRODUCT_LINK_ATTRIBUTE` FOREIGN KEY (`product_link_attribute_id`) REFERENCES `catalog_product_link_attribute` (`product_link_attribute_id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_link_attribute_int`
--

LOCK TABLES `catalog_product_link_attribute_int` WRITE;
/*!40000 ALTER TABLE `catalog_product_link_attribute_int` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_link_attribute_int` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_link_attribute_varchar`
--

DROP TABLE IF EXISTS `catalog_product_link_attribute_varchar`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_link_attribute_varchar` (
  `value_id` int(11) unsigned NOT NULL auto_increment,
  `product_link_attribute_id` smallint(6) unsigned NOT NULL default '0',
  `link_id` int(11) unsigned default NULL,
  `value` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`value_id`),
  KEY `FK_VARCHAR_PRODUCT_LINK_ATTRIBUTE` (`product_link_attribute_id`),
  KEY `FK_VARCHAR_LINK` (`link_id`),
  CONSTRAINT `FK_VARCHAR_LINK` FOREIGN KEY (`link_id`) REFERENCES `catalog_product_link` (`link_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_VARCHAR_PRODUCT_LINK_ATTRIBUTE` FOREIGN KEY (`product_link_attribute_id`) REFERENCES `catalog_product_link_attribute` (`product_link_attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Varchar attributes values';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_link_attribute_varchar`
--

LOCK TABLES `catalog_product_link_attribute_varchar` WRITE;
/*!40000 ALTER TABLE `catalog_product_link_attribute_varchar` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_link_attribute_varchar` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_link_type`
--

DROP TABLE IF EXISTS `catalog_product_link_type`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_link_type` (
  `link_type_id` tinyint(3) unsigned NOT NULL auto_increment,
  `code` varchar(32) NOT NULL default '',
  PRIMARY KEY  (`link_type_id`)
) ENGINE=InnoDB AUTO_INCREMENT=6 DEFAULT CHARSET=utf8 COMMENT='Types of product link(Related, superproduct, bundles)';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_link_type`
--

LOCK TABLES `catalog_product_link_type` WRITE;
/*!40000 ALTER TABLE `catalog_product_link_type` DISABLE KEYS */;
INSERT INTO `catalog_product_link_type` VALUES (1,'relation'),(2,'bundle'),(3,'super'),(4,'up_sell'),(5,'cross_sell');
/*!40000 ALTER TABLE `catalog_product_link_type` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_option`
--

DROP TABLE IF EXISTS `catalog_product_option`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_option` (
  `option_id` int(10) unsigned NOT NULL auto_increment,
  `product_id` int(10) unsigned NOT NULL default '0',
  `type` varchar(50) NOT NULL default '',
  `is_require` tinyint(1) NOT NULL default '1',
  `sku` varchar(64) NOT NULL default '',
  `max_characters` int(10) unsigned default NULL,
  `file_extension` varchar(50) default NULL,
  `image_size_x` smallint(5) unsigned NOT NULL,
  `image_size_y` smallint(5) unsigned NOT NULL,
  `sort_order` int(10) unsigned NOT NULL default '0',
  PRIMARY KEY  (`option_id`),
  KEY `CATALOG_PRODUCT_OPTION_PRODUCT` (`product_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_OPTION_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_option`
--

LOCK TABLES `catalog_product_option` WRITE;
/*!40000 ALTER TABLE `catalog_product_option` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_option` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_option_price`
--

DROP TABLE IF EXISTS `catalog_product_option_price`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_option_price` (
  `option_price_id` int(10) unsigned NOT NULL auto_increment,
  `option_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `price` decimal(12,4) NOT NULL default '0.0000',
  `price_type` enum('fixed','percent') NOT NULL default 'fixed',
  PRIMARY KEY  (`option_price_id`),
  KEY `CATALOG_PRODUCT_OPTION_PRICE_OPTION` (`option_id`),
  KEY `CATALOG_PRODUCT_OPTION_TITLE_STORE` (`store_id`),
  KEY `IDX_CATALOG_PRODUCT_OPTION_PRICE_SI_OI` (`store_id`,`option_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_OPTION_PRICE_OPTION` FOREIGN KEY (`option_id`) REFERENCES `catalog_product_option` (`option_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_OPTION_PRICE_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_option_price`
--

LOCK TABLES `catalog_product_option_price` WRITE;
/*!40000 ALTER TABLE `catalog_product_option_price` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_option_price` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_option_title`
--

DROP TABLE IF EXISTS `catalog_product_option_title`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_option_title` (
  `option_title_id` int(10) unsigned NOT NULL auto_increment,
  `option_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `title` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`option_title_id`),
  KEY `CATALOG_PRODUCT_OPTION_TITLE_OPTION` (`option_id`),
  KEY `CATALOG_PRODUCT_OPTION_TITLE_STORE` (`store_id`),
  KEY `IDX_CATALOG_PRODUCT_OPTION_TITLE_SI_OI` (`store_id`,`option_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_OPTION_TITLE_OPTION` FOREIGN KEY (`option_id`) REFERENCES `catalog_product_option` (`option_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_OPTION_TITLE_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_option_title`
--

LOCK TABLES `catalog_product_option_title` WRITE;
/*!40000 ALTER TABLE `catalog_product_option_title` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_option_title` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_option_type_price`
--

DROP TABLE IF EXISTS `catalog_product_option_type_price`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_option_type_price` (
  `option_type_price_id` int(10) unsigned NOT NULL auto_increment,
  `option_type_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `price` decimal(12,4) NOT NULL default '0.0000',
  `price_type` enum('fixed','percent') NOT NULL default 'fixed',
  PRIMARY KEY  (`option_type_price_id`),
  KEY `CATALOG_PRODUCT_OPTION_TYPE_PRICE_OPTION_TYPE` (`option_type_id`),
  KEY `CATALOG_PRODUCT_OPTION_TYPE_PRICE_STORE` (`store_id`),
  KEY `IDX_CATALOG_PRODUCT_OPTION_TYPE_PRICE_SI_OTI` (`store_id`,`option_type_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_OPTION_TYPE_PRICE_OPTION` FOREIGN KEY (`option_type_id`) REFERENCES `catalog_product_option_type_value` (`option_type_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_OPTION_TYPE_PRICE_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_option_type_price`
--

LOCK TABLES `catalog_product_option_type_price` WRITE;
/*!40000 ALTER TABLE `catalog_product_option_type_price` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_option_type_price` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_option_type_title`
--

DROP TABLE IF EXISTS `catalog_product_option_type_title`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_option_type_title` (
  `option_type_title_id` int(10) unsigned NOT NULL auto_increment,
  `option_type_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `title` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`option_type_title_id`),
  KEY `CATALOG_PRODUCT_OPTION_TYPE_TITLE_OPTION` (`option_type_id`),
  KEY `CATALOG_PRODUCT_OPTION_TYPE_TITLE_STORE` (`store_id`),
  KEY `IDX_CATALOG_PRODUCT_OPTION_TYPE_TITLE_SI_OTI` (`store_id`,`option_type_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_OPTION_TYPE_TITLE_OPTION` FOREIGN KEY (`option_type_id`) REFERENCES `catalog_product_option_type_value` (`option_type_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_PRODUCT_OPTION_TYPE_TITLE_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_option_type_title`
--

LOCK TABLES `catalog_product_option_type_title` WRITE;
/*!40000 ALTER TABLE `catalog_product_option_type_title` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_option_type_title` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_option_type_value`
--

DROP TABLE IF EXISTS `catalog_product_option_type_value`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_option_type_value` (
  `option_type_id` int(10) unsigned NOT NULL auto_increment,
  `option_id` int(10) unsigned NOT NULL default '0',
  `sku` varchar(64) NOT NULL default '',
  `sort_order` int(10) unsigned NOT NULL default '0',
  PRIMARY KEY  (`option_type_id`),
  KEY `CATALOG_PRODUCT_OPTION_TYPE_VALUE_OPTION` (`option_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_OPTION_TYPE_VALUE_OPTION` FOREIGN KEY (`option_id`) REFERENCES `catalog_product_option` (`option_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_option_type_value`
--

LOCK TABLES `catalog_product_option_type_value` WRITE;
/*!40000 ALTER TABLE `catalog_product_option_type_value` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_option_type_value` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_super_attribute`
--

DROP TABLE IF EXISTS `catalog_product_super_attribute`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_super_attribute` (
  `product_super_attribute_id` int(10) unsigned NOT NULL auto_increment,
  `product_id` int(10) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `position` smallint(5) unsigned NOT NULL default '0',
  PRIMARY KEY  (`product_super_attribute_id`),
  KEY `FK_SUPER_PRODUCT_ATTRIBUTE_PRODUCT` (`product_id`),
  CONSTRAINT `FK_SUPER_PRODUCT_ATTRIBUTE_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_super_attribute`
--

LOCK TABLES `catalog_product_super_attribute` WRITE;
/*!40000 ALTER TABLE `catalog_product_super_attribute` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_super_attribute` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_super_attribute_label`
--

DROP TABLE IF EXISTS `catalog_product_super_attribute_label`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_super_attribute_label` (
  `value_id` int(10) unsigned NOT NULL auto_increment,
  `product_super_attribute_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `value` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`value_id`),
  KEY `FK_SUPER_PRODUCT_ATTRIBUTE_LABEL` (`product_super_attribute_id`),
  KEY `IDX_CATALOG_PRODUCT_SUPER_ATTRIBUTE_STORE_PSAI_SI` (`product_super_attribute_id`,`store_id`),
  CONSTRAINT `FK_SUPER_PRODUCT_ATTRIBUTE_LABEL` FOREIGN KEY (`product_super_attribute_id`) REFERENCES `catalog_product_super_attribute` (`product_super_attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_super_attribute_label`
--

LOCK TABLES `catalog_product_super_attribute_label` WRITE;
/*!40000 ALTER TABLE `catalog_product_super_attribute_label` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_super_attribute_label` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_super_attribute_pricing`
--

DROP TABLE IF EXISTS `catalog_product_super_attribute_pricing`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_super_attribute_pricing` (
  `value_id` int(10) unsigned NOT NULL auto_increment,
  `product_super_attribute_id` int(10) unsigned NOT NULL default '0',
  `value_index` varchar(255) NOT NULL default '',
  `is_percent` tinyint(1) unsigned default '0',
  `pricing_value` decimal(10,4) default NULL,
  `website_id` smallint(5) unsigned NOT NULL default '0',
  PRIMARY KEY  (`value_id`),
  KEY `FK_SUPER_PRODUCT_ATTRIBUTE_PRICING` (`product_super_attribute_id`),
  KEY `FK_CATALOG_PRODUCT_SUPER_PRICE_WEBSITE` (`website_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_SUPER_PRICE_WEBSITE` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_SUPER_PRODUCT_ATTRIBUTE_PRICING` FOREIGN KEY (`product_super_attribute_id`) REFERENCES `catalog_product_super_attribute` (`product_super_attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_super_attribute_pricing`
--

LOCK TABLES `catalog_product_super_attribute_pricing` WRITE;
/*!40000 ALTER TABLE `catalog_product_super_attribute_pricing` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_super_attribute_pricing` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_super_link`
--

DROP TABLE IF EXISTS `catalog_product_super_link`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_super_link` (
  `link_id` int(10) unsigned NOT NULL auto_increment,
  `product_id` int(10) unsigned NOT NULL default '0',
  `parent_id` int(10) unsigned NOT NULL default '0',
  PRIMARY KEY  (`link_id`),
  KEY `FK_SUPER_PRODUCT_LINK_PARENT` (`parent_id`),
  KEY `FK_catalog_product_super_link` (`product_id`),
  CONSTRAINT `FK_SUPER_PRODUCT_LINK_ENTITY` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_SUPER_PRODUCT_LINK_PARENT` FOREIGN KEY (`parent_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_super_link`
--

LOCK TABLES `catalog_product_super_link` WRITE;
/*!40000 ALTER TABLE `catalog_product_super_link` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalog_product_super_link` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalog_product_website`
--

DROP TABLE IF EXISTS `catalog_product_website`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalog_product_website` (
  `product_id` int(10) unsigned NOT NULL auto_increment,
  `website_id` smallint(5) unsigned NOT NULL,
  PRIMARY KEY  (`product_id`,`website_id`),
  KEY `FK_CATALOG_PRODUCT_WEBSITE_WEBSITE` (`website_id`),
  CONSTRAINT `FK_CATALOG_PRODUCT_WEBSITE_WEBSITE` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOG_WEBSITE_PRODUCT_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=3 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalog_product_website`
--

LOCK TABLES `catalog_product_website` WRITE;
/*!40000 ALTER TABLE `catalog_product_website` DISABLE KEYS */;
INSERT INTO `catalog_product_website` VALUES (1,1),(2,1);
/*!40000 ALTER TABLE `catalog_product_website` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalogindex_aggregation`
--

DROP TABLE IF EXISTS `catalogindex_aggregation`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalogindex_aggregation` (
  `aggregation_id` int(10) unsigned NOT NULL auto_increment,
  `store_id` smallint(5) unsigned NOT NULL,
  `created_at` datetime NOT NULL,
  `key` varchar(255) default NULL,
  `data` mediumtext,
  PRIMARY KEY  (`aggregation_id`),
  UNIQUE KEY `IDX_STORE_KEY` (`store_id`,`key`),
  CONSTRAINT `FK_CATALOGINDEX_AGGREGATION_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalogindex_aggregation`
--

LOCK TABLES `catalogindex_aggregation` WRITE;
/*!40000 ALTER TABLE `catalogindex_aggregation` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalogindex_aggregation` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalogindex_aggregation_tag`
--

DROP TABLE IF EXISTS `catalogindex_aggregation_tag`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalogindex_aggregation_tag` (
  `tag_id` int(10) unsigned NOT NULL auto_increment,
  `tag_code` varchar(255) NOT NULL,
  PRIMARY KEY  (`tag_id`),
  UNIQUE KEY `IDX_CODE` (`tag_code`)
) ENGINE=InnoDB AUTO_INCREMENT=16 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalogindex_aggregation_tag`
--

LOCK TABLES `catalogindex_aggregation_tag` WRITE;
/*!40000 ALTER TABLE `catalogindex_aggregation_tag` DISABLE KEYS */;
INSERT INTO `catalogindex_aggregation_tag` VALUES (1,'catalog_category1'),(12,'catalog_category10'),(13,'catalog_category11'),(14,'catalog_category12'),(15,'catalog_category13'),(3,'catalog_category2'),(2,'catalog_category3'),(4,'catalog_category4'),(7,'catalog_category5'),(8,'catalog_category6'),(9,'catalog_category7'),(10,'catalog_category8'),(11,'catalog_category9'),(6,'PRODUCT_PRICE'),(5,'SEARCH_QUERY');
/*!40000 ALTER TABLE `catalogindex_aggregation_tag` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalogindex_aggregation_to_tag`
--

DROP TABLE IF EXISTS `catalogindex_aggregation_to_tag`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalogindex_aggregation_to_tag` (
  `aggregation_id` int(10) unsigned NOT NULL,
  `tag_id` int(10) unsigned NOT NULL,
  UNIQUE KEY `IDX_AGGREGATION_TAG` (`aggregation_id`,`tag_id`),
  KEY `FK_CATALOGINDEX_AGGREGATION_TO_TAG_TAG` (`tag_id`),
  CONSTRAINT `FK_CATALOGINDEX_AGGREGATION_TO_TAG_AGGREGATION` FOREIGN KEY (`aggregation_id`) REFERENCES `catalogindex_aggregation` (`aggregation_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOGINDEX_AGGREGATION_TO_TAG_TAG` FOREIGN KEY (`tag_id`) REFERENCES `catalogindex_aggregation_tag` (`tag_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalogindex_aggregation_to_tag`
--

LOCK TABLES `catalogindex_aggregation_to_tag` WRITE;
/*!40000 ALTER TABLE `catalogindex_aggregation_to_tag` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalogindex_aggregation_to_tag` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalogindex_eav`
--

DROP TABLE IF EXISTS `catalogindex_eav`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalogindex_eav` (
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `value` int(11) NOT NULL default '0',
  PRIMARY KEY  (`store_id`,`entity_id`,`attribute_id`,`value`),
  KEY `IDX_VALUE` (`value`),
  KEY `FK_CATALOGINDEX_EAV_ENTITY` (`entity_id`),
  KEY `FK_CATALOGINDEX_EAV_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CATALOGINDEX_EAV_STORE` (`store_id`),
  CONSTRAINT `FK_CATALOGINDEX_EAV_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOGINDEX_EAV_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOGINDEX_EAV_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalogindex_eav`
--

LOCK TABLES `catalogindex_eav` WRITE;
/*!40000 ALTER TABLE `catalogindex_eav` DISABLE KEYS */;
INSERT INTO `catalogindex_eav` VALUES (1,1,81,2),(1,2,81,2);
/*!40000 ALTER TABLE `catalogindex_eav` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalogindex_minimal_price`
--

DROP TABLE IF EXISTS `catalogindex_minimal_price`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalogindex_minimal_price` (
  `index_id` int(10) unsigned NOT NULL auto_increment,
  `entity_id` int(10) unsigned NOT NULL default '0',
  `customer_group_id` smallint(3) unsigned NOT NULL default '0',
  `qty` decimal(12,4) unsigned NOT NULL default '0.0000',
  `value` decimal(12,4) NOT NULL default '0.0000',
  `tax_class_id` smallint(6) NOT NULL default '0',
  `website_id` smallint(5) unsigned default NULL,
  PRIMARY KEY  (`index_id`),
  KEY `IDX_VALUE` (`value`),
  KEY `IDX_QTY` (`qty`),
  KEY `FK_CATALOGINDEX_MINIMAL_PRICE_CUSTOMER_GROUP` (`customer_group_id`),
  KEY `FK_CI_MINIMAL_PRICE_WEBSITE_ID` (`website_id`),
  KEY `IDX_FULL` (`entity_id`,`qty`,`customer_group_id`,`value`,`website_id`),
  CONSTRAINT `FK_CATALOGINDEX_MINIMAL_PRICE_CUSTOMER_GROUP` FOREIGN KEY (`customer_group_id`) REFERENCES `customer_group` (`customer_group_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOGINDEX_MINIMAL_PRICE_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CI_MINIMAL_PRICE_WEBSITE_ID` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalogindex_minimal_price`
--

LOCK TABLES `catalogindex_minimal_price` WRITE;
/*!40000 ALTER TABLE `catalogindex_minimal_price` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalogindex_minimal_price` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalogindex_price`
--

DROP TABLE IF EXISTS `catalogindex_price`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalogindex_price` (
  `entity_id` int(10) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `customer_group_id` smallint(3) unsigned NOT NULL default '0',
  `qty` decimal(12,4) unsigned NOT NULL default '0.0000',
  `value` decimal(12,4) NOT NULL default '0.0000',
  `tax_class_id` smallint(6) NOT NULL default '0',
  `website_id` smallint(5) unsigned default NULL,
  KEY `IDX_VALUE` (`value`),
  KEY `IDX_QTY` (`qty`),
  KEY `FK_CATALOGINDEX_PRICE_ENTITY` (`entity_id`),
  KEY `FK_CATALOGINDEX_PRICE_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CATALOGINDEX_PRICE_CUSTOMER_GROUP` (`customer_group_id`),
  KEY `IDX_RANGE_VALUE` (`entity_id`,`attribute_id`,`customer_group_id`,`value`),
  KEY `FK_CI_PRICE_WEBSITE_ID` (`website_id`),
  KEY `IDX_FULL` (`entity_id`,`attribute_id`,`customer_group_id`,`value`,`website_id`),
  CONSTRAINT `FK_CATALOGINDEX_PRICE_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOGINDEX_PRICE_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CI_PRICE_WEBSITE_ID` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalogindex_price`
--

LOCK TABLES `catalogindex_price` WRITE;
/*!40000 ALTER TABLE `catalogindex_price` DISABLE KEYS */;
INSERT INTO `catalogindex_price` VALUES (1,60,0,'0.0000','1.2500',2,1),(1,60,1,'0.0000','1.2500',2,1),(1,60,2,'0.0000','1.2500',2,1),(1,60,3,'0.0000','1.2500',2,1),(2,60,0,'0.0000','35.0000',2,1),(2,60,1,'0.0000','35.0000',2,1),(2,60,2,'0.0000','35.0000',2,1),(2,60,3,'0.0000','35.0000',2,1);
/*!40000 ALTER TABLE `catalogindex_price` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `cataloginventory_stock`
--

DROP TABLE IF EXISTS `cataloginventory_stock`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `cataloginventory_stock` (
  `stock_id` smallint(4) unsigned NOT NULL auto_increment,
  `stock_name` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`stock_id`)
) ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=utf8 COMMENT='Catalog inventory Stocks list';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `cataloginventory_stock`
--

LOCK TABLES `cataloginventory_stock` WRITE;
/*!40000 ALTER TABLE `cataloginventory_stock` DISABLE KEYS */;
INSERT INTO `cataloginventory_stock` VALUES (1,'Default');
/*!40000 ALTER TABLE `cataloginventory_stock` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `cataloginventory_stock_item`
--

DROP TABLE IF EXISTS `cataloginventory_stock_item`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `cataloginventory_stock_item` (
  `item_id` int(10) unsigned NOT NULL auto_increment,
  `product_id` int(10) unsigned NOT NULL default '0',
  `stock_id` smallint(4) unsigned NOT NULL default '0',
  `qty` decimal(12,4) NOT NULL default '0.0000',
  `min_qty` decimal(12,4) NOT NULL default '0.0000',
  `use_config_min_qty` tinyint(1) unsigned NOT NULL default '1',
  `is_qty_decimal` tinyint(1) unsigned NOT NULL default '0',
  `backorders` tinyint(3) unsigned NOT NULL default '0',
  `use_config_backorders` tinyint(1) unsigned NOT NULL default '1',
  `min_sale_qty` decimal(12,4) NOT NULL default '1.0000',
  `use_config_min_sale_qty` tinyint(1) unsigned NOT NULL default '1',
  `max_sale_qty` decimal(12,4) NOT NULL default '0.0000',
  `use_config_max_sale_qty` tinyint(1) unsigned NOT NULL default '1',
  `is_in_stock` tinyint(1) unsigned NOT NULL default '0',
  `low_stock_date` datetime default NULL,
  `notify_stock_qty` decimal(12,4) default NULL,
  `use_config_notify_stock_qty` tinyint(1) unsigned NOT NULL default '1',
  `manage_stock` tinyint(1) unsigned NOT NULL default '0',
  `use_config_manage_stock` tinyint(1) unsigned NOT NULL default '1',
  `stock_status_changed_automatically` tinyint(1) unsigned NOT NULL default '0',
  PRIMARY KEY  (`item_id`),
  UNIQUE KEY `IDX_STOCK_PRODUCT` (`product_id`,`stock_id`),
  KEY `FK_CATALOGINVENTORY_STOCK_ITEM_PRODUCT` (`product_id`),
  KEY `FK_CATALOGINVENTORY_STOCK_ITEM_STOCK` (`stock_id`),
  CONSTRAINT `FK_CATALOGINVENTORY_STOCK_ITEM_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOGINVENTORY_STOCK_ITEM_STOCK` FOREIGN KEY (`stock_id`) REFERENCES `cataloginventory_stock` (`stock_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=3 DEFAULT CHARSET=utf8 COMMENT='Inventory Stock Item Data';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `cataloginventory_stock_item`
--

LOCK TABLES `cataloginventory_stock_item` WRITE;
/*!40000 ALTER TABLE `cataloginventory_stock_item` DISABLE KEYS */;
INSERT INTO `cataloginventory_stock_item` VALUES (1,1,1,'0.0000','0.0000',1,0,0,1,'1.0000',1,'0.0000',1,0,'2009-06-24 17:12:43',NULL,1,0,1,1),(2,2,1,'3.0000','0.0000',1,0,0,1,'1.0000',1,'0.0000',1,0,NULL,NULL,1,0,1,0);
/*!40000 ALTER TABLE `cataloginventory_stock_item` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `cataloginventory_stock_status`
--

DROP TABLE IF EXISTS `cataloginventory_stock_status`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `cataloginventory_stock_status` (
  `product_id` int(10) unsigned NOT NULL,
  `website_id` smallint(5) unsigned NOT NULL,
  `stock_id` smallint(4) unsigned NOT NULL,
  `qty` decimal(12,4) NOT NULL default '0.0000',
  `stock_status` tinyint(3) unsigned NOT NULL,
  PRIMARY KEY  (`product_id`,`website_id`,`stock_id`),
  KEY `FK_CATALOGINVENTORY_STOCK_STATUS_STOCK` (`stock_id`),
  KEY `FK_CATALOGINVENTORY_STOCK_STATUS_WEBSITE` (`website_id`),
  CONSTRAINT `FK_CATALOGINVENTORY_STOCK_STATUS_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOGINVENTORY_STOCK_STATUS_STOCK` FOREIGN KEY (`stock_id`) REFERENCES `cataloginventory_stock` (`stock_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOGINVENTORY_STOCK_STATUS_WEBSITE` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `cataloginventory_stock_status`
--

LOCK TABLES `cataloginventory_stock_status` WRITE;
/*!40000 ALTER TABLE `cataloginventory_stock_status` DISABLE KEYS */;
INSERT INTO `cataloginventory_stock_status` VALUES (1,1,1,'0.0000',0),(2,1,1,'3.0000',0);
/*!40000 ALTER TABLE `cataloginventory_stock_status` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalogrule`
--

DROP TABLE IF EXISTS `catalogrule`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalogrule` (
  `rule_id` int(10) unsigned NOT NULL auto_increment,
  `name` varchar(255) NOT NULL default '',
  `description` text NOT NULL,
  `from_date` date default NULL,
  `to_date` date default NULL,
  `customer_group_ids` varchar(255) NOT NULL default '',
  `is_active` tinyint(1) NOT NULL default '0',
  `conditions_serialized` mediumtext NOT NULL,
  `actions_serialized` mediumtext NOT NULL,
  `stop_rules_processing` tinyint(1) NOT NULL default '1',
  `sort_order` int(10) unsigned NOT NULL default '0',
  `simple_action` varchar(32) NOT NULL,
  `discount_amount` decimal(12,4) NOT NULL,
  `website_ids` text,
  PRIMARY KEY  (`rule_id`),
  KEY `sort_order` (`is_active`,`sort_order`,`to_date`,`from_date`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalogrule`
--

LOCK TABLES `catalogrule` WRITE;
/*!40000 ALTER TABLE `catalogrule` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalogrule` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalogrule_affected_product`
--

DROP TABLE IF EXISTS `catalogrule_affected_product`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalogrule_affected_product` (
  `product_id` int(10) unsigned NOT NULL,
  PRIMARY KEY  (`product_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalogrule_affected_product`
--

LOCK TABLES `catalogrule_affected_product` WRITE;
/*!40000 ALTER TABLE `catalogrule_affected_product` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalogrule_affected_product` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalogrule_product`
--

DROP TABLE IF EXISTS `catalogrule_product`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalogrule_product` (
  `rule_product_id` int(10) unsigned NOT NULL auto_increment,
  `rule_id` int(10) unsigned NOT NULL default '0',
  `from_time` int(10) unsigned NOT NULL default '0',
  `to_time` int(10) unsigned NOT NULL default '0',
  `customer_group_id` smallint(5) unsigned NOT NULL default '0',
  `product_id` int(10) unsigned NOT NULL default '0',
  `action_operator` enum('to_fixed','to_percent','by_fixed','by_percent') NOT NULL default 'to_fixed',
  `action_amount` decimal(12,4) NOT NULL default '0.0000',
  `action_stop` tinyint(1) NOT NULL default '0',
  `sort_order` int(10) unsigned NOT NULL default '0',
  `website_id` smallint(5) unsigned NOT NULL,
  PRIMARY KEY  (`rule_product_id`),
  UNIQUE KEY `sort_order` (`rule_id`,`from_time`,`to_time`,`website_id`,`customer_group_id`,`product_id`,`sort_order`),
  KEY `FK_catalogrule_product_rule` (`rule_id`),
  KEY `FK_catalogrule_product_customergroup` (`customer_group_id`),
  KEY `FK_catalogrule_product_website` (`website_id`),
  KEY `FK_CATALOGRULE_PRODUCT_PRODUCT` (`product_id`),
  CONSTRAINT `FK_catalogrule_product_customergroup` FOREIGN KEY (`customer_group_id`) REFERENCES `customer_group` (`customer_group_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOGRULE_PRODUCT_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_catalogrule_product_rule` FOREIGN KEY (`rule_id`) REFERENCES `catalogrule` (`rule_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_catalogrule_product_website` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalogrule_product`
--

LOCK TABLES `catalogrule_product` WRITE;
/*!40000 ALTER TABLE `catalogrule_product` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalogrule_product` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalogrule_product_price`
--

DROP TABLE IF EXISTS `catalogrule_product_price`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalogrule_product_price` (
  `rule_product_price_id` int(10) unsigned NOT NULL auto_increment,
  `rule_date` date NOT NULL default '0000-00-00',
  `customer_group_id` smallint(5) unsigned NOT NULL default '0',
  `product_id` int(10) unsigned NOT NULL default '0',
  `rule_price` decimal(12,4) NOT NULL default '0.0000',
  `website_id` smallint(5) unsigned NOT NULL,
  `latest_start_date` date default NULL,
  `earliest_end_date` date default NULL,
  PRIMARY KEY  (`rule_product_price_id`),
  UNIQUE KEY `rule_date` (`rule_date`,`website_id`,`customer_group_id`,`product_id`),
  KEY `FK_catalogrule_product_price_customergroup` (`customer_group_id`),
  KEY `FK_catalogrule_product_price_website` (`website_id`),
  KEY `FK_CATALOGRULE_PRODUCT_PRICE_PRODUCT` (`product_id`),
  CONSTRAINT `FK_catalogrule_product_price_customergroup` FOREIGN KEY (`customer_group_id`) REFERENCES `customer_group` (`customer_group_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOGRULE_PRODUCT_PRICE_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_catalogrule_product_price_website` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalogrule_product_price`
--

LOCK TABLES `catalogrule_product_price` WRITE;
/*!40000 ALTER TABLE `catalogrule_product_price` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalogrule_product_price` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalogsearch_fulltext`
--

DROP TABLE IF EXISTS `catalogsearch_fulltext`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalogsearch_fulltext` (
  `product_id` int(10) unsigned NOT NULL,
  `store_id` smallint(5) unsigned NOT NULL,
  `data_index` longtext NOT NULL,
  PRIMARY KEY  (`product_id`,`store_id`),
  FULLTEXT KEY `data_index` (`data_index`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalogsearch_fulltext`
--

LOCK TABLES `catalogsearch_fulltext` WRITE;
/*!40000 ALTER TABLE `catalogsearch_fulltext` DISABLE KEYS */;
INSERT INTO `catalogsearch_fulltext` VALUES (1,1,'Codigo Habilitado IVA General PR1 Decripcion Descripcion corta'),(2,1,'3-CHD7-RE-L Habilitado IVA General Medallón doble vulcano espiral Medallón doble 7/5 redondo espiral largo Medallón doble 7/5 redondo espiral largo');
/*!40000 ALTER TABLE `catalogsearch_fulltext` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalogsearch_query`
--

DROP TABLE IF EXISTS `catalogsearch_query`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalogsearch_query` (
  `query_id` int(10) unsigned NOT NULL auto_increment,
  `query_text` varchar(255) NOT NULL default '',
  `num_results` int(10) unsigned NOT NULL default '0',
  `popularity` int(10) unsigned NOT NULL default '0',
  `redirect` varchar(255) NOT NULL default '',
  `synonim_for` varchar(255) NOT NULL default '',
  `store_id` smallint(5) unsigned NOT NULL,
  `display_in_terms` tinyint(1) NOT NULL default '1',
  `is_active` tinyint(1) default '1',
  `is_processed` tinyint(1) default '0',
  `updated_at` datetime NOT NULL,
  PRIMARY KEY  (`query_id`),
  KEY `FK_CATALOGSEARCH_QUERY_STORE` (`store_id`),
  KEY `IDX_SEARCH_QUERY` (`query_text`,`store_id`,`popularity`),
  CONSTRAINT `FK_CATALOGSEARCH_QUERY_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=3 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalogsearch_query`
--

LOCK TABLES `catalogsearch_query` WRITE;
/*!40000 ALTER TABLE `catalogsearch_query` DISABLE KEYS */;
INSERT INTO `catalogsearch_query` VALUES (1,'lk',0,1,'','',1,1,1,0,'2009-06-24 17:40:47'),(2,'lkkjk',0,1,'','',1,1,1,0,'2009-06-24 17:40:57');
/*!40000 ALTER TABLE `catalogsearch_query` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `catalogsearch_result`
--

DROP TABLE IF EXISTS `catalogsearch_result`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `catalogsearch_result` (
  `query_id` int(10) unsigned NOT NULL,
  `product_id` int(10) unsigned NOT NULL,
  `relevance` decimal(6,4) NOT NULL default '0.0000',
  PRIMARY KEY  (`query_id`,`product_id`),
  KEY `IDX_QUERY` (`query_id`),
  KEY `IDX_PRODUCT` (`product_id`),
  KEY `IDX_RELEVANCE` (`query_id`,`relevance`),
  CONSTRAINT `FK_CATALOGSEARCH_RESULT_CATALOG_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CATALOGSEARCH_RESULT_QUERY` FOREIGN KEY (`query_id`) REFERENCES `catalogsearch_query` (`query_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `catalogsearch_result`
--

LOCK TABLES `catalogsearch_result` WRITE;
/*!40000 ALTER TABLE `catalogsearch_result` DISABLE KEYS */;
/*!40000 ALTER TABLE `catalogsearch_result` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `checkout_agreement`
--

DROP TABLE IF EXISTS `checkout_agreement`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `checkout_agreement` (
  `agreement_id` int(10) unsigned NOT NULL auto_increment,
  `name` varchar(255) NOT NULL default '',
  `content` text NOT NULL,
  `content_height` varchar(25) default NULL,
  `checkbox_text` text NOT NULL,
  `is_active` tinyint(4) NOT NULL default '0',
  `is_html` tinyint(4) NOT NULL default '0',
  PRIMARY KEY  (`agreement_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `checkout_agreement`
--

LOCK TABLES `checkout_agreement` WRITE;
/*!40000 ALTER TABLE `checkout_agreement` DISABLE KEYS */;
/*!40000 ALTER TABLE `checkout_agreement` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `checkout_agreement_store`
--

DROP TABLE IF EXISTS `checkout_agreement_store`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `checkout_agreement_store` (
  `agreement_id` int(10) unsigned NOT NULL,
  `store_id` smallint(5) unsigned NOT NULL,
  UNIQUE KEY `agreement_id` (`agreement_id`,`store_id`),
  KEY `FK_CHECKOUT_AGREEMENT_STORE` (`store_id`),
  CONSTRAINT `FK_CHECKOUT_AGREEMENT` FOREIGN KEY (`agreement_id`) REFERENCES `checkout_agreement` (`agreement_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CHECKOUT_AGREEMENT_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `checkout_agreement_store`
--

LOCK TABLES `checkout_agreement_store` WRITE;
/*!40000 ALTER TABLE `checkout_agreement_store` DISABLE KEYS */;
/*!40000 ALTER TABLE `checkout_agreement_store` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `cms_block`
--

DROP TABLE IF EXISTS `cms_block`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `cms_block` (
  `block_id` smallint(6) NOT NULL auto_increment,
  `title` varchar(255) NOT NULL default '',
  `identifier` varchar(255) NOT NULL default '',
  `content` text,
  `creation_time` datetime default NULL,
  `update_time` datetime default NULL,
  `is_active` tinyint(1) NOT NULL default '1',
  PRIMARY KEY  (`block_id`)
) ENGINE=InnoDB AUTO_INCREMENT=6 DEFAULT CHARSET=utf8 COMMENT='CMS Blocks';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `cms_block`
--

LOCK TABLES `cms_block` WRITE;
/*!40000 ALTER TABLE `cms_block` DISABLE KEYS */;
INSERT INTO `cms_block` VALUES (5,'Footer Links','footer_links','<ul>\r\n<li><a href=\"{{store url=\"\"}}About-us\">Quien somos</a></li>\r\n<li class=\"last\"><a href=\"{{store url=\"\"}}customer-service\">Servicio a clientes</a></li>\r\n<li class=\"last\"><a href=\"{{store url=\"\"}}condiciones-generales\">Condiciones generales de venta</a></li>\r\n<li class=\"last\"><a href=\"{{store url=\"\"}}garantia-devolucion\">Garantia y devoluciones</a></li>\r\n</ul>','2009-06-03 13:53:29','2009-06-24 23:01:01',1);
/*!40000 ALTER TABLE `cms_block` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `cms_block_store`
--

DROP TABLE IF EXISTS `cms_block_store`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `cms_block_store` (
  `block_id` smallint(6) NOT NULL,
  `store_id` smallint(5) unsigned NOT NULL,
  PRIMARY KEY  (`block_id`,`store_id`),
  KEY `FK_CMS_BLOCK_STORE_STORE` (`store_id`),
  CONSTRAINT `FK_CMS_BLOCK_STORE_BLOCK` FOREIGN KEY (`block_id`) REFERENCES `cms_block` (`block_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CMS_BLOCK_STORE_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='CMS Blocks to Stores';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `cms_block_store`
--

LOCK TABLES `cms_block_store` WRITE;
/*!40000 ALTER TABLE `cms_block_store` DISABLE KEYS */;
INSERT INTO `cms_block_store` VALUES (5,1);
/*!40000 ALTER TABLE `cms_block_store` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `cms_page`
--

DROP TABLE IF EXISTS `cms_page`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `cms_page` (
  `page_id` smallint(6) NOT NULL auto_increment,
  `title` varchar(255) NOT NULL default '',
  `root_template` varchar(255) NOT NULL default '',
  `meta_keywords` text NOT NULL,
  `meta_description` text NOT NULL,
  `identifier` varchar(100) NOT NULL default '',
  `content` text,
  `creation_time` datetime default NULL,
  `update_time` datetime default NULL,
  `is_active` tinyint(1) NOT NULL default '1',
  `sort_order` tinyint(4) NOT NULL default '0',
  `layout_update_xml` text,
  `custom_theme` varchar(100) default NULL,
  `custom_theme_from` date default NULL,
  `custom_theme_to` date default NULL,
  PRIMARY KEY  (`page_id`),
  KEY `identifier` (`identifier`)
) ENGINE=InnoDB AUTO_INCREMENT=8 DEFAULT CHARSET=utf8 COMMENT='CMS pages';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `cms_page`
--

LOCK TABLES `cms_page` WRITE;
/*!40000 ALTER TABLE `cms_page` DISABLE KEYS */;
INSERT INTO `cms_page` VALUES (1,'404 Not Found 1','two_columns_right','Page keywords','Page description','no-route','<div class=\"page-head-alt\"><h3>Whoops, our bad...</h3></div>\r\n<dl>\r\n<dt>The page you requested was not found, and we have a fine guess why.</dt>\r\n<dd>\r\n<ul class=\"disc\">\r\n<li>If you typed the URL directly, please make sure the spelling is correct.</li>\r\n<li>If you clicked on a link to get here, the link is outdated.</li>\r\n</ul></dd>\r\n</dl>\r\n<br/>\r\n<dl>\r\n<dt>What can you do?</dt>\r\n<dd>Have no fear, help is near! There are many ways you can get back on track with Magento Demo Store.</dd>\r\n<dd>\r\n<ul class=\"disc\">\r\n<li><a href=\"#\" onclick=\"history.go(-1);\">Go back</a> to the previous page.</li>\r\n<li>Use the search bar at the top of the page to search for your products.</li>\r\n<li>Follow these links to get you back on track!<br/><a href=\"{{store url=\"\"}}\">Store Home</a><br/><a href=\"{{store url=\"customer/account\"}}\">My Account</a></li></ul></dd></dl><br/>\r\n<p><img src=\"{{skin url=\'images/media/404_callout1.jpg\'}}\" style=\"margin-right:15px;\"/><img src=\"{{skin url=\'images/media/404_callout2.jpg\'}}\" /></p>','2007-06-20 18:38:32','2007-08-26 19:11:13',1,0,NULL,NULL,NULL,NULL),(2,'Inicio','two_columns_right','','','Inicio','<h1>Maclaonline</h1>\r\n<p>Sitio en construcción</p>\r\n<p>Disculpen las molestias</p>\r\n','2007-08-23 10:03:25','2009-06-22 13:08:25',1,0,'<!--<reference name=\"content\">\r\n<block type=\"catalog/product_new\" name=\"home.catalog.product.new\" alias=\"product_new\" template=\"catalog/product/new.phtml\" after=\"cms_page\"><action method=\"addPriceBlockType\"><type>bundle</type><block>bundle/catalog_product_price</block><template>bundle/catalog/product/price.phtml</template></action></block>\r\n<block type=\"reports/product_viewed\" name=\"home.reports.product.viewed\" alias=\"product_viewed\" template=\"reports/home_product_viewed.phtml\" after=\"product_new\"><action method=\"addPriceBlockType\"><type>bundle</type><block>bundle/catalog_product_price</block><template>bundle/catalog/product/price.phtml</template></action></block>\r\n<block type=\"reports/product_compared\" name=\"home.reports.product.compared\" template=\"reports/home_product_compared.phtml\" after=\"product_viewed\"><action method=\"addPriceBlockType\"><type>bundle</type><block>bundle/catalog_product_price</block><template>bundle/catalog/product/price.phtml</template></action></block>\r\n</reference><reference name=\"right\">\r\n<action method=\"unsetChild\"><alias>right.reports.product.viewed</alias></action>\r\n<action method=\"unsetChild\"><alias>right.reports.product.compared</alias></action>\r\n</reference>-->','',NULL,NULL),(3,'Quien somos','one_column','','','about-us','<div class=\"page-head\">\r\n<h3>About Magento  Demo Store</h3>\r\n</div>\r\n<div class=\"col3-set\">\r\n<div class=\"col-1\">\r\n<p><img src=\"{{skin url=\'images/media/about_us_img.jpg\'}}\" alt=\"Varien office pic\" /></p>\r\n<p style=\"line-height:1.2em;\"><small>Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Morbi luctus. Duis lobortis. Nulla nec velit. Mauris pulvinar erat non massa. Suspendisse tortor turpis, porta nec, tempus vitae, iaculis semper, pede.</small></p>\r\n<p style=\"color:#888; font:1.2em/1.4em georgia, serif;\">Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Morbi luctus. Duis lobortis. Nulla nec velit. Mauris pulvinar erat non massa. Suspendisse tortor turpis, porta nec, tempus vitae, iaculis semper, pede. Cras vel libero id lectus rhoncus porta.</p>\r\n</div>\r\n<div class=\"col-2\">\r\n<p><strong style=\"color:#de036f;\">Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Morbi luctus. Duis lobortis. Nulla nec velit.</strong></p>\r\n<p>Vivamus tortor nisl, lobortis in, faucibus et, tempus at, dui. Nunc risus. Proin scelerisque augue. Nam ullamcorper. Phasellus id massa. Pellentesque nisl. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Nunc augue. Aenean sed justo non leo vehicula laoreet. Praesent ipsum libero, auctor ac, tempus nec, tempor nec, justo.</p>\r\n<p>Maecenas ullamcorper, odio vel tempus egestas, dui orci faucibus orci, sit amet aliquet lectus dolor et quam. Pellentesque consequat luctus purus. Nunc et risus. Etiam a nibh. Phasellus dignissim metus eget nisi. Vestibulum sapien dolor, aliquet nec, porta ac, malesuada a, libero. Praesent feugiat purus eget est. Nulla facilisi. Vestibulum tincidunt sapien eu velit. Mauris purus. Maecenas eget mauris eu orci accumsan feugiat. Pellentesque eget velit. Nunc tincidunt.</p>\r\n</div>\r\n<div class=\"col-3\">\r\n<p>Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Morbi luctus. Duis lobortis. Nulla nec velit. Mauris pulvinar erat non massa. Suspendisse tortor turpis, porta nec, tempus vitae, iaculis semper, pede. Cras vel libero id lectus rhoncus porta. Suspendisse convallis felis ac enim. Vivamus tortor nisl, lobortis in, faucibus et, tempus at, dui. Nunc risus. Proin scelerisque augue. Nam ullamcorper</p>\r\n<p><strong style=\"color:#de036f;\">Maecenas ullamcorper, odio vel tempus egestas, dui orci faucibus orci, sit amet aliquet lectus dolor et quam. Pellentesque consequat luctus purus.</strong></p>\r\n<p>Nunc et risus. Etiam a nibh. Phasellus dignissim metus eget nisi.</p>\r\n<p>To all of you, from all of us at Magento Demo Store - Thank you and Happy eCommerce!</p>\r\n<p style=\"line-height:1.2em;\"><strong style=\"font:italic 2em Georgia, serif;\">John Doe</strong><br /><small>Some important guy</small></p>\r\n</div>\r\n</div>','2007-08-30 14:01:18','2009-06-24 22:49:23',1,0,'','',NULL,NULL),(4,'Servicio a clientes','three_columns','','','customer-service','<div class=\"page-head\">\r\n<h3>Customer Service</h3>\r\n</div>\r\n<ul class=\"disc\" style=\"margin-bottom:15px;\">\r\n<li><a href=\"#answer1\">Shipping &amp; Delivery</a></li>\r\n<li><a href=\"#answer2\">Privacy &amp; Security</a></li>\r\n<li><a href=\"#answer3\">Returns &amp; Replacements</a></li>\r\n<li><a href=\"#answer4\">Ordering</a></li>\r\n<li><a href=\"#answer5\">Payment, Pricing &amp; Promotions</a></li>\r\n<li><a href=\"#answer6\">Viewing Orders</a></li>\r\n<li><a href=\"#answer7\">Updating Account Information</a></li>\r\n</ul>\r\n<dl> <dt id=\"answer1\">Shipping &amp; Delivery</dt> <dd style=\"margin-bottom:10px;\">Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Morbi luctus. Duis lobortis. Nulla nec velit. Mauris pulvinar erat non massa. Suspendisse tortor turpis, porta nec, tempus vitae, iaculis semper, pede. Cras vel libero id lectus rhoncus porta. Suspendisse convallis felis ac enim. Vivamus tortor nisl, lobortis in, faucibus et, tempus at, dui. Nunc risus. Proin scelerisque augue. Nam ullamcorper. Phasellus id massa. Pellentesque nisl. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Nunc augue. Aenean sed justo non leo vehicula laoreet. Praesent ipsum libero, auctor ac, tempus nec, tempor nec, justo.</dd> <dt id=\"answer2\">Privacy &amp; Security</dt> <dd style=\"margin-bottom:10px;\">Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Morbi luctus. Duis lobortis. Nulla nec velit. Mauris pulvinar erat non massa. Suspendisse tortor turpis, porta nec, tempus vitae, iaculis semper, pede. Cras vel libero id lectus rhoncus porta. Suspendisse convallis felis ac enim. Vivamus tortor nisl, lobortis in, faucibus et, tempus at, dui. Nunc risus. Proin scelerisque augue. Nam ullamcorper. Phasellus id massa. Pellentesque nisl. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Nunc augue. Aenean sed justo non leo vehicula laoreet. Praesent ipsum libero, auctor ac, tempus nec, tempor nec, justo.</dd> <dt id=\"answer3\">Returns &amp; Replacements</dt> <dd style=\"margin-bottom:10px;\">Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Morbi luctus. Duis lobortis. Nulla nec velit. Mauris pulvinar erat non massa. Suspendisse tortor turpis, porta nec, tempus vitae, iaculis semper, pede. Cras vel libero id lectus rhoncus porta. Suspendisse convallis felis ac enim. Vivamus tortor nisl, lobortis in, faucibus et, tempus at, dui. Nunc risus. Proin scelerisque augue. Nam ullamcorper. Phasellus id massa. Pellentesque nisl. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Nunc augue. Aenean sed justo non leo vehicula laoreet. Praesent ipsum libero, auctor ac, tempus nec, tempor nec, justo.</dd> <dt id=\"answer4\">Ordering</dt> <dd style=\"margin-bottom:10px;\">Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Morbi luctus. Duis lobortis. Nulla nec velit. Mauris pulvinar erat non massa. Suspendisse tortor turpis, porta nec, tempus vitae, iaculis semper, pede. Cras vel libero id lectus rhoncus porta. Suspendisse convallis felis ac enim. Vivamus tortor nisl, lobortis in, faucibus et, tempus at, dui. Nunc risus. Proin scelerisque augue. Nam ullamcorper. Phasellus id massa. Pellentesque nisl. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Nunc augue. Aenean sed justo non leo vehicula laoreet. Praesent ipsum libero, auctor ac, tempus nec, tempor nec, justo.</dd> <dt id=\"answer5\">Payment, Pricing &amp; Promotions</dt> <dd style=\"margin-bottom:10px;\">Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Morbi luctus. Duis lobortis. Nulla nec velit. Mauris pulvinar erat non massa. Suspendisse tortor turpis, porta nec, tempus vitae, iaculis semper, pede. Cras vel libero id lectus rhoncus porta. Suspendisse convallis felis ac enim. Vivamus tortor nisl, lobortis in, faucibus et, tempus at, dui. Nunc risus. Proin scelerisque augue. Nam ullamcorper. Phasellus id massa. Pellentesque nisl. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Nunc augue. Aenean sed justo non leo vehicula laoreet. Praesent ipsum libero, auctor ac, tempus nec, tempor nec, justo.</dd> <dt id=\"answer6\">Viewing Orders</dt> <dd style=\"margin-bottom:10px;\">Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Morbi luctus. Duis lobortis. Nulla nec velit. Mauris pulvinar erat non massa. Suspendisse tortor turpis, porta nec, tempus vitae, iaculis semper, pede. Cras vel libero id lectus rhoncus porta. Suspendisse convallis felis ac enim. Vivamus tortor nisl, lobortis in, faucibus et, tempus at, dui. Nunc risus. Proin scelerisque augue. Nam ullamcorper. Phasellus id massa. Pellentesque nisl. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Nunc augue. Aenean sed justo non leo vehicula laoreet. Praesent ipsum libero, auctor ac, tempus nec, tempor nec, justo.</dd> <dt id=\"answer7\">Updating Account Information</dt> <dd style=\"margin-bottom:10px;\">Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Morbi luctus. Duis lobortis. Nulla nec velit. Mauris pulvinar erat non massa. Suspendisse tortor turpis, porta nec, tempus vitae, iaculis semper, pede. Cras vel libero id lectus rhoncus porta. Suspendisse convallis felis ac enim. Vivamus tortor nisl, lobortis in, faucibus et, tempus at, dui. Nunc risus. Proin scelerisque augue. Nam ullamcorper. Phasellus id massa. Pellentesque nisl. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Nunc augue. Aenean sed justo non leo vehicula laoreet. Praesent ipsum libero, auctor ac, tempus nec, tempor nec, justo.</dd> </dl>','2007-08-30 14:02:20','2009-06-24 22:52:38',1,0,'','',NULL,NULL),(5,'Enable Cookies','one_column','','','enable-cookies','<div class=\"std\">\r\n    <ul class=\"messages\">\r\n        <li class=\"notice-msg\">\r\n            <ul>\r\n                <li>Please enable cookies in your web browser to continue.</li>\r\n            </ul>\r\n        </li>\r\n    </ul>\r\n    <div class=\"page-head\">\r\n        <h3><a name=\"top\"></a>What are Cookies?</h3>\r\n    </div>\r\n    <p>Cookies are short pieces of data that are sent to your computer when you visit a website. On later visits, this data is then returned to that website. Cookies allow us to recognize you automatically whenever you visit our site so that we can personalize your experience and provide you with better service. We also use cookies (and similar browser data, such as Flash cookies) for fraud prevention and other purposes. If your web browser is set to refuse cookies from our website, you will not be able to complete a purchase or take advantage of certain features of our website, such as storing items in your Shopping Cart or receiving personalized recommendations. As a result, we strongly encourage you to configure your web browser to accept cookies from our website.</p>\r\n    <h3>Enabling Cookies</h3>\r\n    <ul>\r\n        <li><a href=\"#ie7\">Internet Explorer 7.x</a></li>\r\n        <li><a href=\"#ie6\">Internet Explorer 6.x</a></li>\r\n        <li><a href=\"#firefox\">Mozilla/Firefox</a></li>\r\n        <li><a href=\"#opera\">Opera 7.x</a></li>\r\n    </ul>\r\n    <h4><a name=\"ie7\"></a>Internet Explorer 7.x</h4>\r\n    <ol>\r\n        <li>\r\n            <p>Start Internet Explorer</p>\r\n        </li>\r\n        <li>\r\n            <p>Under the <strong>Tools</strong> menu, click <strong>Internet Options</strong></p>\r\n            <p><img src=\"{{skin url=\"images/cookies/ie7-1.gif\"}}\" alt=\"\" /></p>\r\n        </li>\r\n        <li>\r\n            <p>Click the <strong>Privacy</strong> tab</p>\r\n            <p><img src=\"{{skin url=\"images/cookies/ie7-2.gif\"}}\" alt=\"\" /></p>\r\n        </li>\r\n        <li>\r\n            <p>Click the <strong>Advanced</strong> button</p>\r\n            <p><img src=\"{{skin url=\"images/cookies/ie7-3.gif\"}}\" alt=\"\" /></p>\r\n        </li>\r\n        <li>\r\n            <p>Put a check mark in the box for <strong>Override Automatic Cookie Handling</strong>, put another check mark in the <strong>Always accept session cookies </strong>box</p>\r\n            <p><img src=\"{{skin url=\"images/cookies/ie7-4.gif\"}}\" alt=\"\" /></p>\r\n        </li>\r\n        <li>\r\n            <p>Click <strong>OK</strong></p>\r\n            <p><img src=\"{{skin url=\"images/cookies/ie7-5.gif\"}}\" alt=\"\" /></p>\r\n        </li>\r\n        <li>\r\n            <p>Click <strong>OK</strong></p>\r\n            <p><img src=\"{{skin url=\"images/cookies/ie7-6.gif\"}}\" alt=\"\" /></p>\r\n        </li>\r\n        <li>\r\n            <p>Restart Internet Explore</p>\r\n        </li>\r\n    </ol>\r\n    <p class=\"a-top\"><a href=\"#top\">Back to Top</a></p>\r\n    <h4><a name=\"ie6\"></a>Internet Explorer 6.x</h4>\r\n    <ol>\r\n        <li>\r\n            <p>Select <strong>Internet Options</strong> from the Tools menu</p>\r\n            <p><img src=\"{{skin url=\"images/cookies/ie6-1.gif\"}}\" alt=\"\" /></p>\r\n        </li>\r\n        <li>\r\n            <p>Click on the <strong>Privacy</strong> tab</p>\r\n        </li>\r\n        <li>\r\n            <p>Click the <strong>Default</strong> button (or manually slide the bar down to <strong>Medium</strong>) under <strong>Settings</strong>. Click <strong>OK</strong></p>\r\n            <p><img src=\"{{skin url=\"images/cookies/ie6-2.gif\"}}\" alt=\"\" /></p>\r\n        </li>\r\n    </ol>\r\n    <p class=\"a-top\"><a href=\"#top\">Back to Top</a></p>\r\n    <h4><a name=\"firefox\"></a>Mozilla/Firefox</h4>\r\n    <ol>\r\n        <li>\r\n            <p>Click on the <strong>Tools</strong>-menu in Mozilla</p>\r\n        </li>\r\n        <li>\r\n            <p>Click on the <strong>Options...</strong> item in the menu - a new window open</p>\r\n        </li>\r\n        <li>\r\n            <p>Click on the <strong>Privacy</strong> selection in the left part of the window. (See image below)</p>\r\n            <p><img src=\"{{skin url=\"images/cookies/firefox.png\"}}\" alt=\"\" /></p>\r\n        </li>\r\n        <li>\r\n            <p>Expand the <strong>Cookies</strong> section</p>\r\n        </li>\r\n        <li>\r\n            <p>Check the <strong>Enable cookies</strong> and <strong>Accept cookies normally</strong> checkboxes</p>\r\n        </li>\r\n        <li>\r\n            <p>Save changes by clicking <strong>Ok</strong>.</p>\r\n        </li>\r\n    </ol>\r\n    <p class=\"a-top\"><a href=\"#top\">Back to Top</a></p>\r\n    <h4><a name=\"opera\"></a>Opera 7.x</h4>\r\n    <ol>\r\n        <li>\r\n            <p>Click on the <strong>Tools</strong> menu in Opera</p>\r\n        </li>\r\n        <li>\r\n            <p>Click on the <strong>Preferences...</strong> item in the menu - a new window open</p>\r\n        </li>\r\n        <li>\r\n            <p>Click on the <strong>Privacy</strong> selection near the bottom left of the window. (See image below)</p>\r\n            <p><img src=\"{{skin url=\"images/cookies/opera.png\"}}\" alt=\"\" /></p>\r\n        </li>\r\n        <li>\r\n            <p>The <strong>Enable cookies</strong> checkbox must be checked, and <strong>Accept all cookies</strong> should be selected in the &quot;<strong>Normal cookies</strong>&quot; drop-down</p>\r\n        </li>\r\n        <li>\r\n            <p>Save changes by clicking <strong>Ok</strong></p>\r\n        </li>\r\n    </ol>\r\n    <p class=\"a-top\"><a href=\"#top\">Back to Top</a></p>\r\n</div>\r\n','2009-06-03 19:53:31','2009-06-03 19:53:31',1,0,NULL,NULL,NULL,NULL),(6,'Condiciones generales de venta','one_column','','','condiciones-generales','<p><strong>Condiciones generales de venta</strong></p>','2009-06-24 22:54:09','2009-06-24 23:02:53',1,0,'','',NULL,NULL),(7,'Garantia y devolución','one_column','','','garantia-devolucion','<p><strong>Garantia y devoluci&oacute;n</strong></p>','2009-06-24 22:55:02','2009-06-24 23:03:14',1,0,'','',NULL,NULL);
/*!40000 ALTER TABLE `cms_page` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `cms_page_store`
--

DROP TABLE IF EXISTS `cms_page_store`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `cms_page_store` (
  `page_id` smallint(6) NOT NULL,
  `store_id` smallint(5) unsigned NOT NULL,
  PRIMARY KEY  (`page_id`,`store_id`),
  KEY `FK_CMS_PAGE_STORE_STORE` (`store_id`),
  CONSTRAINT `FK_CMS_PAGE_STORE_PAGE` FOREIGN KEY (`page_id`) REFERENCES `cms_page` (`page_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CMS_PAGE_STORE_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='CMS Pages to Stores';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `cms_page_store`
--

LOCK TABLES `cms_page_store` WRITE;
/*!40000 ALTER TABLE `cms_page_store` DISABLE KEYS */;
INSERT INTO `cms_page_store` VALUES (1,0),(5,0),(2,1),(3,1),(4,1),(6,1),(7,1);
/*!40000 ALTER TABLE `cms_page_store` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `core_config_data`
--

DROP TABLE IF EXISTS `core_config_data`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `core_config_data` (
  `config_id` int(10) unsigned NOT NULL auto_increment,
  `scope` enum('default','websites','stores','config') NOT NULL default 'default',
  `scope_id` int(11) NOT NULL default '0',
  `path` varchar(255) NOT NULL default 'general',
  `value` text NOT NULL,
  PRIMARY KEY  (`config_id`),
  UNIQUE KEY `config_scope` (`scope`,`scope_id`,`path`)
) ENGINE=InnoDB AUTO_INCREMENT=422 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `core_config_data`
--

LOCK TABLES `core_config_data` WRITE;
/*!40000 ALTER TABLE `core_config_data` DISABLE KEYS */;
INSERT INTO `core_config_data` VALUES (1,'default',0,'catalog/category/root_id','2'),(2,'default',0,'web/unsecure/base_url','http://www.maclaonline.es/tienda/'),(3,'default',0,'web/secure/base_url','http://www.maclaonline.es/tienda/'),(4,'default',0,'general/locale/code','es_ES'),(5,'default',0,'general/locale/timezone','Europe/Berlin'),(6,'default',0,'currency/options/base','EUR'),(7,'default',0,'currency/options/default','EUR'),(8,'default',0,'currency/options/allow','EUR'),(9,'default',0,'general/country/default','ES'),(10,'default',0,'general/country/allow','ES'),(11,'default',0,'general/locale/firstday','1'),(12,'default',0,'general/locale/weekend','0,6'),(13,'default',0,'web/url/use_store','0'),(14,'default',0,'web/seo/use_rewrites','0'),(15,'default',0,'web/unsecure/base_link_url','{{unsecure_base_url}}'),(16,'default',0,'web/unsecure/base_skin_url','{{unsecure_base_url}}skin/'),(17,'default',0,'web/unsecure/base_media_url','{{unsecure_base_url}}media/'),(18,'default',0,'web/unsecure/base_js_url','{{unsecure_base_url}}js/'),(19,'default',0,'web/secure/base_link_url','{{secure_base_url}}'),(20,'default',0,'web/secure/base_skin_url','{{secure_base_url}}skin/'),(21,'default',0,'web/secure/base_media_url','{{secure_base_url}}media/'),(22,'default',0,'web/secure/base_js_url','{{secure_base_url}}js/'),(23,'default',0,'web/secure/use_in_frontend','0'),(24,'default',0,'web/secure/use_in_adminhtml','0'),(25,'default',0,'web/default/front','cms'),(26,'default',0,'web/default/cms_home_page','Inicio'),(27,'default',0,'web/default/no_route','cms/index/noRoute'),(28,'default',0,'web/default/cms_no_route','no-route'),(29,'default',0,'web/default/cms_no_cookies','enable-cookies'),(30,'default',0,'web/default/show_cms_breadcrumbs','1'),(31,'default',0,'web/polls/poll_check_by_ip','0'),(32,'default',0,'web/cookie/cookie_domain','maclaonline.es'),(33,'default',0,'web/cookie/cookie_path',''),(34,'default',0,'web/cookie/cookie_lifetime','3600'),(35,'default',0,'web/cookie/cookie_httponly','1'),(36,'default',0,'web/session/use_remote_addr','0'),(37,'default',0,'web/session/use_http_via','0'),(38,'default',0,'web/session/use_http_x_forwarded_for','0'),(39,'default',0,'web/session/use_http_user_agent','0'),(40,'default',0,'web/browser_capabilities/cookies','1'),(41,'default',0,'web/browser_capabilities/javascript','1'),(42,'default',0,'design/package/name','default'),(43,'default',0,'design/package/ua_regexp','a:0:{}'),(44,'default',0,'design/theme/locale','default'),(45,'default',0,'design/theme/template','modern'),(46,'default',0,'design/theme/template_ua_regexp','a:0:{}'),(47,'default',0,'design/theme/skin','modern'),(48,'default',0,'design/theme/skin_ua_regexp','a:0:{}'),(49,'default',0,'design/theme/layout','modern'),(50,'default',0,'design/theme/layout_ua_regexp','a:0:{}'),(51,'default',0,'design/theme/default','default'),(52,'default',0,'design/theme/default_ua_regexp','a:0:{}'),(53,'default',0,'design/head/default_title','Macla Online'),(54,'default',0,'design/head/title_prefix',''),(55,'default',0,'design/head/title_suffix',''),(56,'default',0,'design/head/default_description','Tienda Maclaonline'),(57,'default',0,'design/head/default_keywords','Macla, tienda, online'),(58,'default',0,'design/head/default_robots','INDEX,FOLLOW'),(59,'default',0,'design/head/includes',''),(60,'default',0,'design/head/demonotice','0'),(61,'default',0,'design/header/logo_src','images/logo.gif'),(62,'default',0,'design/header/logo_alt','Macla Online'),(63,'default',0,'design/header/welcome','Bienvenido a Maclaonline'),(64,'default',0,'design/footer/copyright','&copy; 2008 Maclaonline.'),(65,'default',0,'design/footer/absolute_footer',''),(66,'default',0,'design/watermark/image_size',''),(67,'default',0,'design/watermark/image_position','stretch'),(68,'default',0,'design/watermark/small_image_size',''),(69,'default',0,'design/watermark/small_image_position','stretch'),(70,'default',0,'design/watermark/thumbnail_size',''),(71,'default',0,'design/watermark/thumbnail_position','stretch'),(72,'default',0,'currency/options/trim_sign','0'),(73,'default',0,'currency/webservicex/timeout','100'),(74,'default',0,'currency/import/enabled','0'),(75,'default',0,'currency/import/service','webservicex'),(76,'default',0,'crontab/jobs/currency_rates_update/schedule/cron_expr','0 0 * * *'),(77,'default',0,'currency/import/time','00,00,00'),(78,'default',0,'currency/import/frequency','D'),(79,'default',0,'currency/import/error_email',''),(80,'default',0,'currency/import/error_email_identity','general'),(81,'default',0,'currency/import/error_email_template','currency_import_error_email_template'),(82,'default',0,'trans_email/ident_general/name','Maclaonline'),(83,'default',0,'trans_email/ident_general/email','info@maclaonline.es'),(84,'default',0,'trans_email/ident_sales/name','Maclaonline'),(85,'default',0,'trans_email/ident_sales/email','info@maclaonline.es'),(86,'default',0,'trans_email/ident_support/name','Maclaonline'),(87,'default',0,'trans_email/ident_support/email','info@maclaonline.es'),(88,'default',0,'trans_email/ident_custom1/name','Maclaonline'),(89,'default',0,'trans_email/ident_custom1/email','info@maclaonline.es'),(90,'default',0,'trans_email/ident_custom2/name','Maclaonline'),(91,'default',0,'trans_email/ident_custom2/email','info@maclaonline.es'),(92,'default',0,'contacts/contacts/enabled','1'),(93,'default',0,'contacts/email/recipient_email','info@maclaonline.es'),(94,'default',0,'contacts/email/sender_email_identity','custom2'),(95,'default',0,'contacts/email/email_template','contacts_email_email_template'),(96,'default',0,'system/cron/schedule_generate_every','15'),(97,'default',0,'system/cron/schedule_ahead_for','20'),(98,'default',0,'system/cron/schedule_lifetime','15'),(99,'default',0,'system/cron/history_cleanup_every','10'),(100,'default',0,'system/cron/history_success_lifetime','60'),(101,'default',0,'system/cron/history_failure_lifetime','600'),(102,'default',0,'system/smtp/disable','0'),(103,'default',0,'system/smtp/host','localhost'),(104,'default',0,'system/smtp/port','25'),(105,'default',0,'system/currency/installed','EUR'),(106,'default',0,'system/log/clean_after_day','180'),(107,'default',0,'system/log/enabled','0'),(108,'default',0,'system/log/time','00,00,00'),(109,'default',0,'system/log/frequency','D'),(110,'default',0,'crontab/jobs/log_clean/schedule/cron_expr',''),(111,'default',0,'crontab/jobs/log_clean/run/model','log/cron::logClean'),(112,'default',0,'system/log/error_email',''),(113,'default',0,'system/log/error_email_identity','general'),(114,'default',0,'system/log/error_email_template','system_log_error_email_template'),(115,'default',0,'system/adminnotification/use_https','0'),(116,'default',0,'system/adminnotification/frequency','1'),(117,'default',0,'payment/ccsave/active','0'),(118,'default',0,'payment/ccsave/title','Credit Card (saved)'),(119,'default',0,'payment/ccsave/order_status','pending'),(120,'default',0,'payment/ccsave/cctypes','AE,VI,MC,DI'),(121,'default',0,'payment/ccsave/useccv','0'),(122,'default',0,'payment/ccsave/allowspecific','0'),(123,'default',0,'payment/ccsave/min_order_total',''),(124,'default',0,'payment/ccsave/max_order_total',''),(125,'default',0,'payment/ccsave/sort_order',''),(126,'default',0,'payment/free/title','No es necesaria información de pago'),(127,'default',0,'payment/free/active','1'),(128,'default',0,'payment/free/order_status','pending'),(129,'default',0,'payment/free/payment_action',''),(130,'default',0,'payment/free/allowspecific','0'),(131,'default',0,'payment/free/sort_order','1'),(132,'default',0,'payment/checkmo/active','0'),(133,'default',0,'payment/checkmo/title','Check / Money order'),(134,'default',0,'payment/checkmo/order_status','pending'),(135,'default',0,'payment/checkmo/allowspecific','0'),(136,'default',0,'payment/checkmo/payable_to',''),(137,'default',0,'payment/checkmo/mailing_address',''),(138,'default',0,'payment/checkmo/min_order_total',''),(139,'default',0,'payment/checkmo/max_order_total',''),(140,'default',0,'payment/checkmo/sort_order',''),(141,'default',0,'payment/purchaseorder/active','0'),(142,'default',0,'payment/purchaseorder/title','Purchase Order'),(143,'default',0,'payment/purchaseorder/order_status','pending'),(144,'default',0,'payment/purchaseorder/allowspecific','0'),(145,'default',0,'payment/purchaseorder/min_order_total',''),(146,'default',0,'payment/purchaseorder/max_order_total',''),(147,'default',0,'payment/purchaseorder/sort_order',''),(148,'default',0,'payment/authorizenet/active','0'),(149,'default',0,'payment/authorizenet/title','Credit Card (Authorize.net)'),(150,'default',0,'payment/authorizenet/login',''),(151,'default',0,'payment/authorizenet/payment_action','authorize'),(152,'default',0,'payment/authorizenet/trans_key',''),(153,'default',0,'payment/authorizenet/order_status',''),(154,'default',0,'payment/authorizenet/test','1'),(155,'default',0,'payment/authorizenet/cgi_url','https://secure.authorize.net/gateway/transact.dll'),(156,'default',0,'payment/authorizenet/debug','0'),(157,'default',0,'payment/authorizenet/email_customer','0'),(158,'default',0,'payment/authorizenet/merchant_email',''),(159,'default',0,'payment/authorizenet/cctypes','AE,VI,MC,DI'),(160,'default',0,'payment/authorizenet/useccv','0'),(161,'default',0,'payment/authorizenet/allowspecific','0'),(162,'default',0,'payment/authorizenet/min_order_total',''),(163,'default',0,'payment/authorizenet/max_order_total',''),(164,'default',0,'payment/authorizenet/sort_order',''),(165,'default',0,'payment/verisign/active','0'),(166,'default',0,'payment/verisign/title','Credit Card (Payflow Pro)'),(167,'default',0,'payment/verisign/url','https://pilot-payflowpro.verisign.com/transaction'),(168,'default',0,'payment/verisign/verbosity','MEDIUM'),(169,'default',0,'payment/verisign/payment_action','authorize'),(170,'default',0,'payment/verisign/tender','C'),(171,'default',0,'payment/verisign/fraud_order_status','holded'),(172,'default',0,'payment/verisign/order_status',''),(173,'default',0,'payment/verisign/partner',''),(174,'default',0,'payment/verisign/vendor',''),(175,'default',0,'payment/verisign/user',''),(176,'default',0,'payment/verisign/pwd',''),(177,'default',0,'payment/verisign/debug','0'),(178,'default',0,'payment/verisign/cctypes','AE,VI,MC,DI'),(179,'default',0,'payment/verisign/useccv','0'),(180,'default',0,'payment/verisign/use_proxy','0'),(181,'default',0,'payment/verisign/proxy_host',''),(182,'default',0,'payment/verisign/proxy_port',''),(183,'default',0,'payment/verisign/allowspecific','0'),(184,'default',0,'payment/verisign/min_order_total',''),(185,'default',0,'payment/verisign/max_order_total',''),(186,'default',0,'payment/verisign/sort_order',''),(187,'default',0,'payment/paypal_express/active','0'),(188,'default',0,'payment/paypal_express/title','Paypal Express'),(189,'default',0,'payment/paypal_express/payment_action','Authorization'),(190,'default',0,'payment/paypal_express/order_status',''),(191,'default',0,'payment/paypal_express/allowspecific','0'),(192,'default',0,'payment/paypal_express/sort_order',''),(193,'default',0,'payment/paypal_direct/active','0'),(194,'default',0,'payment/paypal_direct/title','Paypal Direct'),(195,'default',0,'payment/paypal_direct/order_status',''),(196,'default',0,'payment/paypal_direct/payment_action','Authorization'),(197,'default',0,'payment/paypal_direct/cctypes','AE,VI,MC,DI'),(198,'default',0,'payment/paypal_direct/allowspecific','0'),(199,'default',0,'payment/paypal_direct/sort_order',''),(200,'default',0,'payment/paypal_standard/active','0'),(201,'default',0,'payment/paypal_standard/title','Paypal Standard'),(202,'default',0,'payment/paypal_standard/payment_action','AUTHORIZATION'),(203,'default',0,'payment/paypal_standard/types','IPN'),(204,'default',0,'payment/paypal_standard/order_status',''),(205,'default',0,'payment/paypal_standard/transaction_type','O'),(206,'default',0,'payment/paypal_standard/allowspecific','0'),(207,'default',0,'payment/paypal_standard/sort_order',''),(208,'default',0,'payment/paypaluk_express/active','0'),(209,'default',0,'payment/paypaluk_express/title','PaypalUk Express'),(210,'default',0,'payment/paypaluk_express/payment_action','authorize'),(211,'default',0,'payment/paypaluk_express/order_status',''),(212,'default',0,'payment/paypaluk_express/allowspecific','0'),(213,'default',0,'payment/paypaluk_express/sort_order',''),(214,'default',0,'payment/paypaluk_direct/active','0'),(215,'default',0,'payment/paypaluk_direct/title','PaypalUk Direct'),(216,'default',0,'payment/paypaluk_direct/order_status',''),(217,'default',0,'payment/paypaluk_direct/payment_action','authorize'),(218,'default',0,'payment/paypaluk_direct/cctypes','AE,VI,MC,DI'),(219,'default',0,'payment/paypaluk_direct/allowspecific','0'),(220,'default',0,'payment/paypaluk_direct/sort_order',''),(221,'default',0,'payment/amazonpayments_asp/title','Amazon Simple Pay'),(222,'default',0,'payment/amazonpayments_asp/active','0'),(223,'default',0,'payment/amazonpayments_asp/is_sandbox','1'),(224,'default',0,'payment/amazonpayments_asp/pay_process_immediate','1'),(225,'default',0,'payment/amazonpayments_asp/access_key',''),(226,'default',0,'payment/amazonpayments_asp/secret_key',''),(227,'default',0,'payment/amazonpayments_asp/pay_immediate_return','1'),(228,'default',0,'payment/amazonpayments_asp/pay_description','MAGENTO ASP PAYMENT'),(229,'default',0,'payment/amazonpayments_asp/refund_description','MAGENTO ASP REFUND'),(230,'default',0,'payment/amazonpayments_asp/cancel_description','MAGENTO ASP CANCEL'),(231,'default',0,'payment/amazonpayments_asp/report_email',''),(232,'default',0,'payment/amazonpayments_asp/email_sender_identity','general'),(233,'default',0,'payment/amazonpayments_asp/email_template_notofication_error','payment_amazonpayments_asp_email_template_notofication_error'),(234,'default',0,'payment/amazonpayments_asp/report_error_to_email','0'),(235,'default',0,'payment/amazonpayments_asp/error_log','0'),(236,'default',0,'payment/amazonpayments_asp/debug_log','0'),(237,'default',0,'payment/amazonpayments_cba/title','Checkout by Amazon'),(238,'default',0,'payment/amazonpayments_cba/active','0'),(239,'default',0,'payment/amazonpayments_cba/sandbox_flag','1'),(240,'default',0,'payment/amazonpayments_cba/debug_flag','0'),(241,'default',0,'payment/amazonpayments_cba/merchant_id',''),(242,'default',0,'payment/amazonpayments_cba/merchant_name',''),(243,'default',0,'payment/amazonpayments_cba/accesskey_id',''),(244,'default',0,'payment/amazonpayments_cba/secretkey_id',''),(245,'default',0,'payment/amazonpayments_cba/merchant_tocken',''),(246,'default',0,'payment/amazonpayments_cba/merchant_login',''),(247,'default',0,'payment/amazonpayments_cba/merchant_pass',''),(248,'default',0,'payment/amazonpayments_cba/order_status','pending'),(249,'default',0,'payment/amazonpayments_cba/standard_rate','a:1:{s:6:\"method\";s:4:\"None\";}'),(250,'default',0,'payment/amazonpayments_cba/expedited_rate','a:1:{s:6:\"method\";s:4:\"None\";}'),(251,'default',0,'payment/amazonpayments_cba/oneday_rate','a:1:{s:6:\"method\";s:4:\"None\";}'),(252,'default',0,'payment/amazonpayments_cba/twoday_rate','a:1:{s:6:\"method\";s:4:\"None\";}'),(253,'default',0,'payment/cashondelivery/active','1'),(254,'default',0,'payment/cashondelivery/title','Contrareembolso'),(255,'default',0,'payment/cashondelivery/order_status','processing'),(256,'default',0,'payment/cashondelivery/sort_order',''),(257,'default',0,'payment/cashondelivery/allowspecific','1'),(258,'default',0,'payment/cashondelivery/specificcountry','ES'),(259,'default',0,'payment/cashondelivery/inlandcosts','0'),(260,'default',0,'payment/cashondelivery/foreigncountrycosts','0'),(261,'default',0,'payment/cashondelivery/customtext',''),(262,'default',0,'shipping/origin/country_id','ES'),(263,'default',0,'shipping/origin/region_id','161'),(264,'default',0,'shipping/origin/postcode','28016'),(265,'default',0,'shipping/origin/city','Madrid'),(266,'default',0,'shipping/option/checkout_multiple','0'),(267,'default',0,'shipping/option/checkout_multiple_maximum_qty','100'),(268,'default',0,'carriers/flatrate/active','0'),(269,'default',0,'carriers/flatrate/title','Flat Rate'),(270,'default',0,'carriers/flatrate/name','Fixed'),(271,'default',0,'carriers/flatrate/type','I'),(272,'default',0,'carriers/flatrate/price','5.00'),(273,'default',0,'carriers/flatrate/handling_type','F'),(274,'default',0,'carriers/flatrate/handling_fee',''),(275,'default',0,'carriers/flatrate/specificerrmsg','This shipping method is currently unavailable. If you would like to ship using this shipping method, please contact us.'),(276,'default',0,'carriers/flatrate/sallowspecific','0'),(277,'default',0,'carriers/flatrate/showmethod','0'),(278,'default',0,'carriers/flatrate/sort_order',''),(279,'default',0,'carriers/tablerate/active','0'),(280,'default',0,'carriers/tablerate/title','Best Way'),(281,'default',0,'carriers/tablerate/name','Table Rate'),(282,'default',0,'carriers/tablerate/condition_name','package_weight'),(283,'default',0,'carriers/tablerate/handling_type','F'),(284,'default',0,'carriers/tablerate/handling_fee',''),(285,'default',0,'carriers/tablerate/specificerrmsg','This shipping method is currently unavailable. If you would like to ship using this shipping method, please contact us.'),(286,'default',0,'carriers/tablerate/sallowspecific','0'),(287,'default',0,'carriers/tablerate/showmethod','0'),(288,'default',0,'carriers/tablerate/sort_order',''),(289,'default',0,'carriers/freeshipping/active','1'),(290,'default',0,'carriers/freeshipping/title','Envío gratuito'),(291,'default',0,'carriers/freeshipping/name','Correos'),(292,'default',0,'carriers/freeshipping/free_shipping_subtotal','0'),(293,'default',0,'carriers/freeshipping/specificerrmsg','This shipping method is currently unavailable. If you would like to ship using this shipping method, please contact us.'),(294,'default',0,'carriers/freeshipping/sallowspecific','1'),(295,'default',0,'carriers/freeshipping/showmethod','0'),(296,'default',0,'carriers/freeshipping/sort_order',''),(297,'default',0,'carriers/ups/active','0'),(298,'default',0,'carriers/ups/type','UPS'),(299,'default',0,'carriers/ups/access_license_number',''),(300,'default',0,'carriers/ups/password',''),(301,'default',0,'carriers/ups/username',''),(302,'default',0,'carriers/ups/gateway_xml_url','https://www.ups.com/ups.app/xml/Rate'),(303,'default',0,'carriers/ups/origin_shipment','United States Domestic Shipments'),(304,'default',0,'carriers/ups/title','United Parcel Service'),(305,'default',0,'carriers/ups/negotiated_active','0'),(306,'default',0,'carriers/ups/gateway_url','http://www.ups.com/using/services/rave/qcostcgi.cgi'),(307,'default',0,'carriers/ups/shipper_number',''),(308,'default',0,'carriers/ups/container','CP'),(309,'default',0,'carriers/ups/unit_of_measure','LBS'),(310,'default',0,'carriers/ups/tracking_xml_url','https://www.ups.com/ups.app/xml/Track'),(311,'default',0,'carriers/ups/dest_type','RES'),(312,'default',0,'carriers/ups/pickup','CC'),(313,'default',0,'carriers/ups/max_package_weight','150'),(314,'default',0,'carriers/ups/handling_type','F'),(315,'default',0,'carriers/ups/handling_action','O'),(316,'default',0,'carriers/ups/handling_fee',''),(317,'default',0,'carriers/ups/allowed_methods','1DM,1DML,1DA,1DAL,1DAPI,1DP,1DPL,2DM,2DML,2DA,2DAL,3DS,GND,GNDCOM,GNDRES,STD,XPR,WXS,XPRL,XDM,XDML,XPD'),(318,'default',0,'carriers/ups/free_method','GND'),(319,'default',0,'carriers/ups/free_shipping_enable','0'),(320,'default',0,'carriers/ups/free_shipping_subtotal',''),(321,'default',0,'carriers/ups/specificerrmsg','This shipping method is currently unavailable. If you would like to ship using this shipping method, please contact us.'),(322,'default',0,'carriers/ups/sallowspecific','0'),(323,'default',0,'carriers/ups/showmethod','0'),(324,'default',0,'carriers/ups/sort_order',''),(325,'default',0,'carriers/usps/active','0'),(326,'default',0,'carriers/usps/gateway_url','http://production.shippingapis.com/ShippingAPI.dll'),(327,'default',0,'carriers/usps/title','United States Postal Service'),(328,'default',0,'carriers/usps/userid',''),(329,'default',0,'carriers/usps/container','VARIABLE'),(330,'default',0,'carriers/usps/size','REGULAR'),(331,'default',0,'carriers/usps/machinable','true'),(332,'default',0,'carriers/usps/max_package_weight','70'),(333,'default',0,'carriers/usps/handling_type','F'),(334,'default',0,'carriers/usps/handling_action','O'),(335,'default',0,'carriers/usps/handling_fee',''),(336,'default',0,'carriers/usps/allowed_methods','Bound Printed Matter,Express Mail,Express Mail Flat-Rate Envelope,Express Mail Flat-Rate Envelope Hold For Pickup,Express Mail Flat-Rate Envelope Sunday/Holiday Guarantee,Express Mail Hold For Pickup,Express Mail International (EMS),Express Mail International (EMS) Flat-Rate Envelope,Express Mail PO to PO,Express Mail Sunday/Holiday Guarantee,First Class Mail International Large Envelope,First Class Mail International Letters,First Class Mail International Package,First-Class,First-Class Mail,First-Class Mail Flat,First-Class Mail International,First-Class Mail Letter,First-Class Mail Parcel,Global Express Guaranteed,Global Express Guaranteed Non-Document Non-Rectangular,Global Express Guaranteed Non-Document Rectangular,Library Mail,Media Mail,Parcel Post,Priority Mail,Priority Mail Flat-Rate Box,Priority Mail Flat-Rate Envelope,Priority Mail International,Priority Mail International Flat-Rate Box,Priority Mail International Flat-Rate Envelope,Priority Mail International Large Flat-Rate Box,Priority Mail Large Flat-Rate Box,USPS GXG Envelopes'),(337,'default',0,'carriers/usps/free_method',''),(338,'default',0,'carriers/usps/free_shipping_enable','0'),(339,'default',0,'carriers/usps/free_shipping_subtotal',''),(340,'default',0,'carriers/usps/specificerrmsg','This shipping method is currently unavailable. If you would like to ship using this shipping method, please contact us.'),(341,'default',0,'carriers/usps/sallowspecific','0'),(342,'default',0,'carriers/usps/showmethod','0'),(343,'default',0,'carriers/usps/sort_order',''),(344,'default',0,'carriers/fedex/active','0'),(345,'default',0,'carriers/fedex/gateway_url','https://gateway.fedex.com/GatewayDC'),(346,'default',0,'carriers/fedex/title','Federal Express'),(347,'default',0,'carriers/fedex/account',''),(348,'default',0,'carriers/fedex/packaging','YOURPACKAGING'),(349,'default',0,'carriers/fedex/dropoff','REGULARPICKUP'),(350,'default',0,'carriers/fedex/max_package_weight','150'),(351,'default',0,'carriers/fedex/handling_type','F'),(352,'default',0,'carriers/fedex/handling_action','O'),(353,'default',0,'carriers/fedex/handling_fee',''),(354,'default',0,'carriers/fedex/residence_delivery','0'),(355,'default',0,'carriers/fedex/allowed_methods','PRIORITYOVERNIGHT,STANDARDOVERNIGHT,FIRSTOVERNIGHT,FEDEX2DAY,FEDEXEXPRESSSAVER,INTERNATIONALPRIORITY,INTERNATIONALECONOMY,INTERNATIONALFIRST,FEDEX1DAYFREIGHT,FEDEX2DAYFREIGHT,FEDEX3DAYFREIGHT,FEDEXGROUND,GROUNDHOMEDELIVERY,INTERNATIONALPRIORITY FREIGHT,INTERNATIONALECONOMY FREIGHT,EUROPEFIRSTINTERNATIONALPRIORITY'),(356,'default',0,'carriers/fedex/free_method','FEDEXGROUND'),(357,'default',0,'carriers/fedex/free_shipping_enable','0'),(358,'default',0,'carriers/fedex/free_shipping_subtotal',''),(359,'default',0,'carriers/fedex/specificerrmsg','This shipping method is currently unavailable. If you would like to ship using this shipping method, please contact us.'),(360,'default',0,'carriers/fedex/sallowspecific','0'),(361,'default',0,'carriers/fedex/showmethod','0'),(362,'default',0,'carriers/fedex/sort_order',''),(363,'default',0,'carriers/dhl/active','0'),(364,'default',0,'carriers/dhl/gateway_url','https://eCommerce.airborne.com/ApiLandingTest.asp'),(365,'default',0,'carriers/dhl/title','DHL'),(366,'default',0,'carriers/dhl/id',''),(367,'default',0,'carriers/dhl/password',''),(368,'default',0,'carriers/dhl/account',''),(369,'default',0,'carriers/dhl/shipping_key',''),(370,'default',0,'carriers/dhl/shipping_intlkey',''),(371,'default',0,'carriers/dhl/shipment_type','P'),(372,'default',0,'carriers/dhl/handling_type','F'),(373,'default',0,'carriers/dhl/handling_action','O'),(374,'default',0,'carriers/dhl/contentdesc','Big Box'),(375,'default',0,'carriers/dhl/handling_fee',''),(376,'default',0,'carriers/dhl/dutiable','0'),(377,'default',0,'carriers/dhl/max_package_weight','150'),(378,'default',0,'carriers/dhl/dutypaymenttype','R'),(379,'default',0,'carriers/dhl/allowed_methods','IE,E SAT,E 10:30AM,E,N,S,G'),(380,'default',0,'carriers/dhl/specificerrmsg','This shipping method is currently unavailable. If you would like to ship using this shipping method, please contact us.'),(381,'default',0,'carriers/dhl/showmethod','0'),(382,'default',0,'carriers/dhl/free_method','G'),(383,'default',0,'carriers/dhl/free_shipping_enable','0'),(384,'default',0,'carriers/dhl/free_shipping_subtotal',''),(385,'default',0,'carriers/dhl/additional_protection_enabled','0'),(386,'default',0,'carriers/dhl/additional_protection_min_value',''),(387,'default',0,'carriers/dhl/additional_protection_use_subtotal','0'),(388,'default',0,'carriers/dhl/additional_protection_value',''),(389,'default',0,'carriers/dhl/additional_protection_rounding','0'),(390,'default',0,'carriers/dhl/hazardous_materials','0'),(391,'default',0,'carriers/dhl/default_length',''),(392,'default',0,'carriers/dhl/default_width',''),(393,'default',0,'carriers/dhl/default_height',''),(394,'default',0,'carriers/dhl/shipment_days','1,2,3,4,5,6'),(395,'default',0,'carriers/dhl/intl_shipment_days','1,2,3,4,5'),(396,'default',0,'carriers/dhl/sallowspecific','0'),(397,'default',0,'carriers/dhl/sort_order',''),(398,'default',0,'carriers/freeshipping/specificcountry','ES'),(399,'default',0,'admin/emails/forgot_email_template','admin_emails_forgot_email_template'),(400,'default',0,'admin/emails/forgot_email_identity','general'),(401,'default',0,'admin/startup/page','dashboard'),(402,'default',0,'admin/url/use_custom','0'),(403,'default',0,'admin/url/custom',''),(404,'default',0,'admin/security/use_form_key','1'),(405,'default',0,'admin/security/session_cookie_lifetime',''),(406,'default',0,'admin/bouncingorange_tinymce/active','1'),(407,'default',0,'admin/bouncingorange_tinymce/active_gzip','0'),(408,'default',0,'admin/bouncingorange_tinymce/elements','description,short_description,page_content,group_3description'),(409,'default',0,'admin/bouncingorange_tinymce/language','es'),(410,'default',0,'admin/bouncingorange_tinymce/theme','advanced'),(411,'default',0,'admin/bouncingorange_tinymce/extra_config','width: \"100%\",\r\nheight: \"300px\",\r\n\r\ntheme_advanced_toolbar_location : \"top\",\r\ntheme_advanced_toolbar_align : \"left\",\r\ntheme_advanced_statusbar_location : \"none\",\r\n\r\ntheme_advanced_buttons1 : \"mybutton,bold,italic,underline,strikethrough,separator,bullist,numlist,separator,outdent,indent,separator,undo,redo,separator,link,unlink,separator,code\",\r\ntheme_advanced_buttons2 : \"\",\r\ntheme_advanced_buttons3 : \"\"\r\n				'),(412,'default',0,'dev/restrict/allow_ips',''),(413,'default',0,'dev/debug/profiler','0'),(414,'default',0,'dev/translate_inline/active','0'),(415,'default',0,'dev/translate_inline/active_admin','0'),(416,'default',0,'dev/log/active','0'),(417,'default',0,'dev/log/file','system.log'),(418,'default',0,'dev/log/exception_file','exception.log'),(419,'default',0,'dev/js/merge_files','1'),(420,'default',0,'dev/js/deprecation','0'),(421,'default',0,'admin/bouncingorange_tinymce/plugins','contextmenu,layer,paste,preview,table');
/*!40000 ALTER TABLE `core_config_data` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `core_email_template`
--

DROP TABLE IF EXISTS `core_email_template`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `core_email_template` (
  `template_id` int(7) unsigned NOT NULL auto_increment,
  `template_code` varchar(150) default NULL,
  `template_text` text,
  `template_type` int(3) unsigned default NULL,
  `template_subject` varchar(200) default NULL,
  `template_sender_name` varchar(200) default NULL,
  `template_sender_email` varchar(200) character set latin1 collate latin1_general_ci default NULL,
  `added_at` datetime default NULL,
  `modified_at` datetime default NULL,
  PRIMARY KEY  (`template_id`),
  UNIQUE KEY `template_code` (`template_code`),
  KEY `added_at` (`added_at`),
  KEY `modified_at` (`modified_at`)
) ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=utf8 COMMENT='Email templates';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `core_email_template`
--

LOCK TABLES `core_email_template` WRITE;
/*!40000 ALTER TABLE `core_email_template` DISABLE KEYS */;
INSERT INTO `core_email_template` VALUES (1,'Send product to a friend','Welcome, {{var name}}<br /><br />Please look at <a href=\"{{var product.getProductUrl()}}\">{{var product.name}}</a><br /><br />Here is message: <br />{{var message}}<br /><br />',2,'Welcome, {{var name}}',NULL,NULL,'2009-06-03 13:52:22','2009-06-03 13:52:22');
/*!40000 ALTER TABLE `core_email_template` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `core_flag`
--

DROP TABLE IF EXISTS `core_flag`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `core_flag` (
  `flag_id` smallint(5) unsigned NOT NULL auto_increment,
  `flag_code` varchar(255) NOT NULL,
  `state` smallint(5) unsigned NOT NULL default '0',
  `flag_data` text,
  `last_update` datetime NOT NULL,
  PRIMARY KEY  (`flag_id`),
  KEY `last_update` (`last_update`)
) ENGINE=InnoDB AUTO_INCREMENT=6 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `core_flag`
--

LOCK TABLES `core_flag` WRITE;
/*!40000 ALTER TABLE `core_flag` DISABLE KEYS */;
INSERT INTO `core_flag` VALUES (1,'catalog_product_flat',0,'a:1:{s:8:\"is_built\";b:0;}','2009-06-03 19:52:49');
/*!40000 ALTER TABLE `core_flag` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `core_layout_link`
--

DROP TABLE IF EXISTS `core_layout_link`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `core_layout_link` (
  `layout_link_id` int(10) unsigned NOT NULL auto_increment,
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `package` varchar(64) NOT NULL default '',
  `theme` varchar(64) NOT NULL default '',
  `layout_update_id` int(10) unsigned NOT NULL default '0',
  PRIMARY KEY  (`layout_link_id`),
  UNIQUE KEY `store_id` (`store_id`,`package`,`theme`,`layout_update_id`),
  KEY `FK_core_layout_link_update` (`layout_update_id`),
  CONSTRAINT `FK_CORE_LAYOUT_LINK_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CORE_LAYOUT_LINK_UPDATE` FOREIGN KEY (`layout_update_id`) REFERENCES `core_layout_update` (`layout_update_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `core_layout_link`
--

LOCK TABLES `core_layout_link` WRITE;
/*!40000 ALTER TABLE `core_layout_link` DISABLE KEYS */;
/*!40000 ALTER TABLE `core_layout_link` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `core_layout_update`
--

DROP TABLE IF EXISTS `core_layout_update`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `core_layout_update` (
  `layout_update_id` int(10) unsigned NOT NULL auto_increment,
  `handle` varchar(255) default NULL,
  `xml` text,
  PRIMARY KEY  (`layout_update_id`),
  KEY `handle` (`handle`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `core_layout_update`
--

LOCK TABLES `core_layout_update` WRITE;
/*!40000 ALTER TABLE `core_layout_update` DISABLE KEYS */;
/*!40000 ALTER TABLE `core_layout_update` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `core_resource`
--

DROP TABLE IF EXISTS `core_resource`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `core_resource` (
  `code` varchar(50) NOT NULL default '',
  `version` varchar(50) NOT NULL default '',
  PRIMARY KEY  (`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Resource version registry';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `core_resource`
--

LOCK TABLES `core_resource` WRITE;
/*!40000 ALTER TABLE `core_resource` DISABLE KEYS */;
INSERT INTO `core_resource` VALUES ('adminnotification_setup','1.0.0'),('admin_setup','0.7.1'),('amazonpayments_setup','0.1.2'),('api_setup','0.8.1'),('backup_setup','0.7.0'),('bundle_setup','0.1.7'),('cashondelivery_setup','0.1.0'),('catalogindex_setup','0.7.10'),('cataloginventory_setup','0.7.5'),('catalogrule_setup','0.7.7'),('catalogsearch_setup','0.7.6'),('catalog_setup','0.7.69'),('checkout_setup','0.9.3'),('cms_setup','0.7.8'),('compiler_setup','0.1.0'),('contacts_setup','0.8.0'),('core_setup','0.8.13'),('cron_setup','0.7.1'),('customer_setup','0.8.10'),('dataflow_setup','0.7.4'),('directory_setup','0.8.4'),('downloadable_setup','0.1.14'),('eav_setup','0.7.13'),('giftmessage_setup','0.7.2'),('googlebase_setup','0.1.1'),('googlecheckout_setup','0.7.3'),('googleoptimizer_setup','0.1.2'),('log_setup','0.7.6'),('newsletter_setup','0.8.0'),('paygate_setup','0.7.0'),('payment_setup','0.7.0'),('paypaluk_setup','0.7.0'),('paypal_setup','0.7.2'),('poll_setup','0.7.2'),('productalert_setup','0.7.2'),('rating_setup','0.7.2'),('reports_setup','0.7.7'),('review_setup','0.7.4'),('salesrule_setup','0.7.7'),('sales_setup','0.9.38'),('sendfriend_setup','0.7.2'),('shipping_setup','0.7.0'),('sitemap_setup','0.7.2'),('tag_setup','0.7.2'),('tax_setup','0.7.8'),('usa_setup','0.7.0'),('weee_setup','0.13'),('wishlist_setup','0.7.4');
/*!40000 ALTER TABLE `core_resource` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `core_session`
--

DROP TABLE IF EXISTS `core_session`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `core_session` (
  `session_id` varchar(255) NOT NULL default '',
  `website_id` smallint(5) unsigned default NULL,
  `session_expires` int(10) unsigned NOT NULL default '0',
  `session_data` mediumblob NOT NULL,
  PRIMARY KEY  (`session_id`),
  KEY `FK_SESSION_WEBSITE` (`website_id`),
  CONSTRAINT `FK_SESSION_WEBSITE` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Session data store';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `core_session`
--

LOCK TABLES `core_session` WRITE;
/*!40000 ALTER TABLE `core_session` DISABLE KEYS */;
INSERT INTO `core_session` VALUES ('0166c3f1393d6e61c07b45a85086fb33',NULL,1246575157,'core|a:5:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}s:12:\"visitor_data\";a:17:{s:0:\"\";N;s:11:\"server_addr\";i:1163467184;s:11:\"remote_addr\";i:1474176362;s:11:\"http_secure\";b:0;s:9:\"http_host\";s:18:\"www.maclaonline.es\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";s:20:\"http_accept_language\";s:35:\"es-es,es;q=0.8,en-us;q=0.5,en;q=0.3\";s:19:\"http_accept_charset\";s:30:\"ISO-8859-1,utf-8;q=0.7,*;q=0.7\";s:11:\"request_uri\";s:49:\"/tienda/index.php/vulcano/medallones-vulcano.html\";s:10:\"session_id\";s:32:\"0166c3f1393d6e61c07b45a85086fb33\";s:12:\"http_referer\";s:55:\"http://www.maclaonline.es/tienda/index.php/vulcano.html\";s:14:\"first_visit_at\";s:19:\"2009-07-02 22:27:45\";s:14:\"is_new_visitor\";b:0;s:13:\"last_visit_at\";s:19:\"2009-07-02 22:28:37\";s:10:\"visitor_id\";s:2:\"28\";s:27:\"catalog_compare_items_count\";i:0;s:11:\"last_url_id\";s:3:\"306\";}s:8:\"last_url\";s:65:\"http://www.maclaonline.es/tienda/index.php/catalog/category/view/\";}_cookie_revalidate|d:1246574565;catalog|a:5:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:24:\"last_visited_category_id\";s:1:\"6\";s:23:\"last_viewed_category_id\";s:1:\"6\";s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}}customer_base|a:2:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}}checkout|a:3:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}}store_default|a:2:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}}'),('706eb67da7d48e7eecf8bd6a060f7a82',NULL,1246575367,'core|a:5:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:12:\"67.222.37.11\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:0:\"\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}s:12:\"visitor_data\";a:16:{s:0:\"\";N;s:11:\"server_addr\";i:1163467184;s:11:\"remote_addr\";i:1138631947;s:11:\"http_secure\";b:0;s:9:\"http_host\";s:18:\"www.maclaonline.es\";s:15:\"http_user_agent\";s:0:\"\";s:20:\"http_accept_language\";s:0:\"\";s:19:\"http_accept_charset\";s:0:\"\";s:11:\"request_uri\";s:8:\"/tienda/\";s:10:\"session_id\";s:32:\"706eb67da7d48e7eecf8bd6a060f7a82\";s:12:\"http_referer\";s:0:\"\";s:14:\"first_visit_at\";s:19:\"2009-07-02 22:32:07\";s:14:\"is_new_visitor\";b:0;s:13:\"last_visit_at\";s:19:\"2009-07-02 22:32:07\";s:10:\"visitor_id\";s:2:\"29\";s:11:\"last_url_id\";s:3:\"307\";}s:8:\"last_url\";s:59:\"http://www.maclaonline.es/tienda/index.php/cms/index/index/\";}_cookie_revalidate|d:1246574827;customer_base|a:2:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:12:\"67.222.37.11\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:0:\"\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}}checkout|a:3:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:12:\"67.222.37.11\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:0:\"\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}}catalog|a:3:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:12:\"67.222.37.11\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:0:\"\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}}'),('7884ff26f86c0c69e029798a59eba19b',NULL,1246575370,'core|a:5:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:12:\"67.222.37.11\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:0:\"\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}s:12:\"visitor_data\";a:16:{s:0:\"\";N;s:11:\"server_addr\";i:1163467184;s:11:\"remote_addr\";i:1138631947;s:11:\"http_secure\";b:0;s:9:\"http_host\";s:18:\"www.maclaonline.es\";s:15:\"http_user_agent\";s:0:\"\";s:20:\"http_accept_language\";s:0:\"\";s:19:\"http_accept_charset\";s:0:\"\";s:11:\"request_uri\";s:8:\"/tienda/\";s:10:\"session_id\";s:32:\"7884ff26f86c0c69e029798a59eba19b\";s:12:\"http_referer\";s:0:\"\";s:14:\"first_visit_at\";s:19:\"2009-07-02 22:32:10\";s:14:\"is_new_visitor\";b:0;s:13:\"last_visit_at\";s:19:\"2009-07-02 22:32:10\";s:10:\"visitor_id\";s:2:\"30\";s:11:\"last_url_id\";s:3:\"308\";}s:8:\"last_url\";s:59:\"http://www.maclaonline.es/tienda/index.php/cms/index/index/\";}_cookie_revalidate|d:1246574830;customer_base|a:2:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:12:\"67.222.37.11\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:0:\"\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}}checkout|a:3:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:12:\"67.222.37.11\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:0:\"\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}}catalog|a:3:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:12:\"67.222.37.11\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:0:\"\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}}'),('987ff71c964ae972bd7fdbeee9210d95',NULL,1246574942,'core|a:5:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}s:12:\"visitor_data\";a:17:{s:0:\"\";N;s:11:\"server_addr\";i:1163467184;s:11:\"remote_addr\";i:1474176362;s:11:\"http_secure\";b:0;s:9:\"http_host\";s:18:\"www.maclaonline.es\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";s:20:\"http_accept_language\";s:35:\"es-es,es;q=0.8,en-us;q=0.5,en;q=0.3\";s:19:\"http_accept_charset\";s:30:\"ISO-8859-1,utf-8;q=0.7,*;q=0.7\";s:11:\"request_uri\";s:54:\"/tienda/index.php/review/product/list/id/2/category/6/\";s:10:\"session_id\";s:32:\"987ff71c964ae972bd7fdbeee9210d95\";s:12:\"http_referer\";s:105:\"http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html\";s:14:\"first_visit_at\";s:19:\"2009-07-02 22:20:53\";s:14:\"is_new_visitor\";b:0;s:13:\"last_visit_at\";s:19:\"2009-07-02 22:25:02\";s:10:\"visitor_id\";s:2:\"27\";s:27:\"catalog_compare_items_count\";i:0;s:11:\"last_url_id\";s:3:\"299\";}s:8:\"last_url\";s:63:\"http://www.maclaonline.es/tienda/index.php/review/product/list/\";}_cookie_revalidate|d:1246574153;catalog|a:6:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:24:\"last_visited_category_id\";s:1:\"6\";s:23:\"last_viewed_category_id\";s:1:\"6\";s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}s:22:\"last_viewed_product_id\";s:1:\"2\";}customer_base|a:3:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:2:\"id\";N;}checkout|a:3:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}}store_default|a:2:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}}reports|a:3:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:15:\"viewed_products\";b:1;}tag|a:3:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}}review|a:3:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}}'),('9c664286b89b843d95803b5706cb8350',NULL,1246575776,'core|a:4:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}s:9:\"_form_key\";s:16:\"rDT1TNOFV1KrXT5f\";}_cookie_revalidate|d:1246574670;adminhtml|a:4:{s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";O:31:\"Mage_Core_Model_Message_Success\":6:{s:8:\"\0*\0_type\";s:7:\"success\";s:8:\"\0*\0_code\";s:43:\"La copia de seguridad fue creada con éxito\";s:9:\"\0*\0_class\";s:0:\"\";s:10:\"\0*\0_method\";s:0:\"\";s:14:\"\0*\0_identifier\";N;s:12:\"\0*\0_isSticky\";b:0;}}s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:6:\"locale\";s:5:\"es_ES\";}admin|a:4:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:13:\"87.222.37.106\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:105:\"Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:4:\"user\";O:21:\"Mage_Admin_Model_User\":12:{s:15:\"\0*\0_eventPrefix\";s:10:\"admin_user\";s:8:\"\0*\0_role\";N;s:15:\"\0*\0_eventObject\";s:6:\"object\";s:16:\"\0*\0_resourceName\";s:10:\"admin/user\";s:12:\"\0*\0_resource\";N;s:26:\"\0*\0_resourceCollectionName\";s:21:\"admin/user_collection\";s:12:\"\0*\0_cacheTag\";b:0;s:19:\"\0*\0_dataSaveAllowed\";b:1;s:8:\"\0*\0_data\";a:13:{s:7:\"user_id\";s:1:\"2\";s:9:\"firstname\";s:8:\"Leopoldo\";s:8:\"lastname\";s:16:\"Martinez-Cubells\";s:5:\"email\";s:16:\"polo@lasbolas.es\";s:8:\"username\";s:5:\"admin\";s:8:\"password\";s:35:\"1278590f3737367e8ec5b0da8a151c3c:1I\";s:7:\"created\";s:19:\"2009-06-22 06:40:44\";s:8:\"modified\";s:19:\"2009-06-22 12:45:15\";s:7:\"logdate\";s:19:\"2009-07-02 22:35:33\";s:6:\"lognum\";s:2:\"17\";s:15:\"reload_acl_flag\";s:1:\"0\";s:9:\"is_active\";s:1:\"1\";s:5:\"extra\";a:1:{s:11:\"configState\";a:65:{s:15:\"general_country\";s:1:\"1\";s:14:\"general_locale\";s:1:\"1\";s:7:\"web_url\";s:1:\"1\";s:7:\"web_seo\";s:1:\"1\";s:12:\"web_unsecure\";s:1:\"1\";s:10:\"web_secure\";s:1:\"1\";s:11:\"web_default\";s:1:\"1\";s:9:\"web_polls\";s:1:\"1\";s:10:\"web_cookie\";s:1:\"1\";s:11:\"web_session\";s:1:\"1\";s:24:\"web_browser_capabilities\";s:1:\"1\";s:14:\"design_package\";s:1:\"1\";s:12:\"design_theme\";s:1:\"1\";s:11:\"design_head\";s:1:\"1\";s:13:\"design_header\";s:1:\"1\";s:13:\"design_footer\";s:1:\"1\";s:16:\"design_watermark\";s:1:\"0\";s:16:\"currency_options\";s:1:\"1\";s:20:\"currency_webservicex\";s:1:\"1\";s:15:\"currency_import\";s:1:\"1\";s:25:\"trans_email_ident_general\";s:1:\"1\";s:23:\"trans_email_ident_sales\";s:1:\"1\";s:25:\"trans_email_ident_support\";s:1:\"1\";s:25:\"trans_email_ident_custom1\";s:1:\"1\";s:25:\"trans_email_ident_custom2\";s:1:\"1\";s:17:\"contacts_contacts\";s:1:\"1\";s:14:\"contacts_email\";s:1:\"1\";s:11:\"system_cron\";s:1:\"1\";s:11:\"system_smtp\";s:1:\"1\";s:15:\"system_currency\";s:1:\"1\";s:10:\"system_log\";s:1:\"1\";s:24:\"system_adminnotification\";s:1:\"1\";s:14:\"payment_ccsave\";s:1:\"0\";s:12:\"payment_free\";s:1:\"0\";s:15:\"payment_checkmo\";s:1:\"0\";s:21:\"payment_purchaseorder\";s:1:\"0\";s:20:\"payment_authorizenet\";s:1:\"0\";s:16:\"payment_verisign\";s:1:\"0\";s:22:\"payment_paypal_express\";s:1:\"0\";s:21:\"payment_paypal_direct\";s:1:\"0\";s:23:\"payment_paypal_standard\";s:1:\"0\";s:24:\"payment_paypaluk_express\";s:1:\"0\";s:23:\"payment_paypaluk_direct\";s:1:\"0\";s:26:\"payment_amazonpayments_asp\";s:1:\"0\";s:26:\"payment_amazonpayments_cba\";s:1:\"0\";s:22:\"payment_cashondelivery\";s:1:\"1\";s:15:\"shipping_origin\";s:1:\"1\";s:15:\"shipping_option\";s:1:\"1\";s:17:\"carriers_flatrate\";s:1:\"1\";s:18:\"carriers_tablerate\";s:1:\"0\";s:21:\"carriers_freeshipping\";s:1:\"1\";s:12:\"carriers_ups\";s:1:\"1\";s:13:\"carriers_usps\";s:1:\"1\";s:14:\"carriers_fedex\";s:1:\"1\";s:12:\"carriers_dhl\";s:1:\"1\";s:12:\"admin_emails\";s:1:\"0\";s:13:\"admin_startup\";s:1:\"0\";s:9:\"admin_url\";s:1:\"0\";s:14:\"admin_security\";s:1:\"0\";s:28:\"admin_bouncingorange_tinymce\";s:1:\"1\";s:12:\"dev_restrict\";s:1:\"1\";s:9:\"dev_debug\";s:1:\"1\";s:20:\"dev_translate_inline\";s:1:\"1\";s:7:\"dev_log\";s:1:\"0\";s:6:\"dev_js\";s:1:\"0\";}}}s:12:\"\0*\0_origData\";a:13:{s:7:\"user_id\";s:1:\"2\";s:9:\"firstname\";s:8:\"Leopoldo\";s:8:\"lastname\";s:16:\"Martinez-Cubells\";s:5:\"email\";s:16:\"polo@lasbolas.es\";s:8:\"username\";s:5:\"admin\";s:8:\"password\";s:35:\"1278590f3737367e8ec5b0da8a151c3c:1I\";s:7:\"created\";s:19:\"2009-06-22 06:40:44\";s:8:\"modified\";s:19:\"2009-06-22 12:45:15\";s:7:\"logdate\";s:19:\"2009-07-02 22:35:33\";s:6:\"lognum\";s:2:\"17\";s:15:\"reload_acl_flag\";s:1:\"0\";s:9:\"is_active\";s:1:\"1\";s:5:\"extra\";a:1:{s:11:\"configState\";a:65:{s:15:\"general_country\";s:1:\"1\";s:14:\"general_locale\";s:1:\"1\";s:7:\"web_url\";s:1:\"1\";s:7:\"web_seo\";s:1:\"1\";s:12:\"web_unsecure\";s:1:\"1\";s:10:\"web_secure\";s:1:\"1\";s:11:\"web_default\";s:1:\"1\";s:9:\"web_polls\";s:1:\"1\";s:10:\"web_cookie\";s:1:\"1\";s:11:\"web_session\";s:1:\"1\";s:24:\"web_browser_capabilities\";s:1:\"1\";s:14:\"design_package\";s:1:\"1\";s:12:\"design_theme\";s:1:\"1\";s:11:\"design_head\";s:1:\"1\";s:13:\"design_header\";s:1:\"1\";s:13:\"design_footer\";s:1:\"1\";s:16:\"design_watermark\";s:1:\"0\";s:16:\"currency_options\";s:1:\"1\";s:20:\"currency_webservicex\";s:1:\"1\";s:15:\"currency_import\";s:1:\"1\";s:25:\"trans_email_ident_general\";s:1:\"1\";s:23:\"trans_email_ident_sales\";s:1:\"1\";s:25:\"trans_email_ident_support\";s:1:\"1\";s:25:\"trans_email_ident_custom1\";s:1:\"1\";s:25:\"trans_email_ident_custom2\";s:1:\"1\";s:17:\"contacts_contacts\";s:1:\"1\";s:14:\"contacts_email\";s:1:\"1\";s:11:\"system_cron\";s:1:\"1\";s:11:\"system_smtp\";s:1:\"1\";s:15:\"system_currency\";s:1:\"1\";s:10:\"system_log\";s:1:\"1\";s:24:\"system_adminnotification\";s:1:\"1\";s:14:\"payment_ccsave\";s:1:\"0\";s:12:\"payment_free\";s:1:\"0\";s:15:\"payment_checkmo\";s:1:\"0\";s:21:\"payment_purchaseorder\";s:1:\"0\";s:20:\"payment_authorizenet\";s:1:\"0\";s:16:\"payment_verisign\";s:1:\"0\";s:22:\"payment_paypal_express\";s:1:\"0\";s:21:\"payment_paypal_direct\";s:1:\"0\";s:23:\"payment_paypal_standard\";s:1:\"0\";s:24:\"payment_paypaluk_express\";s:1:\"0\";s:23:\"payment_paypaluk_direct\";s:1:\"0\";s:26:\"payment_amazonpayments_asp\";s:1:\"0\";s:26:\"payment_amazonpayments_cba\";s:1:\"0\";s:22:\"payment_cashondelivery\";s:1:\"1\";s:15:\"shipping_origin\";s:1:\"1\";s:15:\"shipping_option\";s:1:\"1\";s:17:\"carriers_flatrate\";s:1:\"1\";s:18:\"carriers_tablerate\";s:1:\"0\";s:21:\"carriers_freeshipping\";s:1:\"1\";s:12:\"carriers_ups\";s:1:\"1\";s:13:\"carriers_usps\";s:1:\"1\";s:14:\"carriers_fedex\";s:1:\"1\";s:12:\"carriers_dhl\";s:1:\"1\";s:12:\"admin_emails\";s:1:\"0\";s:13:\"admin_startup\";s:1:\"0\";s:9:\"admin_url\";s:1:\"0\";s:14:\"admin_security\";s:1:\"0\";s:28:\"admin_bouncingorange_tinymce\";s:1:\"1\";s:12:\"dev_restrict\";s:1:\"1\";s:9:\"dev_debug\";s:1:\"1\";s:20:\"dev_translate_inline\";s:1:\"1\";s:7:\"dev_log\";s:1:\"0\";s:6:\"dev_js\";s:1:\"0\";}}}s:15:\"\0*\0_idFieldName\";s:7:\"user_id\";s:13:\"\0*\0_isDeleted\";b:0;}s:3:\"acl\";O:20:\"Mage_Admin_Model_Acl\":3:{s:16:\"\0*\0_roleRegistry\";O:34:\"Mage_Admin_Model_Acl_Role_Registry\":1:{s:9:\"\0*\0_roles\";a:3:{s:2:\"G1\";a:3:{s:8:\"instance\";O:31:\"Mage_Admin_Model_Acl_Role_Group\":1:{s:10:\"\0*\0_roleId\";s:2:\"G1\";}s:7:\"parents\";a:0:{}s:8:\"children\";a:2:{s:2:\"U2\";O:30:\"Mage_Admin_Model_Acl_Role_User\":1:{s:10:\"\0*\0_roleId\";s:2:\"U2\";}s:2:\"U3\";O:30:\"Mage_Admin_Model_Acl_Role_User\":1:{s:10:\"\0*\0_roleId\";s:2:\"U3\";}}}s:2:\"U2\";a:3:{s:8:\"instance\";r:219;s:7:\"parents\";a:1:{s:2:\"G1\";r:215;}s:8:\"children\";a:0:{}}s:2:\"U3\";a:3:{s:8:\"instance\";r:221;s:7:\"parents\";a:1:{s:2:\"G1\";r:215;}s:8:\"children\";a:0:{}}}}s:13:\"\0*\0_resources\";a:153:{s:3:\"all\";a:3:{s:8:\"instance\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:3:\"all\";}s:6:\"parent\";N;s:8:\"children\";a:0:{}}s:5:\"admin\";a:3:{s:8:\"instance\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:5:\"admin\";}s:6:\"parent\";N;s:8:\"children\";a:10:{s:15:\"admin/dashboard\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:15:\"admin/dashboard\";}s:12:\"admin/system\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:12:\"admin/system\";}s:19:\"admin/global_search\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:19:\"admin/global_search\";}s:14:\"admin/customer\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:14:\"admin/customer\";}s:13:\"admin/catalog\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:13:\"admin/catalog\";}s:11:\"admin/promo\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:11:\"admin/promo\";}s:11:\"admin/sales\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:11:\"admin/sales\";}s:9:\"admin/cms\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:9:\"admin/cms\";}s:12:\"admin/report\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:12:\"admin/report\";}s:16:\"admin/newsletter\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:16:\"admin/newsletter\";}}}s:15:\"admin/dashboard\";a:3:{s:8:\"instance\";r:244;s:6:\"parent\";r:240;s:8:\"children\";a:0:{}}s:12:\"admin/system\";a:3:{s:8:\"instance\";r:246;s:6:\"parent\";r:240;s:8:\"children\";a:13:{s:16:\"admin/system/acl\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:16:\"admin/system/acl\";}s:18:\"admin/system/store\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:18:\"admin/system/store\";}s:19:\"admin/system/design\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:19:\"admin/system/design\";}s:19:\"admin/system/config\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:19:\"admin/system/config\";}s:21:\"admin/system/currency\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:21:\"admin/system/currency\";}s:27:\"admin/system/email_template\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:27:\"admin/system/email_template\";}s:22:\"admin/system/myaccount\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:22:\"admin/system/myaccount\";}s:18:\"admin/system/tools\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:18:\"admin/system/tools\";}s:20:\"admin/system/convert\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:20:\"admin/system/convert\";}s:18:\"admin/system/cache\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:18:\"admin/system/cache\";}s:23:\"admin/system/extensions\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:23:\"admin/system/extensions\";}s:30:\"admin/system/adminnotification\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:30:\"admin/system/adminnotification\";}s:16:\"admin/system/api\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:16:\"admin/system/api\";}}}s:16:\"admin/system/acl\";a:3:{s:8:\"instance\";r:272;s:6:\"parent\";r:246;s:8:\"children\";a:2:{s:22:\"admin/system/acl/roles\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:22:\"admin/system/acl/roles\";}s:22:\"admin/system/acl/users\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:22:\"admin/system/acl/users\";}}}s:22:\"admin/system/acl/roles\";a:3:{s:8:\"instance\";r:302;s:6:\"parent\";r:272;s:8:\"children\";a:0:{}}s:22:\"admin/system/acl/users\";a:3:{s:8:\"instance\";r:304;s:6:\"parent\";r:272;s:8:\"children\";a:0:{}}s:18:\"admin/system/store\";a:3:{s:8:\"instance\";r:274;s:6:\"parent\";r:246;s:8:\"children\";a:0:{}}s:19:\"admin/system/design\";a:3:{s:8:\"instance\";r:276;s:6:\"parent\";r:246;s:8:\"children\";a:0:{}}s:19:\"admin/system/config\";a:3:{s:8:\"instance\";r:278;s:6:\"parent\";r:246;s:8:\"children\";a:30:{s:27:\"admin/system/config/general\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:27:\"admin/system/config/general\";}s:23:\"admin/system/config/web\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:23:\"admin/system/config/web\";}s:26:\"admin/system/config/design\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:26:\"admin/system/config/design\";}s:26:\"admin/system/config/system\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:26:\"admin/system/config/system\";}s:28:\"admin/system/config/advanced\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:28:\"admin/system/config/advanced\";}s:31:\"admin/system/config/trans_email\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:31:\"admin/system/config/trans_email\";}s:23:\"admin/system/config/dev\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:23:\"admin/system/config/dev\";}s:28:\"admin/system/config/currency\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:28:\"admin/system/config/currency\";}s:30:\"admin/system/config/sendfriend\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:30:\"admin/system/config/sendfriend\";}s:25:\"admin/system/config/admin\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:25:\"admin/system/config/admin\";}s:28:\"admin/system/config/customer\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:28:\"admin/system/config/customer\";}s:27:\"admin/system/config/catalog\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:27:\"admin/system/config/catalog\";}s:25:\"admin/system/config/sales\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:25:\"admin/system/config/sales\";}s:31:\"admin/system/config/sales_email\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:31:\"admin/system/config/sales_email\";}s:29:\"admin/system/config/sales_pdf\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:29:\"admin/system/config/sales_pdf\";}s:28:\"admin/system/config/checkout\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:28:\"admin/system/config/checkout\";}s:28:\"admin/system/config/shipping\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:28:\"admin/system/config/shipping\";}s:28:\"admin/system/config/carriers\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:28:\"admin/system/config/carriers\";}s:27:\"admin/system/config/payment\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:27:\"admin/system/config/payment\";}s:26:\"admin/system/config/paypal\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:26:\"admin/system/config/paypal\";}s:26:\"admin/system/config/google\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:26:\"admin/system/config/google\";}s:30:\"admin/system/config/newsletter\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:30:\"admin/system/config/newsletter\";}s:23:\"admin/system/config/tax\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:23:\"admin/system/config/tax\";}s:28:\"admin/system/config/wishlist\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:28:\"admin/system/config/wishlist\";}s:36:\"admin/system/config/cataloginventory\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:36:\"admin/system/config/cataloginventory\";}s:28:\"admin/system/config/contacts\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:28:\"admin/system/config/contacts\";}s:27:\"admin/system/config/sitemap\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:27:\"admin/system/config/sitemap\";}s:23:\"admin/system/config/rss\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:23:\"admin/system/config/rss\";}s:23:\"admin/system/config/api\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:23:\"admin/system/config/api\";}s:32:\"admin/system/config/downloadable\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:32:\"admin/system/config/downloadable\";}}}s:27:\"admin/system/config/general\";a:3:{s:8:\"instance\";r:326;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:23:\"admin/system/config/web\";a:3:{s:8:\"instance\";r:328;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:26:\"admin/system/config/design\";a:3:{s:8:\"instance\";r:330;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:26:\"admin/system/config/system\";a:3:{s:8:\"instance\";r:332;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:28:\"admin/system/config/advanced\";a:3:{s:8:\"instance\";r:334;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:31:\"admin/system/config/trans_email\";a:3:{s:8:\"instance\";r:336;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:23:\"admin/system/config/dev\";a:3:{s:8:\"instance\";r:338;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:28:\"admin/system/config/currency\";a:3:{s:8:\"instance\";r:340;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:30:\"admin/system/config/sendfriend\";a:3:{s:8:\"instance\";r:342;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:25:\"admin/system/config/admin\";a:3:{s:8:\"instance\";r:344;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:28:\"admin/system/config/customer\";a:3:{s:8:\"instance\";r:346;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:27:\"admin/system/config/catalog\";a:3:{s:8:\"instance\";r:348;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:25:\"admin/system/config/sales\";a:3:{s:8:\"instance\";r:350;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:31:\"admin/system/config/sales_email\";a:3:{s:8:\"instance\";r:352;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:29:\"admin/system/config/sales_pdf\";a:3:{s:8:\"instance\";r:354;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:28:\"admin/system/config/checkout\";a:3:{s:8:\"instance\";r:356;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:28:\"admin/system/config/shipping\";a:3:{s:8:\"instance\";r:358;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:28:\"admin/system/config/carriers\";a:3:{s:8:\"instance\";r:360;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:27:\"admin/system/config/payment\";a:3:{s:8:\"instance\";r:362;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:26:\"admin/system/config/paypal\";a:3:{s:8:\"instance\";r:364;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:26:\"admin/system/config/google\";a:3:{s:8:\"instance\";r:366;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:30:\"admin/system/config/newsletter\";a:3:{s:8:\"instance\";r:368;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:23:\"admin/system/config/tax\";a:3:{s:8:\"instance\";r:370;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:28:\"admin/system/config/wishlist\";a:3:{s:8:\"instance\";r:372;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:36:\"admin/system/config/cataloginventory\";a:3:{s:8:\"instance\";r:374;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:28:\"admin/system/config/contacts\";a:3:{s:8:\"instance\";r:376;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:27:\"admin/system/config/sitemap\";a:3:{s:8:\"instance\";r:378;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:23:\"admin/system/config/rss\";a:3:{s:8:\"instance\";r:380;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:23:\"admin/system/config/api\";a:3:{s:8:\"instance\";r:382;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:32:\"admin/system/config/downloadable\";a:3:{s:8:\"instance\";r:384;s:6:\"parent\";r:278;s:8:\"children\";a:0:{}}s:21:\"admin/system/currency\";a:3:{s:8:\"instance\";r:280;s:6:\"parent\";r:246;s:8:\"children\";a:0:{}}s:27:\"admin/system/email_template\";a:3:{s:8:\"instance\";r:282;s:6:\"parent\";r:246;s:8:\"children\";a:0:{}}s:22:\"admin/system/myaccount\";a:3:{s:8:\"instance\";r:284;s:6:\"parent\";r:246;s:8:\"children\";a:0:{}}s:18:\"admin/system/tools\";a:3:{s:8:\"instance\";r:286;s:6:\"parent\";r:246;s:8:\"children\";a:2:{s:25:\"admin/system/tools/backup\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:25:\"admin/system/tools/backup\";}s:27:\"admin/system/tools/compiler\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:27:\"admin/system/tools/compiler\";}}}s:25:\"admin/system/tools/backup\";a:3:{s:8:\"instance\";r:522;s:6:\"parent\";r:286;s:8:\"children\";a:0:{}}s:27:\"admin/system/tools/compiler\";a:3:{s:8:\"instance\";r:524;s:6:\"parent\";r:286;s:8:\"children\";a:0:{}}s:20:\"admin/system/convert\";a:3:{s:8:\"instance\";r:288;s:6:\"parent\";r:246;s:8:\"children\";a:2:{s:24:\"admin/system/convert/gui\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:24:\"admin/system/convert/gui\";}s:29:\"admin/system/convert/profiles\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:29:\"admin/system/convert/profiles\";}}}s:24:\"admin/system/convert/gui\";a:3:{s:8:\"instance\";r:538;s:6:\"parent\";r:288;s:8:\"children\";a:0:{}}s:29:\"admin/system/convert/profiles\";a:3:{s:8:\"instance\";r:540;s:6:\"parent\";r:288;s:8:\"children\";a:0:{}}s:18:\"admin/system/cache\";a:3:{s:8:\"instance\";r:290;s:6:\"parent\";r:246;s:8:\"children\";a:0:{}}s:23:\"admin/system/extensions\";a:3:{s:8:\"instance\";r:292;s:6:\"parent\";r:246;s:8:\"children\";a:2:{s:29:\"admin/system/extensions/local\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:29:\"admin/system/extensions/local\";}s:30:\"admin/system/extensions/custom\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:30:\"admin/system/extensions/custom\";}}}s:29:\"admin/system/extensions/local\";a:3:{s:8:\"instance\";r:558;s:6:\"parent\";r:292;s:8:\"children\";a:0:{}}s:30:\"admin/system/extensions/custom\";a:3:{s:8:\"instance\";r:560;s:6:\"parent\";r:292;s:8:\"children\";a:0:{}}s:30:\"admin/system/adminnotification\";a:3:{s:8:\"instance\";r:294;s:6:\"parent\";r:246;s:8:\"children\";a:4:{s:43:\"admin/system/adminnotification/show_toolbar\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:43:\"admin/system/adminnotification/show_toolbar\";}s:40:\"admin/system/adminnotification/show_list\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:40:\"admin/system/adminnotification/show_list\";}s:43:\"admin/system/adminnotification/mark_as_read\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:43:\"admin/system/adminnotification/mark_as_read\";}s:37:\"admin/system/adminnotification/remove\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:37:\"admin/system/adminnotification/remove\";}}}s:43:\"admin/system/adminnotification/show_toolbar\";a:3:{s:8:\"instance\";r:574;s:6:\"parent\";r:294;s:8:\"children\";a:0:{}}s:40:\"admin/system/adminnotification/show_list\";a:3:{s:8:\"instance\";r:576;s:6:\"parent\";r:294;s:8:\"children\";a:0:{}}s:43:\"admin/system/adminnotification/mark_as_read\";a:3:{s:8:\"instance\";r:578;s:6:\"parent\";r:294;s:8:\"children\";a:0:{}}s:37:\"admin/system/adminnotification/remove\";a:3:{s:8:\"instance\";r:580;s:6:\"parent\";r:294;s:8:\"children\";a:0:{}}s:16:\"admin/system/api\";a:3:{s:8:\"instance\";r:296;s:6:\"parent\";r:246;s:8:\"children\";a:2:{s:22:\"admin/system/api/users\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:22:\"admin/system/api/users\";}s:22:\"admin/system/api/roles\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:22:\"admin/system/api/roles\";}}}s:22:\"admin/system/api/users\";a:3:{s:8:\"instance\";r:602;s:6:\"parent\";r:296;s:8:\"children\";a:0:{}}s:22:\"admin/system/api/roles\";a:3:{s:8:\"instance\";r:604;s:6:\"parent\";r:296;s:8:\"children\";a:0:{}}s:19:\"admin/global_search\";a:3:{s:8:\"instance\";r:248;s:6:\"parent\";r:240;s:8:\"children\";a:0:{}}s:14:\"admin/customer\";a:3:{s:8:\"instance\";r:250;s:6:\"parent\";r:240;s:8:\"children\";a:3:{s:20:\"admin/customer/group\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:20:\"admin/customer/group\";}s:21:\"admin/customer/manage\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:21:\"admin/customer/manage\";}s:21:\"admin/customer/online\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:21:\"admin/customer/online\";}}}s:20:\"admin/customer/group\";a:3:{s:8:\"instance\";r:622;s:6:\"parent\";r:250;s:8:\"children\";a:0:{}}s:21:\"admin/customer/manage\";a:3:{s:8:\"instance\";r:624;s:6:\"parent\";r:250;s:8:\"children\";a:0:{}}s:21:\"admin/customer/online\";a:3:{s:8:\"instance\";r:626;s:6:\"parent\";r:250;s:8:\"children\";a:0:{}}s:13:\"admin/catalog\";a:3:{s:8:\"instance\";r:252;s:6:\"parent\";r:240;s:8:\"children\";a:10:{s:24:\"admin/catalog/attributes\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:24:\"admin/catalog/attributes\";}s:24:\"admin/catalog/categories\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:24:\"admin/catalog/categories\";}s:22:\"admin/catalog/products\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:22:\"admin/catalog/products\";}s:31:\"admin/catalog/update_attributes\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:31:\"admin/catalog/update_attributes\";}s:20:\"admin/catalog/search\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:20:\"admin/catalog/search\";}s:24:\"admin/catalog/urlrewrite\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:24:\"admin/catalog/urlrewrite\";}s:29:\"admin/catalog/reviews_ratings\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:29:\"admin/catalog/reviews_ratings\";}s:17:\"admin/catalog/tag\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:17:\"admin/catalog/tag\";}s:21:\"admin/catalog/sitemap\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:21:\"admin/catalog/sitemap\";}s:24:\"admin/catalog/googlebase\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:24:\"admin/catalog/googlebase\";}}}s:24:\"admin/catalog/attributes\";a:3:{s:8:\"instance\";r:644;s:6:\"parent\";r:252;s:8:\"children\";a:2:{s:35:\"admin/catalog/attributes/attributes\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:35:\"admin/catalog/attributes/attributes\";}s:29:\"admin/catalog/attributes/sets\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:29:\"admin/catalog/attributes/sets\";}}}s:35:\"admin/catalog/attributes/attributes\";a:3:{s:8:\"instance\";r:668;s:6:\"parent\";r:644;s:8:\"children\";a:0:{}}s:29:\"admin/catalog/attributes/sets\";a:3:{s:8:\"instance\";r:670;s:6:\"parent\";r:644;s:8:\"children\";a:0:{}}s:24:\"admin/catalog/categories\";a:3:{s:8:\"instance\";r:646;s:6:\"parent\";r:252;s:8:\"children\";a:0:{}}s:22:\"admin/catalog/products\";a:3:{s:8:\"instance\";r:648;s:6:\"parent\";r:252;s:8:\"children\";a:0:{}}s:31:\"admin/catalog/update_attributes\";a:3:{s:8:\"instance\";r:650;s:6:\"parent\";r:252;s:8:\"children\";a:0:{}}s:20:\"admin/catalog/search\";a:3:{s:8:\"instance\";r:652;s:6:\"parent\";r:252;s:8:\"children\";a:0:{}}s:24:\"admin/catalog/urlrewrite\";a:3:{s:8:\"instance\";r:654;s:6:\"parent\";r:252;s:8:\"children\";a:0:{}}s:29:\"admin/catalog/reviews_ratings\";a:3:{s:8:\"instance\";r:656;s:6:\"parent\";r:252;s:8:\"children\";a:2:{s:37:\"admin/catalog/reviews_ratings/ratings\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:37:\"admin/catalog/reviews_ratings/ratings\";}s:37:\"admin/catalog/reviews_ratings/reviews\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:37:\"admin/catalog/reviews_ratings/reviews\";}}}s:37:\"admin/catalog/reviews_ratings/ratings\";a:3:{s:8:\"instance\";r:704;s:6:\"parent\";r:656;s:8:\"children\";a:0:{}}s:37:\"admin/catalog/reviews_ratings/reviews\";a:3:{s:8:\"instance\";r:706;s:6:\"parent\";r:656;s:8:\"children\";a:2:{s:41:\"admin/catalog/reviews_ratings/reviews/all\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:41:\"admin/catalog/reviews_ratings/reviews/all\";}s:45:\"admin/catalog/reviews_ratings/reviews/pending\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:45:\"admin/catalog/reviews_ratings/reviews/pending\";}}}s:41:\"admin/catalog/reviews_ratings/reviews/all\";a:3:{s:8:\"instance\";r:716;s:6:\"parent\";r:706;s:8:\"children\";a:0:{}}s:45:\"admin/catalog/reviews_ratings/reviews/pending\";a:3:{s:8:\"instance\";r:718;s:6:\"parent\";r:706;s:8:\"children\";a:0:{}}s:17:\"admin/catalog/tag\";a:3:{s:8:\"instance\";r:658;s:6:\"parent\";r:252;s:8:\"children\";a:2:{s:21:\"admin/catalog/tag/all\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:21:\"admin/catalog/tag/all\";}s:25:\"admin/catalog/tag/pending\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:25:\"admin/catalog/tag/pending\";}}}s:21:\"admin/catalog/tag/all\";a:3:{s:8:\"instance\";r:732;s:6:\"parent\";r:658;s:8:\"children\";a:0:{}}s:25:\"admin/catalog/tag/pending\";a:3:{s:8:\"instance\";r:734;s:6:\"parent\";r:658;s:8:\"children\";a:0:{}}s:21:\"admin/catalog/sitemap\";a:3:{s:8:\"instance\";r:660;s:6:\"parent\";r:252;s:8:\"children\";a:0:{}}s:24:\"admin/catalog/googlebase\";a:3:{s:8:\"instance\";r:662;s:6:\"parent\";r:252;s:8:\"children\";a:2:{s:30:\"admin/catalog/googlebase/types\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:30:\"admin/catalog/googlebase/types\";}s:30:\"admin/catalog/googlebase/items\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:30:\"admin/catalog/googlebase/items\";}}}s:30:\"admin/catalog/googlebase/types\";a:3:{s:8:\"instance\";r:752;s:6:\"parent\";r:662;s:8:\"children\";a:0:{}}s:30:\"admin/catalog/googlebase/items\";a:3:{s:8:\"instance\";r:754;s:6:\"parent\";r:662;s:8:\"children\";a:0:{}}s:11:\"admin/promo\";a:3:{s:8:\"instance\";r:254;s:6:\"parent\";r:240;s:8:\"children\";a:2:{s:19:\"admin/promo/catalog\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:19:\"admin/promo/catalog\";}s:17:\"admin/promo/quote\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:17:\"admin/promo/quote\";}}}s:19:\"admin/promo/catalog\";a:3:{s:8:\"instance\";r:768;s:6:\"parent\";r:254;s:8:\"children\";a:0:{}}s:17:\"admin/promo/quote\";a:3:{s:8:\"instance\";r:770;s:6:\"parent\";r:254;s:8:\"children\";a:0:{}}s:11:\"admin/sales\";a:3:{s:8:\"instance\";r:256;s:6:\"parent\";r:240;s:8:\"children\";a:6:{s:17:\"admin/sales/order\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:17:\"admin/sales/order\";}s:19:\"admin/sales/invoice\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:19:\"admin/sales/invoice\";}s:20:\"admin/sales/shipment\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:20:\"admin/sales/shipment\";}s:22:\"admin/sales/creditmemo\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:22:\"admin/sales/creditmemo\";}s:29:\"admin/sales/checkoutagreement\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:29:\"admin/sales/checkoutagreement\";}s:15:\"admin/sales/tax\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:15:\"admin/sales/tax\";}}}s:17:\"admin/sales/order\";a:3:{s:8:\"instance\";r:784;s:6:\"parent\";r:256;s:8:\"children\";a:1:{s:25:\"admin/sales/order/actions\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:25:\"admin/sales/order/actions\";}}}s:25:\"admin/sales/order/actions\";a:3:{s:8:\"instance\";r:800;s:6:\"parent\";r:784;s:8:\"children\";a:12:{s:32:\"admin/sales/order/actions/create\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:32:\"admin/sales/order/actions/create\";}s:30:\"admin/sales/order/actions/view\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:30:\"admin/sales/order/actions/view\";}s:33:\"admin/sales/order/actions/reorder\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:33:\"admin/sales/order/actions/reorder\";}s:30:\"admin/sales/order/actions/edit\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:30:\"admin/sales/order/actions/edit\";}s:32:\"admin/sales/order/actions/cancel\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:32:\"admin/sales/order/actions/cancel\";}s:33:\"admin/sales/order/actions/capture\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:33:\"admin/sales/order/actions/capture\";}s:33:\"admin/sales/order/actions/invoice\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:33:\"admin/sales/order/actions/invoice\";}s:36:\"admin/sales/order/actions/creditmemo\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:36:\"admin/sales/order/actions/creditmemo\";}s:30:\"admin/sales/order/actions/hold\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:30:\"admin/sales/order/actions/hold\";}s:32:\"admin/sales/order/actions/unhold\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:32:\"admin/sales/order/actions/unhold\";}s:30:\"admin/sales/order/actions/ship\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:30:\"admin/sales/order/actions/ship\";}s:33:\"admin/sales/order/actions/comment\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:33:\"admin/sales/order/actions/comment\";}}}s:32:\"admin/sales/order/actions/create\";a:3:{s:8:\"instance\";r:806;s:6:\"parent\";r:800;s:8:\"children\";a:0:{}}s:30:\"admin/sales/order/actions/view\";a:3:{s:8:\"instance\";r:808;s:6:\"parent\";r:800;s:8:\"children\";a:0:{}}s:33:\"admin/sales/order/actions/reorder\";a:3:{s:8:\"instance\";r:810;s:6:\"parent\";r:800;s:8:\"children\";a:0:{}}s:30:\"admin/sales/order/actions/edit\";a:3:{s:8:\"instance\";r:812;s:6:\"parent\";r:800;s:8:\"children\";a:0:{}}s:32:\"admin/sales/order/actions/cancel\";a:3:{s:8:\"instance\";r:814;s:6:\"parent\";r:800;s:8:\"children\";a:0:{}}s:33:\"admin/sales/order/actions/capture\";a:3:{s:8:\"instance\";r:816;s:6:\"parent\";r:800;s:8:\"children\";a:0:{}}s:33:\"admin/sales/order/actions/invoice\";a:3:{s:8:\"instance\";r:818;s:6:\"parent\";r:800;s:8:\"children\";a:0:{}}s:36:\"admin/sales/order/actions/creditmemo\";a:3:{s:8:\"instance\";r:820;s:6:\"parent\";r:800;s:8:\"children\";a:0:{}}s:30:\"admin/sales/order/actions/hold\";a:3:{s:8:\"instance\";r:822;s:6:\"parent\";r:800;s:8:\"children\";a:0:{}}s:32:\"admin/sales/order/actions/unhold\";a:3:{s:8:\"instance\";r:824;s:6:\"parent\";r:800;s:8:\"children\";a:0:{}}s:30:\"admin/sales/order/actions/ship\";a:3:{s:8:\"instance\";r:826;s:6:\"parent\";r:800;s:8:\"children\";a:0:{}}s:33:\"admin/sales/order/actions/comment\";a:3:{s:8:\"instance\";r:828;s:6:\"parent\";r:800;s:8:\"children\";a:0:{}}s:19:\"admin/sales/invoice\";a:3:{s:8:\"instance\";r:786;s:6:\"parent\";r:256;s:8:\"children\";a:0:{}}s:20:\"admin/sales/shipment\";a:3:{s:8:\"instance\";r:788;s:6:\"parent\";r:256;s:8:\"children\";a:0:{}}s:22:\"admin/sales/creditmemo\";a:3:{s:8:\"instance\";r:790;s:6:\"parent\";r:256;s:8:\"children\";a:0:{}}s:29:\"admin/sales/checkoutagreement\";a:3:{s:8:\"instance\";r:792;s:6:\"parent\";r:256;s:8:\"children\";a:0:{}}s:15:\"admin/sales/tax\";a:3:{s:8:\"instance\";r:794;s:6:\"parent\";r:256;s:8:\"children\";a:5:{s:32:\"admin/sales/tax/classes_customer\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:32:\"admin/sales/tax/classes_customer\";}s:31:\"admin/sales/tax/classes_product\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:31:\"admin/sales/tax/classes_product\";}s:29:\"admin/sales/tax/import_export\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:29:\"admin/sales/tax/import_export\";}s:21:\"admin/sales/tax/rates\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:21:\"admin/sales/tax/rates\";}s:21:\"admin/sales/tax/rules\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:21:\"admin/sales/tax/rules\";}}}s:32:\"admin/sales/tax/classes_customer\";a:3:{s:8:\"instance\";r:898;s:6:\"parent\";r:794;s:8:\"children\";a:0:{}}s:31:\"admin/sales/tax/classes_product\";a:3:{s:8:\"instance\";r:900;s:6:\"parent\";r:794;s:8:\"children\";a:0:{}}s:29:\"admin/sales/tax/import_export\";a:3:{s:8:\"instance\";r:902;s:6:\"parent\";r:794;s:8:\"children\";a:0:{}}s:21:\"admin/sales/tax/rates\";a:3:{s:8:\"instance\";r:904;s:6:\"parent\";r:794;s:8:\"children\";a:0:{}}s:21:\"admin/sales/tax/rules\";a:3:{s:8:\"instance\";r:906;s:6:\"parent\";r:794;s:8:\"children\";a:0:{}}s:9:\"admin/cms\";a:3:{s:8:\"instance\";r:258;s:6:\"parent\";r:240;s:8:\"children\";a:3:{s:15:\"admin/cms/block\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:15:\"admin/cms/block\";}s:14:\"admin/cms/page\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:14:\"admin/cms/page\";}s:14:\"admin/cms/poll\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:14:\"admin/cms/poll\";}}}s:15:\"admin/cms/block\";a:3:{s:8:\"instance\";r:932;s:6:\"parent\";r:258;s:8:\"children\";a:0:{}}s:14:\"admin/cms/page\";a:3:{s:8:\"instance\";r:934;s:6:\"parent\";r:258;s:8:\"children\";a:0:{}}s:14:\"admin/cms/poll\";a:3:{s:8:\"instance\";r:936;s:6:\"parent\";r:258;s:8:\"children\";a:0:{}}s:12:\"admin/report\";a:3:{s:8:\"instance\";r:260;s:6:\"parent\";r:240;s:8:\"children\";a:7:{s:22:\"admin/report/salesroot\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:22:\"admin/report/salesroot\";}s:21:\"admin/report/shopcart\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:21:\"admin/report/shopcart\";}s:21:\"admin/report/products\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:21:\"admin/report/products\";}s:22:\"admin/report/customers\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:22:\"admin/report/customers\";}s:19:\"admin/report/review\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:19:\"admin/report/review\";}s:17:\"admin/report/tags\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:17:\"admin/report/tags\";}s:19:\"admin/report/search\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:19:\"admin/report/search\";}}}s:22:\"admin/report/salesroot\";a:3:{s:8:\"instance\";r:954;s:6:\"parent\";r:260;s:8:\"children\";a:6:{s:28:\"admin/report/salesroot/sales\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:28:\"admin/report/salesroot/sales\";}s:26:\"admin/report/salesroot/tax\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:26:\"admin/report/salesroot/tax\";}s:31:\"admin/report/salesroot/shipping\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:31:\"admin/report/salesroot/shipping\";}s:31:\"admin/report/salesroot/invoiced\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:31:\"admin/report/salesroot/invoiced\";}s:31:\"admin/report/salesroot/refunded\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:31:\"admin/report/salesroot/refunded\";}s:30:\"admin/report/salesroot/coupons\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:30:\"admin/report/salesroot/coupons\";}}}s:28:\"admin/report/salesroot/sales\";a:3:{s:8:\"instance\";r:972;s:6:\"parent\";r:954;s:8:\"children\";a:0:{}}s:26:\"admin/report/salesroot/tax\";a:3:{s:8:\"instance\";r:974;s:6:\"parent\";r:954;s:8:\"children\";a:0:{}}s:31:\"admin/report/salesroot/shipping\";a:3:{s:8:\"instance\";r:976;s:6:\"parent\";r:954;s:8:\"children\";a:0:{}}s:31:\"admin/report/salesroot/invoiced\";a:3:{s:8:\"instance\";r:978;s:6:\"parent\";r:954;s:8:\"children\";a:0:{}}s:31:\"admin/report/salesroot/refunded\";a:3:{s:8:\"instance\";r:980;s:6:\"parent\";r:954;s:8:\"children\";a:0:{}}s:30:\"admin/report/salesroot/coupons\";a:3:{s:8:\"instance\";r:982;s:6:\"parent\";r:954;s:8:\"children\";a:0:{}}s:21:\"admin/report/shopcart\";a:3:{s:8:\"instance\";r:956;s:6:\"parent\";r:260;s:8:\"children\";a:2:{s:29:\"admin/report/shopcart/product\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:29:\"admin/report/shopcart/product\";}s:31:\"admin/report/shopcart/abandoned\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:31:\"admin/report/shopcart/abandoned\";}}}s:29:\"admin/report/shopcart/product\";a:3:{s:8:\"instance\";r:1012;s:6:\"parent\";r:956;s:8:\"children\";a:0:{}}s:31:\"admin/report/shopcart/abandoned\";a:3:{s:8:\"instance\";r:1014;s:6:\"parent\";r:956;s:8:\"children\";a:0:{}}s:21:\"admin/report/products\";a:3:{s:8:\"instance\";r:958;s:6:\"parent\";r:260;s:8:\"children\";a:5:{s:29:\"admin/report/products/ordered\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:29:\"admin/report/products/ordered\";}s:26:\"admin/report/products/sold\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:26:\"admin/report/products/sold\";}s:28:\"admin/report/products/viewed\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:28:\"admin/report/products/viewed\";}s:30:\"admin/report/products/lowstock\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:30:\"admin/report/products/lowstock\";}s:31:\"admin/report/products/downloads\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:31:\"admin/report/products/downloads\";}}}s:29:\"admin/report/products/ordered\";a:3:{s:8:\"instance\";r:1028;s:6:\"parent\";r:958;s:8:\"children\";a:0:{}}s:26:\"admin/report/products/sold\";a:3:{s:8:\"instance\";r:1030;s:6:\"parent\";r:958;s:8:\"children\";a:0:{}}s:28:\"admin/report/products/viewed\";a:3:{s:8:\"instance\";r:1032;s:6:\"parent\";r:958;s:8:\"children\";a:0:{}}s:30:\"admin/report/products/lowstock\";a:3:{s:8:\"instance\";r:1034;s:6:\"parent\";r:958;s:8:\"children\";a:0:{}}s:31:\"admin/report/products/downloads\";a:3:{s:8:\"instance\";r:1036;s:6:\"parent\";r:958;s:8:\"children\";a:0:{}}s:22:\"admin/report/customers\";a:3:{s:8:\"instance\";r:960;s:6:\"parent\";r:260;s:8:\"children\";a:3:{s:31:\"admin/report/customers/accounts\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:31:\"admin/report/customers/accounts\";}s:29:\"admin/report/customers/totals\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:29:\"admin/report/customers/totals\";}s:29:\"admin/report/customers/orders\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:29:\"admin/report/customers/orders\";}}}s:31:\"admin/report/customers/accounts\";a:3:{s:8:\"instance\";r:1062;s:6:\"parent\";r:960;s:8:\"children\";a:0:{}}s:29:\"admin/report/customers/totals\";a:3:{s:8:\"instance\";r:1064;s:6:\"parent\";r:960;s:8:\"children\";a:0:{}}s:29:\"admin/report/customers/orders\";a:3:{s:8:\"instance\";r:1066;s:6:\"parent\";r:960;s:8:\"children\";a:0:{}}s:19:\"admin/report/review\";a:3:{s:8:\"instance\";r:962;s:6:\"parent\";r:260;s:8:\"children\";a:2:{s:28:\"admin/report/review/customer\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:28:\"admin/report/review/customer\";}s:27:\"admin/report/review/product\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:27:\"admin/report/review/product\";}}}s:28:\"admin/report/review/customer\";a:3:{s:8:\"instance\";r:1084;s:6:\"parent\";r:962;s:8:\"children\";a:0:{}}s:27:\"admin/report/review/product\";a:3:{s:8:\"instance\";r:1086;s:6:\"parent\";r:962;s:8:\"children\";a:0:{}}s:17:\"admin/report/tags\";a:3:{s:8:\"instance\";r:964;s:6:\"parent\";r:260;s:8:\"children\";a:3:{s:26:\"admin/report/tags/customer\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:26:\"admin/report/tags/customer\";}s:25:\"admin/report/tags/popular\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:25:\"admin/report/tags/popular\";}s:25:\"admin/report/tags/product\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:25:\"admin/report/tags/product\";}}}s:26:\"admin/report/tags/customer\";a:3:{s:8:\"instance\";r:1100;s:6:\"parent\";r:964;s:8:\"children\";a:0:{}}s:25:\"admin/report/tags/popular\";a:3:{s:8:\"instance\";r:1102;s:6:\"parent\";r:964;s:8:\"children\";a:0:{}}s:25:\"admin/report/tags/product\";a:3:{s:8:\"instance\";r:1104;s:6:\"parent\";r:964;s:8:\"children\";a:0:{}}s:19:\"admin/report/search\";a:3:{s:8:\"instance\";r:966;s:6:\"parent\";r:260;s:8:\"children\";a:0:{}}s:16:\"admin/newsletter\";a:3:{s:8:\"instance\";r:262;s:6:\"parent\";r:240;s:8:\"children\";a:4:{s:24:\"admin/newsletter/problem\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:24:\"admin/newsletter/problem\";}s:22:\"admin/newsletter/queue\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:22:\"admin/newsletter/queue\";}s:27:\"admin/newsletter/subscriber\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:27:\"admin/newsletter/subscriber\";}s:25:\"admin/newsletter/template\";O:29:\"Mage_Admin_Model_Acl_Resource\":1:{s:14:\"\0*\0_resourceId\";s:25:\"admin/newsletter/template\";}}}s:24:\"admin/newsletter/problem\";a:3:{s:8:\"instance\";r:1126;s:6:\"parent\";r:262;s:8:\"children\";a:0:{}}s:22:\"admin/newsletter/queue\";a:3:{s:8:\"instance\";r:1128;s:6:\"parent\";r:262;s:8:\"children\";a:0:{}}s:27:\"admin/newsletter/subscriber\";a:3:{s:8:\"instance\";r:1130;s:6:\"parent\";r:262;s:8:\"children\";a:0:{}}s:25:\"admin/newsletter/template\";a:3:{s:8:\"instance\";r:1132;s:6:\"parent\";r:262;s:8:\"children\";a:0:{}}}s:9:\"\0*\0_rules\";a:2:{s:12:\"allResources\";a:2:{s:8:\"allRoles\";a:2:{s:13:\"allPrivileges\";a:2:{s:4:\"type\";s:9:\"TYPE_DENY\";s:6:\"assert\";N;}s:13:\"byPrivilegeId\";a:0:{}}s:8:\"byRoleId\";a:1:{s:2:\"G1\";a:2:{s:13:\"byPrivilegeId\";a:0:{}s:13:\"allPrivileges\";a:2:{s:4:\"type\";s:10:\"TYPE_ALLOW\";s:6:\"assert\";N;}}}}s:12:\"byResourceId\";a:1:{s:3:\"all\";a:1:{s:8:\"byRoleId\";a:1:{s:2:\"G1\";a:2:{s:13:\"byPrivilegeId\";a:0:{}s:13:\"allPrivileges\";a:2:{s:4:\"type\";s:10:\"TYPE_ALLOW\";s:6:\"assert\";N;}}}}}}}}'),('ccfa2a82a2f91b7a99dba0593b2e57bb',NULL,1246575834,'core|a:5:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:12:\"67.222.37.11\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:0:\"\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}s:12:\"visitor_data\";a:16:{s:0:\"\";N;s:11:\"server_addr\";i:1163467184;s:11:\"remote_addr\";i:1138631947;s:11:\"http_secure\";b:0;s:9:\"http_host\";s:18:\"www.maclaonline.es\";s:15:\"http_user_agent\";s:0:\"\";s:20:\"http_accept_language\";s:0:\"\";s:19:\"http_accept_charset\";s:0:\"\";s:11:\"request_uri\";s:8:\"/tienda/\";s:10:\"session_id\";s:32:\"ccfa2a82a2f91b7a99dba0593b2e57bb\";s:12:\"http_referer\";s:0:\"\";s:14:\"first_visit_at\";s:19:\"2009-07-02 22:39:53\";s:14:\"is_new_visitor\";b:0;s:13:\"last_visit_at\";s:19:\"2009-07-02 22:39:54\";s:10:\"visitor_id\";s:2:\"31\";s:11:\"last_url_id\";s:3:\"309\";}s:8:\"last_url\";s:59:\"http://www.maclaonline.es/tienda/index.php/cms/index/index/\";}_cookie_revalidate|d:1246575293;customer_base|a:2:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:12:\"67.222.37.11\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:0:\"\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}}checkout|a:3:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:12:\"67.222.37.11\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:0:\"\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}}catalog|a:3:{s:23:\"_session_validator_data\";a:4:{s:11:\"remote_addr\";s:12:\"67.222.37.11\";s:8:\"http_via\";s:0:\"\";s:20:\"http_x_forwarded_for\";s:0:\"\";s:15:\"http_user_agent\";s:0:\"\";}s:13:\"session_hosts\";a:1:{s:18:\"www.maclaonline.es\";b:1;}s:8:\"messages\";O:34:\"Mage_Core_Model_Message_Collection\":2:{s:12:\"\0*\0_messages\";a:0:{}s:20:\"\0*\0_lastAddedMessage\";N;}}');
/*!40000 ALTER TABLE `core_session` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `core_store`
--

DROP TABLE IF EXISTS `core_store`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `core_store` (
  `store_id` smallint(5) unsigned NOT NULL auto_increment,
  `code` varchar(32) NOT NULL default '',
  `website_id` smallint(5) unsigned default '0',
  `group_id` smallint(5) unsigned NOT NULL default '0',
  `name` varchar(255) NOT NULL,
  `sort_order` smallint(5) unsigned NOT NULL default '0',
  `is_active` tinyint(1) unsigned NOT NULL default '0',
  PRIMARY KEY  (`store_id`),
  UNIQUE KEY `code` (`code`),
  KEY `FK_STORE_WEBSITE` (`website_id`),
  KEY `is_active` (`is_active`,`sort_order`),
  KEY `FK_STORE_GROUP` (`group_id`),
  CONSTRAINT `FK_STORE_GROUP_STORE` FOREIGN KEY (`group_id`) REFERENCES `core_store_group` (`group_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_STORE_WEBSITE` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=utf8 COMMENT='Stores';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `core_store`
--

LOCK TABLES `core_store` WRITE;
/*!40000 ALTER TABLE `core_store` DISABLE KEYS */;
INSERT INTO `core_store` VALUES (0,'admin',0,0,'Admin',0,1),(1,'default',1,1,'Default Store View',0,1);
/*!40000 ALTER TABLE `core_store` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `core_store_group`
--

DROP TABLE IF EXISTS `core_store_group`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `core_store_group` (
  `group_id` smallint(5) unsigned NOT NULL auto_increment,
  `website_id` smallint(5) unsigned NOT NULL default '0',
  `name` varchar(255) NOT NULL,
  `root_category_id` int(10) unsigned NOT NULL default '0',
  `default_store_id` smallint(5) unsigned NOT NULL default '0',
  PRIMARY KEY  (`group_id`),
  KEY `FK_STORE_GROUP_WEBSITE` (`website_id`),
  KEY `default_store_id` (`default_store_id`),
  CONSTRAINT `FK_STORE_GROUP_WEBSITE` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `core_store_group`
--

LOCK TABLES `core_store_group` WRITE;
/*!40000 ALTER TABLE `core_store_group` DISABLE KEYS */;
INSERT INTO `core_store_group` VALUES (0,0,'Default',0,0),(1,1,'Main Website Store',2,1);
/*!40000 ALTER TABLE `core_store_group` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `core_translate`
--

DROP TABLE IF EXISTS `core_translate`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `core_translate` (
  `key_id` int(10) unsigned NOT NULL auto_increment,
  `string` varchar(255) NOT NULL default '',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `translate` varchar(255) NOT NULL default '',
  `locale` varchar(20) NOT NULL default 'en_US',
  PRIMARY KEY  (`key_id`),
  UNIQUE KEY `IDX_CODE` (`store_id`,`locale`,`string`),
  KEY `FK_CORE_TRANSLATE_STORE` (`store_id`),
  CONSTRAINT `FK_CORE_TRANSLATE_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Translation data';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `core_translate`
--

LOCK TABLES `core_translate` WRITE;
/*!40000 ALTER TABLE `core_translate` DISABLE KEYS */;
/*!40000 ALTER TABLE `core_translate` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `core_url_rewrite`
--

DROP TABLE IF EXISTS `core_url_rewrite`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `core_url_rewrite` (
  `url_rewrite_id` int(10) unsigned NOT NULL auto_increment,
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `category_id` int(10) unsigned default NULL,
  `product_id` int(10) unsigned default NULL,
  `id_path` varchar(255) NOT NULL default '',
  `request_path` varchar(255) NOT NULL default '',
  `target_path` varchar(255) NOT NULL default '',
  `is_system` tinyint(1) unsigned default '1',
  `options` varchar(255) NOT NULL default '',
  `description` varchar(255) default NULL,
  PRIMARY KEY  (`url_rewrite_id`),
  UNIQUE KEY `UNQ_REQUEST_PATH` (`store_id`,`request_path`),
  UNIQUE KEY `UNQ_PATH` (`store_id`,`id_path`,`is_system`),
  KEY `FK_CORE_URL_REWRITE_STORE` (`store_id`),
  KEY `IDX_TARGET_PATH` (`store_id`,`target_path`),
  KEY `IDX_ID_PATH` (`id_path`),
  KEY `FK_CORE_URL_REWRITE_PRODUCT` (`product_id`),
  KEY `IDX_CATEGORY_REWRITE` (`category_id`,`is_system`,`product_id`,`store_id`,`id_path`),
  CONSTRAINT `FK_CORE_URL_REWRITE_CATEGORY` FOREIGN KEY (`category_id`) REFERENCES `catalog_category_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CORE_URL_REWRITE_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CORE_URL_REWRITE_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=18 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `core_url_rewrite`
--

LOCK TABLES `core_url_rewrite` WRITE;
/*!40000 ALTER TABLE `core_url_rewrite` DISABLE KEYS */;
INSERT INTO `core_url_rewrite` VALUES (3,1,NULL,1,'product/1','pr1.html','catalog/product/view/id/1',1,'',NULL),(4,1,NULL,2,'product/2','medallon-doble-vulcano-espiral.html','catalog/product/view/id/2',1,'',NULL),(5,1,4,NULL,'category/4','vulcano.html','catalog/category/view/id/4',1,'',NULL),(8,1,5,NULL,'category/5','vulcano/colgante-piedra.html','catalog/category/view/id/5',1,'',NULL),(9,1,6,NULL,'category/6','vulcano/medallones-vulcano.html','catalog/category/view/id/6',1,'',NULL),(10,1,6,2,'product/2/6','vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html','catalog/product/view/id/2/category/6',1,'',NULL),(11,1,7,NULL,'category/7','dominica.html','catalog/category/view/id/7',1,'',NULL),(12,1,8,NULL,'category/8','dominica/piedras-dominica.html','catalog/category/view/id/8',1,'',NULL),(13,1,9,NULL,'category/9','dominica/placas-dominica.html','catalog/category/view/id/9',1,'',NULL),(14,1,10,NULL,'category/10','dominica/medallones-dominica.html','catalog/category/view/id/10',1,'',NULL),(15,1,11,NULL,'category/11','hindu.html','catalog/category/view/id/11',1,'',NULL),(16,1,12,NULL,'category/12','hindu/medallones-hindu.html','catalog/category/view/id/12',1,'',NULL),(17,1,13,NULL,'category/13','hindu/plumas-hindu.html','catalog/category/view/id/13',1,'',NULL);
/*!40000 ALTER TABLE `core_url_rewrite` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `core_website`
--

DROP TABLE IF EXISTS `core_website`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `core_website` (
  `website_id` smallint(5) unsigned NOT NULL auto_increment,
  `code` varchar(32) NOT NULL default '',
  `name` varchar(64) NOT NULL default '',
  `sort_order` smallint(5) unsigned NOT NULL default '0',
  `default_group_id` smallint(5) unsigned NOT NULL default '0',
  `is_default` tinyint(1) unsigned default '0',
  PRIMARY KEY  (`website_id`),
  UNIQUE KEY `code` (`code`),
  KEY `sort_order` (`sort_order`),
  KEY `default_group_id` (`default_group_id`)
) ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=utf8 COMMENT='Websites';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `core_website`
--

LOCK TABLES `core_website` WRITE;
/*!40000 ALTER TABLE `core_website` DISABLE KEYS */;
INSERT INTO `core_website` VALUES (0,'admin','Admin',0,0,0),(1,'base','Main Website',0,1,1);
/*!40000 ALTER TABLE `core_website` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `cron_schedule`
--

DROP TABLE IF EXISTS `cron_schedule`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `cron_schedule` (
  `schedule_id` int(10) unsigned NOT NULL auto_increment,
  `job_code` varchar(255) NOT NULL default '0',
  `status` enum('pending','running','success','missed','error') NOT NULL default 'pending',
  `messages` text,
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `scheduled_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `executed_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `finished_at` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`schedule_id`),
  KEY `task_name` (`job_code`),
  KEY `scheduled_at` (`scheduled_at`,`status`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `cron_schedule`
--

LOCK TABLES `cron_schedule` WRITE;
/*!40000 ALTER TABLE `cron_schedule` DISABLE KEYS */;
/*!40000 ALTER TABLE `cron_schedule` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `customer_address_entity`
--

DROP TABLE IF EXISTS `customer_address_entity`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `customer_address_entity` (
  `entity_id` int(10) unsigned NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_set_id` smallint(5) unsigned NOT NULL default '0',
  `increment_id` varchar(50) NOT NULL default '',
  `parent_id` int(10) unsigned default NULL,
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `is_active` tinyint(1) unsigned NOT NULL default '1',
  PRIMARY KEY  (`entity_id`),
  KEY `FK_CUSTOMER_ADDRESS_CUSTOMER_ID` (`parent_id`),
  CONSTRAINT `FK_CUSTOMER_ADDRESS_CUSTOMER_ID` FOREIGN KEY (`parent_id`) REFERENCES `customer_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC COMMENT='Customer Address Entities';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `customer_address_entity`
--

LOCK TABLES `customer_address_entity` WRITE;
/*!40000 ALTER TABLE `customer_address_entity` DISABLE KEYS */;
/*!40000 ALTER TABLE `customer_address_entity` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `customer_address_entity_datetime`
--

DROP TABLE IF EXISTS `customer_address_entity_datetime`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `customer_address_entity_datetime` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`),
  KEY `FK_CUSTOMER_ADDRESS_DATETIME_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_CUSTOMER_ADDRESS_DATETIME_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CUSTOMER_ADDRESS_DATETIME_ENTITY` (`entity_id`),
  KEY `IDX_VALUE` (`entity_id`,`attribute_id`,`value`),
  CONSTRAINT `FK_CUSTOMER_ADDRESS_DATETIME_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_ADDRESS_DATETIME_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `customer_address_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_ADDRESS_DATETIME_ENTITY_TYPE` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `customer_address_entity_datetime`
--

LOCK TABLES `customer_address_entity_datetime` WRITE;
/*!40000 ALTER TABLE `customer_address_entity_datetime` DISABLE KEYS */;
/*!40000 ALTER TABLE `customer_address_entity_datetime` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `customer_address_entity_decimal`
--

DROP TABLE IF EXISTS `customer_address_entity_decimal`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `customer_address_entity_decimal` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` decimal(12,4) NOT NULL default '0.0000',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`),
  KEY `FK_CUSTOMER_ADDRESS_DECIMAL_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_CUSTOMER_ADDRESS_DECIMAL_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CUSTOMER_ADDRESS_DECIMAL_ENTITY` (`entity_id`),
  KEY `IDX_VALUE` (`entity_id`,`attribute_id`,`value`),
  CONSTRAINT `FK_CUSTOMER_ADDRESS_DECIMAL_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_ADDRESS_DECIMAL_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `customer_address_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_ADDRESS_DECIMAL_ENTITY_TYPE` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `customer_address_entity_decimal`
--

LOCK TABLES `customer_address_entity_decimal` WRITE;
/*!40000 ALTER TABLE `customer_address_entity_decimal` DISABLE KEYS */;
/*!40000 ALTER TABLE `customer_address_entity_decimal` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `customer_address_entity_int`
--

DROP TABLE IF EXISTS `customer_address_entity_int`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `customer_address_entity_int` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` int(11) NOT NULL default '0',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`),
  KEY `FK_CUSTOMER_ADDRESS_INT_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_CUSTOMER_ADDRESS_INT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CUSTOMER_ADDRESS_INT_ENTITY` (`entity_id`),
  KEY `IDX_VALUE` (`entity_id`,`attribute_id`,`value`),
  CONSTRAINT `FK_CUSTOMER_ADDRESS_INT_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_ADDRESS_INT_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `customer_address_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_ADDRESS_INT_ENTITY_TYPE` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `customer_address_entity_int`
--

LOCK TABLES `customer_address_entity_int` WRITE;
/*!40000 ALTER TABLE `customer_address_entity_int` DISABLE KEYS */;
/*!40000 ALTER TABLE `customer_address_entity_int` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `customer_address_entity_text`
--

DROP TABLE IF EXISTS `customer_address_entity_text`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `customer_address_entity_text` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` text NOT NULL,
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`),
  KEY `FK_CUSTOMER_ADDRESS_TEXT_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_CUSTOMER_ADDRESS_TEXT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CUSTOMER_ADDRESS_TEXT_ENTITY` (`entity_id`),
  KEY `IDX_VALUE` (`entity_id`,`attribute_id`),
  CONSTRAINT `FK_CUSTOMER_ADDRESS_TEXT_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_ADDRESS_TEXT_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `customer_address_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_ADDRESS_TEXT_ENTITY_TYPE` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `customer_address_entity_text`
--

LOCK TABLES `customer_address_entity_text` WRITE;
/*!40000 ALTER TABLE `customer_address_entity_text` DISABLE KEYS */;
/*!40000 ALTER TABLE `customer_address_entity_text` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `customer_address_entity_varchar`
--

DROP TABLE IF EXISTS `customer_address_entity_varchar`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `customer_address_entity_varchar` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`),
  KEY `FK_CUSTOMER_ADDRESS_VARCHAR_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_CUSTOMER_ADDRESS_VARCHAR_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CUSTOMER_ADDRESS_VARCHAR_ENTITY` (`entity_id`),
  KEY `IDX_VALUE` (`entity_id`,`attribute_id`,`value`),
  CONSTRAINT `FK_CUSTOMER_ADDRESS_VARCHAR_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_ADDRESS_VARCHAR_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `customer_address_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_ADDRESS_VARCHAR_ENTITY_TYPE` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `customer_address_entity_varchar`
--

LOCK TABLES `customer_address_entity_varchar` WRITE;
/*!40000 ALTER TABLE `customer_address_entity_varchar` DISABLE KEYS */;
/*!40000 ALTER TABLE `customer_address_entity_varchar` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `customer_entity`
--

DROP TABLE IF EXISTS `customer_entity`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `customer_entity` (
  `entity_id` int(10) unsigned NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_set_id` smallint(5) unsigned NOT NULL default '0',
  `website_id` smallint(5) unsigned default NULL,
  `email` varchar(255) NOT NULL default '',
  `group_id` smallint(3) unsigned NOT NULL default '0',
  `increment_id` varchar(50) NOT NULL default '',
  `store_id` smallint(5) unsigned default '0',
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `is_active` tinyint(1) unsigned NOT NULL default '1',
  PRIMARY KEY  (`entity_id`),
  KEY `FK_CUSTOMER_ENTITY_STORE` (`store_id`),
  KEY `IDX_ENTITY_TYPE` (`entity_type_id`),
  KEY `IDX_AUTH` (`email`,`website_id`),
  KEY `FK_CUSTOMER_WEBSITE` (`website_id`),
  CONSTRAINT `FK_CUSTOMER_ENTITY_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE SET NULL ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_WEBSITE` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE SET NULL ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC COMMENT='Customer Entityies';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `customer_entity`
--

LOCK TABLES `customer_entity` WRITE;
/*!40000 ALTER TABLE `customer_entity` DISABLE KEYS */;
/*!40000 ALTER TABLE `customer_entity` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `customer_entity_datetime`
--

DROP TABLE IF EXISTS `customer_entity_datetime`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `customer_entity_datetime` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`),
  KEY `FK_CUSTOMER_DATETIME_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_CUSTOMER_DATETIME_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CUSTOMER_DATETIME_ENTITY` (`entity_id`),
  KEY `IDX_VALUE` (`entity_id`,`attribute_id`,`value`),
  CONSTRAINT `FK_CUSTOMER_DATETIME_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_DATETIME_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `customer_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_DATETIME_ENTITY_TYPE` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `customer_entity_datetime`
--

LOCK TABLES `customer_entity_datetime` WRITE;
/*!40000 ALTER TABLE `customer_entity_datetime` DISABLE KEYS */;
/*!40000 ALTER TABLE `customer_entity_datetime` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `customer_entity_decimal`
--

DROP TABLE IF EXISTS `customer_entity_decimal`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `customer_entity_decimal` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` decimal(12,4) NOT NULL default '0.0000',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`),
  KEY `FK_CUSTOMER_DECIMAL_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_CUSTOMER_DECIMAL_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CUSTOMER_DECIMAL_ENTITY` (`entity_id`),
  KEY `IDX_VALUE` (`entity_id`,`attribute_id`,`value`),
  CONSTRAINT `FK_CUSTOMER_DECIMAL_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_DECIMAL_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `customer_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_DECIMAL_ENTITY_TYPE` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `customer_entity_decimal`
--

LOCK TABLES `customer_entity_decimal` WRITE;
/*!40000 ALTER TABLE `customer_entity_decimal` DISABLE KEYS */;
/*!40000 ALTER TABLE `customer_entity_decimal` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `customer_entity_int`
--

DROP TABLE IF EXISTS `customer_entity_int`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `customer_entity_int` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` int(11) NOT NULL default '0',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`),
  KEY `FK_CUSTOMER_INT_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_CUSTOMER_INT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CUSTOMER_INT_ENTITY` (`entity_id`),
  KEY `IDX_VALUE` (`entity_id`,`attribute_id`,`value`),
  CONSTRAINT `FK_CUSTOMER_INT_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_INT_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `customer_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_INT_ENTITY_TYPE` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `customer_entity_int`
--

LOCK TABLES `customer_entity_int` WRITE;
/*!40000 ALTER TABLE `customer_entity_int` DISABLE KEYS */;
/*!40000 ALTER TABLE `customer_entity_int` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `customer_entity_text`
--

DROP TABLE IF EXISTS `customer_entity_text`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `customer_entity_text` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` text NOT NULL,
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`),
  KEY `FK_CUSTOMER_TEXT_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_CUSTOMER_TEXT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CUSTOMER_TEXT_ENTITY` (`entity_id`),
  KEY `IDX_VALUE` (`entity_id`,`attribute_id`),
  CONSTRAINT `FK_CUSTOMER_TEXT_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_TEXT_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `customer_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_TEXT_ENTITY_TYPE` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `customer_entity_text`
--

LOCK TABLES `customer_entity_text` WRITE;
/*!40000 ALTER TABLE `customer_entity_text` DISABLE KEYS */;
/*!40000 ALTER TABLE `customer_entity_text` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `customer_entity_varchar`
--

DROP TABLE IF EXISTS `customer_entity_varchar`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `customer_entity_varchar` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `IDX_ATTRIBUTE_VALUE` (`entity_id`,`attribute_id`),
  KEY `FK_CUSTOMER_VARCHAR_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_CUSTOMER_VARCHAR_ATTRIBUTE` (`attribute_id`),
  KEY `FK_CUSTOMER_VARCHAR_ENTITY` (`entity_id`),
  KEY `IDX_VALUE` (`entity_id`,`attribute_id`,`value`),
  CONSTRAINT `FK_CUSTOMER_VARCHAR_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_VARCHAR_ENTITY` FOREIGN KEY (`entity_id`) REFERENCES `customer_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_CUSTOMER_VARCHAR_ENTITY_TYPE` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `customer_entity_varchar`
--

LOCK TABLES `customer_entity_varchar` WRITE;
/*!40000 ALTER TABLE `customer_entity_varchar` DISABLE KEYS */;
/*!40000 ALTER TABLE `customer_entity_varchar` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `customer_group`
--

DROP TABLE IF EXISTS `customer_group`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `customer_group` (
  `customer_group_id` smallint(3) unsigned NOT NULL auto_increment,
  `customer_group_code` varchar(32) NOT NULL default '',
  `tax_class_id` int(10) unsigned NOT NULL default '0',
  PRIMARY KEY  (`customer_group_id`)
) ENGINE=InnoDB AUTO_INCREMENT=4 DEFAULT CHARSET=utf8 COMMENT='Customer groups';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `customer_group`
--

LOCK TABLES `customer_group` WRITE;
/*!40000 ALTER TABLE `customer_group` DISABLE KEYS */;
INSERT INTO `customer_group` VALUES (0,'NOT LOGGED IN',3),(1,'General',3),(2,'Wholesale',3),(3,'Retailer',3);
/*!40000 ALTER TABLE `customer_group` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `dataflow_batch`
--

DROP TABLE IF EXISTS `dataflow_batch`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `dataflow_batch` (
  `batch_id` int(10) unsigned NOT NULL auto_increment,
  `profile_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `adapter` varchar(128) default NULL,
  `params` text,
  `created_at` datetime default NULL,
  PRIMARY KEY  (`batch_id`),
  KEY `FK_DATAFLOW_BATCH_PROFILE` (`profile_id`),
  KEY `FK_DATAFLOW_BATCH_STORE` (`store_id`),
  KEY `IDX_CREATED_AT` (`created_at`),
  CONSTRAINT `FK_DATAFLOW_BATCH_PROFILE` FOREIGN KEY (`profile_id`) REFERENCES `dataflow_profile` (`profile_id`) ON DELETE CASCADE,
  CONSTRAINT `FK_DATAFLOW_BATCH_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `dataflow_batch`
--

LOCK TABLES `dataflow_batch` WRITE;
/*!40000 ALTER TABLE `dataflow_batch` DISABLE KEYS */;
/*!40000 ALTER TABLE `dataflow_batch` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `dataflow_batch_export`
--

DROP TABLE IF EXISTS `dataflow_batch_export`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `dataflow_batch_export` (
  `batch_export_id` bigint(20) unsigned NOT NULL auto_increment,
  `batch_id` int(10) unsigned NOT NULL default '0',
  `batch_data` longtext,
  `status` tinyint(3) unsigned NOT NULL default '0',
  PRIMARY KEY  (`batch_export_id`),
  KEY `FK_DATAFLOW_BATCH_EXPORT_BATCH` (`batch_id`),
  CONSTRAINT `FK_DATAFLOW_BATCH_EXPORT_BATCH` FOREIGN KEY (`batch_id`) REFERENCES `dataflow_batch` (`batch_id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `dataflow_batch_export`
--

LOCK TABLES `dataflow_batch_export` WRITE;
/*!40000 ALTER TABLE `dataflow_batch_export` DISABLE KEYS */;
/*!40000 ALTER TABLE `dataflow_batch_export` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `dataflow_batch_import`
--

DROP TABLE IF EXISTS `dataflow_batch_import`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `dataflow_batch_import` (
  `batch_import_id` bigint(20) unsigned NOT NULL auto_increment,
  `batch_id` int(10) unsigned NOT NULL default '0',
  `batch_data` longtext,
  `status` tinyint(3) unsigned NOT NULL default '0',
  PRIMARY KEY  (`batch_import_id`),
  KEY `FK_DATAFLOW_BATCH_IMPORT_BATCH` (`batch_id`),
  CONSTRAINT `FK_DATAFLOW_BATCH_IMPORT_BATCH` FOREIGN KEY (`batch_id`) REFERENCES `dataflow_batch` (`batch_id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `dataflow_batch_import`
--

LOCK TABLES `dataflow_batch_import` WRITE;
/*!40000 ALTER TABLE `dataflow_batch_import` DISABLE KEYS */;
/*!40000 ALTER TABLE `dataflow_batch_import` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `dataflow_import_data`
--

DROP TABLE IF EXISTS `dataflow_import_data`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `dataflow_import_data` (
  `import_id` int(11) NOT NULL auto_increment,
  `session_id` int(11) default NULL,
  `serial_number` int(11) NOT NULL default '0',
  `value` text,
  `status` int(1) NOT NULL default '0',
  PRIMARY KEY  (`import_id`),
  KEY `FK_dataflow_import_data` (`session_id`),
  CONSTRAINT `FK_dataflow_import_data` FOREIGN KEY (`session_id`) REFERENCES `dataflow_session` (`session_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `dataflow_import_data`
--

LOCK TABLES `dataflow_import_data` WRITE;
/*!40000 ALTER TABLE `dataflow_import_data` DISABLE KEYS */;
/*!40000 ALTER TABLE `dataflow_import_data` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `dataflow_profile`
--

DROP TABLE IF EXISTS `dataflow_profile`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `dataflow_profile` (
  `profile_id` int(10) unsigned NOT NULL auto_increment,
  `name` varchar(255) NOT NULL default '',
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `actions_xml` text,
  `gui_data` text,
  `direction` enum('import','export') default NULL,
  `entity_type` varchar(64) NOT NULL default '',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `data_transfer` enum('file','interactive') default NULL,
  PRIMARY KEY  (`profile_id`)
) ENGINE=InnoDB AUTO_INCREMENT=7 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `dataflow_profile`
--

LOCK TABLES `dataflow_profile` WRITE;
/*!40000 ALTER TABLE `dataflow_profile` DISABLE KEYS */;
INSERT INTO `dataflow_profile` VALUES (1,'Export All Products','2009-06-03 13:52:14','2009-06-03 13:52:14','<action type=\"catalog/convert_adapter_product\" method=\"load\">\r\n    <var name=\"store\"><![CDATA[0]]></var>\r\n</action>\r\n\r\n<action type=\"catalog/convert_parser_product\" method=\"unparse\">\r\n    <var name=\"store\"><![CDATA[0]]></var>\r\n</action>\r\n\r\n<action type=\"dataflow/convert_mapper_column\" method=\"map\">\r\n</action>\r\n\r\n<action type=\"dataflow/convert_parser_csv\" method=\"unparse\">\r\n    <var name=\"delimiter\"><![CDATA[,]]></var>\r\n    <var name=\"enclose\"><![CDATA[\"]]></var>\r\n    <var name=\"fieldnames\">true</var>\r\n</action>\r\n\r\n<action type=\"dataflow/convert_adapter_io\" method=\"save\">\r\n    <var name=\"type\">file</var>\r\n    <var name=\"path\">var/export</var>\r\n    <var name=\"filename\"><![CDATA[export_all_products.csv]]></var>\r\n</action>\r\n\r\n','a:5:{s:4:\"file\";a:7:{s:4:\"type\";s:4:\"file\";s:8:\"filename\";s:23:\"export_all_products.csv\";s:4:\"path\";s:10:\"var/export\";s:4:\"host\";s:0:\"\";s:4:\"user\";s:0:\"\";s:8:\"password\";s:0:\"\";s:7:\"passive\";s:0:\"\";}s:5:\"parse\";a:5:{s:4:\"type\";s:3:\"csv\";s:12:\"single_sheet\";s:0:\"\";s:9:\"delimiter\";s:1:\",\";s:7:\"enclose\";s:1:\"\"\";s:10:\"fieldnames\";s:4:\"true\";}s:3:\"map\";a:3:{s:14:\"only_specified\";s:0:\"\";s:7:\"product\";a:2:{s:2:\"db\";a:0:{}s:4:\"file\";a:0:{}}s:8:\"customer\";a:2:{s:2:\"db\";a:0:{}s:4:\"file\";a:0:{}}}s:7:\"product\";a:1:{s:6:\"filter\";a:8:{s:4:\"name\";s:0:\"\";s:3:\"sku\";s:0:\"\";s:4:\"type\";s:1:\"0\";s:13:\"attribute_set\";s:0:\"\";s:5:\"price\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}s:3:\"qty\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}s:10:\"visibility\";s:1:\"0\";s:6:\"status\";s:1:\"0\";}}s:8:\"customer\";a:1:{s:6:\"filter\";a:10:{s:9:\"firstname\";s:0:\"\";s:8:\"lastname\";s:0:\"\";s:5:\"email\";s:0:\"\";s:5:\"group\";s:1:\"0\";s:10:\"adressType\";s:15:\"default_billing\";s:9:\"telephone\";s:0:\"\";s:8:\"postcode\";s:0:\"\";s:7:\"country\";s:0:\"\";s:6:\"region\";s:0:\"\";s:10:\"created_at\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}}}}','export','product',0,'file'),(2,'Export Product Stocks','2009-06-03 13:52:14','2009-06-03 13:52:14','<action type=\"catalog/convert_adapter_product\" method=\"load\">\r\n    <var name=\"store\"><![CDATA[0]]></var>\r\n</action>\r\n\r\n<action type=\"catalog/convert_parser_product\" method=\"unparse\">\r\n    <var name=\"store\"><![CDATA[0]]></var>\r\n</action>\r\n\r\n<action type=\"dataflow/convert_mapper_column\" method=\"map\">\r\n    <var name=\"map\">\r\n        <map name=\"store\"><![CDATA[store]]></map>\r\n        <map name=\"sku\"><![CDATA[sku]]></map>\r\n        <map name=\"qty\"><![CDATA[qty]]></map>\r\n        <map name=\"is_in_stock\"><![CDATA[is_in_stock]]></map>\r\n    </var>\r\n    <var name=\"_only_specified\">true</var>\r\n</action>\r\n\r\n<action type=\"dataflow/convert_parser_csv\" method=\"unparse\">\r\n    <var name=\"delimiter\"><![CDATA[,]]></var>\r\n    <var name=\"enclose\"><![CDATA[\"]]></var>\r\n    <var name=\"fieldnames\">true</var>\r\n</action>\r\n\r\n<action type=\"dataflow/convert_adapter_io\" method=\"save\">\r\n    <var name=\"type\">file</var>\r\n    <var name=\"path\">var/export</var>\r\n    <var name=\"filename\"><![CDATA[export_product_stocks.csv]]></var>\r\n</action>\r\n\r\n','a:5:{s:4:\"file\";a:7:{s:4:\"type\";s:4:\"file\";s:8:\"filename\";s:25:\"export_product_stocks.csv\";s:4:\"path\";s:10:\"var/export\";s:4:\"host\";s:0:\"\";s:4:\"user\";s:0:\"\";s:8:\"password\";s:0:\"\";s:7:\"passive\";s:0:\"\";}s:5:\"parse\";a:5:{s:4:\"type\";s:3:\"csv\";s:12:\"single_sheet\";s:0:\"\";s:9:\"delimiter\";s:1:\",\";s:7:\"enclose\";s:1:\"\"\";s:10:\"fieldnames\";s:4:\"true\";}s:3:\"map\";a:3:{s:14:\"only_specified\";s:4:\"true\";s:7:\"product\";a:2:{s:2:\"db\";a:4:{i:1;s:5:\"store\";i:2;s:3:\"sku\";i:3;s:3:\"qty\";i:4;s:11:\"is_in_stock\";}s:4:\"file\";a:4:{i:1;s:5:\"store\";i:2;s:3:\"sku\";i:3;s:3:\"qty\";i:4;s:11:\"is_in_stock\";}}s:8:\"customer\";a:2:{s:2:\"db\";a:0:{}s:4:\"file\";a:0:{}}}s:7:\"product\";a:1:{s:6:\"filter\";a:8:{s:4:\"name\";s:0:\"\";s:3:\"sku\";s:0:\"\";s:4:\"type\";s:1:\"0\";s:13:\"attribute_set\";s:0:\"\";s:5:\"price\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}s:3:\"qty\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}s:10:\"visibility\";s:1:\"0\";s:6:\"status\";s:1:\"0\";}}s:8:\"customer\";a:1:{s:6:\"filter\";a:10:{s:9:\"firstname\";s:0:\"\";s:8:\"lastname\";s:0:\"\";s:5:\"email\";s:0:\"\";s:5:\"group\";s:1:\"0\";s:10:\"adressType\";s:15:\"default_billing\";s:9:\"telephone\";s:0:\"\";s:8:\"postcode\";s:0:\"\";s:7:\"country\";s:0:\"\";s:6:\"region\";s:0:\"\";s:10:\"created_at\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}}}}','export','product',0,'file'),(3,'Import All Products','2009-06-03 13:52:14','2009-06-03 13:52:14','<action type=\"dataflow/convert_parser_csv\" method=\"parse\">\r\n    <var name=\"delimiter\"><![CDATA[,]]></var>\r\n    <var name=\"enclose\"><![CDATA[\"]]></var>\r\n    <var name=\"fieldnames\">true</var>\r\n    <var name=\"store\"><![CDATA[0]]></var>\r\n    <var name=\"adapter\">catalog/convert_adapter_product</var>\r\n    <var name=\"method\">parse</var>\r\n</action>','a:5:{s:4:\"file\";a:7:{s:4:\"type\";s:4:\"file\";s:8:\"filename\";s:23:\"export_all_products.csv\";s:4:\"path\";s:10:\"var/export\";s:4:\"host\";s:0:\"\";s:4:\"user\";s:0:\"\";s:8:\"password\";s:0:\"\";s:7:\"passive\";s:0:\"\";}s:5:\"parse\";a:5:{s:4:\"type\";s:3:\"csv\";s:12:\"single_sheet\";s:0:\"\";s:9:\"delimiter\";s:1:\",\";s:7:\"enclose\";s:1:\"\"\";s:10:\"fieldnames\";s:4:\"true\";}s:3:\"map\";a:3:{s:14:\"only_specified\";s:0:\"\";s:7:\"product\";a:2:{s:2:\"db\";a:0:{}s:4:\"file\";a:0:{}}s:8:\"customer\";a:2:{s:2:\"db\";a:0:{}s:4:\"file\";a:0:{}}}s:7:\"product\";a:1:{s:6:\"filter\";a:8:{s:4:\"name\";s:0:\"\";s:3:\"sku\";s:0:\"\";s:4:\"type\";s:1:\"0\";s:13:\"attribute_set\";s:0:\"\";s:5:\"price\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}s:3:\"qty\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}s:10:\"visibility\";s:1:\"0\";s:6:\"status\";s:1:\"0\";}}s:8:\"customer\";a:1:{s:6:\"filter\";a:10:{s:9:\"firstname\";s:0:\"\";s:8:\"lastname\";s:0:\"\";s:5:\"email\";s:0:\"\";s:5:\"group\";s:1:\"0\";s:10:\"adressType\";s:15:\"default_billing\";s:9:\"telephone\";s:0:\"\";s:8:\"postcode\";s:0:\"\";s:7:\"country\";s:0:\"\";s:6:\"region\";s:0:\"\";s:10:\"created_at\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}}}}','import','product',0,'interactive'),(4,'Import Product Stocks','2009-06-03 13:52:14','2009-06-03 13:52:14','<action type=\"dataflow/convert_parser_csv\" method=\"parse\">\r\n    <var name=\"delimiter\"><![CDATA[,]]></var>\r\n    <var name=\"enclose\"><![CDATA[\"]]></var>\r\n    <var name=\"fieldnames\">true</var>\r\n    <var name=\"store\"><![CDATA[0]]></var>\r\n    <var name=\"adapter\">catalog/convert_adapter_product</var>\r\n    <var name=\"method\">parse</var>\r\n</action>','a:5:{s:4:\"file\";a:7:{s:4:\"type\";s:4:\"file\";s:8:\"filename\";s:18:\"export_product.csv\";s:4:\"path\";s:10:\"var/export\";s:4:\"host\";s:0:\"\";s:4:\"user\";s:0:\"\";s:8:\"password\";s:0:\"\";s:7:\"passive\";s:0:\"\";}s:5:\"parse\";a:5:{s:4:\"type\";s:3:\"csv\";s:12:\"single_sheet\";s:0:\"\";s:9:\"delimiter\";s:1:\",\";s:7:\"enclose\";s:1:\"\"\";s:10:\"fieldnames\";s:4:\"true\";}s:3:\"map\";a:3:{s:14:\"only_specified\";s:0:\"\";s:7:\"product\";a:2:{s:2:\"db\";a:0:{}s:4:\"file\";a:0:{}}s:8:\"customer\";a:2:{s:2:\"db\";a:0:{}s:4:\"file\";a:0:{}}}s:7:\"product\";a:1:{s:6:\"filter\";a:8:{s:4:\"name\";s:0:\"\";s:3:\"sku\";s:0:\"\";s:4:\"type\";s:1:\"0\";s:13:\"attribute_set\";s:0:\"\";s:5:\"price\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}s:3:\"qty\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}s:10:\"visibility\";s:1:\"0\";s:6:\"status\";s:1:\"0\";}}s:8:\"customer\";a:1:{s:6:\"filter\";a:10:{s:9:\"firstname\";s:0:\"\";s:8:\"lastname\";s:0:\"\";s:5:\"email\";s:0:\"\";s:5:\"group\";s:1:\"0\";s:10:\"adressType\";s:15:\"default_billing\";s:9:\"telephone\";s:0:\"\";s:8:\"postcode\";s:0:\"\";s:7:\"country\";s:0:\"\";s:6:\"region\";s:0:\"\";s:10:\"created_at\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}}}}','import','product',0,'interactive'),(5,'Export Customers','2009-06-03 13:52:14','2009-06-03 13:52:14','<action type=\"customer/convert_adapter_customer\" method=\"load\">\r\n    <var name=\"store\"><![CDATA[0]]></var>\r\n    <var name=\"filter/adressType\"><![CDATA[default_billing]]></var>\r\n</action>\r\n\r\n<action type=\"customer/convert_parser_customer\" method=\"unparse\">\r\n    <var name=\"store\"><![CDATA[0]]></var>\r\n</action>\r\n\r\n<action type=\"dataflow/convert_mapper_column\" method=\"map\">\r\n</action>\r\n\r\n<action type=\"dataflow/convert_parser_csv\" method=\"unparse\">\r\n    <var name=\"delimiter\"><![CDATA[,]]></var>\r\n    <var name=\"enclose\"><![CDATA[\"]]></var>\r\n    <var name=\"fieldnames\">true</var>\r\n</action>\r\n\r\n<action type=\"dataflow/convert_adapter_io\" method=\"save\">\r\n    <var name=\"type\">file</var>\r\n    <var name=\"path\">var/export</var>\r\n    <var name=\"filename\"><![CDATA[export_customers.csv]]></var>\r\n</action>\r\n\r\n','a:5:{s:4:\"file\";a:7:{s:4:\"type\";s:4:\"file\";s:8:\"filename\";s:20:\"export_customers.csv\";s:4:\"path\";s:10:\"var/export\";s:4:\"host\";s:0:\"\";s:4:\"user\";s:0:\"\";s:8:\"password\";s:0:\"\";s:7:\"passive\";s:0:\"\";}s:5:\"parse\";a:5:{s:4:\"type\";s:3:\"csv\";s:12:\"single_sheet\";s:0:\"\";s:9:\"delimiter\";s:1:\",\";s:7:\"enclose\";s:1:\"\"\";s:10:\"fieldnames\";s:4:\"true\";}s:3:\"map\";a:3:{s:14:\"only_specified\";s:0:\"\";s:7:\"product\";a:2:{s:2:\"db\";a:0:{}s:4:\"file\";a:0:{}}s:8:\"customer\";a:2:{s:2:\"db\";a:0:{}s:4:\"file\";a:0:{}}}s:7:\"product\";a:1:{s:6:\"filter\";a:8:{s:4:\"name\";s:0:\"\";s:3:\"sku\";s:0:\"\";s:4:\"type\";s:1:\"0\";s:13:\"attribute_set\";s:0:\"\";s:5:\"price\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}s:3:\"qty\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}s:10:\"visibility\";s:1:\"0\";s:6:\"status\";s:1:\"0\";}}s:8:\"customer\";a:1:{s:6:\"filter\";a:10:{s:9:\"firstname\";s:0:\"\";s:8:\"lastname\";s:0:\"\";s:5:\"email\";s:0:\"\";s:5:\"group\";s:1:\"0\";s:10:\"adressType\";s:15:\"default_billing\";s:9:\"telephone\";s:0:\"\";s:8:\"postcode\";s:0:\"\";s:7:\"country\";s:0:\"\";s:6:\"region\";s:0:\"\";s:10:\"created_at\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}}}}','export','customer',0,'file'),(6,'Import Customers','2009-06-03 13:52:14','2009-06-03 13:52:14','<action type=\"dataflow/convert_parser_csv\" method=\"parse\">\r\n    <var name=\"delimiter\"><![CDATA[,]]></var>\r\n    <var name=\"enclose\"><![CDATA[\"]]></var>\r\n    <var name=\"fieldnames\">true</var>\r\n    <var name=\"store\"><![CDATA[0]]></var>\r\n    <var name=\"adapter\">customer/convert_adapter_customer</var>\r\n    <var name=\"method\">parse</var>\r\n</action>','a:5:{s:4:\"file\";a:7:{s:4:\"type\";s:4:\"file\";s:8:\"filename\";s:19:\"export_customer.csv\";s:4:\"path\";s:10:\"var/export\";s:4:\"host\";s:0:\"\";s:4:\"user\";s:0:\"\";s:8:\"password\";s:0:\"\";s:7:\"passive\";s:0:\"\";}s:5:\"parse\";a:5:{s:4:\"type\";s:3:\"csv\";s:12:\"single_sheet\";s:0:\"\";s:9:\"delimiter\";s:1:\",\";s:7:\"enclose\";s:1:\"\"\";s:10:\"fieldnames\";s:4:\"true\";}s:3:\"map\";a:3:{s:14:\"only_specified\";s:0:\"\";s:7:\"product\";a:2:{s:2:\"db\";a:0:{}s:4:\"file\";a:0:{}}s:8:\"customer\";a:2:{s:2:\"db\";a:0:{}s:4:\"file\";a:0:{}}}s:7:\"product\";a:1:{s:6:\"filter\";a:8:{s:4:\"name\";s:0:\"\";s:3:\"sku\";s:0:\"\";s:4:\"type\";s:1:\"0\";s:13:\"attribute_set\";s:0:\"\";s:5:\"price\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}s:3:\"qty\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}s:10:\"visibility\";s:1:\"0\";s:6:\"status\";s:1:\"0\";}}s:8:\"customer\";a:1:{s:6:\"filter\";a:10:{s:9:\"firstname\";s:0:\"\";s:8:\"lastname\";s:0:\"\";s:5:\"email\";s:0:\"\";s:5:\"group\";s:1:\"0\";s:10:\"adressType\";s:15:\"default_billing\";s:9:\"telephone\";s:0:\"\";s:8:\"postcode\";s:0:\"\";s:7:\"country\";s:0:\"\";s:6:\"region\";s:0:\"\";s:10:\"created_at\";a:2:{s:4:\"from\";s:0:\"\";s:2:\"to\";s:0:\"\";}}}}','import','customer',0,'interactive');
/*!40000 ALTER TABLE `dataflow_profile` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `dataflow_profile_history`
--

DROP TABLE IF EXISTS `dataflow_profile_history`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `dataflow_profile_history` (
  `history_id` int(10) unsigned NOT NULL auto_increment,
  `profile_id` int(10) unsigned NOT NULL default '0',
  `action_code` varchar(64) default NULL,
  `user_id` int(10) unsigned NOT NULL default '0',
  `performed_at` datetime default NULL,
  PRIMARY KEY  (`history_id`),
  KEY `FK_dataflow_profile_history` (`profile_id`),
  CONSTRAINT `FK_dataflow_profile_history` FOREIGN KEY (`profile_id`) REFERENCES `dataflow_profile` (`profile_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `dataflow_profile_history`
--

LOCK TABLES `dataflow_profile_history` WRITE;
/*!40000 ALTER TABLE `dataflow_profile_history` DISABLE KEYS */;
/*!40000 ALTER TABLE `dataflow_profile_history` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `dataflow_session`
--

DROP TABLE IF EXISTS `dataflow_session`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `dataflow_session` (
  `session_id` int(11) NOT NULL auto_increment,
  `user_id` int(11) NOT NULL,
  `created_date` datetime default NULL,
  `file` varchar(255) default NULL,
  `type` varchar(32) default NULL,
  `direction` varchar(32) default NULL,
  `comment` varchar(255) default NULL,
  PRIMARY KEY  (`session_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `dataflow_session`
--

LOCK TABLES `dataflow_session` WRITE;
/*!40000 ALTER TABLE `dataflow_session` DISABLE KEYS */;
/*!40000 ALTER TABLE `dataflow_session` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `design_change`
--

DROP TABLE IF EXISTS `design_change`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `design_change` (
  `design_change_id` int(11) NOT NULL auto_increment,
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `design` varchar(255) NOT NULL default '',
  `date_from` date default NULL,
  `date_to` date default NULL,
  PRIMARY KEY  (`design_change_id`),
  KEY `FK_DESIGN_CHANGE_STORE` (`store_id`),
  CONSTRAINT `FK_DESIGN_CHANGE_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `design_change`
--

LOCK TABLES `design_change` WRITE;
/*!40000 ALTER TABLE `design_change` DISABLE KEYS */;
/*!40000 ALTER TABLE `design_change` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `directory_country`
--

DROP TABLE IF EXISTS `directory_country`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `directory_country` (
  `country_id` varchar(2) NOT NULL default '',
  `iso2_code` varchar(2) NOT NULL default '',
  `iso3_code` varchar(3) NOT NULL default '',
  PRIMARY KEY  (`country_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Countries';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `directory_country`
--

LOCK TABLES `directory_country` WRITE;
/*!40000 ALTER TABLE `directory_country` DISABLE KEYS */;
INSERT INTO `directory_country` VALUES ('AD','AD','AND'),('AE','AE','ARE'),('AF','AF','AFG'),('AG','AG','ATG'),('AI','AI','AIA'),('AL','AL','ALB'),('AM','AM','ARM'),('AN','AN','ANT'),('AO','AO','AGO'),('AQ','AQ','ATA'),('AR','AR','ARG'),('AS','AS','ASM'),('AT','AT','AUT'),('AU','AU','AUS'),('AW','AW','ABW'),('AX','AX','ALA'),('AZ','AZ','AZE'),('BA','BA','BIH'),('BB','BB','BRB'),('BD','BD','BGD'),('BE','BE','BEL'),('BF','BF','BFA'),('BG','BG','BGR'),('BH','BH','BHR'),('BI','BI','BDI'),('BJ','BJ','BEN'),('BL','BL','BLM'),('BM','BM','BMU'),('BN','BN','BRN'),('BO','BO','BOL'),('BR','BR','BRA'),('BS','BS','BHS'),('BT','BT','BTN'),('BV','BV','BVT'),('BW','BW','BWA'),('BY','BY','BLR'),('BZ','BZ','BLZ'),('CA','CA','CAN'),('CC','CC','CCK'),('CD','CD','COD'),('CF','CF','CAF'),('CG','CG','COG'),('CH','CH','CHE'),('CI','CI','CIV'),('CK','CK','COK'),('CL','CL','CHL'),('CM','CM','CMR'),('CN','CN','CHN'),('CO','CO','COL'),('CR','CR','CRI'),('CS','CS','SCG'),('CU','CU','CUB'),('CV','CV','CPV'),('CX','CX','CXR'),('CY','CY','CYP'),('CZ','CZ','CZE'),('DE','DE','DEU'),('DJ','DJ','DJI'),('DK','DK','DNK'),('DM','DM','DMA'),('DO','DO','DOM'),('DZ','DZ','DZA'),('EC','EC','ECU'),('EE','EE','EST'),('EG','EG','EGY'),('EH','EH','ESH'),('ER','ER','ERI'),('ES','ES','ESP'),('ET','ET','ETH'),('FI','FI','FIN'),('FJ','FJ','FJI'),('FK','FK','FLK'),('FM','FM','FSM'),('FO','FO','FRO'),('FR','FR','FRA'),('FX','FX','FXX'),('GA','GA','GAB'),('GB','GB','GBR'),('GD','GD','GRD'),('GE','GE','GEO'),('GF','GF','GUF'),('GG','GG','GGY'),('GH','GH','GHA'),('GI','GI','GIB'),('GL','GL','GRL'),('GM','GM','GMB'),('GN','GN','GIN'),('GP','GP','GLP'),('GQ','GQ','GNQ'),('GR','GR','GRC'),('GS','GS','SGS'),('GT','GT','GTM'),('GU','GU','GUM'),('GW','GW','GNB'),('GY','GY','GUY'),('HK','HK','HKG'),('HM','HM','HMD'),('HN','HN','HND'),('HR','HR','HRV'),('HT','HT','HTI'),('HU','HU','HUN'),('ID','ID','IDN'),('IE','IE','IRL'),('IL','IL','ISR'),('IM','IM','IMN'),('IN','IN','IND'),('IO','IO','IOT'),('IQ','IQ','IRQ'),('IR','IR','IRN'),('IS','IS','ISL'),('IT','IT','ITA'),('JE','JE','JEY'),('JM','JM','JAM'),('JO','JO','JOR'),('JP','JP','JPN'),('KE','KE','KEN'),('KG','KG','KGZ'),('KH','KH','KHM'),('KI','KI','KIR'),('KM','KM','COM'),('KN','KN','KNA'),('KP','KP','PRK'),('KR','KR','KOR'),('KW','KW','KWT'),('KY','KY','CYM'),('KZ','KZ','KAZ'),('LA','LA','LAO'),('LB','LB','LBN'),('LC','LC','LCA'),('LI','LI','LIE'),('LK','LK','LKA'),('LR','LR','LBR'),('LS','LS','LSO'),('LT','LT','LTU'),('LU','LU','LUX'),('LV','LV','LVA'),('LY','LY','LBY'),('MA','MA','MAR'),('MC','MC','MCO'),('MD','MD','MDA'),('ME','ME','MNE'),('MF','MF','MAF'),('MG','MG','MDG'),('MH','MH','MHL'),('MK','MK','MKD'),('ML','ML','MLI'),('MM','MM','MMR'),('MN','MN','MNG'),('MO','MO','MAC'),('MP','MP','MNP'),('MQ','MQ','MTQ'),('MR','MR','MRT'),('MS','MS','MSR'),('MT','MT','MLT'),('MU','MU','MUS'),('MV','MV','MDV'),('MW','MW','MWI'),('MX','MX','MEX'),('MY','MY','MYS'),('MZ','MZ','MOZ'),('NA','NA','NAM'),('NC','NC','NCL'),('NE','NE','NER'),('NF','NF','NFK'),('NG','NG','NGA'),('NI','NI','NIC'),('NL','NL','NLD'),('NO','NO','NOR'),('NP','NP','NPL'),('NR','NR','NRU'),('NU','NU','NIU'),('NZ','NZ','NZL'),('OM','OM','OMN'),('PA','PA','PAN'),('PE','PE','PER'),('PF','PF','PYF'),('PG','PG','PNG'),('PH','PH','PHL'),('PK','PK','PAK'),('PL','PL','POL'),('PM','PM','SPM'),('PN','PN','PCN'),('PR','PR','PRI'),('PS','PS','PSE'),('PT','PT','PRT'),('PW','PW','PLW'),('PY','PY','PRY'),('QA','QA','QAT'),('RE','RE','REU'),('RO','RO','ROM'),('RS','RS','SRB'),('RU','RU','RUS'),('RW','RW','RWA'),('SA','SA','SAU'),('SB','SB','SLB'),('SC','SC','SYC'),('SD','SD','SDN'),('SE','SE','SWE'),('SG','SG','SGP'),('SH','SH','SHN'),('SI','SI','SVN'),('SJ','SJ','SJM'),('SK','SK','SVK'),('SL','SL','SLE'),('SM','SM','SMR'),('SN','SN','SEN'),('SO','SO','SOM'),('SR','SR','SUR'),('ST','ST','STP'),('SV','SV','SLV'),('SY','SY','SYR'),('SZ','SZ','SWZ'),('TC','TC','TCA'),('TD','TD','TCD'),('TF','TF','ATF'),('TG','TG','TGO'),('TH','TH','THA'),('TJ','TJ','TJK'),('TK','TK','TKL'),('TL','TL','TLS'),('TM','TM','TKM'),('TN','TN','TUN'),('TO','TO','TON'),('TR','TR','TUR'),('TT','TT','TTO'),('TV','TV','TUV'),('TW','TW','TWN'),('TZ','TZ','TZA'),('UA','UA','UKR'),('UG','UG','UGA'),('UM','UM','UMI'),('US','US','USA'),('UY','UY','URY'),('UZ','UZ','UZB'),('VA','VA','VAT'),('VC','VC','VCT'),('VE','VE','VEN'),('VG','VG','VGB'),('VI','VI','VIR'),('VN','VN','VNM'),('VU','VU','VUT'),('WF','WF','WLF'),('WS','WS','WSM'),('YE','YE','YEM'),('YT','YT','MYT'),('ZA','ZA','ZAF'),('ZM','ZM','ZMB'),('ZW','ZW','ZWE');
/*!40000 ALTER TABLE `directory_country` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `directory_country_format`
--

DROP TABLE IF EXISTS `directory_country_format`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `directory_country_format` (
  `country_format_id` int(10) unsigned NOT NULL auto_increment,
  `country_id` varchar(2) NOT NULL default '',
  `type` varchar(30) NOT NULL default '',
  `format` text NOT NULL,
  PRIMARY KEY  (`country_format_id`),
  UNIQUE KEY `country_type` (`country_id`,`type`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Countries format';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `directory_country_format`
--

LOCK TABLES `directory_country_format` WRITE;
/*!40000 ALTER TABLE `directory_country_format` DISABLE KEYS */;
/*!40000 ALTER TABLE `directory_country_format` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `directory_country_region`
--

DROP TABLE IF EXISTS `directory_country_region`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `directory_country_region` (
  `region_id` mediumint(8) unsigned NOT NULL auto_increment,
  `country_id` varchar(4) NOT NULL default '0',
  `code` varchar(32) NOT NULL default '',
  `default_name` varchar(255) default NULL,
  PRIMARY KEY  (`region_id`),
  KEY `FK_REGION_COUNTRY` (`country_id`)
) ENGINE=InnoDB AUTO_INCREMENT=278 DEFAULT CHARSET=utf8 COMMENT='Country regions';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `directory_country_region`
--

LOCK TABLES `directory_country_region` WRITE;
/*!40000 ALTER TABLE `directory_country_region` DISABLE KEYS */;
INSERT INTO `directory_country_region` VALUES (1,'US','AL','Alabama'),(2,'US','AK','Alaska'),(3,'US','AS','American Samoa'),(4,'US','AZ','Arizona'),(5,'US','AR','Arkansas'),(6,'US','AF','Armed Forces Africa'),(7,'US','AA','Armed Forces Americas'),(8,'US','AC','Armed Forces Canada'),(9,'US','AE','Armed Forces Europe'),(10,'US','AM','Armed Forces Middle East'),(11,'US','AP','Armed Forces Pacific'),(12,'US','CA','California'),(13,'US','CO','Colorado'),(14,'US','CT','Connecticut'),(15,'US','DE','Delaware'),(16,'US','DC','District of Columbia'),(17,'US','FM','Federated States Of Micronesia'),(18,'US','FL','Florida'),(19,'US','GA','Georgia'),(20,'US','GU','Guam'),(21,'US','HI','Hawaii'),(22,'US','ID','Idaho'),(23,'US','IL','Illinois'),(24,'US','IN','Indiana'),(25,'US','IA','Iowa'),(26,'US','KS','Kansas'),(27,'US','KY','Kentucky'),(28,'US','LA','Louisiana'),(29,'US','ME','Maine'),(30,'US','MH','Marshall Islands'),(31,'US','MD','Maryland'),(32,'US','MA','Massachusetts'),(33,'US','MI','Michigan'),(34,'US','MN','Minnesota'),(35,'US','MS','Mississippi'),(36,'US','MO','Missouri'),(37,'US','MT','Montana'),(38,'US','NE','Nebraska'),(39,'US','NV','Nevada'),(40,'US','NH','New Hampshire'),(41,'US','NJ','New Jersey'),(42,'US','NM','New Mexico'),(43,'US','NY','New York'),(44,'US','NC','North Carolina'),(45,'US','ND','North Dakota'),(46,'US','MP','Northern Mariana Islands'),(47,'US','OH','Ohio'),(48,'US','OK','Oklahoma'),(49,'US','OR','Oregon'),(50,'US','PW','Palau'),(51,'US','PA','Pennsylvania'),(52,'US','PR','Puerto Rico'),(53,'US','RI','Rhode Island'),(54,'US','SC','South Carolina'),(55,'US','SD','South Dakota'),(56,'US','TN','Tennessee'),(57,'US','TX','Texas'),(58,'US','UT','Utah'),(59,'US','VT','Vermont'),(60,'US','VI','Virgin Islands'),(61,'US','VA','Virginia'),(62,'US','WA','Washington'),(63,'US','WV','West Virginia'),(64,'US','WI','Wisconsin'),(65,'US','WY','Wyoming'),(66,'CA','AB','Alberta'),(67,'CA','BC','British Columbia'),(68,'CA','MB','Manitoba'),(69,'CA','NF','Newfoundland'),(70,'CA','NB','New Brunswick'),(71,'CA','NS','Nova Scotia'),(72,'CA','NT','Northwest Territories'),(73,'CA','NU','Nunavut'),(74,'CA','ON','Ontario'),(75,'CA','PE','Prince Edward Island'),(76,'CA','QC','Quebec'),(77,'CA','SK','Saskatchewan'),(78,'CA','YT','Yukon Territory'),(79,'DE','NDS','Niedersachsen'),(80,'DE','BAW','Baden-WÃ¼rttemberg'),(81,'DE','BAY','Bayern'),(82,'DE','BER','Berlin'),(83,'DE','BRG','Brandenburg'),(84,'DE','BRE','Bremen'),(85,'DE','HAM','Hamburg'),(86,'DE','HES','Hessen'),(87,'DE','MEC','Mecklenburg-Vorpommern'),(88,'DE','NRW','Nordrhein-Westfalen'),(89,'DE','RHE','Rheinland-Pfalz'),(90,'DE','SAR','Saarland'),(91,'DE','SAS','Sachsen'),(92,'DE','SAC','Sachsen-Anhalt'),(93,'DE','SCN','Schleswig-Holstein'),(94,'DE','THE','ThÃ¼ringen'),(95,'AT','WI','Wien'),(96,'AT','NO','NiederÃ¶sterreich'),(97,'AT','OO','OberÃ¶sterreich'),(98,'AT','SB','Salzburg'),(99,'AT','KN','KÃ¤rnten'),(100,'AT','ST','Steiermark'),(101,'AT','TI','Tirol'),(102,'AT','BL','Burgenland'),(103,'AT','VB','Voralberg'),(104,'CH','AG','Aargau'),(105,'CH','AI','Appenzell Innerrhoden'),(106,'CH','AR','Appenzell Ausserrhoden'),(107,'CH','BE','Bern'),(108,'CH','BL','Basel-Landschaft'),(109,'CH','BS','Basel-Stadt'),(110,'CH','FR','Freiburg'),(111,'CH','GE','Genf'),(112,'CH','GL','Glarus'),(113,'CH','JU','GraubÃ¼nden'),(114,'CH','JU','Jura'),(115,'CH','LU','Luzern'),(116,'CH','NE','Neuenburg'),(117,'CH','NW','Nidwalden'),(118,'CH','OW','Obwalden'),(119,'CH','SG','St. Gallen'),(120,'CH','SH','Schaffhausen'),(121,'CH','SO','Solothurn'),(122,'CH','SZ','Schwyz'),(123,'CH','TG','Thurgau'),(124,'CH','TI','Tessin'),(125,'CH','UR','Uri'),(126,'CH','VD','Waadt'),(127,'CH','VS','Wallis'),(128,'CH','ZG','Zug'),(129,'CH','ZH','ZÃ¼rich'),(130,'ES','A CoruÑa','A CoruÃ±a'),(131,'ES','Alava','Alava'),(132,'ES','Albacete','Albacete'),(133,'ES','Alicante','Alicante'),(134,'ES','Almeria','Almeria'),(135,'ES','Asturias','Asturias'),(136,'ES','Avila','Avila'),(137,'ES','Badajoz','Badajoz'),(138,'ES','Baleares','Baleares'),(139,'ES','Barcelona','Barcelona'),(140,'ES','Burgos','Burgos'),(141,'ES','Caceres','Caceres'),(142,'ES','Cadiz','Cadiz'),(143,'ES','Cantabria','Cantabria'),(144,'ES','Castellon','Castellon'),(145,'ES','Ceuta','Ceuta'),(146,'ES','Ciudad Real','Ciudad Real'),(147,'ES','Cordoba','Cordoba'),(148,'ES','Cuenca','Cuenca'),(149,'ES','Girona','Girona'),(150,'ES','Granada','Granada'),(151,'ES','Guadalajara','Guadalajara'),(152,'ES','Guipuzcoa','Guipuzcoa'),(153,'ES','Huelva','Huelva'),(154,'ES','Huesca','Huesca'),(155,'ES','Jaen','Jaen'),(156,'ES','La Rioja','La Rioja'),(157,'ES','Las Palmas','Las Palmas'),(158,'ES','Leon','Leon'),(159,'ES','Lleida','Lleida'),(160,'ES','Lugo','Lugo'),(161,'ES','Madrid','Madrid'),(162,'ES','Malaga','Malaga'),(163,'ES','Melilla','Melilla'),(164,'ES','Murcia','Murcia'),(165,'ES','Navarra','Navarra'),(166,'ES','Ourense','Ourense'),(167,'ES','Palencia','Palencia'),(168,'ES','Pontevedra','Pontevedra'),(169,'ES','Salamanca','Salamanca'),(170,'ES','Santa Cruz de Tenerife','Santa Cruz de Tenerife'),(171,'ES','Segovia','Segovia'),(172,'ES','Sevilla','Sevilla'),(173,'ES','Soria','Soria'),(174,'ES','Tarragona','Tarragona'),(175,'ES','Teruel','Teruel'),(176,'ES','Toledo','Toledo'),(177,'ES','Valencia','Valencia'),(178,'ES','Valladolid','Valladolid'),(179,'ES','Vizcaya','Vizcaya'),(180,'ES','Zamora','Zamora'),(181,'ES','Zaragoza','Zaragoza'),(182,'FR','01','Ain'),(183,'FR','02','Aisne'),(184,'FR','03','Allier'),(185,'FR','04','Alpes-de-Haute-Provence'),(186,'FR','05','Hautes-Alpes'),(187,'FR','06','Alpes-Maritimes'),(188,'FR','07','ArdÃ¨che'),(189,'FR','08','Ardennes'),(190,'FR','09','AriÃ¨ge'),(191,'FR','10','Aube'),(192,'FR','11','Aude'),(193,'FR','12','Aveyron'),(194,'FR','13','Bouches-du-RhÃ´ne'),(195,'FR','14','Calvados'),(196,'FR','15','Cantal'),(197,'FR','16','Charente'),(198,'FR','17','Charente-Maritime'),(199,'FR','18','Cher'),(200,'FR','19','CorrÃ¨ze'),(201,'FR','2A','Corse-du-Sud'),(202,'FR','2B','Haute-Corse'),(203,'FR','21','CÃ´te-d\'Or'),(204,'FR','22','CÃ´tes-d\'Armor'),(205,'FR','23','Creuse'),(206,'FR','24','Dordogne'),(207,'FR','25','Doubs'),(208,'FR','26','DrÃ´me'),(209,'FR','27','Eure'),(210,'FR','28','Eure-et-Loir'),(211,'FR','29','FinistÃ¨re'),(212,'FR','30','Gard'),(213,'FR','31','Haute-Garonne'),(214,'FR','32','Gers'),(215,'FR','33','Gironde'),(216,'FR','34','HÃ©rault'),(217,'FR','35','Ille-et-Vilaine'),(218,'FR','36','Indre'),(219,'FR','37','Indre-et-Loire'),(220,'FR','38','IsÃ¨re'),(221,'FR','39','Jura'),(222,'FR','40','Landes'),(223,'FR','41','Loir-et-Cher'),(224,'FR','42','Loire'),(225,'FR','43','Haute-Loire'),(226,'FR','44','Loire-Atlantique'),(227,'FR','45','Loiret'),(228,'FR','46','Lot'),(229,'FR','47','Lot-et-Garonne'),(230,'FR','48','LozÃ¨re'),(231,'FR','49','Maine-et-Loire'),(232,'FR','50','Manche'),(233,'FR','51','Marne'),(234,'FR','52','Haute-Marne'),(235,'FR','53','Mayenne'),(236,'FR','54','Meurthe-et-Moselle'),(237,'FR','55','Meuse'),(238,'FR','56','Morbihan'),(239,'FR','57','Moselle'),(240,'FR','58','NiÃ¨vre'),(241,'FR','59','Nord'),(242,'FR','60','Oise'),(243,'FR','61','Orne'),(244,'FR','62','Pas-de-Calais'),(245,'FR','63','Puy-de-DÃ´me'),(246,'FR','64','PyrÃ©nÃ©es-Atlantiques'),(247,'FR','65','Hautes-PyrÃ©nÃ©es'),(248,'FR','66','PyrÃ©nÃ©es-Orientales'),(249,'FR','67','Bas-Rhin'),(250,'FR','68','Haut-Rhin'),(251,'FR','69','RhÃ´ne'),(252,'FR','70','Haute-SaÃ´ne'),(253,'FR','71','SaÃ´ne-et-Loire'),(254,'FR','72','Sarthe'),(255,'FR','73','Savoie'),(256,'FR','74','Haute-Savoie'),(257,'FR','75','Paris'),(258,'FR','76','Seine-Maritime'),(259,'FR','77','Seine-et-Marne'),(260,'FR','78','Yvelines'),(261,'FR','79','Deux-SÃ¨vres'),(262,'FR','80','Somme'),(263,'FR','81','Tarn'),(264,'FR','82','Tarn-et-Garonne'),(265,'FR','83','Var'),(266,'FR','84','Vaucluse'),(267,'FR','85','VendÃ©e'),(268,'FR','86','Vienne'),(269,'FR','87','Haute-Vienne'),(270,'FR','88','Vosges'),(271,'FR','89','Yonne'),(272,'FR','90','Territoire-de-Belfort'),(273,'FR','91','Essonne'),(274,'FR','92','Hauts-de-Seine'),(275,'FR','93','Seine-Saint-Denis'),(276,'FR','94','Val-de-Marne'),(277,'FR','95','Val-d\'Oise');
/*!40000 ALTER TABLE `directory_country_region` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `directory_country_region_name`
--

DROP TABLE IF EXISTS `directory_country_region_name`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `directory_country_region_name` (
  `locale` varchar(8) NOT NULL default '',
  `region_id` mediumint(8) unsigned NOT NULL default '0',
  `name` varchar(64) NOT NULL default '',
  PRIMARY KEY  (`locale`,`region_id`),
  KEY `FK_DIRECTORY_REGION_NAME_REGION` (`region_id`),
  CONSTRAINT `FK_DIRECTORY_REGION_NAME_REGION` FOREIGN KEY (`region_id`) REFERENCES `directory_country_region` (`region_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Regions names';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `directory_country_region_name`
--

LOCK TABLES `directory_country_region_name` WRITE;
/*!40000 ALTER TABLE `directory_country_region_name` DISABLE KEYS */;
INSERT INTO `directory_country_region_name` VALUES ('en_US',1,'Alabama'),('en_US',2,'Alaska'),('en_US',3,'American Samoa'),('en_US',4,'Arizona'),('en_US',5,'Arkansas'),('en_US',6,'Armed Forces Africa'),('en_US',7,'Armed Forces Americas'),('en_US',8,'Armed Forces Canada'),('en_US',9,'Armed Forces Europe'),('en_US',10,'Armed Forces Middle East'),('en_US',11,'Armed Forces Pacific'),('en_US',12,'California'),('en_US',13,'Colorado'),('en_US',14,'Connecticut'),('en_US',15,'Delaware'),('en_US',16,'District of Columbia'),('en_US',17,'Federated States Of Micronesia'),('en_US',18,'Florida'),('en_US',19,'Georgia'),('en_US',20,'Guam'),('en_US',21,'Hawaii'),('en_US',22,'Idaho'),('en_US',23,'Illinois'),('en_US',24,'Indiana'),('en_US',25,'Iowa'),('en_US',26,'Kansas'),('en_US',27,'Kentucky'),('en_US',28,'Louisiana'),('en_US',29,'Maine'),('en_US',30,'Marshall Islands'),('en_US',31,'Maryland'),('en_US',32,'Massachusetts'),('en_US',33,'Michigan'),('en_US',34,'Minnesota'),('en_US',35,'Mississippi'),('en_US',36,'Missouri'),('en_US',37,'Montana'),('en_US',38,'Nebraska'),('en_US',39,'Nevada'),('en_US',40,'New Hampshire'),('en_US',41,'New Jersey'),('en_US',42,'New Mexico'),('en_US',43,'New York'),('en_US',44,'North Carolina'),('en_US',45,'North Dakota'),('en_US',46,'Northern Mariana Islands'),('en_US',47,'Ohio'),('en_US',48,'Oklahoma'),('en_US',49,'Oregon'),('en_US',50,'Palau'),('en_US',51,'Pennsylvania'),('en_US',52,'Puerto Rico'),('en_US',53,'Rhode Island'),('en_US',54,'South Carolina'),('en_US',55,'South Dakota'),('en_US',56,'Tennessee'),('en_US',57,'Texas'),('en_US',58,'Utah'),('en_US',59,'Vermont'),('en_US',60,'Virgin Islands'),('en_US',61,'Virginia'),('en_US',62,'Washington'),('en_US',63,'West Virginia'),('en_US',64,'Wisconsin'),('en_US',65,'Wyoming'),('en_US',66,'Alberta'),('en_US',67,'British Columbia'),('en_US',68,'Manitoba'),('en_US',69,'Newfoundland'),('en_US',70,'New Brunswick'),('en_US',71,'Nova Scotia'),('en_US',72,'Northwest Territories'),('en_US',73,'Nunavut'),('en_US',74,'Ontario'),('en_US',75,'Prince Edward Island'),('en_US',76,'Quebec'),('en_US',77,'Saskatchewan'),('en_US',78,'Yukon Territory'),('en_US',79,'Niedersachsen'),('en_US',80,'Baden-WÃ¼rttemberg'),('en_US',81,'Bayern'),('en_US',82,'Berlin'),('en_US',83,'Brandenburg'),('en_US',84,'Bremen'),('en_US',85,'Hamburg'),('en_US',86,'Hessen'),('en_US',87,'Mecklenburg-Vorpommern'),('en_US',88,'Nordrhein-Westfalen'),('en_US',89,'Rheinland-Pfalz'),('en_US',90,'Saarland'),('en_US',91,'Sachsen'),('en_US',92,'Sachsen-Anhalt'),('en_US',93,'Schleswig-Holstein'),('en_US',94,'ThÃ¼ringen'),('en_US',95,'Wien'),('en_US',96,'NiederÃ¶sterreich'),('en_US',97,'OberÃ¶sterreich'),('en_US',98,'Salzburg'),('en_US',99,'KÃ¤rnten'),('en_US',100,'Steiermark'),('en_US',101,'Tirol'),('en_US',102,'Burgenland'),('en_US',103,'Voralberg'),('en_US',104,'Aargau'),('en_US',105,'Appenzell Innerrhoden'),('en_US',106,'Appenzell Ausserrhoden'),('en_US',107,'Bern'),('en_US',108,'Basel-Landschaft'),('en_US',109,'Basel-Stadt'),('en_US',110,'Freiburg'),('en_US',111,'Genf'),('en_US',112,'Glarus'),('en_US',113,'GraubÃ¼nden'),('en_US',114,'Jura'),('en_US',115,'Luzern'),('en_US',116,'Neuenburg'),('en_US',117,'Nidwalden'),('en_US',118,'Obwalden'),('en_US',119,'St. Gallen'),('en_US',120,'Schaffhausen'),('en_US',121,'Solothurn'),('en_US',122,'Schwyz'),('en_US',123,'Thurgau'),('en_US',124,'Tessin'),('en_US',125,'Uri'),('en_US',126,'Waadt'),('en_US',127,'Wallis'),('en_US',128,'Zug'),('en_US',129,'ZÃ¼rich'),('en_US',130,'A CoruÃ±a'),('en_US',131,'Alava'),('en_US',132,'Albacete'),('en_US',133,'Alicante'),('en_US',134,'Almeria'),('en_US',135,'Asturias'),('en_US',136,'Avila'),('en_US',137,'Badajoz'),('en_US',138,'Baleares'),('en_US',139,'Barcelona'),('en_US',140,'Burgos'),('en_US',141,'Caceres'),('en_US',142,'Cadiz'),('en_US',143,'Cantabria'),('en_US',144,'Castellon'),('en_US',145,'Ceuta'),('en_US',146,'Ciudad Real'),('en_US',147,'Cordoba'),('en_US',148,'Cuenca'),('en_US',149,'Girona'),('en_US',150,'Granada'),('en_US',151,'Guadalajara'),('en_US',152,'Guipuzcoa'),('en_US',153,'Huelva'),('en_US',154,'Huesca'),('en_US',155,'Jaen'),('en_US',156,'La Rioja'),('en_US',157,'Las Palmas'),('en_US',158,'Leon'),('en_US',159,'Lleida'),('en_US',160,'Lugo'),('en_US',161,'Madrid'),('en_US',162,'Malaga'),('en_US',163,'Melilla'),('en_US',164,'Murcia'),('en_US',165,'Navarra'),('en_US',166,'Ourense'),('en_US',167,'Palencia'),('en_US',168,'Pontevedra'),('en_US',169,'Salamanca'),('en_US',170,'Santa Cruz de Tenerife'),('en_US',171,'Segovia'),('en_US',172,'Sevilla'),('en_US',173,'Soria'),('en_US',174,'Tarragona'),('en_US',175,'Teruel'),('en_US',176,'Toledo'),('en_US',177,'Valencia'),('en_US',178,'Valladolid'),('en_US',179,'Vizcaya'),('en_US',180,'Zamora'),('en_US',181,'Zaragoza'),('en_US',182,'Ain'),('en_US',183,'Aisne'),('en_US',184,'Allier'),('en_US',185,'Alpes-de-Haute-Provence'),('en_US',186,'Hautes-Alpes'),('en_US',187,'Alpes-Maritimes'),('en_US',188,'ArdÃ¨che'),('en_US',189,'Ardennes'),('en_US',190,'AriÃ¨ge'),('en_US',191,'Aube'),('en_US',192,'Aude'),('en_US',193,'Aveyron'),('en_US',194,'Bouches-du-RhÃ´ne'),('en_US',195,'Calvados'),('en_US',196,'Cantal'),('en_US',197,'Charente'),('en_US',198,'Charente-Maritime'),('en_US',199,'Cher'),('en_US',200,'CorrÃ¨ze'),('en_US',201,'Corse-du-Sud'),('en_US',202,'Haute-Corse'),('en_US',203,'CÃ´te-d\'Or'),('en_US',204,'CÃ´tes-d\'Armor'),('en_US',205,'Creuse'),('en_US',206,'Dordogne'),('en_US',207,'Doubs'),('en_US',208,'DrÃ´me'),('en_US',209,'Eure'),('en_US',210,'Eure-et-Loir'),('en_US',211,'FinistÃ¨re'),('en_US',212,'Gard'),('en_US',213,'Haute-Garonne'),('en_US',214,'Gers'),('en_US',215,'Gironde'),('en_US',216,'HÃ©rault'),('en_US',217,'Ille-et-Vilaine'),('en_US',218,'Indre'),('en_US',219,'Indre-et-Loire'),('en_US',220,'IsÃ¨re'),('en_US',221,'Jura'),('en_US',222,'Landes'),('en_US',223,'Loir-et-Cher'),('en_US',224,'Loire'),('en_US',225,'Haute-Loire'),('en_US',226,'Loire-Atlantique'),('en_US',227,'Loiret'),('en_US',228,'Lot'),('en_US',229,'Lot-et-Garonne'),('en_US',230,'LozÃ¨re'),('en_US',231,'Maine-et-Loire'),('en_US',232,'Manche'),('en_US',233,'Marne'),('en_US',234,'Haute-Marne'),('en_US',235,'Mayenne'),('en_US',236,'Meurthe-et-Moselle'),('en_US',237,'Meuse'),('en_US',238,'Morbihan'),('en_US',239,'Moselle'),('en_US',240,'NiÃ¨vre'),('en_US',241,'Nord'),('en_US',242,'Oise'),('en_US',243,'Orne'),('en_US',244,'Pas-de-Calais'),('en_US',245,'Puy-de-DÃ´me'),('en_US',246,'PyrÃ©nÃ©es-Atlantiques'),('en_US',247,'Hautes-PyrÃ©nÃ©es'),('en_US',248,'PyrÃ©nÃ©es-Orientales'),('en_US',249,'Bas-Rhin'),('en_US',250,'Haut-Rhin'),('en_US',251,'RhÃ´ne'),('en_US',252,'Haute-SaÃ´ne'),('en_US',253,'SaÃ´ne-et-Loire'),('en_US',254,'Sarthe'),('en_US',255,'Savoie'),('en_US',256,'Haute-Savoie'),('en_US',257,'Paris'),('en_US',258,'Seine-Maritime'),('en_US',259,'Seine-et-Marne'),('en_US',260,'Yvelines'),('en_US',261,'Deux-SÃ¨vres'),('en_US',262,'Somme'),('en_US',263,'Tarn'),('en_US',264,'Tarn-et-Garonne'),('en_US',265,'Var'),('en_US',266,'Vaucluse'),('en_US',267,'VendÃ©e'),('en_US',268,'Vienne'),('en_US',269,'Haute-Vienne'),('en_US',270,'Vosges'),('en_US',271,'Yonne'),('en_US',272,'Territoire-de-Belfort'),('en_US',273,'Essonne'),('en_US',274,'Hauts-de-Seine'),('en_US',275,'Seine-Saint-Denis'),('en_US',276,'Val-de-Marne'),('en_US',277,'Val-d\'Oise');
/*!40000 ALTER TABLE `directory_country_region_name` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `directory_currency_rate`
--

DROP TABLE IF EXISTS `directory_currency_rate`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `directory_currency_rate` (
  `currency_from` char(3) NOT NULL default '',
  `currency_to` char(3) NOT NULL default '',
  `rate` decimal(24,12) NOT NULL default '0.000000000000',
  PRIMARY KEY  (`currency_from`,`currency_to`),
  KEY `FK_CURRENCY_RATE_TO` (`currency_to`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `directory_currency_rate`
--

LOCK TABLES `directory_currency_rate` WRITE;
/*!40000 ALTER TABLE `directory_currency_rate` DISABLE KEYS */;
INSERT INTO `directory_currency_rate` VALUES ('EUR','EUR','1.000000000000'),('EUR','USD','1.415000000000'),('USD','EUR','0.706700000000'),('USD','USD','1.000000000000');
/*!40000 ALTER TABLE `directory_currency_rate` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `downloadable_link`
--

DROP TABLE IF EXISTS `downloadable_link`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `downloadable_link` (
  `link_id` int(10) unsigned NOT NULL auto_increment,
  `product_id` int(10) unsigned NOT NULL default '0',
  `sort_order` int(10) unsigned NOT NULL default '0',
  `number_of_downloads` int(10) unsigned default NULL,
  `is_shareable` smallint(1) unsigned NOT NULL default '0',
  `link_url` varchar(255) NOT NULL default '',
  `link_file` varchar(255) NOT NULL default '',
  `link_type` varchar(20) NOT NULL default '',
  `sample_url` varchar(255) NOT NULL default '',
  `sample_file` varchar(255) NOT NULL default '',
  `sample_type` varchar(20) NOT NULL default '',
  PRIMARY KEY  (`link_id`),
  KEY `DOWNLODABLE_LINK_PRODUCT` (`product_id`),
  KEY `DOWNLODABLE_LINK_PRODUCT_SORT_ORDER` (`product_id`,`sort_order`),
  CONSTRAINT `FK_DOWNLODABLE_LINK_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `downloadable_link`
--

LOCK TABLES `downloadable_link` WRITE;
/*!40000 ALTER TABLE `downloadable_link` DISABLE KEYS */;
/*!40000 ALTER TABLE `downloadable_link` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `downloadable_link_price`
--

DROP TABLE IF EXISTS `downloadable_link_price`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `downloadable_link_price` (
  `price_id` int(10) unsigned NOT NULL auto_increment,
  `link_id` int(10) unsigned NOT NULL default '0',
  `website_id` smallint(5) unsigned NOT NULL default '0',
  `price` decimal(12,4) NOT NULL default '0.0000',
  PRIMARY KEY  (`price_id`),
  KEY `DOWNLOADABLE_LINK_PRICE_LINK` (`link_id`),
  KEY `DOWNLOADABLE_LINK_PRICE_WEBSITE` (`website_id`),
  CONSTRAINT `FK_DOWNLOADABLE_LINK_PRICE_LINK` FOREIGN KEY (`link_id`) REFERENCES `downloadable_link` (`link_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_DOWNLOADABLE_LINK_PRICE_WEBSITE` FOREIGN KEY (`website_id`) REFERENCES `core_website` (`website_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `downloadable_link_price`
--

LOCK TABLES `downloadable_link_price` WRITE;
/*!40000 ALTER TABLE `downloadable_link_price` DISABLE KEYS */;
/*!40000 ALTER TABLE `downloadable_link_price` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `downloadable_link_purchased`
--

DROP TABLE IF EXISTS `downloadable_link_purchased`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `downloadable_link_purchased` (
  `purchased_id` int(10) unsigned NOT NULL auto_increment,
  `order_id` int(10) unsigned NOT NULL default '0',
  `order_increment_id` varchar(50) NOT NULL default '',
  `order_item_id` int(10) unsigned NOT NULL default '0',
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `customer_id` int(10) unsigned NOT NULL default '0',
  `product_name` varchar(255) NOT NULL default '',
  `product_sku` varchar(255) NOT NULL default '',
  `link_section_title` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`purchased_id`),
  KEY `DOWNLOADABLE_ORDER_ID` (`order_id`),
  KEY `DOWNLOADABLE_CUSTOMER_ID` (`customer_id`),
  KEY `KEY_DOWNLOADABLE_ORDER_ITEM_ID` (`order_item_id`),
  CONSTRAINT `FK_DOWNLOADABLE_ORDER_ID` FOREIGN KEY (`order_id`) REFERENCES `sales_order` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_DOWNLOADABLE_PURCHASED_ORDER_ITEM_ID` FOREIGN KEY (`order_item_id`) REFERENCES `sales_flat_order_item` (`item_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `downloadable_link_purchased`
--

LOCK TABLES `downloadable_link_purchased` WRITE;
/*!40000 ALTER TABLE `downloadable_link_purchased` DISABLE KEYS */;
/*!40000 ALTER TABLE `downloadable_link_purchased` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `downloadable_link_purchased_item`
--

DROP TABLE IF EXISTS `downloadable_link_purchased_item`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `downloadable_link_purchased_item` (
  `item_id` int(10) unsigned NOT NULL auto_increment,
  `purchased_id` int(10) unsigned NOT NULL default '0',
  `order_item_id` int(10) unsigned NOT NULL default '0',
  `product_id` int(10) unsigned default '0',
  `link_hash` varchar(255) NOT NULL default '',
  `number_of_downloads_bought` int(10) unsigned NOT NULL default '0',
  `number_of_downloads_used` int(10) unsigned NOT NULL default '0',
  `link_id` int(20) unsigned NOT NULL default '0',
  `link_title` varchar(255) NOT NULL default '',
  `is_shareable` smallint(1) unsigned NOT NULL default '0',
  `link_url` varchar(255) NOT NULL default '',
  `link_file` varchar(255) NOT NULL default '',
  `link_type` varchar(255) NOT NULL default '',
  `status` varchar(50) NOT NULL default '',
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`item_id`),
  KEY `DOWNLOADABLE_LINK_PURCHASED_ID` (`purchased_id`),
  KEY `DOWNLOADABLE_ORDER_ITEM_ID` (`order_item_id`),
  KEY `DOWNLOADALBE_LINK_HASH` (`link_hash`),
  CONSTRAINT `FK_DOWNLOADABLE_LINK_PURCHASED_ID` FOREIGN KEY (`purchased_id`) REFERENCES `downloadable_link_purchased` (`purchased_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_DOWNLOADABLE_ORDER_ITEM_ID` FOREIGN KEY (`order_item_id`) REFERENCES `sales_flat_order_item` (`item_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `downloadable_link_purchased_item`
--

LOCK TABLES `downloadable_link_purchased_item` WRITE;
/*!40000 ALTER TABLE `downloadable_link_purchased_item` DISABLE KEYS */;
/*!40000 ALTER TABLE `downloadable_link_purchased_item` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `downloadable_link_title`
--

DROP TABLE IF EXISTS `downloadable_link_title`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `downloadable_link_title` (
  `title_id` int(10) unsigned NOT NULL auto_increment,
  `link_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `title` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`title_id`),
  KEY `DOWNLOADABLE_LINK_TITLE_LINK` (`link_id`),
  KEY `DOWNLOADABLE_LINK_TITLE_STORE` (`store_id`),
  CONSTRAINT `FK_DOWNLOADABLE_LINK_TITLE_LINK` FOREIGN KEY (`link_id`) REFERENCES `downloadable_link` (`link_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_DOWNLOADABLE_LINK_TITLE_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `downloadable_link_title`
--

LOCK TABLES `downloadable_link_title` WRITE;
/*!40000 ALTER TABLE `downloadable_link_title` DISABLE KEYS */;
/*!40000 ALTER TABLE `downloadable_link_title` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `downloadable_sample`
--

DROP TABLE IF EXISTS `downloadable_sample`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `downloadable_sample` (
  `sample_id` int(10) unsigned NOT NULL auto_increment,
  `product_id` int(10) unsigned NOT NULL default '0',
  `sample_url` varchar(255) NOT NULL default '',
  `sample_file` varchar(255) NOT NULL default '',
  `sample_type` varchar(20) NOT NULL default '',
  `sort_order` int(10) unsigned NOT NULL default '0',
  PRIMARY KEY  (`sample_id`),
  KEY `DOWNLODABLE_SAMPLE_PRODUCT` (`product_id`),
  CONSTRAINT `FK_DOWNLODABLE_SAMPLE_PRODUCT` FOREIGN KEY (`product_id`) REFERENCES `catalog_product_entity` (`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `downloadable_sample`
--

LOCK TABLES `downloadable_sample` WRITE;
/*!40000 ALTER TABLE `downloadable_sample` DISABLE KEYS */;
/*!40000 ALTER TABLE `downloadable_sample` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `downloadable_sample_title`
--

DROP TABLE IF EXISTS `downloadable_sample_title`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `downloadable_sample_title` (
  `title_id` int(10) unsigned NOT NULL auto_increment,
  `sample_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `title` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`title_id`),
  KEY `DOWNLOADABLE_SAMPLE_TITLE_SAMPLE` (`sample_id`),
  KEY `DOWNLOADABLE_SAMPLE_TITLE_STORE` (`store_id`),
  CONSTRAINT `FK_DOWNLOADABLE_SAMPLE_TITLE_SAMPLE` FOREIGN KEY (`sample_id`) REFERENCES `downloadable_sample` (`sample_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_DOWNLOADABLE_SAMPLE_TITLE_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `downloadable_sample_title`
--

LOCK TABLES `downloadable_sample_title` WRITE;
/*!40000 ALTER TABLE `downloadable_sample_title` DISABLE KEYS */;
/*!40000 ALTER TABLE `downloadable_sample_title` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_attribute`
--

DROP TABLE IF EXISTS `eav_attribute`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_attribute` (
  `attribute_id` smallint(5) unsigned NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_code` varchar(255) NOT NULL default '',
  `attribute_model` varchar(255) default NULL,
  `backend_model` varchar(255) default NULL,
  `backend_type` enum('static','datetime','decimal','int','text','varchar') NOT NULL default 'static',
  `backend_table` varchar(255) default NULL,
  `frontend_model` varchar(255) default NULL,
  `frontend_input` varchar(50) default NULL,
  `frontend_input_renderer` varchar(255) default NULL,
  `frontend_label` varchar(255) default NULL,
  `frontend_class` varchar(255) default NULL,
  `source_model` varchar(255) default NULL,
  `is_global` tinyint(1) unsigned NOT NULL default '1',
  `is_visible` tinyint(1) unsigned NOT NULL default '1',
  `is_required` tinyint(1) unsigned NOT NULL default '0',
  `is_user_defined` tinyint(1) unsigned NOT NULL default '0',
  `default_value` text,
  `is_searchable` tinyint(1) unsigned NOT NULL default '0',
  `is_filterable` tinyint(1) unsigned NOT NULL default '0',
  `is_comparable` tinyint(1) unsigned NOT NULL default '0',
  `is_visible_on_front` tinyint(1) unsigned NOT NULL default '0',
  `is_html_allowed_on_front` tinyint(1) unsigned NOT NULL default '0',
  `is_unique` tinyint(1) unsigned NOT NULL default '0',
  `is_used_for_price_rules` tinyint(1) unsigned NOT NULL default '0',
  `is_filterable_in_search` tinyint(1) unsigned NOT NULL default '0',
  `used_in_product_listing` tinyint(1) unsigned NOT NULL default '0',
  `used_for_sort_by` tinyint(1) unsigned NOT NULL default '0',
  `is_configurable` tinyint(1) unsigned NOT NULL default '1',
  `apply_to` varchar(255) NOT NULL,
  `position` int(11) NOT NULL,
  `note` varchar(255) NOT NULL,
  `is_visible_in_advanced_search` tinyint(1) unsigned NOT NULL default '0',
  PRIMARY KEY  (`attribute_id`),
  UNIQUE KEY `entity_type_id` (`entity_type_id`,`attribute_code`),
  KEY `IDX_USED_FOR_SORT_BY` (`entity_type_id`,`used_for_sort_by`),
  KEY `IDX_USED_IN_PRODUCT_LISTING` (`entity_type_id`,`used_in_product_listing`),
  CONSTRAINT `FK_eav_attribute` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=496 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_attribute`
--

LOCK TABLES `eav_attribute` WRITE;
/*!40000 ALTER TABLE `eav_attribute` DISABLE KEYS */;
INSERT INTO `eav_attribute` VALUES (1,1,'website_id',NULL,'customer/customer_attribute_backend_website','static','','','select','','Associate to Website',NULL,'customer/customer_attribute_source_website',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(2,1,'store_id',NULL,'customer/customer_attribute_backend_store','static','','','select','','Create In',NULL,'customer/customer_attribute_source_store',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(3,1,'created_in',NULL,'','varchar','','','text','','Created From',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(4,1,'prefix',NULL,'','varchar','','','text','','Prefix',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(5,1,'firstname',NULL,'','varchar','','','text','','First Name',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(6,1,'middlename',NULL,'','varchar','','','text','','Middle Name/Initial',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(7,1,'lastname',NULL,'','varchar','','','text','','Last Name',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(8,1,'suffix',NULL,'','varchar','','','text','','Suffix',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(9,1,'email',NULL,'','static','','','text','','Email',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(10,1,'group_id',NULL,'','static','','','select','','Customer Group',NULL,'customer/customer_attribute_source_group',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(11,1,'dob',NULL,'eav/entity_attribute_backend_datetime','datetime','','','date','','Date Of Birth',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(12,1,'password_hash',NULL,'customer/customer_attribute_backend_password','varchar','','','hidden','','',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(13,1,'default_billing',NULL,'customer/customer_attribute_backend_billing','int','','','text','','',NULL,'',1,0,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(14,1,'default_shipping',NULL,'customer/customer_attribute_backend_shipping','int','','','text','','',NULL,'',1,0,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(15,1,'taxvat',NULL,'','varchar','','','text','','Tax/VAT number',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(16,1,'confirmation',NULL,'','varchar','','','text','','Is confirmed',NULL,'',1,0,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(17,2,'prefix',NULL,'','varchar','','','text','','Prefix',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(18,2,'firstname',NULL,'','varchar','','','text','','First Name',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(19,2,'middlename',NULL,'','varchar','','','text','','Middle Name/Initial',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(20,2,'lastname',NULL,'','varchar','','','text','','Last Name',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(21,2,'suffix',NULL,'','varchar','','','text','','Suffix',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(22,2,'company',NULL,'','varchar','','','text','','Company',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(23,2,'street',NULL,'customer_entity/address_attribute_backend_street','text','','','multiline','','Street Address',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(24,2,'city',NULL,'','varchar','','','text','','City',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(25,2,'country_id',NULL,'','varchar','','','select','','Country',NULL,'customer_entity/address_attribute_source_country',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(26,2,'region',NULL,'customer_entity/address_attribute_backend_region','varchar','','','text','','State/Province',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(27,2,'region_id',NULL,'','int','','','hidden','','',NULL,'customer_entity/address_attribute_source_region',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(28,2,'postcode',NULL,'','varchar','','','text','','Zip/Postal Code',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(29,2,'telephone',NULL,'','varchar','','','text','','Telephone',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(30,2,'fax',NULL,'','varchar','','','text','','Fax',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(31,3,'name',NULL,'','varchar','','','text','','Name',NULL,'',0,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(32,3,'is_active',NULL,'','int','','','select','','Is Active',NULL,'eav/entity_attribute_source_boolean',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(33,3,'url_key',NULL,'catalog/category_attribute_backend_urlkey','varchar','','','text','','URL key',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(34,3,'description',NULL,'','text','','','textarea','','Description',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(35,3,'image',NULL,'catalog/category_attribute_backend_image','varchar','','','image','','Image',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(36,3,'meta_title',NULL,'','varchar','','','text','','Page Title',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(37,3,'meta_keywords',NULL,'','text','','','textarea','','Meta Keywords',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(38,3,'meta_description',NULL,'','text','','','textarea','','Meta Description',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(39,3,'display_mode',NULL,'','varchar','','','select','','Display Mode',NULL,'catalog/category_attribute_source_mode',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(40,3,'landing_page',NULL,'','int','','','select','','CMS Block',NULL,'catalog/category_attribute_source_page',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(41,3,'is_anchor',NULL,'','int','','','select','','Is Anchor',NULL,'eav/entity_attribute_source_boolean',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(42,3,'path',NULL,'','static','','','','','Path',NULL,'',1,0,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(43,3,'position',NULL,'','static','','','','','Position',NULL,'',1,0,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(44,3,'all_children',NULL,'','text','','','','','',NULL,'',1,0,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(45,3,'path_in_store',NULL,'','text','','','','','',NULL,'',1,0,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(46,3,'children',NULL,'','text','','','','','',NULL,'',1,0,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(47,3,'url_path',NULL,'','varchar','','','','','',NULL,'',0,0,0,0,'',0,0,0,0,0,1,1,0,0,0,1,'',1,'',0),(48,3,'custom_design',NULL,'','varchar','','','select','','Custom Design',NULL,'core/design_source_design',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(49,3,'custom_design_apply',NULL,'','int','','','select','','Apply To',NULL,'core/design_source_apply',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(50,3,'custom_design_from',NULL,'eav/entity_attribute_backend_datetime','datetime','','','date','','Active From',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(51,3,'custom_design_to',NULL,'eav/entity_attribute_backend_datetime','datetime','','','date','','Active To',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(52,3,'page_layout',NULL,'','varchar','','','select','','Page Layout',NULL,'catalog/category_attribute_source_layout',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(53,3,'custom_layout_update',NULL,'','text','','','textarea','','Custom Layout Update',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(54,3,'level',NULL,'','static','','','','','Level',NULL,'',1,0,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(55,3,'children_count',NULL,'','static','','','','','Children Count',NULL,'',1,0,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(56,4,'name',NULL,'','varchar','','','text','','Name',NULL,'',0,1,1,0,'',1,0,0,0,0,0,1,0,1,1,1,'',1,'',1),(57,4,'description',NULL,'','text','','','textarea','','Description',NULL,'',0,1,1,0,'',1,0,1,0,0,0,1,0,0,0,1,'',1,'',1),(58,4,'short_description',NULL,'','text','','','textarea','','Short Description',NULL,'',0,1,1,0,'',1,0,1,0,0,0,1,0,1,0,1,'',1,'',1),(59,4,'sku',NULL,'','static','','','text','','SKU',NULL,'',1,1,1,0,'',1,0,1,0,0,1,1,0,0,0,1,'',1,'',1),(60,4,'price',NULL,'catalog/product_attribute_backend_price','decimal','','','price','','Price',NULL,'',2,1,1,0,'',1,1,0,0,0,0,1,0,1,1,1,'simple,configurable,virtual,bundle,downloadable',0,'',1),(61,4,'special_price',NULL,'catalog/product_attribute_backend_price','decimal','','','price','','Special Price',NULL,'',2,1,0,0,'',0,0,0,0,0,0,1,0,1,0,1,'simple,configurable,virtual,bundle,downloadable',1,'',0),(62,4,'special_from_date',NULL,'catalog/product_attribute_backend_startdate','datetime','','','date','','Special Price From Date',NULL,'',2,1,0,0,'',0,0,0,0,0,0,1,0,1,0,1,'simple,configurable,virtual,bundle,downloadable',1,'',0),(63,4,'special_to_date',NULL,'eav/entity_attribute_backend_datetime','datetime','','','date','','Special Price To Date',NULL,'',2,1,0,0,'',0,0,0,0,0,0,1,0,1,0,1,'simple,configurable,virtual,bundle,downloadable',1,'',0),(64,4,'cost',NULL,'catalog/product_attribute_backend_price','decimal','','','price','','Cost',NULL,'',2,1,0,1,'',0,0,0,0,0,0,1,0,0,0,1,'simple,configurable,virtual,bundle,downloadable',1,'',0),(65,4,'weight',NULL,'','decimal','','','text','','Weight',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'simple,bundle',1,'',0),(66,4,'manufacturer',NULL,'','int','','','select','','Manufacturer',NULL,'',1,1,0,1,'',1,1,1,0,0,0,1,0,0,0,1,'simple',1,'',1),(67,4,'meta_title',NULL,'','varchar','','','text','','Meta Title',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(68,4,'meta_keyword',NULL,'','text','','','textarea','','Meta Keywords',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(69,4,'meta_description',NULL,'','varchar','','','textarea','','Meta Description',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'Maximum 255 chars',0),(70,4,'image',NULL,'','varchar','','catalog/product_attribute_frontend_image','media_image','','Base Image',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(71,4,'small_image',NULL,'','varchar','','catalog/product_attribute_frontend_image','media_image','','Small Image',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,1,0,1,'',1,'',0),(72,4,'thumbnail',NULL,'','varchar','','catalog/product_attribute_frontend_image','media_image','','Thumbnail',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,1,0,1,'',1,'',0),(73,4,'media_gallery',NULL,'catalog/product_attribute_backend_media','varchar','','','gallery','','Media Gallery',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(74,4,'old_id',NULL,'','int','','','','','',NULL,'',1,0,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(75,4,'tier_price',NULL,'catalog/product_attribute_backend_tierprice','decimal','','','text','','Tier Price',NULL,'',2,1,0,0,'',0,0,0,0,0,0,0,0,0,0,1,'simple,configurable,virtual,bundle,downloadable',1,'',0),(76,4,'color',NULL,'','int','','','select','','Color',NULL,'',1,1,0,1,'',1,1,1,0,0,0,1,0,0,0,1,'simple',1,'',1),(77,4,'news_from_date',NULL,'eav/entity_attribute_backend_datetime','datetime','','','date','','Set Product as New from Date',NULL,'',2,1,0,0,'',0,0,0,0,0,0,1,0,1,0,1,'',1,'',0),(78,4,'news_to_date',NULL,'eav/entity_attribute_backend_datetime','datetime','','','date','','Set Product as New to Date',NULL,'',2,1,0,0,'',0,0,0,0,0,0,1,0,1,0,1,'',1,'',0),(79,4,'gallery',NULL,'','varchar','','','gallery','','Image Gallery',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(80,4,'status',NULL,'','int','','','select','','Status',NULL,'catalog/product_status',2,1,1,0,'',1,0,0,0,0,0,1,0,1,0,1,'',1,'',0),(81,4,'tax_class_id',NULL,'','int','','','select','','Tax Class',NULL,'tax/class_source_product',2,1,1,0,'',1,0,0,0,0,0,1,0,1,0,1,'simple,configurable,virtual,bundle,downloadable',1,'',1),(82,4,'url_key',NULL,'catalog/product_attribute_backend_urlkey','varchar','','','text','','URL key',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,1,0,1,'',1,'',0),(83,4,'url_path',NULL,'','varchar','','','','','',NULL,'',0,0,0,0,'',0,0,0,0,0,1,1,0,0,0,1,'',1,'',0),(84,4,'minimal_price',NULL,'','decimal','','','price','','Minimal Price',NULL,'',0,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'simple,configurable,virtual,bundle,downloadable',1,'',0),(85,4,'visibility',NULL,'','int','','','select','','Visibility',NULL,'catalog/product_visibility',0,1,1,0,'4',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(86,4,'custom_design',NULL,'','varchar','','','select','','Custom Design',NULL,'core/design_source_design',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(87,4,'custom_design_from',NULL,'eav/entity_attribute_backend_datetime','datetime','','','date','','Active From',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(88,4,'custom_design_to',NULL,'eav/entity_attribute_backend_datetime','datetime','','','date','','Active To',NULL,'',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(89,4,'custom_layout_update',NULL,'','text','','','textarea','','Custom Layout Update',NULL,'',1,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(90,4,'page_layout',NULL,'','varchar','','','select','','Page Layout',NULL,'catalog/product_attribute_source_layout',0,1,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(91,4,'category_ids',NULL,'','static','','','','','',NULL,'',1,0,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',1,'',0),(92,4,'options_container',NULL,'','varchar','','','select','','Display product options in',NULL,'catalog/entity_product_attribute_design_options_container',0,1,0,0,'container2',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(93,4,'required_options',NULL,'','static','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,1,0,1,'',0,'',0),(94,4,'has_options',NULL,'','static','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(95,4,'image_label',NULL,'','varchar','','','text','','Image Label',NULL,'',0,0,0,0,'',0,0,0,0,0,0,1,0,1,0,0,'',0,'',0),(96,4,'small_image_label',NULL,'','varchar','','','text','','Small Image Label',NULL,'',0,0,0,0,'',0,0,0,0,0,0,1,0,1,0,0,'',0,'',0),(97,4,'thumbnail_label',NULL,'','varchar','','','text','','Thumbnail Label',NULL,'',0,0,0,0,'',0,0,0,0,0,0,1,0,1,0,0,'',0,'',0),(98,3,'available_sort_by',NULL,'catalog/category_attribute_backend_sortby','text','','','multiselect','adminhtml/catalog_category_helper_sortby_available','Available Product Listing Sort by',NULL,'catalog/category_attribute_source_sortby',0,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(99,3,'default_sort_by',NULL,'catalog/category_attribute_backend_sortby','varchar','','','select','adminhtml/catalog_category_helper_sortby_default','Default Product Listing Sort by',NULL,'catalog/category_attribute_source_sortby',0,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(100,4,'created_at',NULL,'eav/entity_attribute_backend_time_created','static','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(101,4,'updated_at',NULL,'eav/entity_attribute_backend_time_updated','static','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(102,11,'entity_id',NULL,'sales_entity/order_attribute_backend_parent','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(103,11,'store_id',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(104,11,'store_name',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(105,11,'remote_ip',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(106,11,'status',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(107,11,'state',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(108,11,'hold_before_status',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(109,11,'hold_before_state',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(110,11,'relation_parent_id',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(111,11,'relation_parent_real_id',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(112,11,'relation_child_id',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(113,11,'relation_child_real_id',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(114,11,'original_increment_id',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(115,11,'edit_increment',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(116,11,'ext_order_id',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(117,11,'ext_customer_id',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(118,11,'quote_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(119,11,'quote_address_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(120,11,'billing_address_id',NULL,'sales_entity/order_attribute_backend_billing','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(121,11,'shipping_address_id',NULL,'sales_entity/order_attribute_backend_shipping','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(122,11,'coupon_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(123,11,'applied_rule_ids',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(124,11,'giftcert_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(125,11,'global_currency_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(126,11,'base_currency_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(127,11,'store_currency_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(128,11,'order_currency_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(129,11,'store_to_base_rate',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(130,11,'store_to_order_rate',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(131,11,'base_to_global_rate',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(132,11,'base_to_order_rate',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(133,11,'is_virtual',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(134,11,'shipping_method',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(135,11,'shipping_description',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(136,11,'weight',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(137,11,'tax_amount',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(138,11,'shipping_amount',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(139,11,'shipping_tax_amount',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(140,11,'discount_amount',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(141,11,'giftcert_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(142,11,'subtotal',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(143,11,'grand_total',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(144,11,'total_paid',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(145,11,'total_due',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(146,11,'total_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(147,11,'total_qty_ordered',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(148,11,'total_canceled',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(149,11,'total_invoiced',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(150,11,'total_online_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(151,11,'total_offline_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(152,11,'adjustment_positive',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(153,11,'adjustment_negative',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(154,11,'base_tax_amount',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(155,11,'base_shipping_amount',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(156,11,'base_shipping_tax_amount',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(157,11,'base_discount_amount',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(158,11,'base_giftcert_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(159,11,'base_subtotal',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(160,11,'base_grand_total',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(161,11,'base_total_paid',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(162,11,'base_total_due',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(163,11,'base_total_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(164,11,'base_total_qty_ordered',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(165,11,'base_total_canceled',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(166,11,'base_total_invoiced',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(167,11,'base_total_online_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(168,11,'base_total_offline_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(169,11,'base_adjustment_positive',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(170,11,'base_adjustment_negative',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(171,11,'subtotal_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(172,11,'subtotal_canceled',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(173,11,'discount_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(174,11,'discount_canceled',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(175,11,'discount_invoiced',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(176,11,'subtotal_invoiced',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(177,11,'tax_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(178,11,'tax_canceled',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(179,11,'tax_invoiced',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(180,11,'shipping_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(181,11,'shipping_canceled',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(182,11,'shipping_invoiced',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(183,11,'base_subtotal_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(184,11,'base_subtotal_canceled',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(185,11,'base_discount_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(186,11,'base_discount_canceled',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(187,11,'base_discount_invoiced',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(188,11,'base_subtotal_invoiced',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(189,11,'base_tax_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(190,11,'base_tax_canceled',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(191,11,'base_tax_invoiced',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(192,11,'base_shipping_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(193,11,'base_shipping_canceled',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(194,11,'base_shipping_invoiced',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(195,11,'customer_id',NULL,'','static','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(196,11,'customer_group_id',NULL,'','int','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(197,11,'customer_email',NULL,'','varchar','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(198,11,'customer_prefix',NULL,'','varchar','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(199,11,'customer_firstname',NULL,'','varchar','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(200,11,'customer_middlename',NULL,'','varchar','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(201,11,'customer_lastname',NULL,'','varchar','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(202,11,'customer_suffix',NULL,'','varchar','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(203,11,'customer_note',NULL,'','text','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(204,11,'customer_note_notify',NULL,'','int','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(205,11,'customer_is_guest',NULL,'','int','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(206,11,'email_sent',NULL,'','int','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(207,11,'customer_taxvat',NULL,'','varchar','','','text','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(208,11,'customer_dob',NULL,'eav/entity_attribute_backend_datetime','datetime','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(209,12,'parent_id',NULL,'sales_entity/order_attribute_backend_child','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(210,12,'quote_address_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(211,12,'address_type',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(212,12,'customer_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(213,12,'customer_address_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(214,12,'email',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(215,12,'prefix',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(216,12,'firstname',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(217,12,'middlename',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(218,12,'lastname',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(219,12,'suffix',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(220,12,'company',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(221,12,'street',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(222,12,'city',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(223,12,'region',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(224,12,'region_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(225,12,'postcode',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(226,12,'country_id',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(227,12,'telephone',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(228,12,'fax',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(229,13,'parent_id',NULL,'sales_entity/order_attribute_backend_child','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(230,13,'quote_item_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(231,13,'product_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(232,13,'super_product_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(233,13,'parent_product_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(234,13,'is_virtual',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(235,13,'sku',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(236,13,'name',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(237,13,'description',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(238,13,'weight',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(239,13,'is_qty_decimal',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(240,13,'qty_ordered',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(241,13,'qty_backordered',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(242,13,'qty_invoiced',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(243,13,'qty_canceled',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(244,13,'qty_shipped',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(245,13,'qty_refunded',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(246,13,'original_price',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(247,13,'price',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(248,13,'cost',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(249,13,'discount_percent',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(250,13,'discount_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(251,13,'discount_invoiced',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(252,13,'tax_percent',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(253,13,'tax_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(254,13,'tax_invoiced',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(255,13,'row_total',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(256,13,'row_weight',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(257,13,'row_invoiced',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(258,13,'invoiced_total',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(259,13,'amount_refunded',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(260,13,'base_price',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(261,13,'base_original_price',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(262,13,'base_discount_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(263,13,'base_discount_invoiced',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(264,13,'base_tax_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(265,13,'base_tax_invoiced',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(266,13,'base_row_total',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(267,13,'base_row_invoiced',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(268,13,'base_invoiced_total',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(269,13,'base_amount_refunded',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(270,13,'applied_rule_ids',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(271,13,'additional_data',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(272,14,'parent_id',NULL,'sales_entity/order_attribute_backend_child','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(273,14,'quote_payment_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(274,14,'method',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(275,14,'additional_data',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(276,14,'last_trans_id',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(277,14,'po_number',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(278,14,'cc_type',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(279,14,'cc_number_enc',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(280,14,'cc_last4',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(281,14,'cc_owner',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(282,14,'cc_exp_month',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(283,14,'cc_exp_year',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(284,14,'cc_ss_issue',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(285,14,'cc_ss_start_month',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(286,14,'cc_ss_start_year',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(287,14,'cc_status',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(288,14,'cc_status_description',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(289,14,'cc_trans_id',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(290,14,'cc_approval',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(291,14,'cc_avs_status',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(292,14,'cc_cid_status',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(293,14,'cc_debug_request_body',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(294,14,'cc_debug_response_body',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(295,14,'cc_debug_response_serialized',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(296,14,'anet_trans_method',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(297,14,'echeck_routing_number',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(298,14,'echeck_bank_name',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(299,14,'echeck_account_type',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(300,14,'echeck_account_name',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(301,14,'echeck_type',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(302,14,'amount_ordered',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(303,14,'amount_authorized',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(304,14,'amount_paid',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(305,14,'amount_canceled',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(306,14,'amount_refunded',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(307,14,'shipping_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(308,14,'shipping_captured',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(309,14,'shipping_refunded',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(310,14,'base_amount_ordered',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(311,14,'base_amount_authorized',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(312,14,'base_amount_paid',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(313,14,'base_amount_canceled',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(314,14,'base_amount_refunded',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(315,14,'base_shipping_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(316,14,'base_shipping_captured',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(317,14,'base_shipping_refunded',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(318,15,'parent_id',NULL,'sales_entity/order_attribute_backend_child','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(319,15,'status',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(320,15,'comment',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(321,15,'is_customer_notified',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(322,16,'entity_id',NULL,'sales_entity/order_invoice_attribute_backend_parent','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(323,16,'state',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(324,16,'is_used_for_refund',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(325,16,'transaction_id',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(326,16,'order_id',NULL,'sales_entity/order_invoice_attribute_backend_order','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(327,16,'billing_address_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(328,16,'shipping_address_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(329,16,'global_currency_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(330,16,'base_currency_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(331,16,'store_currency_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(332,16,'order_currency_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(333,16,'store_to_base_rate',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(334,16,'store_to_order_rate',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(335,16,'base_to_global_rate',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(336,16,'base_to_order_rate',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(337,16,'subtotal',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(338,16,'discount_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(339,16,'tax_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(340,16,'shipping_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(341,16,'grand_total',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(342,16,'total_qty',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(343,16,'can_void_flag',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(344,16,'base_subtotal',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(345,16,'base_discount_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(346,16,'base_tax_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(347,16,'base_shipping_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(348,16,'base_grand_total',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(349,16,'email_sent',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(350,16,'store_id',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(351,17,'parent_id',NULL,'sales_entity/order_invoice_attribute_backend_child','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(352,17,'order_item_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(353,17,'product_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(354,17,'name',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(355,17,'description',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(356,17,'sku',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(357,17,'qty',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(358,17,'cost',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(359,17,'price',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(360,17,'discount_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(361,17,'tax_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(362,17,'row_total',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(363,17,'base_price',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(364,17,'base_discount_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(365,17,'base_tax_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(366,17,'base_row_total',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(367,17,'additional_data',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(368,18,'parent_id',NULL,'sales_entity/order_invoice_attribute_backend_child','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(369,18,'comment',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(370,18,'is_customer_notified',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(371,19,'entity_id',NULL,'sales_entity/order_shipment_attribute_backend_parent','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(372,19,'customer_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(373,19,'order_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(374,19,'shipment_status',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(375,19,'billing_address_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(376,19,'shipping_address_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(377,19,'total_qty',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(378,19,'total_weight',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(379,19,'email_sent',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(380,19,'store_id',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(381,20,'parent_id',NULL,'sales_entity/order_shipment_attribute_backend_child','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(382,20,'order_item_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(383,20,'product_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(384,20,'name',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(385,20,'description',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(386,20,'sku',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(387,20,'qty',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(388,20,'price',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(389,20,'weight',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(390,20,'row_total',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(391,20,'additional_data',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(392,21,'parent_id',NULL,'sales_entity/order_shipment_attribute_backend_child','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(393,21,'comment',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(394,21,'is_customer_notified',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(395,22,'parent_id',NULL,'sales_entity/order_shipment_attribute_backend_child','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(396,22,'order_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(397,22,'number',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(398,22,'carrier_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(399,22,'title',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(400,22,'description',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(401,22,'qty',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(402,22,'weight',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(403,23,'entity_id',NULL,'sales_entity/order_creditmemo_attribute_backend_parent','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(404,23,'state',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(405,23,'invoice_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(406,23,'transaction_id',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(407,23,'order_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(408,23,'creditmemo_status',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(409,23,'billing_address_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(410,23,'shipping_address_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(411,23,'global_currency_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(412,23,'base_currency_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(413,23,'store_currency_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(414,23,'order_currency_code',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(415,23,'store_to_base_rate',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(416,23,'store_to_order_rate',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(417,23,'base_to_global_rate',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(418,23,'base_to_order_rate',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(419,23,'subtotal',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(420,23,'discount_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(421,23,'tax_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(422,23,'shipping_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(423,23,'adjustment',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(424,23,'adjustment_positive',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(425,23,'adjustment_negative',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(426,23,'grand_total',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(427,23,'base_subtotal',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(428,23,'base_discount_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(429,23,'base_tax_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(430,23,'base_shipping_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(431,23,'base_adjustment',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(432,23,'base_adjustment_positive',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(433,23,'base_adjustment_negative',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(434,23,'base_grand_total',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(435,23,'email_sent',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(436,23,'store_id',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(437,24,'parent_id',NULL,'sales_entity/order_creditmemo_attribute_backend_child','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(438,24,'order_item_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(439,24,'product_id',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(440,24,'name',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(441,24,'description',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(442,24,'sku',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(443,24,'qty',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(444,24,'cost',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(445,24,'price',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(446,24,'discount_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(447,24,'tax_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(448,24,'row_total',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(449,24,'base_price',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(450,24,'base_discount_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(451,24,'base_tax_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(452,24,'base_row_total',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(453,24,'additional_data',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(454,25,'parent_id',NULL,'sales_entity/order_creditmemo_attribute_backend_child','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(455,25,'comment',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(456,25,'is_customer_notified',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(457,13,'product_type',NULL,'','varchar','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(458,11,'can_ship_partially',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(459,11,'can_ship_partially_item',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(460,11,'payment_authorization_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(461,11,'payment_authorization_expiration',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(462,11,'shipping_tax_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(463,11,'base_shipping_tax_refunded',NULL,'','static','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(465,11,'forced_do_shipment_with_invoice',NULL,'','int','','','text','','',NULL,'',1,1,1,0,'0',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(466,11,'paypal_ipn_customer_notified',NULL,'','int','','','text','','',NULL,'',1,0,1,0,'0',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(467,4,'enable_googlecheckout',NULL,'','int','','','select','','Is product available for purchase with Google Checkout',NULL,'eav/entity_attribute_source_boolean',1,1,0,0,'1',0,0,0,0,0,0,1,0,0,0,0,'',0,'',0),(468,11,'gift_message_id',NULL,'','int','','','text','','',NULL,'',1,0,0,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(469,4,'gift_message_available',NULL,'giftmessage/entity_attribute_backend_boolean_config','varchar','','','select','','Allow Gift Message',NULL,'giftmessage/entity_attribute_source_boolean_config',1,1,0,0,'2',0,0,0,0,0,0,1,0,0,0,0,'',0,'',0),(470,4,'price_type',NULL,'','int','','','','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,1,0,0,'bundle',0,'',0),(471,4,'sku_type',NULL,'','int','','','','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,0,'bundle',0,'',0),(472,4,'weight_type',NULL,'','int','','','','','',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,1,0,0,'bundle',0,'',0),(473,4,'price_view',NULL,'','int','','','select','','Price View',NULL,'bundle/product_attribute_source_price_view',1,1,1,0,'',0,0,0,0,0,0,1,0,1,0,0,'bundle',0,'',0),(474,4,'shipment_type',NULL,'','int','','','','','Shipment',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,1,0,0,'bundle',0,'',0),(475,4,'links_purchased_separately',NULL,'','int','','','','','Links can be purchased separately',NULL,'',1,0,1,0,'',0,0,0,0,0,0,1,0,0,0,0,'downloadable',0,'',0),(476,4,'samples_title',NULL,'','varchar','','','','','Samples title',NULL,'',0,0,1,0,'',0,0,0,0,0,0,1,0,0,0,0,'downloadable',0,'',0),(477,4,'links_title',NULL,'','varchar','','','','','Links title',NULL,'',0,0,1,0,'',0,0,0,0,0,0,1,0,0,0,0,'downloadable',0,'',0),(478,24,'base_weee_tax_applied_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(479,24,'base_weee_tax_applied_row_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(480,24,'weee_tax_applied_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(481,24,'weee_tax_applied_row_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(482,17,'base_weee_tax_applied_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(483,17,'base_weee_tax_applied_row_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(484,17,'weee_tax_applied_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(485,17,'weee_tax_applied_row_amount',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(486,17,'weee_tax_applied',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(487,24,'weee_tax_applied',NULL,'','text','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(488,24,'weee_tax_disposition',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(489,24,'weee_tax_row_disposition',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(490,24,'base_weee_tax_disposition',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(491,24,'base_weee_tax_row_disposition',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(492,17,'weee_tax_disposition',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(493,17,'weee_tax_row_disposition',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(494,17,'base_weee_tax_disposition',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0),(495,17,'base_weee_tax_row_disposition',NULL,'','decimal','','','text','','',NULL,'',1,1,1,0,'',0,0,0,0,0,0,1,0,0,0,1,'',0,'',0);
/*!40000 ALTER TABLE `eav_attribute` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_attribute_group`
--

DROP TABLE IF EXISTS `eav_attribute_group`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_attribute_group` (
  `attribute_group_id` smallint(5) unsigned NOT NULL auto_increment,
  `attribute_set_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_group_name` varchar(255) NOT NULL default '',
  `sort_order` smallint(6) NOT NULL default '0',
  `default_id` smallint(5) unsigned default '0',
  PRIMARY KEY  (`attribute_group_id`),
  UNIQUE KEY `attribute_set_id` (`attribute_set_id`,`attribute_group_name`),
  KEY `attribute_set_id_2` (`attribute_set_id`,`sort_order`),
  CONSTRAINT `FK_eav_attribute_group` FOREIGN KEY (`attribute_set_id`) REFERENCES `eav_attribute_set` (`attribute_set_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=32 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_attribute_group`
--

LOCK TABLES `eav_attribute_group` WRITE;
/*!40000 ALTER TABLE `eav_attribute_group` DISABLE KEYS */;
INSERT INTO `eav_attribute_group` VALUES (1,1,'General',1,1),(2,2,'General',1,1),(3,3,'General Information',10,1),(4,4,'General',1,1),(5,4,'Prices',2,0),(6,4,'Meta Information',3,0),(7,4,'Images',4,0),(8,4,'Design',6,0),(9,3,'Display Settings',20,0),(10,3,'Custom Design',30,0),(11,5,'General',1,1),(12,6,'General',1,1),(13,7,'General',1,1),(14,8,'General',1,1),(15,9,'General',1,1),(16,10,'General',1,1),(17,11,'General',1,1),(18,12,'General',1,1),(19,13,'General',1,1),(20,14,'General',1,1),(21,15,'General',1,1),(22,16,'General',1,1),(23,17,'General',1,1),(24,18,'General',1,1),(25,19,'General',1,1),(26,20,'General',1,1),(27,21,'General',1,1),(28,22,'General',1,1),(29,23,'General',1,1),(30,24,'General',1,1),(31,25,'General',1,1);
/*!40000 ALTER TABLE `eav_attribute_group` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_attribute_option`
--

DROP TABLE IF EXISTS `eav_attribute_option`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_attribute_option` (
  `option_id` int(10) unsigned NOT NULL auto_increment,
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `sort_order` smallint(5) unsigned NOT NULL default '0',
  PRIMARY KEY  (`option_id`),
  KEY `FK_ATTRIBUTE_OPTION_ATTRIBUTE` (`attribute_id`),
  CONSTRAINT `FK_ATTRIBUTE_OPTION_ATTRIBUTE` FOREIGN KEY (`attribute_id`) REFERENCES `eav_attribute` (`attribute_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Attributes option (for source model)';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_attribute_option`
--

LOCK TABLES `eav_attribute_option` WRITE;
/*!40000 ALTER TABLE `eav_attribute_option` DISABLE KEYS */;
/*!40000 ALTER TABLE `eav_attribute_option` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_attribute_option_value`
--

DROP TABLE IF EXISTS `eav_attribute_option_value`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_attribute_option_value` (
  `value_id` int(10) unsigned NOT NULL auto_increment,
  `option_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `value` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`value_id`),
  KEY `FK_ATTRIBUTE_OPTION_VALUE_OPTION` (`option_id`),
  KEY `FK_ATTRIBUTE_OPTION_VALUE_STORE` (`store_id`),
  CONSTRAINT `FK_ATTRIBUTE_OPTION_VALUE_OPTION` FOREIGN KEY (`option_id`) REFERENCES `eav_attribute_option` (`option_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_ATTRIBUTE_OPTION_VALUE_STORE` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Attribute option values per store';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_attribute_option_value`
--

LOCK TABLES `eav_attribute_option_value` WRITE;
/*!40000 ALTER TABLE `eav_attribute_option_value` DISABLE KEYS */;
/*!40000 ALTER TABLE `eav_attribute_option_value` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_attribute_set`
--

DROP TABLE IF EXISTS `eav_attribute_set`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_attribute_set` (
  `attribute_set_id` smallint(5) unsigned NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_set_name` varchar(255) character set utf8 collate utf8_swedish_ci NOT NULL default '',
  `sort_order` smallint(6) NOT NULL default '0',
  PRIMARY KEY  (`attribute_set_id`),
  UNIQUE KEY `entity_type_id` (`entity_type_id`,`attribute_set_name`),
  KEY `entity_type_id_2` (`entity_type_id`,`sort_order`),
  CONSTRAINT `FK_eav_attribute_set` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB AUTO_INCREMENT=26 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_attribute_set`
--

LOCK TABLES `eav_attribute_set` WRITE;
/*!40000 ALTER TABLE `eav_attribute_set` DISABLE KEYS */;
INSERT INTO `eav_attribute_set` VALUES (1,1,'Default',3),(2,2,'Default',3),(3,3,'Default',11),(4,4,'Default',11),(5,5,'Default',1),(6,6,'Default',1),(7,7,'Default',1),(8,8,'Default',1),(9,9,'Default',1),(10,10,'Default',1),(11,11,'Default',1),(12,12,'Default',1),(13,13,'Default',1),(14,14,'Default',1),(15,15,'Default',1),(16,16,'Default',1),(17,17,'Default',1),(18,18,'Default',1),(19,19,'Default',1),(20,20,'Default',1),(21,21,'Default',1),(22,22,'Default',1),(23,23,'Default',1),(24,24,'Default',1),(25,25,'Default',1);
/*!40000 ALTER TABLE `eav_attribute_set` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_entity`
--

DROP TABLE IF EXISTS `eav_entity`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_entity` (
  `entity_id` int(10) unsigned NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_set_id` smallint(5) unsigned NOT NULL default '0',
  `increment_id` varchar(50) NOT NULL default '',
  `parent_id` int(11) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `is_active` tinyint(1) unsigned NOT NULL default '1',
  PRIMARY KEY  (`entity_id`),
  KEY `FK_ENTITY_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_ENTITY_STORE` (`store_id`),
  CONSTRAINT `FK_eav_entity` FOREIGN KEY (`entity_type_id`) REFERENCES `eav_entity_type` (`entity_type_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT `FK_eav_entity_store` FOREIGN KEY (`store_id`) REFERENCES `core_store` (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Entityies';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_entity`
--

LOCK TABLES `eav_entity` WRITE;
/*!40000 ALTER TABLE `eav_entity` DISABLE KEYS */;
/*!40000 ALTER TABLE `eav_entity` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_entity_attribute`
--

DROP TABLE IF EXISTS `eav_entity_attribute`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_entity_attribute` (
  `entity_attribute_id` int(10) unsigned NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_set_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_group_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `sort_order` smallint(6) NOT NULL default '0',
  PRIMARY KEY  (`entity_attribute_id`),
  UNIQUE KEY `attribute_set_id_2` (`attribute_set_id`,`attribute_id`),
  UNIQUE KEY `attribute_group_id` (`attribute_group_id`,`attribute_id`),
  KEY `attribute_set_id_3` (`attribute_set_id`,`sort_order`),
  KEY `FK_EAV_ENTITY_ATTRIVUTE_ATTRIBUTE` (`attribute_id`)
) ENGINE=InnoDB AUTO_INCREMENT=496 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_entity_attribute`
--

LOCK TABLES `eav_entity_attribute` WRITE;
/*!40000 ALTER TABLE `eav_entity_attribute` DISABLE KEYS */;
INSERT INTO `eav_entity_attribute` VALUES (1,1,1,1,1,10),(2,1,1,1,2,20),(3,1,1,1,3,30),(4,1,1,1,4,37),(5,1,1,1,5,40),(6,1,1,1,6,43),(7,1,1,1,7,50),(8,1,1,1,8,53),(9,1,1,1,9,60),(10,1,1,1,10,70),(11,1,1,1,11,80),(12,1,1,1,12,81),(13,1,1,1,13,82),(14,1,1,1,14,83),(15,1,1,1,15,84),(16,1,1,1,16,85),(17,2,2,2,17,7),(18,2,2,2,18,10),(19,2,2,2,19,13),(20,2,2,2,20,20),(21,2,2,2,21,23),(22,2,2,2,22,30),(23,2,2,2,23,40),(24,2,2,2,24,50),(25,2,2,2,25,60),(26,2,2,2,26,70),(27,2,2,2,27,80),(28,2,2,2,28,90),(29,2,2,2,29,100),(30,2,2,2,30,110),(31,3,3,3,31,1),(32,3,3,3,32,2),(33,3,3,3,33,3),(34,3,3,3,34,4),(35,3,3,3,35,5),(36,3,3,3,36,6),(37,3,3,3,37,7),(38,3,3,3,38,8),(39,3,3,9,39,10),(40,3,3,9,40,20),(41,3,3,9,41,30),(42,3,3,3,42,12),(43,3,3,3,43,13),(44,3,3,3,44,14),(45,3,3,3,45,15),(46,3,3,3,46,16),(47,3,3,3,47,17),(48,3,3,10,48,10),(49,3,3,10,49,20),(50,3,3,10,50,30),(51,3,3,10,51,40),(52,3,3,10,52,50),(53,3,3,10,53,60),(54,3,3,3,54,24),(55,3,3,3,55,25),(56,4,4,4,56,1),(57,4,4,4,57,2),(58,4,4,4,58,3),(59,4,4,4,59,4),(60,4,4,5,60,1),(61,4,4,5,61,2),(62,4,4,5,62,3),(63,4,4,5,63,4),(64,4,4,5,64,5),(65,4,4,4,65,5),(66,4,4,4,66,6),(67,4,4,6,67,1),(68,4,4,6,68,2),(69,4,4,6,69,3),(70,4,4,7,70,1),(71,4,4,7,71,2),(72,4,4,7,72,3),(73,4,4,7,73,4),(74,4,4,4,74,7),(75,4,4,5,75,10),(76,4,4,4,76,8),(77,4,4,4,77,9),(78,4,4,4,78,10),(79,4,4,7,79,5),(80,4,4,4,80,11),(81,4,4,5,81,7),(82,4,4,4,82,12),(83,4,4,4,83,13),(84,4,4,5,84,8),(85,4,4,4,85,14),(86,4,4,8,86,1),(87,4,4,8,87,2),(88,4,4,8,88,3),(89,4,4,8,89,4),(90,4,4,8,90,5),(91,4,4,4,91,15),(92,4,4,8,92,6),(93,4,4,4,93,16),(94,4,4,4,94,17),(95,4,4,4,95,18),(96,4,4,4,96,19),(97,4,4,4,97,20),(98,3,3,9,98,40),(99,3,3,9,99,50),(100,4,4,4,100,21),(101,4,4,4,101,22),(102,11,11,17,102,1),(103,11,11,17,103,2),(104,11,11,17,104,3),(105,11,11,17,105,4),(106,11,11,17,106,5),(107,11,11,17,107,6),(108,11,11,17,108,7),(109,11,11,17,109,8),(110,11,11,17,110,9),(111,11,11,17,111,10),(112,11,11,17,112,11),(113,11,11,17,113,12),(114,11,11,17,114,13),(115,11,11,17,115,14),(116,11,11,17,116,15),(117,11,11,17,117,16),(118,11,11,17,118,17),(119,11,11,17,119,18),(120,11,11,17,120,19),(121,11,11,17,121,20),(122,11,11,17,122,21),(123,11,11,17,123,22),(124,11,11,17,124,23),(125,11,11,17,125,24),(126,11,11,17,126,25),(127,11,11,17,127,26),(128,11,11,17,128,27),(129,11,11,17,129,28),(130,11,11,17,130,29),(131,11,11,17,131,30),(132,11,11,17,132,31),(133,11,11,17,133,32),(134,11,11,17,134,33),(135,11,11,17,135,34),(136,11,11,17,136,35),(137,11,11,17,137,36),(138,11,11,17,138,37),(139,11,11,17,139,38),(140,11,11,17,140,39),(141,11,11,17,141,40),(142,11,11,17,142,41),(143,11,11,17,143,42),(144,11,11,17,144,43),(145,11,11,17,145,44),(146,11,11,17,146,45),(147,11,11,17,147,46),(148,11,11,17,148,47),(149,11,11,17,149,48),(150,11,11,17,150,49),(151,11,11,17,151,50),(152,11,11,17,152,51),(153,11,11,17,153,52),(154,11,11,17,154,53),(155,11,11,17,155,54),(156,11,11,17,156,55),(157,11,11,17,157,56),(158,11,11,17,158,57),(159,11,11,17,159,58),(160,11,11,17,160,59),(161,11,11,17,161,60),(162,11,11,17,162,61),(163,11,11,17,163,62),(164,11,11,17,164,63),(165,11,11,17,165,64),(166,11,11,17,166,65),(167,11,11,17,167,66),(168,11,11,17,168,67),(169,11,11,17,169,68),(170,11,11,17,170,69),(171,11,11,17,171,70),(172,11,11,17,172,71),(173,11,11,17,173,72),(174,11,11,17,174,73),(175,11,11,17,175,74),(176,11,11,17,176,75),(177,11,11,17,177,76),(178,11,11,17,178,77),(179,11,11,17,179,78),(180,11,11,17,180,79),(181,11,11,17,181,80),(182,11,11,17,182,81),(183,11,11,17,183,82),(184,11,11,17,184,83),(185,11,11,17,185,84),(186,11,11,17,186,85),(187,11,11,17,187,86),(188,11,11,17,188,87),(189,11,11,17,189,88),(190,11,11,17,190,89),(191,11,11,17,191,90),(192,11,11,17,192,91),(193,11,11,17,193,92),(194,11,11,17,194,93),(195,11,11,17,195,94),(196,11,11,17,196,95),(197,11,11,17,197,96),(198,11,11,17,198,97),(199,11,11,17,199,98),(200,11,11,17,200,99),(201,11,11,17,201,100),(202,11,11,17,202,101),(203,11,11,17,203,102),(204,11,11,17,204,103),(205,11,11,17,205,104),(206,11,11,17,206,105),(207,11,11,17,207,106),(208,11,11,17,208,107),(209,12,12,18,209,1),(210,12,12,18,210,2),(211,12,12,18,211,3),(212,12,12,18,212,4),(213,12,12,18,213,5),(214,12,12,18,214,6),(215,12,12,18,215,7),(216,12,12,18,216,8),(217,12,12,18,217,9),(218,12,12,18,218,10),(219,12,12,18,219,11),(220,12,12,18,220,12),(221,12,12,18,221,13),(222,12,12,18,222,14),(223,12,12,18,223,15),(224,12,12,18,224,16),(225,12,12,18,225,17),(226,12,12,18,226,18),(227,12,12,18,227,19),(228,12,12,18,228,20),(229,13,13,19,229,1),(230,13,13,19,230,2),(231,13,13,19,231,3),(232,13,13,19,232,4),(233,13,13,19,233,5),(234,13,13,19,234,6),(235,13,13,19,235,7),(236,13,13,19,236,8),(237,13,13,19,237,9),(238,13,13,19,238,10),(239,13,13,19,239,11),(240,13,13,19,240,12),(241,13,13,19,241,13),(242,13,13,19,242,14),(243,13,13,19,243,15),(244,13,13,19,244,16),(245,13,13,19,245,17),(246,13,13,19,246,18),(247,13,13,19,247,19),(248,13,13,19,248,20),(249,13,13,19,249,21),(250,13,13,19,250,22),(251,13,13,19,251,23),(252,13,13,19,252,24),(253,13,13,19,253,25),(254,13,13,19,254,26),(255,13,13,19,255,27),(256,13,13,19,256,28),(257,13,13,19,257,29),(258,13,13,19,258,30),(259,13,13,19,259,31),(260,13,13,19,260,32),(261,13,13,19,261,33),(262,13,13,19,262,34),(263,13,13,19,263,35),(264,13,13,19,264,36),(265,13,13,19,265,37),(266,13,13,19,266,38),(267,13,13,19,267,39),(268,13,13,19,268,40),(269,13,13,19,269,41),(270,13,13,19,270,42),(271,13,13,19,271,43),(272,14,14,20,272,1),(273,14,14,20,273,2),(274,14,14,20,274,3),(275,14,14,20,275,4),(276,14,14,20,276,5),(277,14,14,20,277,6),(278,14,14,20,278,7),(279,14,14,20,279,8),(280,14,14,20,280,9),(281,14,14,20,281,10),(282,14,14,20,282,11),(283,14,14,20,283,12),(284,14,14,20,284,13),(285,14,14,20,285,14),(286,14,14,20,286,15),(287,14,14,20,287,16),(288,14,14,20,288,17),(289,14,14,20,289,18),(290,14,14,20,290,19),(291,14,14,20,291,20),(292,14,14,20,292,21),(293,14,14,20,293,22),(294,14,14,20,294,23),(295,14,14,20,295,24),(296,14,14,20,296,25),(297,14,14,20,297,26),(298,14,14,20,298,27),(299,14,14,20,299,28),(300,14,14,20,300,29),(301,14,14,20,301,30),(302,14,14,20,302,31),(303,14,14,20,303,32),(304,14,14,20,304,33),(305,14,14,20,305,34),(306,14,14,20,306,35),(307,14,14,20,307,36),(308,14,14,20,308,37),(309,14,14,20,309,38),(310,14,14,20,310,39),(311,14,14,20,311,40),(312,14,14,20,312,41),(313,14,14,20,313,42),(314,14,14,20,314,43),(315,14,14,20,315,44),(316,14,14,20,316,45),(317,14,14,20,317,46),(318,15,15,21,318,1),(319,15,15,21,319,2),(320,15,15,21,320,3),(321,15,15,21,321,4),(322,16,16,22,322,1),(323,16,16,22,323,2),(324,16,16,22,324,3),(325,16,16,22,325,4),(326,16,16,22,326,5),(327,16,16,22,327,6),(328,16,16,22,328,7),(329,16,16,22,329,8),(330,16,16,22,330,9),(331,16,16,22,331,10),(332,16,16,22,332,11),(333,16,16,22,333,12),(334,16,16,22,334,13),(335,16,16,22,335,14),(336,16,16,22,336,15),(337,16,16,22,337,16),(338,16,16,22,338,17),(339,16,16,22,339,18),(340,16,16,22,340,19),(341,16,16,22,341,20),(342,16,16,22,342,21),(343,16,16,22,343,22),(344,16,16,22,344,23),(345,16,16,22,345,24),(346,16,16,22,346,25),(347,16,16,22,347,26),(348,16,16,22,348,27),(349,16,16,22,349,28),(350,16,16,22,350,29),(351,17,17,23,351,1),(352,17,17,23,352,2),(353,17,17,23,353,3),(354,17,17,23,354,4),(355,17,17,23,355,5),(356,17,17,23,356,6),(357,17,17,23,357,7),(358,17,17,23,358,8),(359,17,17,23,359,9),(360,17,17,23,360,10),(361,17,17,23,361,11),(362,17,17,23,362,12),(363,17,17,23,363,13),(364,17,17,23,364,14),(365,17,17,23,365,15),(366,17,17,23,366,16),(367,17,17,23,367,17),(368,18,18,24,368,1),(369,18,18,24,369,2),(370,18,18,24,370,3),(371,19,19,25,371,1),(372,19,19,25,372,2),(373,19,19,25,373,3),(374,19,19,25,374,4),(375,19,19,25,375,5),(376,19,19,25,376,6),(377,19,19,25,377,7),(378,19,19,25,378,8),(379,19,19,25,379,9),(380,19,19,25,380,10),(381,20,20,26,381,1),(382,20,20,26,382,2),(383,20,20,26,383,3),(384,20,20,26,384,4),(385,20,20,26,385,5),(386,20,20,26,386,6),(387,20,20,26,387,7),(388,20,20,26,388,8),(389,20,20,26,389,9),(390,20,20,26,390,10),(391,20,20,26,391,11),(392,21,21,27,392,1),(393,21,21,27,393,2),(394,21,21,27,394,3),(395,22,22,28,395,1),(396,22,22,28,396,2),(397,22,22,28,397,3),(398,22,22,28,398,4),(399,22,22,28,399,5),(400,22,22,28,400,6),(401,22,22,28,401,7),(402,22,22,28,402,8),(403,23,23,29,403,1),(404,23,23,29,404,2),(405,23,23,29,405,3),(406,23,23,29,406,4),(407,23,23,29,407,5),(408,23,23,29,408,6),(409,23,23,29,409,7),(410,23,23,29,410,8),(411,23,23,29,411,9),(412,23,23,29,412,10),(413,23,23,29,413,11),(414,23,23,29,414,12),(415,23,23,29,415,13),(416,23,23,29,416,14),(417,23,23,29,417,15),(418,23,23,29,418,16),(419,23,23,29,419,17),(420,23,23,29,420,18),(421,23,23,29,421,19),(422,23,23,29,422,20),(423,23,23,29,423,21),(424,23,23,29,424,22),(425,23,23,29,425,23),(426,23,23,29,426,24),(427,23,23,29,427,25),(428,23,23,29,428,26),(429,23,23,29,429,27),(430,23,23,29,430,28),(431,23,23,29,431,29),(432,23,23,29,432,30),(433,23,23,29,433,31),(434,23,23,29,434,32),(435,23,23,29,435,33),(436,23,23,29,436,34),(437,24,24,30,437,1),(438,24,24,30,438,2),(439,24,24,30,439,3),(440,24,24,30,440,4),(441,24,24,30,441,5),(442,24,24,30,442,6),(443,24,24,30,443,7),(444,24,24,30,444,8),(445,24,24,30,445,9),(446,24,24,30,446,10),(447,24,24,30,447,11),(448,24,24,30,448,12),(449,24,24,30,449,13),(450,24,24,30,450,14),(451,24,24,30,451,15),(452,24,24,30,452,16),(453,24,24,30,453,17),(454,25,25,31,454,1),(455,25,25,31,455,2),(456,25,25,31,456,3),(457,13,13,19,457,44),(458,11,11,17,458,108),(459,11,11,17,459,109),(460,11,11,17,460,110),(461,11,11,17,461,111),(462,11,11,17,462,112),(463,11,11,17,463,113),(464,11,11,17,464,114),(465,11,11,17,465,115),(466,11,11,17,466,116),(467,4,4,5,467,23),(468,11,11,17,468,117),(469,4,4,4,469,23),(470,4,4,4,470,24),(471,4,4,4,471,25),(472,4,4,4,472,26),(473,4,4,5,473,24),(474,4,4,4,474,27),(475,4,4,4,475,28),(476,4,4,4,476,29),(477,4,4,4,477,30),(478,24,24,30,478,18),(479,24,24,30,479,19),(480,24,24,30,480,20),(481,24,24,30,481,21),(482,17,17,23,482,18),(483,17,17,23,483,19),(484,17,17,23,484,20),(485,17,17,23,485,21),(486,17,17,23,486,22),(487,24,24,30,487,22),(488,24,24,30,488,23),(489,24,24,30,489,24),(490,24,24,30,490,25),(491,24,24,30,491,26),(492,17,17,23,492,23),(493,17,17,23,493,24),(494,17,17,23,494,25),(495,17,17,23,495,26);
/*!40000 ALTER TABLE `eav_entity_attribute` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_entity_datetime`
--

DROP TABLE IF EXISTS `eav_entity_datetime`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_entity_datetime` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`value_id`),
  KEY `FK_ATTRIBUTE_DATETIME_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_ATTRIBUTE_DATETIME_ATTRIBUTE` (`attribute_id`),
  KEY `FK_ATTRIBUTE_DATETIME_STORE` (`store_id`),
  KEY `FK_ATTRIBUTE_DATETIME_ENTITY` (`entity_id`),
  KEY `value_by_attribute` (`attribute_id`,`value`),
  KEY `value_by_entity_type` (`entity_type_id`,`value`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Datetime values of attributes';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_entity_datetime`
--

LOCK TABLES `eav_entity_datetime` WRITE;
/*!40000 ALTER TABLE `eav_entity_datetime` DISABLE KEYS */;
/*!40000 ALTER TABLE `eav_entity_datetime` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_entity_decimal`
--

DROP TABLE IF EXISTS `eav_entity_decimal`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_entity_decimal` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` decimal(12,4) NOT NULL default '0.0000',
  PRIMARY KEY  (`value_id`),
  KEY `FK_ATTRIBUTE_DECIMAL_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_ATTRIBUTE_DECIMAL_ATTRIBUTE` (`attribute_id`),
  KEY `FK_ATTRIBUTE_DECIMAL_STORE` (`store_id`),
  KEY `FK_ATTRIBUTE_DECIMAL_ENTITY` (`entity_id`),
  KEY `value_by_attribute` (`attribute_id`,`value`),
  KEY `value_by_entity_type` (`entity_type_id`,`value`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Decimal values of attributes';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_entity_decimal`
--

LOCK TABLES `eav_entity_decimal` WRITE;
/*!40000 ALTER TABLE `eav_entity_decimal` DISABLE KEYS */;
/*!40000 ALTER TABLE `eav_entity_decimal` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_entity_int`
--

DROP TABLE IF EXISTS `eav_entity_int`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_entity_int` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` int(11) NOT NULL default '0',
  PRIMARY KEY  (`value_id`),
  KEY `FK_ATTRIBUTE_INT_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_ATTRIBUTE_INT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_ATTRIBUTE_INT_STORE` (`store_id`),
  KEY `FK_ATTRIBUTE_INT_ENTITY` (`entity_id`),
  KEY `value_by_attribute` (`attribute_id`,`value`),
  KEY `value_by_entity_type` (`entity_type_id`,`value`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Integer values of attributes';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_entity_int`
--

LOCK TABLES `eav_entity_int` WRITE;
/*!40000 ALTER TABLE `eav_entity_int` DISABLE KEYS */;
/*!40000 ALTER TABLE `eav_entity_int` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_entity_store`
--

DROP TABLE IF EXISTS `eav_entity_store`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_entity_store` (
  `entity_store_id` int(10) unsigned NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `increment_prefix` varchar(20) NOT NULL default '',
  `increment_last_id` varchar(50) NOT NULL default '',
  PRIMARY KEY  (`entity_store_id`),
  KEY `FK_eav_entity_store_entity_type` (`entity_type_id`),
  KEY `FK_eav_entity_store_store` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_entity_store`
--

LOCK TABLES `eav_entity_store` WRITE;
/*!40000 ALTER TABLE `eav_entity_store` DISABLE KEYS */;
/*!40000 ALTER TABLE `eav_entity_store` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_entity_text`
--

DROP TABLE IF EXISTS `eav_entity_text`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_entity_text` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` text NOT NULL,
  PRIMARY KEY  (`value_id`),
  KEY `FK_ATTRIBUTE_TEXT_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_ATTRIBUTE_TEXT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_ATTRIBUTE_TEXT_STORE` (`store_id`),
  KEY `FK_ATTRIBUTE_TEXT_ENTITY` (`entity_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Text values of attributes';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_entity_text`
--

LOCK TABLES `eav_entity_text` WRITE;
/*!40000 ALTER TABLE `eav_entity_text` DISABLE KEYS */;
/*!40000 ALTER TABLE `eav_entity_text` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_entity_type`
--

DROP TABLE IF EXISTS `eav_entity_type`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_entity_type` (
  `entity_type_id` smallint(5) unsigned NOT NULL auto_increment,
  `entity_type_code` varchar(50) NOT NULL default '',
  `entity_model` varchar(255) NOT NULL,
  `attribute_model` varchar(255) NOT NULL,
  `entity_table` varchar(255) NOT NULL default '',
  `value_table_prefix` varchar(255) NOT NULL default '',
  `entity_id_field` varchar(255) NOT NULL default '',
  `is_data_sharing` tinyint(4) unsigned NOT NULL default '1',
  `data_sharing_key` varchar(100) default 'default',
  `default_attribute_set_id` smallint(5) unsigned NOT NULL default '0',
  `increment_model` varchar(255) NOT NULL default '',
  `increment_per_store` tinyint(1) unsigned NOT NULL default '0',
  `increment_pad_length` tinyint(8) unsigned NOT NULL default '8',
  `increment_pad_char` char(1) NOT NULL default '0',
  PRIMARY KEY  (`entity_type_id`),
  KEY `entity_name` (`entity_type_code`)
) ENGINE=InnoDB AUTO_INCREMENT=26 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_entity_type`
--

LOCK TABLES `eav_entity_type` WRITE;
/*!40000 ALTER TABLE `eav_entity_type` DISABLE KEYS */;
INSERT INTO `eav_entity_type` VALUES (1,'customer','customer/customer','','customer/entity','','',1,'default',1,'eav/entity_increment_numeric',0,8,'0'),(2,'customer_address','customer/customer_address','','customer/address_entity','','',1,'default',2,'',0,8,'0'),(3,'catalog_category','catalog/category','catalog/resource_eav_attribute','catalog/category','','',1,'default',3,'',0,8,'0'),(4,'catalog_product','catalog/product','catalog/resource_eav_attribute','catalog/product','','',1,'default',4,'',0,8,'0'),(5,'quote','sales/quote','','sales/quote','','',1,'default',5,'',0,8,'0'),(6,'quote_item','sales/quote_item','','sales/quote_item','','',1,'default',6,'',0,8,'0'),(7,'quote_address','sales/quote_address','','sales/quote_address','','',1,'default',7,'',0,8,'0'),(8,'quote_address_item','sales/quote_address_item','','sales/quote_entity','','',1,'default',8,'',0,8,'0'),(9,'quote_address_rate','sales/quote_address_rate','','sales/quote_entity','','',1,'default',9,'',0,8,'0'),(10,'quote_payment','sales/quote_payment','','sales/quote_entity','','',1,'default',10,'',0,8,'0'),(11,'order','sales/order','','sales/order','','',1,'default',11,'eav/entity_increment_numeric',1,8,'0'),(12,'order_address','sales/order_address','','sales/order_entity','','',1,'default',12,'',0,8,'0'),(13,'order_item','sales/order_item','','sales/order_entity','','',1,'default',13,'',0,8,'0'),(14,'order_payment','sales/order_payment','','sales/order_entity','','',1,'default',14,'',0,8,'0'),(15,'order_status_history','sales/order_status_history','','sales/order_entity','','',1,'default',15,'',0,8,'0'),(16,'invoice','sales/order_invoice','','sales/order_entity','','',1,'default',16,'eav/entity_increment_numeric',1,8,'0'),(17,'invoice_item','sales/order_invoice_item','','sales/order_entity','','',1,'default',17,'',0,8,'0'),(18,'invoice_comment','sales/order_invoice_comment','','sales/order_entity','','',1,'default',18,'',0,8,'0'),(19,'shipment','sales/order_shipment','','sales/order_entity','','',1,'default',19,'eav/entity_increment_numeric',1,8,'0'),(20,'shipment_item','sales/order_shipment_item','','sales/order_entity','','',1,'default',20,'',0,8,'0'),(21,'shipment_comment','sales/order_shipment_comment','','sales/order_entity','','',1,'default',21,'',0,8,'0'),(22,'shipment_track','sales/order_shipment_track','','sales/order_entity','','',1,'default',22,'',0,8,'0'),(23,'creditmemo','sales/order_creditmemo','','sales/order_entity','','',1,'default',23,'eav/entity_increment_numeric',1,8,'0'),(24,'creditmemo_item','sales/order_creditmemo_item','','sales/order_entity','','',1,'default',24,'',0,8,'0'),(25,'creditmemo_comment','sales/order_creditmemo_comment','','sales/order_entity','','',1,'default',25,'',0,8,'0');
/*!40000 ALTER TABLE `eav_entity_type` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `eav_entity_varchar`
--

DROP TABLE IF EXISTS `eav_entity_varchar`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `eav_entity_varchar` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`value_id`),
  KEY `FK_ATTRIBUTE_VARCHAR_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_ATTRIBUTE_VARCHAR_ATTRIBUTE` (`attribute_id`),
  KEY `FK_ATTRIBUTE_VARCHAR_STORE` (`store_id`),
  KEY `FK_ATTRIBUTE_VARCHAR_ENTITY` (`entity_id`),
  KEY `value_by_attribute` (`attribute_id`,`value`),
  KEY `value_by_entity_type` (`entity_type_id`,`value`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Varchar values of attributes';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `eav_entity_varchar`
--

LOCK TABLES `eav_entity_varchar` WRITE;
/*!40000 ALTER TABLE `eav_entity_varchar` DISABLE KEYS */;
/*!40000 ALTER TABLE `eav_entity_varchar` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `gift_message`
--

DROP TABLE IF EXISTS `gift_message`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `gift_message` (
  `gift_message_id` int(7) unsigned NOT NULL auto_increment,
  `customer_id` int(7) unsigned NOT NULL default '0',
  `sender` varchar(255) NOT NULL default '',
  `recipient` varchar(255) NOT NULL default '',
  `message` text NOT NULL,
  PRIMARY KEY  (`gift_message_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `gift_message`
--

LOCK TABLES `gift_message` WRITE;
/*!40000 ALTER TABLE `gift_message` DISABLE KEYS */;
/*!40000 ALTER TABLE `gift_message` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `googlebase_attributes`
--

DROP TABLE IF EXISTS `googlebase_attributes`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `googlebase_attributes` (
  `id` int(10) unsigned NOT NULL auto_increment,
  `attribute_id` smallint(5) unsigned NOT NULL,
  `gbase_attribute` varchar(255) NOT NULL,
  `type_id` int(10) unsigned NOT NULL,
  PRIMARY KEY  (`id`),
  KEY `GOOGLEBASE_ATTRIBUTES_ATTRIBUTE_ID` (`attribute_id`),
  KEY `GOOGLEBASE_ATTRIBUTES_TYPE_ID` (`type_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Google Base Attributes link Product Attributes';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `googlebase_attributes`
--

LOCK TABLES `googlebase_attributes` WRITE;
/*!40000 ALTER TABLE `googlebase_attributes` DISABLE KEYS */;
/*!40000 ALTER TABLE `googlebase_attributes` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `googlebase_items`
--

DROP TABLE IF EXISTS `googlebase_items`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `googlebase_items` (
  `item_id` int(10) unsigned NOT NULL auto_increment,
  `type_id` int(10) unsigned NOT NULL default '0',
  `product_id` int(10) unsigned NOT NULL,
  `gbase_item_id` varchar(255) NOT NULL,
  `store_id` smallint(5) unsigned NOT NULL,
  `published` datetime NOT NULL default '0000-00-00 00:00:00',
  `expires` datetime NOT NULL default '0000-00-00 00:00:00',
  `impr` smallint(5) unsigned NOT NULL default '0',
  `clicks` smallint(5) unsigned NOT NULL default '0',
  `views` smallint(5) unsigned NOT NULL default '0',
  `is_hidden` tinyint(4) NOT NULL default '0',
  PRIMARY KEY  (`item_id`),
  KEY `GOOGLEBASE_ITEMS_PRODUCT_ID` (`product_id`),
  KEY `GOOGLEBASE_ITEMS_STORE_ID` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Google Base Items Products';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `googlebase_items`
--

LOCK TABLES `googlebase_items` WRITE;
/*!40000 ALTER TABLE `googlebase_items` DISABLE KEYS */;
/*!40000 ALTER TABLE `googlebase_items` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `googlebase_types`
--

DROP TABLE IF EXISTS `googlebase_types`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `googlebase_types` (
  `type_id` int(10) unsigned NOT NULL auto_increment,
  `attribute_set_id` smallint(5) unsigned NOT NULL,
  `gbase_itemtype` varchar(255) NOT NULL,
  `target_country` varchar(2) NOT NULL default 'US',
  PRIMARY KEY  (`type_id`),
  KEY `GOOGLEBASE_TYPES_ATTRIBUTE_SET_ID` (`attribute_set_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Google Base Item Types link Attribute Sets';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `googlebase_types`
--

LOCK TABLES `googlebase_types` WRITE;
/*!40000 ALTER TABLE `googlebase_types` DISABLE KEYS */;
/*!40000 ALTER TABLE `googlebase_types` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `googlecheckout_api_debug`
--

DROP TABLE IF EXISTS `googlecheckout_api_debug`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `googlecheckout_api_debug` (
  `debug_id` int(10) unsigned NOT NULL auto_increment,
  `dir` enum('in','out') default NULL,
  `url` varchar(255) default NULL,
  `request_body` text,
  `response_body` text,
  PRIMARY KEY  (`debug_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `googlecheckout_api_debug`
--

LOCK TABLES `googlecheckout_api_debug` WRITE;
/*!40000 ALTER TABLE `googlecheckout_api_debug` DISABLE KEYS */;
/*!40000 ALTER TABLE `googlecheckout_api_debug` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `googleoptimizer_code`
--

DROP TABLE IF EXISTS `googleoptimizer_code`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `googleoptimizer_code` (
  `code_id` int(10) unsigned NOT NULL auto_increment,
  `entity_id` int(10) unsigned NOT NULL,
  `entity_type` varchar(50) NOT NULL default '',
  `store_id` smallint(5) unsigned NOT NULL,
  `control_script` text,
  `tracking_script` text,
  `conversion_script` text,
  `conversion_page` varchar(255) NOT NULL default '',
  `additional_data` text,
  PRIMARY KEY  (`code_id`),
  KEY `GOOGLEOPTIMIZER_CODE_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `googleoptimizer_code`
--

LOCK TABLES `googleoptimizer_code` WRITE;
/*!40000 ALTER TABLE `googleoptimizer_code` DISABLE KEYS */;
/*!40000 ALTER TABLE `googleoptimizer_code` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `log_customer`
--

DROP TABLE IF EXISTS `log_customer`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `log_customer` (
  `log_id` int(10) unsigned NOT NULL auto_increment,
  `visitor_id` bigint(20) unsigned default NULL,
  `customer_id` int(11) NOT NULL default '0',
  `login_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `logout_at` datetime default NULL,
  `store_id` smallint(5) unsigned NOT NULL,
  PRIMARY KEY  (`log_id`),
  KEY `IDX_VISITOR` (`visitor_id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT='Customers log information';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `log_customer`
--

LOCK TABLES `log_customer` WRITE;
/*!40000 ALTER TABLE `log_customer` DISABLE KEYS */;
/*!40000 ALTER TABLE `log_customer` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `log_quote`
--

DROP TABLE IF EXISTS `log_quote`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `log_quote` (
  `quote_id` int(10) unsigned NOT NULL default '0',
  `visitor_id` bigint(20) unsigned default NULL,
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `deleted_at` datetime default NULL,
  PRIMARY KEY  (`quote_id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT='Quote log data';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `log_quote`
--

LOCK TABLES `log_quote` WRITE;
/*!40000 ALTER TABLE `log_quote` DISABLE KEYS */;
INSERT INTO `log_quote` VALUES (1,3,'2009-06-23 14:35:58',NULL);
/*!40000 ALTER TABLE `log_quote` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `log_summary`
--

DROP TABLE IF EXISTS `log_summary`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `log_summary` (
  `summary_id` bigint(20) unsigned NOT NULL auto_increment,
  `store_id` smallint(5) unsigned NOT NULL,
  `type_id` smallint(5) unsigned default NULL,
  `visitor_count` int(11) NOT NULL default '0',
  `customer_count` int(11) NOT NULL default '0',
  `add_date` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`summary_id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT='Summary log information';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `log_summary`
--

LOCK TABLES `log_summary` WRITE;
/*!40000 ALTER TABLE `log_summary` DISABLE KEYS */;
/*!40000 ALTER TABLE `log_summary` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `log_summary_type`
--

DROP TABLE IF EXISTS `log_summary_type`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `log_summary_type` (
  `type_id` smallint(5) unsigned NOT NULL auto_increment,
  `type_code` varchar(64) NOT NULL default '',
  `period` smallint(5) unsigned NOT NULL default '0',
  `period_type` enum('MINUTE','HOUR','DAY','WEEK','MONTH') NOT NULL default 'MINUTE',
  PRIMARY KEY  (`type_id`)
) ENGINE=MyISAM AUTO_INCREMENT=3 DEFAULT CHARSET=utf8 COMMENT='Type of summary information';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `log_summary_type`
--

LOCK TABLES `log_summary_type` WRITE;
/*!40000 ALTER TABLE `log_summary_type` DISABLE KEYS */;
INSERT INTO `log_summary_type` VALUES (1,'hour',1,'HOUR'),(2,'day',1,'DAY');
/*!40000 ALTER TABLE `log_summary_type` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `log_url`
--

DROP TABLE IF EXISTS `log_url`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `log_url` (
  `url_id` bigint(20) unsigned NOT NULL default '0',
  `visitor_id` bigint(20) unsigned default NULL,
  `visit_time` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`url_id`),
  KEY `IDX_VISITOR` (`visitor_id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT='URL visiting history';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `log_url`
--

LOCK TABLES `log_url` WRITE;
/*!40000 ALTER TABLE `log_url` DISABLE KEYS */;
INSERT INTO `log_url` VALUES (1,1,'2009-06-22 12:51:07'),(2,1,'2009-06-22 12:59:17'),(3,1,'2009-06-22 13:00:37'),(4,1,'2009-06-22 13:00:54'),(5,1,'2009-06-22 13:00:59'),(6,1,'2009-06-22 13:01:26'),(7,1,'2009-06-22 13:06:25'),(8,1,'2009-06-22 13:06:25'),(9,1,'2009-06-22 13:07:32'),(10,1,'2009-06-22 13:08:29'),(11,1,'2009-06-22 13:18:45'),(12,1,'2009-06-22 13:19:04'),(13,1,'2009-06-22 13:19:44'),(14,1,'2009-06-22 13:19:54'),(15,1,'2009-06-22 13:20:41'),(16,1,'2009-06-22 13:20:56'),(17,1,'2009-06-22 13:21:43'),(18,1,'2009-06-22 13:21:50'),(19,1,'2009-06-22 13:21:57'),(20,1,'2009-06-22 13:22:11'),(21,1,'2009-06-22 13:22:16'),(22,1,'2009-06-22 13:25:09'),(23,1,'2009-06-22 13:25:21'),(24,1,'2009-06-22 13:26:15'),(25,2,'2009-06-23 09:16:48'),(26,2,'2009-06-23 09:17:20'),(27,3,'2009-06-23 14:13:54'),(28,4,'2009-06-23 14:16:20'),(29,3,'2009-06-23 14:18:59'),(30,3,'2009-06-23 14:20:02'),(31,3,'2009-06-23 14:21:26'),(32,3,'2009-06-23 14:21:37'),(33,3,'2009-06-23 14:21:38'),(34,3,'2009-06-23 14:31:19'),(35,3,'2009-06-23 14:31:24'),(36,3,'2009-06-23 14:31:29'),(37,3,'2009-06-23 14:32:01'),(38,3,'2009-06-23 14:33:51'),(39,3,'2009-06-23 14:34:06'),(40,3,'2009-06-23 14:34:22'),(41,3,'2009-06-23 14:35:58'),(42,3,'2009-06-23 14:36:00'),(43,3,'2009-06-23 14:37:13'),(44,3,'2009-06-23 14:37:41'),(45,3,'2009-06-23 14:38:36'),(46,5,'2009-06-23 15:46:11'),(47,6,'2009-06-23 20:26:07'),(48,6,'2009-06-23 20:26:36'),(49,6,'2009-06-23 20:27:02'),(50,7,'2009-06-24 12:08:53'),(51,11,'2009-06-24 12:56:46'),(52,11,'2009-06-24 12:56:53'),(53,11,'2009-06-24 13:02:57'),(54,11,'2009-06-24 13:04:02'),(55,12,'2009-06-24 16:38:14'),(56,12,'2009-06-24 16:39:43'),(57,12,'2009-06-24 17:02:58'),(58,12,'2009-06-24 17:03:07'),(59,12,'2009-06-24 17:03:14'),(60,12,'2009-06-24 17:03:23'),(61,12,'2009-06-24 17:04:15'),(62,12,'2009-06-24 17:04:20'),(63,12,'2009-06-24 17:05:17'),(64,12,'2009-06-24 17:05:24'),(65,12,'2009-06-24 17:05:33'),(66,12,'2009-06-24 17:08:23'),(67,12,'2009-06-24 17:08:32'),(68,12,'2009-06-24 17:08:36'),(69,12,'2009-06-24 17:08:43'),(70,12,'2009-06-24 17:09:01'),(71,12,'2009-06-24 17:09:04'),(72,12,'2009-06-24 17:09:09'),(73,12,'2009-06-24 17:09:13'),(74,12,'2009-06-24 17:09:20'),(75,12,'2009-06-24 17:09:52'),(76,12,'2009-06-24 17:10:36'),(77,12,'2009-06-24 17:10:41'),(78,12,'2009-06-24 17:10:46'),(79,12,'2009-06-24 17:11:27'),(80,12,'2009-06-24 17:11:34'),(81,12,'2009-06-24 17:11:39'),(82,12,'2009-06-24 17:12:00'),(83,12,'2009-06-24 17:12:07'),(84,12,'2009-06-24 17:12:10'),(85,12,'2009-06-24 17:12:50'),(86,12,'2009-06-24 17:18:37'),(87,12,'2009-06-24 17:18:52'),(88,12,'2009-06-24 17:19:05'),(89,12,'2009-06-24 17:19:26'),(90,12,'2009-06-24 17:24:59'),(91,12,'2009-06-24 17:26:16'),(92,12,'2009-06-24 17:26:44'),(93,12,'2009-06-24 17:26:56'),(94,13,'2009-06-24 17:39:16'),(95,13,'2009-06-24 17:39:33'),(96,13,'2009-06-24 17:39:48'),(97,13,'2009-06-24 17:39:58'),(98,13,'2009-06-24 17:40:08'),(99,13,'2009-06-24 17:40:45'),(100,13,'2009-06-24 17:40:47'),(101,13,'2009-06-24 17:40:55'),(102,13,'2009-06-24 17:40:57'),(103,14,'2009-06-24 19:58:45'),(104,15,'2009-06-24 20:24:12'),(105,15,'2009-06-24 20:24:58'),(106,15,'2009-06-24 20:25:21'),(107,15,'2009-06-24 20:28:08'),(108,15,'2009-06-24 20:29:55'),(109,15,'2009-06-24 20:30:12'),(110,15,'2009-06-24 20:31:54'),(111,15,'2009-06-24 20:33:59'),(112,15,'2009-06-24 20:34:28'),(113,15,'2009-06-24 20:36:23'),(114,15,'2009-06-24 20:47:01'),(115,15,'2009-06-24 20:51:52'),(116,15,'2009-06-24 21:02:10'),(117,15,'2009-06-24 21:06:05'),(118,15,'2009-06-24 21:06:37'),(119,15,'2009-06-24 21:06:47'),(120,15,'2009-06-24 21:10:06'),(121,15,'2009-06-24 21:10:32'),(122,15,'2009-06-24 21:17:35'),(123,15,'2009-06-24 21:27:28'),(124,15,'2009-06-24 21:27:46'),(125,15,'2009-06-24 21:27:53'),(126,15,'2009-06-24 21:28:04'),(127,15,'2009-06-24 21:28:08'),(128,15,'2009-06-24 21:31:42'),(129,15,'2009-06-24 21:32:11'),(130,15,'2009-06-24 21:32:58'),(131,15,'2009-06-24 21:34:18'),(132,15,'2009-06-24 21:52:32'),(133,15,'2009-06-24 21:57:36'),(134,15,'2009-06-24 21:59:51'),(135,15,'2009-06-24 22:00:01'),(136,15,'2009-06-24 22:00:15'),(137,15,'2009-06-24 22:00:44'),(138,15,'2009-06-24 22:08:41'),(139,15,'2009-06-24 22:08:53'),(140,15,'2009-06-24 22:10:23'),(141,15,'2009-06-24 22:11:42'),(142,15,'2009-06-24 22:12:17'),(143,15,'2009-06-24 22:13:58'),(144,15,'2009-06-24 22:27:18'),(145,15,'2009-06-24 22:28:21'),(146,15,'2009-06-24 22:28:36'),(147,15,'2009-06-24 22:29:04'),(148,15,'2009-06-24 22:30:21'),(149,15,'2009-06-24 22:30:53'),(150,15,'2009-06-24 22:31:31'),(151,15,'2009-06-24 22:31:50'),(152,15,'2009-06-24 22:33:24'),(153,15,'2009-06-24 22:33:28'),(154,15,'2009-06-24 22:33:54'),(155,15,'2009-06-24 22:42:23'),(156,15,'2009-06-24 22:43:09'),(157,15,'2009-06-24 22:43:17'),(158,15,'2009-06-24 22:45:14'),(159,15,'2009-06-24 22:45:26'),(160,15,'2009-06-24 22:45:31'),(161,15,'2009-06-24 22:45:34'),(162,15,'2009-06-24 22:45:40'),(163,15,'2009-06-24 22:46:30'),(164,15,'2009-06-24 22:46:42'),(165,15,'2009-06-24 22:46:51'),(166,15,'2009-06-24 22:55:06'),(167,15,'2009-06-24 22:55:17'),(168,15,'2009-06-24 22:55:22'),(169,15,'2009-06-24 22:55:25'),(170,15,'2009-06-24 22:55:29'),(171,15,'2009-06-24 23:01:15'),(172,15,'2009-06-24 23:01:21'),(173,15,'2009-06-24 23:01:24'),(174,15,'2009-06-24 23:01:39'),(175,15,'2009-06-24 23:01:49'),(176,15,'2009-06-24 23:01:52'),(177,15,'2009-06-24 23:01:54'),(178,15,'2009-06-24 23:03:19'),(179,15,'2009-06-24 23:03:22'),(180,15,'2009-06-24 23:03:27'),(181,15,'2009-06-24 23:03:31'),(182,15,'2009-06-24 23:03:36'),(183,15,'2009-06-24 23:03:41'),(184,15,'2009-06-24 23:04:07'),(185,16,'2009-06-25 18:34:46'),(186,17,'2009-06-27 09:23:06'),(187,18,'2009-06-29 12:26:21'),(188,18,'2009-06-29 12:26:30'),(189,18,'2009-06-29 12:26:43'),(190,18,'2009-06-29 12:31:03'),(191,18,'2009-06-29 12:31:21'),(192,18,'2009-06-29 12:32:32'),(193,18,'2009-06-29 12:32:44'),(194,18,'2009-06-29 12:32:48'),(195,18,'2009-06-29 12:47:58'),(196,18,'2009-06-29 12:48:46'),(197,18,'2009-06-29 12:51:53'),(198,18,'2009-06-29 12:54:22'),(199,18,'2009-06-29 12:55:47'),(200,18,'2009-06-29 12:55:57'),(201,18,'2009-06-29 12:56:03'),(202,18,'2009-06-29 12:58:39'),(203,18,'2009-06-29 12:59:11'),(204,18,'2009-06-29 13:02:16'),(205,18,'2009-06-29 13:02:24'),(206,18,'2009-06-29 13:02:40'),(207,18,'2009-06-29 13:02:45'),(208,19,'2009-06-29 13:48:20'),(209,19,'2009-06-29 13:48:27'),(210,19,'2009-06-29 13:48:37'),(211,19,'2009-06-29 13:51:34'),(212,19,'2009-06-29 14:02:23'),(213,19,'2009-06-29 14:02:29'),(214,19,'2009-06-29 14:02:35'),(215,19,'2009-06-29 14:02:42'),(216,20,'2009-07-02 15:23:06'),(217,20,'2009-07-02 15:23:24'),(218,20,'2009-07-02 15:23:47'),(219,20,'2009-07-02 15:24:14'),(220,20,'2009-07-02 15:24:22'),(221,20,'2009-07-02 15:24:25'),(222,20,'2009-07-02 15:24:29'),(223,20,'2009-07-02 15:24:37'),(224,20,'2009-07-02 15:25:25'),(225,21,'2009-07-02 15:59:33'),(226,22,'2009-07-02 16:25:21'),(227,22,'2009-07-02 16:25:33'),(228,22,'2009-07-02 16:25:42'),(229,22,'2009-07-02 16:25:49'),(230,22,'2009-07-02 16:31:13'),(231,22,'2009-07-02 16:31:52'),(232,22,'2009-07-02 16:33:16'),(233,23,'2009-07-02 17:12:58'),(234,23,'2009-07-02 17:13:08'),(235,23,'2009-07-02 17:13:44'),(236,23,'2009-07-02 17:14:07'),(237,23,'2009-07-02 17:14:14'),(238,23,'2009-07-02 17:14:47'),(239,23,'2009-07-02 17:14:58'),(240,23,'2009-07-02 17:16:16'),(241,23,'2009-07-02 17:16:26'),(242,23,'2009-07-02 17:16:38'),(243,23,'2009-07-02 17:16:59'),(244,23,'2009-07-02 17:18:08'),(245,23,'2009-07-02 17:19:03'),(246,24,'2009-07-02 17:46:50'),(247,24,'2009-07-02 17:47:00'),(248,24,'2009-07-02 17:47:31'),(249,24,'2009-07-02 17:47:32'),(250,25,'2009-07-02 19:45:31'),(251,25,'2009-07-02 19:46:01'),(252,25,'2009-07-02 19:46:07'),(253,25,'2009-07-02 19:46:23'),(254,25,'2009-07-02 19:46:30'),(255,25,'2009-07-02 19:46:30'),(256,25,'2009-07-02 19:46:38'),(257,25,'2009-07-02 19:46:43'),(258,25,'2009-07-02 19:46:47'),(259,25,'2009-07-02 19:46:55'),(260,25,'2009-07-02 19:47:02'),(261,25,'2009-07-02 19:47:06'),(262,25,'2009-07-02 19:47:07'),(263,26,'2009-07-02 21:04:15'),(264,26,'2009-07-02 21:04:43'),(265,26,'2009-07-02 21:04:58'),(266,26,'2009-07-02 21:05:02'),(267,26,'2009-07-02 21:05:09'),(268,26,'2009-07-02 21:05:19'),(269,26,'2009-07-02 21:05:35'),(270,26,'2009-07-02 21:05:43'),(271,26,'2009-07-02 21:05:53'),(272,26,'2009-07-02 21:05:57'),(273,26,'2009-07-02 21:06:00'),(274,26,'2009-07-02 21:06:13'),(275,26,'2009-07-02 21:18:46'),(276,26,'2009-07-02 21:25:15'),(277,26,'2009-07-02 21:27:48'),(278,26,'2009-07-02 21:30:49'),(279,26,'2009-07-02 21:31:00'),(280,26,'2009-07-02 21:31:11'),(281,26,'2009-07-02 21:31:19'),(282,26,'2009-07-02 21:32:41'),(283,26,'2009-07-02 21:34:36'),(284,26,'2009-07-02 21:36:10'),(285,26,'2009-07-02 21:36:21'),(286,26,'2009-07-02 21:36:25'),(287,26,'2009-07-02 21:36:53'),(288,26,'2009-07-02 21:36:57'),(289,26,'2009-07-02 21:37:13'),(290,27,'2009-07-02 22:20:54'),(291,27,'2009-07-02 22:21:31'),(292,27,'2009-07-02 22:21:38'),(293,27,'2009-07-02 22:21:55'),(294,27,'2009-07-02 22:23:17'),(295,27,'2009-07-02 22:23:35'),(296,27,'2009-07-02 22:23:51'),(297,27,'2009-07-02 22:24:07'),(298,27,'2009-07-02 22:24:23'),(299,27,'2009-07-02 22:25:02'),(300,28,'2009-07-02 22:27:46'),(301,28,'2009-07-02 22:28:05'),(302,28,'2009-07-02 22:28:13'),(303,28,'2009-07-02 22:28:21'),(304,28,'2009-07-02 22:28:28'),(305,28,'2009-07-02 22:28:33'),(306,28,'2009-07-02 22:28:37'),(307,29,'2009-07-02 22:32:07'),(308,30,'2009-07-02 22:32:10'),(309,31,'2009-07-02 22:39:54');
/*!40000 ALTER TABLE `log_url` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `log_url_info`
--

DROP TABLE IF EXISTS `log_url_info`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `log_url_info` (
  `url_id` bigint(20) unsigned NOT NULL auto_increment,
  `url` varchar(255) NOT NULL default '',
  `referer` varchar(255) default NULL,
  PRIMARY KEY  (`url_id`)
) ENGINE=MyISAM AUTO_INCREMENT=310 DEFAULT CHARSET=utf8 COMMENT='Detale information about url visit';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `log_url_info`
--

LOCK TABLES `log_url_info` WRITE;
/*!40000 ALTER TABLE `log_url_info` DISABLE KEYS */;
INSERT INTO `log_url_info` VALUES (1,'http://www.maclaonline.es/tienda/',''),(2,'http://www.maclaonline.es/tienda/',''),(3,'http://www.maclaonline.es/tienda/',''),(4,'http://www.maclaonline.es/tienda/',''),(5,'http://www.maclaonline.es/tienda/',''),(6,'http://www.maclaonline.es/tienda/',''),(7,'http://www.maclaonline.es/tienda/',''),(8,'http://www.maclaonline.es/tienda/',''),(9,'http://www.maclaonline.es/tienda/',''),(10,'http://www.maclaonline.es/tienda/',''),(11,'http://www.maclaonline.es/tienda/',''),(12,'http://www.maclaonline.es/tienda/',''),(13,'http://www.maclaonline.es/tienda/',''),(14,'http://www.maclaonline.es/tienda/',''),(15,'http://www.maclaonline.es/tienda/',''),(16,'http://www.maclaonline.es/tienda/',''),(17,'http://www.maclaonline.es/tienda/',''),(18,'http://www.maclaonline.es/tienda/index.php/customer-service','http://www.maclaonline.es/tienda/'),(19,'http://www.maclaonline.es/tienda/index.php/contacts/','http://www.maclaonline.es/tienda/index.php/customer-service'),(20,'http://www.maclaonline.es/tienda/index.php/customer-service','http://www.maclaonline.es/tienda/index.php/contacts/'),(21,'http://www.maclaonline.es/tienda/index.php/about-magento-demo-store','http://www.maclaonline.es/tienda/index.php/customer-service'),(22,'http://www.maclaonline.es/tienda/index.php/about-magento-demo-store','http://www.maclaonline.es/tienda/index.php/customer-service'),(23,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/about-magento-demo-store'),(24,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/about-magento-demo-store'),(25,'http://www.maclaonline.es/tienda/',''),(26,'http://www.maclaonline.es/tienda/index.php/customer/account/login/','http://www.maclaonline.es/tienda/'),(27,'http://www.maclaonline.es/tienda/',''),(28,'http://www.maclaonline.es/tienda/',''),(29,'http://www.maclaonline.es/tienda/',''),(30,'http://www.maclaonline.es/tienda/',''),(31,'http://www.maclaonline.es/tienda/',''),(32,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/'),(33,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/'),(34,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/'),(35,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(36,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(37,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(38,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(39,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr001.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(40,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/1/image/2/',''),(41,'http://www.maclaonline.es/tienda/index.php/checkout/cart/add/uenc/aHR0cDovL3d3dy5tYWNsYW9ubGluZS5lcy90aWVuZGEvaW5kZXgucGhwL2NhdDEvc3ViMWNhdDEvcHIwMDEuaHRtbD9fX19TSUQ9VQ,,/product/1/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr001.html'),(42,'http://www.maclaonline.es/tienda/index.php/checkout/cart/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr001.html'),(43,'http://www.maclaonline.es/tienda/index.php/about-magento-demo-store','http://www.maclaonline.es/tienda/index.php/checkout/cart/'),(44,'http://www.maclaonline.es/tienda/index.php/customer-service','http://www.maclaonline.es/tienda/index.php/about-magento-demo-store'),(45,'http://www.maclaonline.es/tienda/index.php/contacts/','http://www.maclaonline.es/tienda/index.php/customer-service'),(46,'http://www.maclaonline.es/tienda/',''),(47,'http://www.maclaonline.es/tienda/',''),(48,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/'),(49,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr001.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(50,'http://www.maclaonline.es/tienda/',''),(51,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr001.html?___SID=U','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(52,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr001.html?___SID=U'),(53,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr001.html?___SID=U'),(54,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(55,'http://www.maclaonline.es/tienda/',''),(56,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/'),(57,'http://www.maclaonline.es/tienda/',''),(58,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/'),(59,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(60,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/1/image/1/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(61,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(62,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/1/image/1/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(63,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(64,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(65,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(66,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(67,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(68,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(69,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/1/image/1/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(70,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(71,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(72,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1.html'),(73,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(74,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(75,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(76,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(77,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/'),(78,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1.html'),(79,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(80,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(81,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(82,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(83,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/'),(84,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(85,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(86,'http://www.maclaonline.es/tienda/',''),(87,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/'),(88,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(89,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/1/image/1/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(90,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(91,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(92,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(93,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(94,'http://www.maclaonline.es/tienda/',''),(95,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/'),(96,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html?order=price&dir=asc','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(97,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html?order=price&dir=asc&mode=list','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html?order=price&dir=asc'),(98,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html?order=price&dir=asc&mode=grid','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html?order=price&dir=asc&mode=list'),(99,'http://www.maclaonline.es/tienda/index.php/catalogsearch/ajax/suggest/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html?order=price&dir=asc&mode=grid'),(100,'http://www.maclaonline.es/tienda/index.php/catalogsearch/result/?q=lk&x=0&y=0','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html?order=price&dir=asc&mode=grid'),(101,'http://www.maclaonline.es/tienda/index.php/catalogsearch/ajax/suggest/','http://www.maclaonline.es/tienda/index.php/catalogsearch/result/?q=lk&x=0&y=0'),(102,'http://www.maclaonline.es/tienda/index.php/catalogsearch/result/?q=lkkjk&x=31&y=17','http://www.maclaonline.es/tienda/index.php/catalogsearch/result/?q=lk&x=0&y=0'),(103,'http://www.maclaonline.es/tienda/',''),(104,'http://www.maclaonline.es/tienda/',''),(105,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/'),(106,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1.html'),(107,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1.html'),(108,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1.html'),(109,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1.html'),(110,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1.html'),(111,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1.html'),(112,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(113,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(114,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(115,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(116,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(117,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(118,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/'),(119,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(120,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(121,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/1/image/1/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(122,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(123,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(124,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/cat1.html'),(125,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1.html'),(126,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(127,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/'),(128,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/'),(129,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/'),(130,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/'),(131,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/cat1.html'),(132,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/cat1.html'),(133,'http://www.maclaonline.es/tienda/index.php/about-magento-demo-store','http://www.maclaonline.es/tienda/index.php/cat1.html'),(134,'http://www.maclaonline.es/tienda/index.php/catalog/seo_sitemap/category/','http://www.maclaonline.es/tienda/index.php/about-magento-demo-store'),(135,'http://www.maclaonline.es/tienda/index.php/catalogsearch/term/popular/','http://www.maclaonline.es/tienda/index.php/catalog/seo_sitemap/category/'),(136,'http://www.maclaonline.es/tienda/index.php/catalogsearch/advanced/','http://www.maclaonline.es/tienda/index.php/catalogsearch/term/popular/'),(137,'http://www.maclaonline.es/tienda/index.php/contacts/','http://www.maclaonline.es/tienda/index.php/catalogsearch/advanced/'),(138,'http://www.maclaonline.es/tienda/index.php/contacts/','http://www.maclaonline.es/tienda/index.php/catalogsearch/advanced/'),(139,'http://www.maclaonline.es/tienda/index.php/contacts/','http://www.maclaonline.es/tienda/index.php/catalogsearch/advanced/'),(140,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/contacts/'),(141,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/contacts/'),(142,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/contacts/'),(143,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/contacts/'),(144,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/contacts/'),(145,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/contacts/'),(146,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/'),(147,'http://www.maclaonline.es/tienda/index.php/checkout/cart/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(148,'http://www.maclaonline.es/tienda/index.php/customer/account/login/','http://www.maclaonline.es/tienda/index.php/checkout/cart/'),(149,'http://www.maclaonline.es/tienda/index.php/customer/account/create/','http://www.maclaonline.es/tienda/index.php/customer/account/login/'),(150,'http://www.maclaonline.es/tienda/index.php/customer/account/login/','http://www.maclaonline.es/tienda/index.php/customer/account/create/'),(151,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/customer/account/login/'),(152,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/customer/account/login/'),(153,'http://www.maclaonline.es/tienda/index.php/cat1.html','http://www.maclaonline.es/tienda/index.php/customer/account/login/'),(154,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1.html'),(155,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1.html'),(156,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1.html'),(157,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1.html'),(158,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1.html'),(159,'http://www.maclaonline.es/tienda/index.php/%7B%7Bstore%20url=','http://www.maclaonline.es/tienda/index.php/'),(160,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1.html'),(161,'http://www.maclaonline.es/tienda/index.php/%7B%7Bstore%20url=','http://www.maclaonline.es/tienda/index.php/'),(162,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1.html'),(163,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1.html'),(164,'http://www.maclaonline.es/tienda/index.php/%7B%7Bstore%20url=','http://www.maclaonline.es/tienda/index.php/'),(165,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1.html'),(166,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1.html'),(167,'http://www.maclaonline.es/tienda/index.php/%7B%7Bstore%20url=','http://www.maclaonline.es/tienda/index.php/'),(168,'http://www.maclaonline.es/tienda/index.php/%7B%7Bstore%20url=','http://www.maclaonline.es/tienda/index.php/%7B%7Bstore%20url='),(169,'http://www.maclaonline.es/tienda/index.php/%7B%7Bstore%20url=','http://www.maclaonline.es/tienda/index.php/%7B%7Bstore%20url='),(170,'http://www.maclaonline.es/tienda/index.php/%7B%7Bstore%20url=','http://www.maclaonline.es/tienda/index.php/%7B%7Bstore%20url='),(171,'http://www.maclaonline.es/tienda/index.php/%7B%7Bstore%20url=','http://www.maclaonline.es/tienda/index.php/%7B%7Bstore%20url='),(172,'http://www.maclaonline.es/tienda/index.php/About-us','http://www.maclaonline.es/tienda/index.php/%7B%7Bstore%20url='),(173,'http://www.maclaonline.es/tienda/index.php/customer-service','http://www.maclaonline.es/tienda/index.php/About-us'),(174,'http://www.maclaonline.es/tienda/index.php/condiciones-generales','http://www.maclaonline.es/tienda/index.php/customer-service'),(175,'http://www.maclaonline.es/tienda/index.php/customer-service',''),(176,'http://www.maclaonline.es/tienda/index.php/garantia-devolucion','http://www.maclaonline.es/tienda/index.php/customer-service'),(177,'http://www.maclaonline.es/tienda/index.php/customer-service',''),(178,'http://www.maclaonline.es/tienda/index.php/About-us','http://www.maclaonline.es/tienda/index.php/customer-service'),(179,'http://www.maclaonline.es/tienda/index.php/customer-service','http://www.maclaonline.es/tienda/index.php/About-us'),(180,'http://www.maclaonline.es/tienda/index.php/condiciones-generales','http://www.maclaonline.es/tienda/index.php/customer-service'),(181,'http://www.maclaonline.es/tienda/index.php/garantia-devolucion','http://www.maclaonline.es/tienda/index.php/condiciones-generales'),(182,'http://www.maclaonline.es/tienda/index.php/catalog/seo_sitemap/category/','http://www.maclaonline.es/tienda/index.php/garantia-devolucion'),(183,'http://www.maclaonline.es/tienda/index.php/catalogsearch/term/popular/','http://www.maclaonline.es/tienda/index.php/catalog/seo_sitemap/category/'),(184,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/catalogsearch/term/popular/'),(185,'http://www.maclaonline.es/tienda/',''),(186,'http://www.maclaonline.es/tienda/',''),(187,'http://www.maclaonline.es/tienda/',''),(188,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/'),(189,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(190,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(191,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(192,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(193,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(194,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/'),(195,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/'),(196,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/'),(197,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/'),(198,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/'),(199,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/'),(200,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/'),(201,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/'),(202,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/'),(203,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(204,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(205,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(206,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(207,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/1/image/1/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(208,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(209,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/1/image/1/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(210,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/1/image/1/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(211,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(212,'http://www.maclaonline.es/tienda/',''),(213,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','http://www.maclaonline.es/tienda/'),(214,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html'),(215,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/1/image/1/','http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1/pr1.html'),(216,'http://www.maclaonline.es/tienda/',''),(217,'http://www.maclaonline.es/tienda/index.php/vulc.html','http://www.maclaonline.es/tienda/'),(218,'http://www.maclaonline.es/tienda/',''),(219,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/'),(220,'http://www.maclaonline.es/tienda/index.php/vulc.html','http://www.maclaonline.es/tienda/index.php/'),(221,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/vulc.html'),(222,'http://www.maclaonline.es/tienda/index.php/vulc.html','http://www.maclaonline.es/tienda/index.php/'),(223,'http://www.maclaonline.es/tienda/index.php/vulc/medallon-largo.html','http://www.maclaonline.es/tienda/index.php/vulc.html'),(224,'http://www.maclaonline.es/tienda/index.php/','http://www.maclaonline.es/tienda/index.php/vulc/medallon-largo.html'),(225,'http://www.maclaonline.es/tienda/',''),(226,'http://www.maclaonline.es/tienda/',''),(227,'http://www.maclaonline.es/tienda/',''),(228,'http://www.maclaonline.es/tienda/',''),(229,'http://www.maclaonline.es/tienda/',''),(230,'http://www.maclaonline.es/tienda/',''),(231,'http://www.maclaonline.es/tienda/index.php/About-us','http://www.maclaonline.es/tienda/'),(232,'http://www.maclaonline.es/tienda/',''),(233,'http://www.maclaonline.es/tienda/',''),(234,'http://www.maclaonline.es/tienda/',''),(235,'http://www.maclaonline.es/tienda/',''),(236,'http://www.maclaonline.es/tienda/',''),(237,'http://www.maclaonline.es/tienda/',''),(238,'http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html','http://www.maclaonline.es/tienda/'),(239,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html','http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html'),(240,'http://www.maclaonline.es/tienda/index.php/vulcano.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html'),(241,'http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html','http://www.maclaonline.es/tienda/index.php/vulcano.html'),(242,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html','http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html'),(243,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html'),(244,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html','http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html'),(245,'http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html','http://www.maclaonline.es/tienda/index.php/vulcano.html'),(246,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html','http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html'),(247,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html'),(248,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/2/image/3/',''),(249,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/2/image/3/maclaonline.es/tienda/skin/frontend/default/modern/css/ie7minus.css','http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/2/image/3/'),(250,'http://www.maclaonline.es/tienda/',''),(251,'http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html','http://www.maclaonline.es/tienda/'),(252,'http://www.maclaonline.es/tienda/index.php/dominica/placas-dominica.html','http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html'),(253,'http://www.maclaonline.es/tienda/index.php/hindu/medallones-hindu.html','http://www.maclaonline.es/tienda/index.php/dominica/placas-dominica.html'),(254,'http://www.maclaonline.es/tienda/index.php/hindu.html','http://www.maclaonline.es/tienda/index.php/hindu/medallones-hindu.html'),(255,'http://www.maclaonline.es/tienda/index.php/hindu/plumas-hindu.html','http://www.maclaonline.es/tienda/index.php/hindu/medallones-hindu.html'),(256,'http://www.maclaonline.es/tienda/index.php/vulcano.html','http://www.maclaonline.es/tienda/index.php/hindu/plumas-hindu.html'),(257,'http://www.maclaonline.es/tienda/index.php/dominica.html','http://www.maclaonline.es/tienda/index.php/vulcano.html'),(258,'http://www.maclaonline.es/tienda/index.php/hindu.html','http://www.maclaonline.es/tienda/index.php/dominica.html'),(259,'http://www.maclaonline.es/tienda/index.php/dominica/medallones-dominica.html','http://www.maclaonline.es/tienda/index.php/hindu.html'),(260,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html','http://www.maclaonline.es/tienda/index.php/dominica/medallones-dominica.html'),(261,'http://www.maclaonline.es/tienda/index.php/hindu.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html'),(262,'http://www.maclaonline.es/tienda/index.php/hindu/medallones-hindu.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html'),(263,'http://www.maclaonline.es/tienda/',''),(264,'http://www.maclaonline.es/tienda/index.php/dominica/piedras-dominica.html','http://www.maclaonline.es/tienda/'),(265,'http://www.maclaonline.es/tienda/index.php/dominica/piedras-dominica.html','http://www.maclaonline.es/tienda/index.php/dominica/piedras-dominica.html'),(266,'http://www.maclaonline.es/tienda/index.php/dominica/placas-dominica.html','http://www.maclaonline.es/tienda/index.php/dominica/piedras-dominica.html'),(267,'http://www.maclaonline.es/tienda/index.php/dominica/medallones-dominica.html','http://www.maclaonline.es/tienda/index.php/dominica/placas-dominica.html'),(268,'http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html','http://www.maclaonline.es/tienda/index.php/dominica/medallones-dominica.html'),(269,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html','http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html'),(270,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html'),(271,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/2/image/3/','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(272,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/image/2/id/2/','http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/2/image/3/'),(273,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/image/3/id/2/','http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/image/2/id/2/'),(274,'http://www.maclaonline.es/tienda/index.php/hindu/medallones-hindu.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(275,'http://www.maclaonline.es/tienda/index.php/hindu/medallones-hindu.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(276,'http://www.maclaonline.es/tienda/index.php/hindu/medallones-hindu.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(277,'http://www.maclaonline.es/tienda/index.php/hindu/medallones-hindu.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(278,'http://www.maclaonline.es/tienda/index.php/hindu/medallones-hindu.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(279,'http://www.maclaonline.es/tienda/index.php/hindu/medallones-hindu.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(280,'http://www.maclaonline.es/tienda/index.php/dominica.html','http://www.maclaonline.es/tienda/index.php/hindu/medallones-hindu.html'),(281,'http://www.maclaonline.es/tienda/index.php/dominica/placas-dominica.html','http://www.maclaonline.es/tienda/index.php/dominica.html'),(282,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html','http://www.maclaonline.es/tienda/index.php/dominica/placas-dominica.html'),(283,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html'),(284,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/2/image/3/','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(285,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/2/image/3/','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(286,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/2/image/3/','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(287,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/2/image/3/','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(288,'http://www.maclaonline.es/tienda/index.php/catalog/product/gallery/id/2/image/2/','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(289,'http://www.maclaonline.es/tienda/index.php/dominica.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(290,'http://www.maclaonline.es/tienda/index.php/dominica.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(291,'http://www.maclaonline.es/tienda/index.php/dominica.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(292,'http://www.maclaonline.es/tienda/index.php/dominica.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(293,'http://www.maclaonline.es/tienda/index.php/hindu/plumas-hindu.html','http://www.maclaonline.es/tienda/index.php/dominica.html'),(294,'http://www.maclaonline.es/tienda/index.php/dominica/piedras-dominica.html','http://www.maclaonline.es/tienda/index.php/hindu/plumas-hindu.html'),(295,'http://www.maclaonline.es/tienda/index.php/vulcano.html','http://www.maclaonline.es/tienda/index.php/dominica/piedras-dominica.html'),(296,'http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html','http://www.maclaonline.es/tienda/index.php/vulcano.html'),(297,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html','http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html'),(298,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html'),(299,'http://www.maclaonline.es/tienda/index.php/review/product/list/id/2/category/6/','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html'),(300,'http://www.maclaonline.es/tienda/index.php/dominica.html','http://www.maclaonline.es/tienda/index.php/review/product/list/id/2/category/6/'),(301,'http://www.maclaonline.es/tienda/index.php/dominica/piedras-dominica.html','http://www.maclaonline.es/tienda/index.php/dominica.html'),(302,'http://www.maclaonline.es/tienda/index.php/vulcano.html','http://www.maclaonline.es/tienda/index.php/dominica/piedras-dominica.html'),(303,'http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html','http://www.maclaonline.es/tienda/index.php/vulcano.html'),(304,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html','http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html'),(305,'http://www.maclaonline.es/tienda/index.php/vulcano.html','http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html'),(306,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano.html','http://www.maclaonline.es/tienda/index.php/vulcano.html'),(307,'http://www.maclaonline.es/tienda/',''),(308,'http://www.maclaonline.es/tienda/',''),(309,'http://www.maclaonline.es/tienda/','');
/*!40000 ALTER TABLE `log_url_info` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `log_visitor`
--

DROP TABLE IF EXISTS `log_visitor`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `log_visitor` (
  `visitor_id` bigint(20) unsigned NOT NULL auto_increment,
  `session_id` char(64) NOT NULL default '',
  `first_visit_at` datetime default NULL,
  `last_visit_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `last_url_id` bigint(20) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL,
  PRIMARY KEY  (`visitor_id`)
) ENGINE=MyISAM AUTO_INCREMENT=32 DEFAULT CHARSET=utf8 COMMENT='System visitors log';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `log_visitor`
--

LOCK TABLES `log_visitor` WRITE;
/*!40000 ALTER TABLE `log_visitor` DISABLE KEYS */;
INSERT INTO `log_visitor` VALUES (1,'360454ca445f63d6dd199e2a9992f5d4','2009-06-22 12:51:03','2009-06-22 13:26:15',24,1),(2,'bf5311ac7cead61e9415d5cb37f6f71f','2009-06-23 09:16:40','2009-06-23 09:17:20',26,1),(3,'091fe1c29724523219e7f87873cc2428','2009-06-23 14:13:53','2009-06-23 14:38:36',45,1),(4,'42a55ef13bb3550857a85bf86d22f477','2009-06-23 14:16:20','2009-06-23 14:16:20',28,1),(5,'580104c0e8def146efbd588329700260','2009-06-23 15:46:10','2009-06-23 15:46:11',46,1),(6,'fa0ba956b52e1ebdf3f59f035d721738','2009-06-23 20:26:05','2009-06-23 20:27:01',49,1),(7,'42428eed5435ce19386209f4f98fad71','2009-06-24 12:08:52','2009-06-24 12:08:53',50,1),(8,'42428eed5435ce19386209f4f98fad71','2009-06-24 12:46:59','2009-06-24 12:46:59',0,1),(9,'42428eed5435ce19386209f4f98fad71','2009-06-24 12:47:02','2009-06-24 12:47:02',0,1),(10,'42428eed5435ce19386209f4f98fad71','2009-06-24 12:50:53','2009-06-24 12:50:53',0,1),(11,'42428eed5435ce19386209f4f98fad71','2009-06-24 12:56:46','2009-06-24 13:04:02',54,1),(12,'d1f5bdb06fa3ffa02c9fd4a0b18f231a','2009-06-24 16:38:13','2009-06-24 17:26:56',93,1),(13,'b9e3880561ea45aafeebddf711c1d1a9','2009-06-24 17:39:15','2009-06-24 17:40:57',102,1),(14,'28e1b56d0ecf6034dbe210c704d58898','2009-06-24 19:58:44','2009-06-24 19:58:45',103,1),(15,'28e1b56d0ecf6034dbe210c704d58898','2009-06-24 20:24:12','2009-06-24 23:04:07',184,1),(16,'75a4a81b98771107edaf2e2263365d77','2009-06-25 18:34:45','2009-06-25 18:34:46',185,1),(17,'e7ce128fae196c5d997750c6a2719c6a','2009-06-27 09:23:05','2009-06-27 09:23:06',186,1),(18,'fd4ab4b67e7cf66d0223e7b089bbc6e4','2009-06-29 12:26:19','2009-06-29 13:02:45',207,1),(19,'5607eff25c2b831f6b443ef9b23abfc8','2009-06-29 13:48:17','2009-06-29 14:02:42',215,1),(20,'4e905097faa1e01ee668e00ca874cfea','2009-07-02 15:23:05','2009-07-02 15:25:25',224,1),(21,'4e905097faa1e01ee668e00ca874cfea','2009-07-02 15:59:33','2009-07-02 15:59:33',225,1),(22,'23d6fea8e8618efe38e4b37e19d3d9f3','2009-07-02 16:25:18','2009-07-02 16:33:16',232,1),(23,'23d6fea8e8618efe38e4b37e19d3d9f3','2009-07-02 17:12:57','2009-07-02 17:19:03',245,1),(24,'4a12203c1f402f5c24c0dcd8c71f1f6f','2009-07-02 17:46:47','2009-07-02 17:47:32',249,1),(25,'dcd2f462994c382d2ae67b56a7d9568b','2009-07-02 19:45:28','2009-07-02 19:47:07',262,1),(26,'987ff71c964ae972bd7fdbeee9210d95','2009-07-02 21:04:14','2009-07-02 21:37:13',289,1),(27,'987ff71c964ae972bd7fdbeee9210d95','2009-07-02 22:20:53','2009-07-02 22:25:02',299,1),(28,'0166c3f1393d6e61c07b45a85086fb33','2009-07-02 22:27:45','2009-07-02 22:28:37',306,1),(29,'706eb67da7d48e7eecf8bd6a060f7a82','2009-07-02 22:32:07','2009-07-02 22:32:07',307,1),(30,'7884ff26f86c0c69e029798a59eba19b','2009-07-02 22:32:10','2009-07-02 22:32:10',308,1),(31,'ccfa2a82a2f91b7a99dba0593b2e57bb','2009-07-02 22:39:53','2009-07-02 22:39:54',309,1);
/*!40000 ALTER TABLE `log_visitor` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `log_visitor_info`
--

DROP TABLE IF EXISTS `log_visitor_info`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `log_visitor_info` (
  `visitor_id` bigint(20) unsigned NOT NULL default '0',
  `http_referer` varchar(255) default NULL,
  `http_user_agent` varchar(255) default NULL,
  `http_accept_charset` varchar(255) default NULL,
  `http_accept_language` varchar(255) default NULL,
  `server_addr` bigint(20) default NULL,
  `remote_addr` bigint(20) default NULL,
  PRIMARY KEY  (`visitor_id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT='Additional information by visitor';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `log_visitor_info`
--

LOCK TABLES `log_visitor_info` WRITE;
/*!40000 ALTER TABLE `log_visitor_info` DISABLE KEYS */;
INSERT INTO `log_visitor_info` VALUES (1,'','Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.0.11) Gecko/2009060215 Firefox/3.0.11 (.NET CLR 3.5.30729)','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es',1163467184,3568815311),(2,'','Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.0.11) Gecko/2009060215 Firefox/3.0.11 (.NET CLR 3.5.30729)','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es',1163467184,3568815311),(3,'','Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; MDDS; .NET CLR 3.5.30729; .NET CLR 3.0.30618; OfficeLiveConnector.1.3; OfficeLivePatch.0.0)','','es',1163467184,1370259326),(4,'','','','',1163467184,1138631947),(5,'','Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.0.11) Gecko/2009060215 Firefox/3.0.11 (.NET CLR 3.5.30729)','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es',1163467184,3568815311),(6,'','Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)','','es',1163467184,1474176362),(7,'','Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.0.11) Gecko/2009060215 Firefox/3.0.11 (.NET CLR 3.5.30729)','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es',1163467184,3568815311),(8,'http://www.maclaonline.es/tienda/','Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.0.11) Gecko/2009060215 Firefox/3.0.11 (.NET CLR 3.5.30729)','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es',1163467184,3568815311),(9,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.0.11) Gecko/2009060215 Firefox/3.0.11 (.NET CLR 3.5.30729)','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es',1163467184,3568815311),(10,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.0.11) Gecko/2009060215 Firefox/3.0.11 (.NET CLR 3.5.30729)','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es',1163467184,3568815311),(11,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.0.11) Gecko/2009060215 Firefox/3.0.11 (.NET CLR 3.5.30729)','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es',1163467184,3568815311),(12,'','Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.0.11) Gecko/2009060215 Firefox/3.0.11 (.NET CLR 3.5.30729)','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es',1163467184,3568815311),(13,'','Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es,es;q=0.8,en-us;q=0.5,en;q=0.3',1163467184,3568815311),(14,'','Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es,es;q=0.8,en-us;q=0.5,en;q=0.3',1163467184,1474176362),(15,'','Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es,es;q=0.8,en-us;q=0.5,en;q=0.3',1163467184,1474176362),(16,'','Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.0.11) Gecko/2009060215 Firefox/3.0.11 (.NET CLR 3.5.30729)','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es',1163467184,3568815311),(17,'','Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es,es;q=0.8,en-us;q=0.5,en;q=0.3',1163467184,1474176362),(18,'','Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.0.11) Gecko/2009060215 Firefox/3.0.11 (.NET CLR 3.5.30729)','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es',1163467184,3568815311),(19,'http://www.maclaonline.es/tienda/index.php/cat1/sub1cat1.html','Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.0.11) Gecko/2009060215 Firefox/3.0.11 (.NET CLR 3.5.30729)','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es',1163467184,3568815311),(20,'','Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322)','','es',1163467184,3575975515),(21,'','Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322)','','es',1163467184,3575975515),(22,'','Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322)','','es',1163467184,3575975515),(23,'','Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322)','','es',1163467184,3575975515),(24,'http://www.maclaonline.es/tienda/index.php/vulcano/colgante-piedra.html','Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322)','','es',1163467184,3575975515),(25,'','Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322)','','es',1163467184,3575975515),(26,'','Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es,es;q=0.8,en-us;q=0.5,en;q=0.3',1163467184,1474176362),(27,'http://www.maclaonline.es/tienda/index.php/vulcano/medallones-vulcano/medallon-doble-vulcano-espiral.html','Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es,es;q=0.8,en-us;q=0.5,en;q=0.3',1163467184,1474176362),(28,'http://www.maclaonline.es/tienda/index.php/review/product/list/id/2/category/6/','Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/2009060308 Ubuntu/9.04 (jaunty) Firefox/3.0.11','ISO-8859-1,utf-8;q=0.7,*;q=0.7','es-es,es;q=0.8,en-us;q=0.5,en;q=0.3',1163467184,1474176362),(29,'','','','',1163467184,1138631947),(30,'','','','',1163467184,1138631947),(31,'','','','',1163467184,1138631947);
/*!40000 ALTER TABLE `log_visitor_info` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `log_visitor_online`
--

DROP TABLE IF EXISTS `log_visitor_online`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `log_visitor_online` (
  `visitor_id` bigint(20) unsigned NOT NULL auto_increment,
  `visitor_type` char(1) NOT NULL,
  `remote_addr` bigint(20) NOT NULL,
  `first_visit_at` datetime default NULL,
  `last_visit_at` datetime default NULL,
  `customer_id` int(10) unsigned default NULL,
  `last_url` varchar(255) default NULL,
  PRIMARY KEY  (`visitor_id`),
  KEY `IDX_VISITOR_TYPE` (`visitor_type`),
  KEY `IDX_VISIT_TIME` (`first_visit_at`,`last_visit_at`),
  KEY `IDX_CUSTOMER` (`customer_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `log_visitor_online`
--

LOCK TABLES `log_visitor_online` WRITE;
/*!40000 ALTER TABLE `log_visitor_online` DISABLE KEYS */;
/*!40000 ALTER TABLE `log_visitor_online` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `newsletter_problem`
--

DROP TABLE IF EXISTS `newsletter_problem`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `newsletter_problem` (
  `problem_id` int(7) unsigned NOT NULL auto_increment,
  `subscriber_id` int(7) unsigned default NULL,
  `queue_id` int(7) unsigned NOT NULL default '0',
  `problem_error_code` int(3) unsigned default '0',
  `problem_error_text` varchar(200) default NULL,
  PRIMARY KEY  (`problem_id`),
  KEY `FK_PROBLEM_SUBSCRIBER` (`subscriber_id`),
  KEY `FK_PROBLEM_QUEUE` (`queue_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Newsletter problems';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `newsletter_problem`
--

LOCK TABLES `newsletter_problem` WRITE;
/*!40000 ALTER TABLE `newsletter_problem` DISABLE KEYS */;
/*!40000 ALTER TABLE `newsletter_problem` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `newsletter_queue`
--

DROP TABLE IF EXISTS `newsletter_queue`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `newsletter_queue` (
  `queue_id` int(7) unsigned NOT NULL auto_increment,
  `template_id` int(7) unsigned NOT NULL default '0',
  `queue_status` int(3) unsigned NOT NULL default '0',
  `queue_start_at` datetime default NULL,
  `queue_finish_at` datetime default NULL,
  PRIMARY KEY  (`queue_id`),
  KEY `FK_QUEUE_TEMPLATE` (`template_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Newsletter queue';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `newsletter_queue`
--

LOCK TABLES `newsletter_queue` WRITE;
/*!40000 ALTER TABLE `newsletter_queue` DISABLE KEYS */;
/*!40000 ALTER TABLE `newsletter_queue` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `newsletter_queue_link`
--

DROP TABLE IF EXISTS `newsletter_queue_link`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `newsletter_queue_link` (
  `queue_link_id` int(9) unsigned NOT NULL auto_increment,
  `queue_id` int(7) unsigned NOT NULL default '0',
  `subscriber_id` int(7) unsigned NOT NULL default '0',
  `letter_sent_at` datetime default NULL,
  PRIMARY KEY  (`queue_link_id`),
  KEY `FK_QUEUE_LINK_SUBSCRIBER` (`subscriber_id`),
  KEY `FK_QUEUE_LINK_QUEUE` (`queue_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Newsletter queue to subscriber link';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `newsletter_queue_link`
--

LOCK TABLES `newsletter_queue_link` WRITE;
/*!40000 ALTER TABLE `newsletter_queue_link` DISABLE KEYS */;
/*!40000 ALTER TABLE `newsletter_queue_link` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `newsletter_queue_store_link`
--

DROP TABLE IF EXISTS `newsletter_queue_store_link`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `newsletter_queue_store_link` (
  `queue_id` int(7) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  PRIMARY KEY  (`queue_id`,`store_id`),
  KEY `FK_NEWSLETTER_QUEUE_STORE_LINK_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `newsletter_queue_store_link`
--

LOCK TABLES `newsletter_queue_store_link` WRITE;
/*!40000 ALTER TABLE `newsletter_queue_store_link` DISABLE KEYS */;
/*!40000 ALTER TABLE `newsletter_queue_store_link` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `newsletter_subscriber`
--

DROP TABLE IF EXISTS `newsletter_subscriber`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `newsletter_subscriber` (
  `subscriber_id` int(7) unsigned NOT NULL auto_increment,
  `store_id` smallint(5) unsigned default '0',
  `change_status_at` datetime default NULL,
  `customer_id` int(11) unsigned NOT NULL default '0',
  `subscriber_email` varchar(150) character set latin1 collate latin1_general_ci NOT NULL default '',
  `subscriber_status` int(3) NOT NULL default '0',
  `subscriber_confirm_code` varchar(32) default 'NULL',
  PRIMARY KEY  (`subscriber_id`),
  KEY `FK_SUBSCRIBER_CUSTOMER` (`customer_id`),
  KEY `FK_NEWSLETTER_SUBSCRIBER_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Newsletter subscribers';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `newsletter_subscriber`
--

LOCK TABLES `newsletter_subscriber` WRITE;
/*!40000 ALTER TABLE `newsletter_subscriber` DISABLE KEYS */;
/*!40000 ALTER TABLE `newsletter_subscriber` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `newsletter_template`
--

DROP TABLE IF EXISTS `newsletter_template`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `newsletter_template` (
  `template_id` int(7) unsigned NOT NULL auto_increment,
  `template_code` varchar(150) default NULL,
  `template_text` text,
  `template_text_preprocessed` text,
  `template_type` int(3) unsigned default NULL,
  `template_subject` varchar(200) default NULL,
  `template_sender_name` varchar(200) default NULL,
  `template_sender_email` varchar(200) character set latin1 collate latin1_general_ci default NULL,
  `template_actual` tinyint(1) unsigned default '1',
  `added_at` datetime default NULL,
  `modified_at` datetime default NULL,
  PRIMARY KEY  (`template_id`),
  KEY `template_actual` (`template_actual`),
  KEY `added_at` (`added_at`),
  KEY `modified_at` (`modified_at`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Newsletter templates';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `newsletter_template`
--

LOCK TABLES `newsletter_template` WRITE;
/*!40000 ALTER TABLE `newsletter_template` DISABLE KEYS */;
/*!40000 ALTER TABLE `newsletter_template` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `paygate_authorizenet_debug`
--

DROP TABLE IF EXISTS `paygate_authorizenet_debug`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `paygate_authorizenet_debug` (
  `debug_id` int(10) unsigned NOT NULL auto_increment,
  `request_body` text,
  `response_body` text,
  `request_serialized` text,
  `result_serialized` text,
  `request_dump` text,
  `result_dump` text,
  PRIMARY KEY  (`debug_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `paygate_authorizenet_debug`
--

LOCK TABLES `paygate_authorizenet_debug` WRITE;
/*!40000 ALTER TABLE `paygate_authorizenet_debug` DISABLE KEYS */;
/*!40000 ALTER TABLE `paygate_authorizenet_debug` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `paypal_api_debug`
--

DROP TABLE IF EXISTS `paypal_api_debug`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `paypal_api_debug` (
  `debug_id` int(10) unsigned NOT NULL auto_increment,
  `debug_at` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `request_body` text,
  `response_body` text,
  PRIMARY KEY  (`debug_id`),
  KEY `debug_at` (`debug_at`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `paypal_api_debug`
--

LOCK TABLES `paypal_api_debug` WRITE;
/*!40000 ALTER TABLE `paypal_api_debug` DISABLE KEYS */;
/*!40000 ALTER TABLE `paypal_api_debug` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `paypaluk_api_debug`
--

DROP TABLE IF EXISTS `paypaluk_api_debug`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `paypaluk_api_debug` (
  `debug_id` int(10) unsigned NOT NULL auto_increment,
  `debug_at` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `request_body` text,
  `response_body` text,
  PRIMARY KEY  (`debug_id`),
  KEY `debug_at` (`debug_at`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `paypaluk_api_debug`
--

LOCK TABLES `paypaluk_api_debug` WRITE;
/*!40000 ALTER TABLE `paypaluk_api_debug` DISABLE KEYS */;
/*!40000 ALTER TABLE `paypaluk_api_debug` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `poll`
--

DROP TABLE IF EXISTS `poll`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `poll` (
  `poll_id` int(10) unsigned NOT NULL auto_increment,
  `poll_title` varchar(255) NOT NULL default '',
  `votes_count` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned default '0',
  `date_posted` datetime NOT NULL default '0000-00-00 00:00:00',
  `date_closed` datetime default NULL,
  `active` smallint(6) NOT NULL default '1',
  `closed` tinyint(1) NOT NULL default '0',
  `answers_display` smallint(6) default NULL,
  PRIMARY KEY  (`poll_id`),
  KEY `FK_POLL_STORE` (`store_id`)
) ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `poll`
--

LOCK TABLES `poll` WRITE;
/*!40000 ALTER TABLE `poll` DISABLE KEYS */;
INSERT INTO `poll` VALUES (1,'What is your favorite color',5,1,'2009-06-03 13:53:25','2009-06-24 21:52:05',1,1,NULL);
/*!40000 ALTER TABLE `poll` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `poll_answer`
--

DROP TABLE IF EXISTS `poll_answer`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `poll_answer` (
  `answer_id` int(10) unsigned NOT NULL auto_increment,
  `poll_id` int(10) unsigned NOT NULL default '0',
  `answer_title` varchar(255) NOT NULL default '',
  `votes_count` int(10) unsigned NOT NULL default '0',
  `answer_order` smallint(6) NOT NULL default '0',
  PRIMARY KEY  (`answer_id`),
  KEY `FK_POLL_PARENT` (`poll_id`)
) ENGINE=InnoDB AUTO_INCREMENT=5 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `poll_answer`
--

LOCK TABLES `poll_answer` WRITE;
/*!40000 ALTER TABLE `poll_answer` DISABLE KEYS */;
INSERT INTO `poll_answer` VALUES (1,1,'Green',4,0),(2,1,'Red',1,0),(3,1,'Black',0,0),(4,1,'Magenta',0,0);
/*!40000 ALTER TABLE `poll_answer` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `poll_store`
--

DROP TABLE IF EXISTS `poll_store`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `poll_store` (
  `poll_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  PRIMARY KEY  (`poll_id`,`store_id`),
  KEY `FK_POLL_STORE_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `poll_store`
--

LOCK TABLES `poll_store` WRITE;
/*!40000 ALTER TABLE `poll_store` DISABLE KEYS */;
INSERT INTO `poll_store` VALUES (1,1);
/*!40000 ALTER TABLE `poll_store` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `poll_vote`
--

DROP TABLE IF EXISTS `poll_vote`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `poll_vote` (
  `vote_id` int(10) unsigned NOT NULL auto_increment,
  `poll_id` int(10) unsigned NOT NULL default '0',
  `poll_answer_id` int(10) unsigned NOT NULL default '0',
  `ip_address` bigint(20) default NULL,
  `customer_id` int(11) default NULL,
  `vote_time` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  PRIMARY KEY  (`vote_id`),
  KEY `FK_POLL_ANSWER` (`poll_answer_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `poll_vote`
--

LOCK TABLES `poll_vote` WRITE;
/*!40000 ALTER TABLE `poll_vote` DISABLE KEYS */;
/*!40000 ALTER TABLE `poll_vote` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `product_alert_price`
--

DROP TABLE IF EXISTS `product_alert_price`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `product_alert_price` (
  `alert_price_id` int(10) unsigned NOT NULL auto_increment,
  `customer_id` int(10) unsigned NOT NULL default '0',
  `product_id` int(10) unsigned NOT NULL default '0',
  `price` decimal(12,4) NOT NULL default '0.0000',
  `website_id` smallint(5) unsigned NOT NULL default '0',
  `add_date` datetime NOT NULL default '0000-00-00 00:00:00',
  `last_send_date` datetime default NULL,
  `send_count` smallint(5) unsigned NOT NULL default '0',
  `status` tinyint(3) unsigned NOT NULL default '0',
  PRIMARY KEY  (`alert_price_id`),
  KEY `FK_PRODUCT_ALERT_PRICE_CUSTOMER` (`customer_id`),
  KEY `FK_PRODUCT_ALERT_PRICE_PRODUCT` (`product_id`),
  KEY `FK_PRODUCT_ALERT_PRICE_WEBSITE` (`website_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `product_alert_price`
--

LOCK TABLES `product_alert_price` WRITE;
/*!40000 ALTER TABLE `product_alert_price` DISABLE KEYS */;
/*!40000 ALTER TABLE `product_alert_price` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `product_alert_stock`
--

DROP TABLE IF EXISTS `product_alert_stock`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `product_alert_stock` (
  `alert_stock_id` int(10) unsigned NOT NULL auto_increment,
  `customer_id` int(10) unsigned NOT NULL default '0',
  `product_id` int(10) unsigned NOT NULL default '0',
  `website_id` smallint(5) unsigned NOT NULL default '0',
  `add_date` datetime NOT NULL default '0000-00-00 00:00:00',
  `send_date` datetime default NULL,
  `send_count` smallint(5) unsigned NOT NULL default '0',
  `status` tinyint(3) unsigned NOT NULL default '0',
  PRIMARY KEY  (`alert_stock_id`),
  KEY `FK_PRODUCT_ALERT_STOCK_CUSTOMER` (`customer_id`),
  KEY `FK_PRODUCT_ALERT_STOCK_PRODUCT` (`product_id`),
  KEY `FK_PRODUCT_ALERT_STOCK_WEBSITE` (`website_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `product_alert_stock`
--

LOCK TABLES `product_alert_stock` WRITE;
/*!40000 ALTER TABLE `product_alert_stock` DISABLE KEYS */;
/*!40000 ALTER TABLE `product_alert_stock` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `rating`
--

DROP TABLE IF EXISTS `rating`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `rating` (
  `rating_id` smallint(6) unsigned NOT NULL auto_increment,
  `entity_id` smallint(6) unsigned NOT NULL default '0',
  `rating_code` varchar(64) NOT NULL default '',
  `position` tinyint(3) unsigned NOT NULL default '0',
  PRIMARY KEY  (`rating_id`),
  UNIQUE KEY `IDX_CODE` (`rating_code`),
  KEY `FK_RATING_ENTITY` (`entity_id`)
) ENGINE=InnoDB AUTO_INCREMENT=4 DEFAULT CHARSET=utf8 COMMENT='ratings';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `rating`
--

LOCK TABLES `rating` WRITE;
/*!40000 ALTER TABLE `rating` DISABLE KEYS */;
INSERT INTO `rating` VALUES (1,1,'Quality',0),(2,1,'Value',0),(3,1,'Price',0);
/*!40000 ALTER TABLE `rating` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `rating_entity`
--

DROP TABLE IF EXISTS `rating_entity`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `rating_entity` (
  `entity_id` smallint(6) unsigned NOT NULL auto_increment,
  `entity_code` varchar(64) NOT NULL default '',
  PRIMARY KEY  (`entity_id`),
  UNIQUE KEY `IDX_CODE` (`entity_code`)
) ENGINE=InnoDB AUTO_INCREMENT=4 DEFAULT CHARSET=utf8 COMMENT='Rating entities';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `rating_entity`
--

LOCK TABLES `rating_entity` WRITE;
/*!40000 ALTER TABLE `rating_entity` DISABLE KEYS */;
INSERT INTO `rating_entity` VALUES (1,'product'),(2,'product_review'),(3,'review');
/*!40000 ALTER TABLE `rating_entity` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `rating_option`
--

DROP TABLE IF EXISTS `rating_option`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `rating_option` (
  `option_id` int(10) unsigned NOT NULL auto_increment,
  `rating_id` smallint(6) unsigned NOT NULL default '0',
  `code` varchar(32) NOT NULL default '',
  `value` tinyint(3) unsigned NOT NULL default '0',
  `position` tinyint(3) unsigned NOT NULL default '0',
  PRIMARY KEY  (`option_id`),
  KEY `FK_RATING_OPTION_RATING` (`rating_id`)
) ENGINE=InnoDB AUTO_INCREMENT=16 DEFAULT CHARSET=utf8 COMMENT='Rating options';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `rating_option`
--

LOCK TABLES `rating_option` WRITE;
/*!40000 ALTER TABLE `rating_option` DISABLE KEYS */;
INSERT INTO `rating_option` VALUES (1,1,'1',1,1),(2,1,'2',2,2),(3,1,'3',3,3),(4,1,'4',4,4),(5,1,'5',5,5),(6,2,'1',1,1),(7,2,'2',2,2),(8,2,'3',3,3),(9,2,'4',4,4),(10,2,'5',5,5),(11,3,'1',1,1),(12,3,'2',2,2),(13,3,'3',3,3),(14,3,'4',4,4),(15,3,'5',5,5);
/*!40000 ALTER TABLE `rating_option` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `rating_option_vote`
--

DROP TABLE IF EXISTS `rating_option_vote`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `rating_option_vote` (
  `vote_id` bigint(20) unsigned NOT NULL auto_increment,
  `option_id` int(10) unsigned NOT NULL default '0',
  `remote_ip` varchar(16) NOT NULL default '',
  `remote_ip_long` int(11) NOT NULL default '0',
  `customer_id` int(11) unsigned default '0',
  `entity_pk_value` bigint(20) unsigned NOT NULL default '0',
  `rating_id` smallint(6) unsigned NOT NULL default '0',
  `review_id` bigint(20) unsigned default NULL,
  `percent` tinyint(3) NOT NULL default '0',
  `value` tinyint(3) NOT NULL default '0',
  PRIMARY KEY  (`vote_id`),
  KEY `FK_RATING_OPTION_VALUE_OPTION` (`option_id`),
  KEY `FK_RATING_OPTION_REVIEW_ID` (`review_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Rating option values';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `rating_option_vote`
--

LOCK TABLES `rating_option_vote` WRITE;
/*!40000 ALTER TABLE `rating_option_vote` DISABLE KEYS */;
/*!40000 ALTER TABLE `rating_option_vote` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `rating_option_vote_aggregated`
--

DROP TABLE IF EXISTS `rating_option_vote_aggregated`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `rating_option_vote_aggregated` (
  `primary_id` int(11) NOT NULL auto_increment,
  `rating_id` smallint(6) unsigned NOT NULL default '0',
  `entity_pk_value` bigint(20) unsigned NOT NULL default '0',
  `vote_count` int(10) unsigned NOT NULL default '0',
  `vote_value_sum` int(10) unsigned NOT NULL default '0',
  `percent` tinyint(3) NOT NULL default '0',
  `percent_approved` tinyint(3) default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  PRIMARY KEY  (`primary_id`),
  KEY `FK_RATING_OPTION_VALUE_AGGREGATE` (`rating_id`),
  KEY `FK_RATING_OPTION_VOTE_AGGREGATED_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `rating_option_vote_aggregated`
--

LOCK TABLES `rating_option_vote_aggregated` WRITE;
/*!40000 ALTER TABLE `rating_option_vote_aggregated` DISABLE KEYS */;
/*!40000 ALTER TABLE `rating_option_vote_aggregated` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `rating_store`
--

DROP TABLE IF EXISTS `rating_store`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `rating_store` (
  `rating_id` smallint(6) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  PRIMARY KEY  (`rating_id`,`store_id`),
  KEY `FK_RATING_STORE_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `rating_store`
--

LOCK TABLES `rating_store` WRITE;
/*!40000 ALTER TABLE `rating_store` DISABLE KEYS */;
/*!40000 ALTER TABLE `rating_store` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `rating_title`
--

DROP TABLE IF EXISTS `rating_title`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `rating_title` (
  `rating_id` smallint(6) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `value` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`rating_id`,`store_id`),
  KEY `FK_RATING_TITLE_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `rating_title`
--

LOCK TABLES `rating_title` WRITE;
/*!40000 ALTER TABLE `rating_title` DISABLE KEYS */;
/*!40000 ALTER TABLE `rating_title` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `report_event`
--

DROP TABLE IF EXISTS `report_event`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `report_event` (
  `event_id` bigint(20) unsigned NOT NULL auto_increment,
  `logged_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `event_type_id` smallint(6) unsigned NOT NULL default '0',
  `object_id` int(10) unsigned NOT NULL default '0',
  `subject_id` int(10) unsigned NOT NULL default '0',
  `subtype` tinyint(3) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL,
  PRIMARY KEY  (`event_id`),
  KEY `IDX_EVENT_TYPE` (`event_type_id`),
  KEY `IDX_SUBJECT` (`subject_id`),
  KEY `IDX_OBJECT` (`object_id`),
  KEY `IDX_SUBTYPE` (`subtype`),
  KEY `FK_REPORT_EVENT_STORE` (`store_id`)
) ENGINE=InnoDB AUTO_INCREMENT=53 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `report_event`
--

LOCK TABLES `report_event` WRITE;
/*!40000 ALTER TABLE `report_event` DISABLE KEYS */;
INSERT INTO `report_event` VALUES (1,'2009-06-23 14:34:05',1,1,3,1,1),(2,'2009-06-23 14:35:58',4,1,3,1,1),(3,'2009-06-23 20:26:59',1,1,6,1,1),(4,'2009-06-24 12:09:12',1,1,7,1,1),(5,'2009-06-24 12:47:02',1,1,9,1,1),(6,'2009-06-24 12:50:53',1,1,10,1,1),(7,'2009-06-24 12:57:23',1,1,11,1,1),(8,'2009-06-24 13:00:55',1,1,11,1,1),(9,'2009-06-24 13:02:19',1,1,11,1,1),(10,'2009-06-24 13:02:29',1,1,11,1,1),(11,'2009-06-24 13:03:13',1,1,11,1,1),(12,'2009-06-24 13:03:20',1,1,11,1,1),(13,'2009-06-24 13:04:00',1,1,11,1,1),(14,'2009-06-24 16:38:45',1,1,12,1,1),(15,'2009-06-24 17:03:14',1,1,12,1,1),(16,'2009-06-24 17:04:14',1,1,12,1,1),(17,'2009-06-24 17:05:17',1,1,12,1,1),(18,'2009-06-24 17:05:32',1,1,12,1,1),(19,'2009-06-24 17:08:22',1,1,12,1,1),(20,'2009-06-24 17:08:36',1,1,12,1,1),(21,'2009-06-24 17:09:12',1,1,12,1,1),(22,'2009-06-24 17:09:20',1,1,12,1,1),(23,'2009-06-24 17:09:52',1,1,12,1,1),(24,'2009-06-24 17:10:23',1,1,12,1,1),(25,'2009-06-24 17:10:49',1,1,12,1,1),(26,'2009-06-24 17:11:27',1,1,12,1,1),(27,'2009-06-24 17:11:34',1,1,12,1,1),(28,'2009-06-24 17:11:39',1,1,12,1,1),(29,'2009-06-24 17:12:09',1,1,12,1,1),(30,'2009-06-24 17:12:49',1,1,12,1,1),(31,'2009-06-24 17:19:04',1,1,12,1,1),(32,'2009-06-24 17:24:58',1,1,12,1,1),(33,'2009-06-24 17:26:16',1,1,12,1,1),(34,'2009-06-24 17:26:43',1,1,12,1,1),(35,'2009-06-24 17:26:55',1,1,12,1,1),(36,'2009-06-24 21:06:46',1,1,15,1,1),(37,'2009-06-24 21:10:05',1,1,15,1,1),(38,'2009-06-29 12:26:41',1,1,18,1,1),(39,'2009-06-29 12:31:01',1,1,18,1,1),(40,'2009-06-29 12:31:19',1,1,18,1,1),(41,'2009-06-29 12:32:31',1,1,18,1,1),(42,'2009-06-29 12:59:10',1,1,18,1,1),(43,'2009-06-29 13:02:15',1,1,18,1,1),(44,'2009-06-29 13:02:40',1,1,18,1,1),(45,'2009-06-29 13:48:17',1,1,19,1,1),(46,'2009-06-29 13:51:33',1,1,19,1,1),(47,'2009-06-29 14:02:34',1,1,19,1,1),(48,'2009-07-02 17:16:57',1,2,23,1,1),(49,'2009-07-02 17:46:58',1,2,24,1,1),(50,'2009-07-02 21:05:42',1,2,26,1,1),(51,'2009-07-02 21:34:36',1,2,26,1,1),(52,'2009-07-02 22:24:21',1,2,27,1,1);
/*!40000 ALTER TABLE `report_event` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `report_event_types`
--

DROP TABLE IF EXISTS `report_event_types`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `report_event_types` (
  `event_type_id` smallint(6) unsigned NOT NULL auto_increment,
  `event_name` varchar(64) NOT NULL,
  `customer_login` tinyint(3) unsigned NOT NULL default '0',
  PRIMARY KEY  (`event_type_id`)
) ENGINE=InnoDB AUTO_INCREMENT=7 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `report_event_types`
--

LOCK TABLES `report_event_types` WRITE;
/*!40000 ALTER TABLE `report_event_types` DISABLE KEYS */;
INSERT INTO `report_event_types` VALUES (1,'catalog_product_view',1),(2,'sendfriend_product',1),(3,'catalog_product_compare_add_product',1),(4,'checkout_cart_add_product',1),(5,'wishlist_add_product',1),(6,'wishlist_share',1);
/*!40000 ALTER TABLE `report_event_types` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `review`
--

DROP TABLE IF EXISTS `review`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `review` (
  `review_id` bigint(20) unsigned NOT NULL auto_increment,
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `entity_id` smallint(5) unsigned NOT NULL default '0',
  `entity_pk_value` int(10) unsigned NOT NULL default '0',
  `status_id` tinyint(3) unsigned NOT NULL default '0',
  PRIMARY KEY  (`review_id`),
  KEY `FK_REVIEW_ENTITY` (`entity_id`),
  KEY `FK_REVIEW_STATUS` (`status_id`),
  KEY `FK_REVIEW_PARENT_PRODUCT` (`entity_pk_value`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Review base information';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `review`
--

LOCK TABLES `review` WRITE;
/*!40000 ALTER TABLE `review` DISABLE KEYS */;
/*!40000 ALTER TABLE `review` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `review_detail`
--

DROP TABLE IF EXISTS `review_detail`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `review_detail` (
  `detail_id` bigint(20) unsigned NOT NULL auto_increment,
  `review_id` bigint(20) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned default '0',
  `title` varchar(255) NOT NULL default '',
  `detail` text NOT NULL,
  `nickname` varchar(128) NOT NULL default '',
  `customer_id` int(10) unsigned default NULL,
  PRIMARY KEY  (`detail_id`),
  KEY `FK_REVIEW_DETAIL_REVIEW` (`review_id`),
  KEY `FK_REVIEW_DETAIL_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Review detail information';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `review_detail`
--

LOCK TABLES `review_detail` WRITE;
/*!40000 ALTER TABLE `review_detail` DISABLE KEYS */;
/*!40000 ALTER TABLE `review_detail` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `review_entity`
--

DROP TABLE IF EXISTS `review_entity`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `review_entity` (
  `entity_id` smallint(5) unsigned NOT NULL auto_increment,
  `entity_code` varchar(32) NOT NULL default '',
  PRIMARY KEY  (`entity_id`)
) ENGINE=InnoDB AUTO_INCREMENT=4 DEFAULT CHARSET=utf8 COMMENT='Review entities';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `review_entity`
--

LOCK TABLES `review_entity` WRITE;
/*!40000 ALTER TABLE `review_entity` DISABLE KEYS */;
INSERT INTO `review_entity` VALUES (1,'product'),(2,'customer'),(3,'category');
/*!40000 ALTER TABLE `review_entity` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `review_entity_summary`
--

DROP TABLE IF EXISTS `review_entity_summary`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `review_entity_summary` (
  `primary_id` bigint(20) NOT NULL auto_increment,
  `entity_pk_value` bigint(20) NOT NULL default '0',
  `entity_type` tinyint(4) NOT NULL default '0',
  `reviews_count` smallint(6) NOT NULL default '0',
  `rating_summary` tinyint(4) NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  PRIMARY KEY  (`primary_id`),
  KEY `FK_REVIEW_ENTITY_SUMMARY_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `review_entity_summary`
--

LOCK TABLES `review_entity_summary` WRITE;
/*!40000 ALTER TABLE `review_entity_summary` DISABLE KEYS */;
/*!40000 ALTER TABLE `review_entity_summary` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `review_status`
--

DROP TABLE IF EXISTS `review_status`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `review_status` (
  `status_id` tinyint(3) unsigned NOT NULL auto_increment,
  `status_code` varchar(32) NOT NULL default '',
  PRIMARY KEY  (`status_id`)
) ENGINE=InnoDB AUTO_INCREMENT=4 DEFAULT CHARSET=utf8 COMMENT='Review statuses';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `review_status`
--

LOCK TABLES `review_status` WRITE;
/*!40000 ALTER TABLE `review_status` DISABLE KEYS */;
INSERT INTO `review_status` VALUES (1,'Approved'),(2,'Pending'),(3,'Not Approved');
/*!40000 ALTER TABLE `review_status` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `review_store`
--

DROP TABLE IF EXISTS `review_store`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `review_store` (
  `review_id` bigint(20) unsigned NOT NULL,
  `store_id` smallint(5) unsigned NOT NULL,
  PRIMARY KEY  (`review_id`,`store_id`),
  KEY `FK_REVIEW_STORE_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `review_store`
--

LOCK TABLES `review_store` WRITE;
/*!40000 ALTER TABLE `review_store` DISABLE KEYS */;
/*!40000 ALTER TABLE `review_store` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_flat_order_item`
--

DROP TABLE IF EXISTS `sales_flat_order_item`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_flat_order_item` (
  `item_id` int(10) unsigned NOT NULL auto_increment,
  `order_id` int(10) unsigned NOT NULL default '0',
  `parent_item_id` int(10) unsigned default NULL,
  `quote_item_id` int(10) unsigned default NULL,
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `product_id` int(10) unsigned default NULL,
  `product_type` varchar(255) default NULL,
  `product_options` text,
  `weight` decimal(12,4) default '0.0000',
  `is_virtual` tinyint(1) unsigned default NULL,
  `sku` varchar(255) NOT NULL default '',
  `name` varchar(255) default NULL,
  `description` text,
  `applied_rule_ids` text,
  `additional_data` text,
  `free_shipping` tinyint(1) unsigned NOT NULL default '0',
  `is_qty_decimal` tinyint(1) unsigned default NULL,
  `no_discount` tinyint(1) unsigned default '0',
  `qty_backordered` decimal(12,4) default '0.0000',
  `qty_canceled` decimal(12,4) default '0.0000',
  `qty_invoiced` decimal(12,4) default '0.0000',
  `qty_ordered` decimal(12,4) default '0.0000',
  `qty_refunded` decimal(12,4) default '0.0000',
  `qty_shipped` decimal(12,4) default '0.0000',
  `cost` decimal(12,4) default '0.0000',
  `price` decimal(12,4) NOT NULL default '0.0000',
  `base_price` decimal(12,4) NOT NULL default '0.0000',
  `original_price` decimal(12,4) default NULL,
  `base_original_price` decimal(12,4) default NULL,
  `tax_percent` decimal(12,4) default '0.0000',
  `tax_amount` decimal(12,4) default '0.0000',
  `base_tax_amount` decimal(12,4) default '0.0000',
  `tax_invoiced` decimal(12,4) default '0.0000',
  `base_tax_invoiced` decimal(12,4) default '0.0000',
  `discount_percent` decimal(12,4) default '0.0000',
  `discount_amount` decimal(12,4) default '0.0000',
  `base_discount_amount` decimal(12,4) default '0.0000',
  `discount_invoiced` decimal(12,4) default '0.0000',
  `base_discount_invoiced` decimal(12,4) default '0.0000',
  `amount_refunded` decimal(12,4) default '0.0000',
  `base_amount_refunded` decimal(12,4) default '0.0000',
  `row_total` decimal(12,4) NOT NULL default '0.0000',
  `base_row_total` decimal(12,4) NOT NULL default '0.0000',
  `row_invoiced` decimal(12,4) NOT NULL default '0.0000',
  `base_row_invoiced` decimal(12,4) NOT NULL default '0.0000',
  `row_weight` decimal(12,4) default '0.0000',
  `gift_message_id` int(10) default NULL,
  `gift_message_available` int(10) default NULL,
  `base_tax_before_discount` decimal(12,4) default NULL,
  `tax_before_discount` decimal(12,4) default NULL,
  `ext_order_item_id` varchar(255) default NULL,
  `locked_do_invoice` int(10) unsigned default NULL,
  `locked_do_ship` int(10) unsigned default NULL,
  `weee_tax_applied` text,
  `weee_tax_applied_amount` decimal(12,4) default NULL,
  `weee_tax_applied_row_amount` decimal(12,4) default NULL,
  `base_weee_tax_applied_amount` decimal(12,4) default NULL,
  `base_weee_tax_applied_row_amount` decimal(12,4) default NULL,
  `weee_tax_disposition` decimal(12,4) default NULL,
  `weee_tax_row_disposition` decimal(12,4) default NULL,
  `base_weee_tax_disposition` decimal(12,4) default NULL,
  `base_weee_tax_row_disposition` decimal(12,4) default NULL,
  PRIMARY KEY  (`item_id`),
  KEY `IDX_ORDER` (`order_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_flat_order_item`
--

LOCK TABLES `sales_flat_order_item` WRITE;
/*!40000 ALTER TABLE `sales_flat_order_item` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_flat_order_item` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_flat_quote`
--

DROP TABLE IF EXISTS `sales_flat_quote`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_flat_quote` (
  `entity_id` int(10) unsigned NOT NULL auto_increment,
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `converted_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `is_active` tinyint(1) unsigned default '1',
  `is_virtual` tinyint(1) unsigned default '0',
  `is_multi_shipping` tinyint(1) unsigned default '0',
  `items_count` int(10) unsigned default '0',
  `items_qty` decimal(12,4) default '0.0000',
  `orig_order_id` int(10) unsigned default '0',
  `store_to_base_rate` decimal(12,4) default '0.0000',
  `store_to_quote_rate` decimal(12,4) default '0.0000',
  `base_currency_code` varchar(255) default NULL,
  `store_currency_code` varchar(255) default NULL,
  `quote_currency_code` varchar(255) default NULL,
  `grand_total` decimal(12,4) default '0.0000',
  `base_grand_total` decimal(12,4) default '0.0000',
  `checkout_method` varchar(255) default NULL,
  `customer_id` int(10) unsigned default '0',
  `customer_tax_class_id` int(10) unsigned default '0',
  `customer_group_id` int(10) unsigned default '0',
  `customer_email` varchar(255) default NULL,
  `customer_prefix` varchar(40) default NULL,
  `customer_firstname` varchar(255) default NULL,
  `customer_middlename` varchar(40) default NULL,
  `customer_lastname` varchar(255) default NULL,
  `customer_suffix` varchar(40) default NULL,
  `customer_dob` datetime default NULL,
  `customer_note` varchar(255) default NULL,
  `customer_note_notify` tinyint(1) unsigned default '1',
  `customer_is_guest` tinyint(1) unsigned default '0',
  `remote_ip` varchar(32) default NULL,
  `applied_rule_ids` varchar(255) default NULL,
  `reserved_order_id` varchar(64) default '',
  `password_hash` varchar(255) default NULL,
  `coupon_code` varchar(255) default NULL,
  `global_currency_code` varchar(255) default NULL,
  `base_to_global_rate` decimal(12,4) default NULL,
  `base_to_quote_rate` decimal(12,4) default NULL,
  `customer_taxvat` varchar(255) default NULL,
  `subtotal` decimal(12,4) default NULL,
  `base_subtotal` decimal(12,4) default NULL,
  `subtotal_with_discount` decimal(12,4) default NULL,
  `base_subtotal_with_discount` decimal(12,4) default NULL,
  `is_changed` int(10) unsigned default NULL,
  `trigger_recollect` tinyint(1) NOT NULL default '0',
  `ext_shipping_info` text,
  `gift_message_id` int(10) unsigned default NULL,
  PRIMARY KEY  (`entity_id`),
  KEY `FK_SALES_QUOTE_STORE` (`store_id`),
  KEY `IDX_CUSTOMER` (`customer_id`,`store_id`,`is_active`)
) ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_flat_quote`
--

LOCK TABLES `sales_flat_quote` WRITE;
/*!40000 ALTER TABLE `sales_flat_quote` DISABLE KEYS */;
INSERT INTO `sales_flat_quote` VALUES (1,1,'2009-06-23 14:35:58','2009-06-23 14:35:59','0000-00-00 00:00:00',1,0,0,0,'0.0000',0,'1.0000','1.0000','EUR','EUR','EUR','25.5000','25.5000','',0,3,0,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,1,0,'81.172.127.126','','',NULL,NULL,'EUR','1.0000','1.0000',NULL,'25.5000','25.5000','25.5000','25.5000',1,0,NULL,NULL);
/*!40000 ALTER TABLE `sales_flat_quote` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_flat_quote_address`
--

DROP TABLE IF EXISTS `sales_flat_quote_address`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_flat_quote_address` (
  `address_id` int(10) unsigned NOT NULL auto_increment,
  `quote_id` int(10) unsigned NOT NULL default '0',
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `customer_id` int(10) unsigned default NULL,
  `save_in_address_book` tinyint(1) default '0',
  `customer_address_id` int(10) unsigned default NULL,
  `address_type` varchar(255) default NULL,
  `email` varchar(255) default NULL,
  `prefix` varchar(40) default NULL,
  `firstname` varchar(255) default NULL,
  `middlename` varchar(40) default NULL,
  `lastname` varchar(255) default NULL,
  `suffix` varchar(40) default NULL,
  `company` varchar(255) default NULL,
  `street` varchar(255) default NULL,
  `city` varchar(255) default NULL,
  `region` varchar(255) default NULL,
  `region_id` int(10) unsigned default NULL,
  `postcode` varchar(255) default NULL,
  `country_id` varchar(255) default NULL,
  `telephone` varchar(255) default NULL,
  `fax` varchar(255) default NULL,
  `same_as_billing` tinyint(1) unsigned NOT NULL default '0',
  `free_shipping` tinyint(1) unsigned NOT NULL default '0',
  `collect_shipping_rates` tinyint(1) unsigned NOT NULL default '0',
  `shipping_method` varchar(255) NOT NULL default '',
  `shipping_description` varchar(255) NOT NULL default '',
  `weight` decimal(12,4) NOT NULL default '0.0000',
  `subtotal` decimal(12,4) NOT NULL default '0.0000',
  `base_subtotal` decimal(12,4) NOT NULL default '0.0000',
  `subtotal_with_discount` decimal(12,4) NOT NULL default '0.0000',
  `base_subtotal_with_discount` decimal(12,4) NOT NULL default '0.0000',
  `tax_amount` decimal(12,4) NOT NULL default '0.0000',
  `base_tax_amount` decimal(12,4) NOT NULL default '0.0000',
  `shipping_amount` decimal(12,4) NOT NULL default '0.0000',
  `base_shipping_amount` decimal(12,4) NOT NULL default '0.0000',
  `shipping_tax_amount` decimal(12,4) default NULL,
  `base_shipping_tax_amount` decimal(12,4) default NULL,
  `discount_amount` decimal(12,4) NOT NULL default '0.0000',
  `base_discount_amount` decimal(12,4) NOT NULL default '0.0000',
  `grand_total` decimal(12,4) NOT NULL default '0.0000',
  `base_grand_total` decimal(12,4) NOT NULL default '0.0000',
  `customer_notes` text,
  `applied_taxes` text,
  `gift_message_id` int(10) unsigned default NULL,
  PRIMARY KEY  (`address_id`),
  KEY `FK_SALES_QUOTE_ADDRESS_SALES_QUOTE` (`quote_id`)
) ENGINE=InnoDB AUTO_INCREMENT=5 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_flat_quote_address`
--

LOCK TABLES `sales_flat_quote_address` WRITE;
/*!40000 ALTER TABLE `sales_flat_quote_address` DISABLE KEYS */;
INSERT INTO `sales_flat_quote_address` VALUES (3,1,'2009-06-23 14:35:59','2009-06-23 14:35:59',NULL,0,NULL,'billing',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0,0,0,'','','0.0000','0.0000','0.0000','0.0000','0.0000','0.0000','0.0000','0.0000','0.0000',NULL,NULL,'0.0000','0.0000','0.0000','0.0000',NULL,'a:0:{}',NULL),(4,1,'2009-06-23 14:35:59','2009-06-23 14:35:59',NULL,0,NULL,'shipping',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0,0,0,'','','0.1000','25.5000','25.5000','25.5000','25.5000','0.0000','0.0000','0.0000','0.0000','0.0000','0.0000','0.0000','0.0000','25.5000','25.5000',NULL,'a:0:{}',NULL);
/*!40000 ALTER TABLE `sales_flat_quote_address` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_flat_quote_address_item`
--

DROP TABLE IF EXISTS `sales_flat_quote_address_item`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_flat_quote_address_item` (
  `address_item_id` int(10) unsigned NOT NULL auto_increment,
  `parent_item_id` int(10) unsigned default NULL,
  `quote_address_id` int(10) unsigned NOT NULL default '0',
  `quote_item_id` int(10) unsigned NOT NULL default '0',
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `applied_rule_ids` text,
  `additional_data` text,
  `weight` decimal(12,4) default '0.0000',
  `qty` decimal(12,4) NOT NULL default '0.0000',
  `discount_amount` decimal(12,4) default '0.0000',
  `tax_amount` decimal(12,4) default '0.0000',
  `row_total` decimal(12,4) NOT NULL default '0.0000',
  `base_row_total` decimal(12,4) NOT NULL default '0.0000',
  `row_total_with_discount` decimal(12,4) default '0.0000',
  `base_discount_amount` decimal(12,4) default '0.0000',
  `base_tax_amount` decimal(12,4) default '0.0000',
  `row_weight` decimal(12,4) default '0.0000',
  `product_id` int(10) unsigned default NULL,
  `super_product_id` int(10) unsigned default NULL,
  `parent_product_id` int(10) unsigned default NULL,
  `sku` varchar(255) default NULL,
  `image` varchar(255) default NULL,
  `name` varchar(255) default NULL,
  `description` text,
  `free_shipping` int(10) unsigned default NULL,
  `is_qty_decimal` int(10) unsigned default NULL,
  `price` decimal(12,4) default NULL,
  `discount_percent` decimal(12,4) default NULL,
  `no_discount` int(10) unsigned default NULL,
  `tax_percent` decimal(12,4) default NULL,
  `base_price` decimal(12,4) default NULL,
  `gift_message_id` int(10) unsigned default NULL,
  PRIMARY KEY  (`address_item_id`),
  KEY `FK_QUOTE_ADDRESS_ITEM_QUOTE_ADDRESS` (`quote_address_id`),
  KEY `FK_SALES_QUOTE_ADDRESS_ITEM_QUOTE_ITEM` (`quote_item_id`),
  KEY `FK_SALES_FLAT_QUOTE_ADDRESS_ITEM_PARENT` (`parent_item_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_flat_quote_address_item`
--

LOCK TABLES `sales_flat_quote_address_item` WRITE;
/*!40000 ALTER TABLE `sales_flat_quote_address_item` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_flat_quote_address_item` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_flat_quote_item`
--

DROP TABLE IF EXISTS `sales_flat_quote_item`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_flat_quote_item` (
  `item_id` int(10) unsigned NOT NULL auto_increment,
  `quote_id` int(10) unsigned NOT NULL default '0',
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `product_id` int(10) unsigned default NULL,
  `parent_item_id` int(10) unsigned default NULL,
  `is_virtual` tinyint(1) unsigned default NULL,
  `sku` varchar(255) NOT NULL default '',
  `name` varchar(255) default NULL,
  `description` text,
  `applied_rule_ids` text,
  `additional_data` text,
  `free_shipping` tinyint(1) unsigned NOT NULL default '0',
  `is_qty_decimal` tinyint(1) unsigned default NULL,
  `no_discount` tinyint(1) unsigned default '0',
  `weight` decimal(12,4) default '0.0000',
  `qty` decimal(12,4) NOT NULL default '0.0000',
  `price` decimal(12,4) NOT NULL default '0.0000',
  `base_price` decimal(12,4) NOT NULL default '0.0000',
  `custom_price` decimal(12,4) default NULL,
  `discount_percent` decimal(12,4) default '0.0000',
  `discount_amount` decimal(12,4) default '0.0000',
  `base_discount_amount` decimal(12,4) default '0.0000',
  `tax_percent` decimal(12,4) default '0.0000',
  `tax_amount` decimal(12,4) default '0.0000',
  `base_tax_amount` decimal(12,4) default '0.0000',
  `row_total` decimal(12,4) NOT NULL default '0.0000',
  `base_row_total` decimal(12,4) NOT NULL default '0.0000',
  `row_total_with_discount` decimal(12,4) default '0.0000',
  `row_weight` decimal(12,4) default '0.0000',
  `product_type` varchar(255) default NULL,
  `base_tax_before_discount` decimal(12,4) default NULL,
  `tax_before_discount` decimal(12,4) default NULL,
  `original_custom_price` decimal(12,4) default NULL,
  `gift_message_id` int(10) unsigned default NULL,
  `weee_tax_applied` text,
  `weee_tax_applied_amount` decimal(12,4) default NULL,
  `weee_tax_applied_row_amount` decimal(12,4) default NULL,
  `base_weee_tax_applied_amount` decimal(12,4) default NULL,
  `base_weee_tax_applied_row_amount` decimal(12,4) default NULL,
  `weee_tax_disposition` decimal(12,4) default NULL,
  `weee_tax_row_disposition` decimal(12,4) default NULL,
  `base_weee_tax_disposition` decimal(12,4) default NULL,
  `base_weee_tax_row_disposition` decimal(12,4) default NULL,
  PRIMARY KEY  (`item_id`),
  KEY `FK_SALES_QUOTE_ITEM_SALES_QUOTE` (`quote_id`),
  KEY `FK_SALES_FLAT_QUOTE_ITEM_PARENT_ITEM` (`parent_item_id`),
  KEY `FK_SALES_QUOTE_ITEM_CATALOG_PRODUCT_ENTITY` (`product_id`)
) ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_flat_quote_item`
--

LOCK TABLES `sales_flat_quote_item` WRITE;
/*!40000 ALTER TABLE `sales_flat_quote_item` DISABLE KEYS */;
INSERT INTO `sales_flat_quote_item` VALUES (1,1,'2009-06-23 14:35:58','2009-06-23 14:35:59',1,NULL,0,'PR001','Producto 1',NULL,'',NULL,0,0,0,'0.1000','1.0000','25.5000','0.0000',NULL,'0.0000','0.0000','0.0000','0.0000','0.0000','0.0000','25.5000','25.5000','25.5000','0.1000','simple','0.0000','0.0000',NULL,NULL,'a:0:{}','0.0000','0.0000','0.0000','0.0000','0.0000','0.0000','0.0000','0.0000');
/*!40000 ALTER TABLE `sales_flat_quote_item` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_flat_quote_item_option`
--

DROP TABLE IF EXISTS `sales_flat_quote_item_option`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_flat_quote_item_option` (
  `option_id` int(10) unsigned NOT NULL auto_increment,
  `item_id` int(10) unsigned NOT NULL,
  `product_id` int(10) unsigned NOT NULL,
  `code` varchar(255) NOT NULL,
  `value` text NOT NULL,
  PRIMARY KEY  (`option_id`),
  KEY `FK_SALES_QUOTE_ITEM_OPTION_ITEM_ID` (`item_id`)
) ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=utf8 COMMENT='Additional options for quote item';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_flat_quote_item_option`
--

LOCK TABLES `sales_flat_quote_item_option` WRITE;
/*!40000 ALTER TABLE `sales_flat_quote_item_option` DISABLE KEYS */;
INSERT INTO `sales_flat_quote_item_option` VALUES (1,1,1,'info_buyRequest','a:4:{s:4:\"uenc\";s:104:\"aHR0cDovL3d3dy5tYWNsYW9ubGluZS5lcy90aWVuZGEvaW5kZXgucGhwL2NhdDEvc3ViMWNhdDEvcHIwMDEuaHRtbD9fX19TSUQ9VQ,,\";s:7:\"product\";s:1:\"1\";s:15:\"related_product\";s:0:\"\";s:3:\"qty\";s:1:\"1\";}');
/*!40000 ALTER TABLE `sales_flat_quote_item_option` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_flat_quote_payment`
--

DROP TABLE IF EXISTS `sales_flat_quote_payment`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_flat_quote_payment` (
  `payment_id` int(10) unsigned NOT NULL auto_increment,
  `quote_id` int(10) unsigned NOT NULL default '0',
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `method` varchar(255) default '',
  `cc_type` varchar(255) default '',
  `cc_number_enc` varchar(255) default '',
  `cc_last4` varchar(255) default '',
  `cc_cid_enc` varchar(255) default '',
  `cc_owner` varchar(255) default '',
  `cc_exp_month` tinyint(2) unsigned default '0',
  `cc_exp_year` smallint(4) unsigned default '0',
  `cc_ss_owner` varchar(255) default '',
  `cc_ss_start_month` tinyint(2) unsigned default '0',
  `cc_ss_start_year` smallint(4) unsigned default '0',
  `cybersource_token` varchar(255) default '',
  `paypal_correlation_id` varchar(255) default '',
  `paypal_payer_id` varchar(255) default '',
  `paypal_payer_status` varchar(255) default '',
  `po_number` varchar(255) default '',
  `additional_data` text,
  `cc_ss_issue` varchar(255) default NULL,
  PRIMARY KEY  (`payment_id`),
  KEY `FK_SALES_QUOTE_PAYMENT_SALES_QUOTE` (`quote_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_flat_quote_payment`
--

LOCK TABLES `sales_flat_quote_payment` WRITE;
/*!40000 ALTER TABLE `sales_flat_quote_payment` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_flat_quote_payment` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_flat_quote_shipping_rate`
--

DROP TABLE IF EXISTS `sales_flat_quote_shipping_rate`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_flat_quote_shipping_rate` (
  `rate_id` int(10) unsigned NOT NULL auto_increment,
  `address_id` int(10) unsigned NOT NULL default '0',
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `carrier` varchar(255) default NULL,
  `carrier_title` varchar(255) default NULL,
  `code` varchar(255) default NULL,
  `method` varchar(255) default NULL,
  `method_description` text,
  `price` decimal(12,4) NOT NULL default '0.0000',
  `error_message` text,
  `method_title` text,
  PRIMARY KEY  (`rate_id`),
  KEY `FK_SALES_QUOTE_SHIPPING_RATE_ADDRESS` (`address_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_flat_quote_shipping_rate`
--

LOCK TABLES `sales_flat_quote_shipping_rate` WRITE;
/*!40000 ALTER TABLE `sales_flat_quote_shipping_rate` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_flat_quote_shipping_rate` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_order`
--

DROP TABLE IF EXISTS `sales_order`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_order` (
  `entity_id` int(10) unsigned NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_set_id` smallint(5) unsigned NOT NULL default '0',
  `increment_id` varchar(50) NOT NULL default '',
  `parent_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned default NULL,
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `is_active` tinyint(1) unsigned NOT NULL default '1',
  `customer_id` int(10) unsigned default NULL,
  `tax_amount` decimal(12,4) NOT NULL default '0.0000',
  `shipping_amount` decimal(12,4) NOT NULL default '0.0000',
  `discount_amount` decimal(12,4) NOT NULL default '0.0000',
  `subtotal` decimal(12,4) NOT NULL default '0.0000',
  `grand_total` decimal(12,4) NOT NULL default '0.0000',
  `total_paid` decimal(12,4) NOT NULL default '0.0000',
  `total_refunded` decimal(12,4) NOT NULL default '0.0000',
  `total_qty_ordered` decimal(12,4) NOT NULL default '0.0000',
  `total_canceled` decimal(12,4) NOT NULL default '0.0000',
  `total_invoiced` decimal(12,4) NOT NULL default '0.0000',
  `total_online_refunded` decimal(12,4) NOT NULL default '0.0000',
  `total_offline_refunded` decimal(12,4) NOT NULL default '0.0000',
  `base_tax_amount` decimal(12,4) NOT NULL default '0.0000',
  `base_shipping_amount` decimal(12,4) NOT NULL default '0.0000',
  `base_discount_amount` decimal(12,4) NOT NULL default '0.0000',
  `base_subtotal` decimal(12,4) NOT NULL default '0.0000',
  `base_grand_total` decimal(12,4) NOT NULL default '0.0000',
  `base_total_paid` decimal(12,4) NOT NULL default '0.0000',
  `base_total_refunded` decimal(12,4) NOT NULL default '0.0000',
  `base_total_qty_ordered` decimal(12,4) NOT NULL default '0.0000',
  `base_total_canceled` decimal(12,4) NOT NULL default '0.0000',
  `base_total_invoiced` decimal(12,4) NOT NULL default '0.0000',
  `base_total_online_refunded` decimal(12,4) NOT NULL default '0.0000',
  `base_total_offline_refunded` decimal(12,4) NOT NULL default '0.0000',
  `subtotal_refunded` decimal(12,4) default NULL,
  `subtotal_canceled` decimal(12,4) default NULL,
  `discount_refunded` decimal(12,4) default NULL,
  `discount_canceled` decimal(12,4) default NULL,
  `discount_invoiced` decimal(12,4) default NULL,
  `tax_refunded` decimal(12,4) default NULL,
  `tax_canceled` decimal(12,4) default NULL,
  `shipping_refunded` decimal(12,4) default NULL,
  `shipping_canceled` decimal(12,4) default NULL,
  `base_subtotal_refunded` decimal(12,4) default NULL,
  `base_subtotal_canceled` decimal(12,4) default NULL,
  `base_discount_refunded` decimal(12,4) default NULL,
  `base_discount_canceled` decimal(12,4) default NULL,
  `base_discount_invoiced` decimal(12,4) default NULL,
  `base_tax_refunded` decimal(12,4) default NULL,
  `base_tax_canceled` decimal(12,4) default NULL,
  `base_shipping_refunded` decimal(12,4) default NULL,
  `base_shipping_canceled` decimal(12,4) default NULL,
  `subtotal_invoiced` decimal(12,4) default NULL,
  `tax_invoiced` decimal(12,4) default NULL,
  `shipping_invoiced` decimal(12,4) default NULL,
  `base_subtotal_invoiced` decimal(12,4) default NULL,
  `base_tax_invoiced` decimal(12,4) default NULL,
  `base_shipping_invoiced` decimal(12,4) default NULL,
  `shipping_tax_amount` decimal(12,4) default NULL,
  `base_shipping_tax_amount` decimal(12,4) default NULL,
  `shipping_tax_refunded` decimal(12,4) default NULL,
  `base_shipping_tax_refunded` decimal(12,4) default NULL,
  PRIMARY KEY  (`entity_id`),
  KEY `FK_SALES_ORDER_TYPE` (`entity_type_id`),
  KEY `FK_SALES_ORDER_STORE` (`store_id`),
  KEY `IDX_CUSTOMER` (`customer_id`),
  KEY `IDX_INCREMENT_ID` (`increment_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_order`
--

LOCK TABLES `sales_order` WRITE;
/*!40000 ALTER TABLE `sales_order` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_order` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_order_datetime`
--

DROP TABLE IF EXISTS `sales_order_datetime`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_order_datetime` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `UNQ_ENTITY_ATTRIBUTE_TYPE` (`entity_id`,`attribute_id`,`entity_type_id`),
  KEY `FK_SALES_ORDER_DATETIME_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_SALES_ORDER_DATETIME_ATTRIBUTE` (`attribute_id`),
  KEY `FK_SALES_ORDER_DATETIME` (`entity_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_order_datetime`
--

LOCK TABLES `sales_order_datetime` WRITE;
/*!40000 ALTER TABLE `sales_order_datetime` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_order_datetime` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_order_decimal`
--

DROP TABLE IF EXISTS `sales_order_decimal`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_order_decimal` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` decimal(12,4) NOT NULL default '0.0000',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `UNQ_ENTITY_ATTRIBUTE_TYPE` (`entity_id`,`attribute_id`,`entity_type_id`),
  KEY `FK_SALES_ORDER_DECIMAL_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_SALES_ORDER_DECIMAL_ATTRIBUTE` (`attribute_id`),
  KEY `FK_SALES_ORDER_DECIMAL` (`entity_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_order_decimal`
--

LOCK TABLES `sales_order_decimal` WRITE;
/*!40000 ALTER TABLE `sales_order_decimal` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_order_decimal` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_order_entity`
--

DROP TABLE IF EXISTS `sales_order_entity`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_order_entity` (
  `entity_id` int(10) unsigned NOT NULL auto_increment,
  `entity_type_id` smallint(8) unsigned NOT NULL default '0',
  `attribute_set_id` smallint(5) unsigned NOT NULL default '0',
  `increment_id` varchar(50) NOT NULL default '',
  `parent_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned default NULL,
  `created_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `updated_at` datetime NOT NULL default '0000-00-00 00:00:00',
  `is_active` tinyint(1) unsigned NOT NULL default '1',
  PRIMARY KEY  (`entity_id`),
  KEY `FK_SALES_ORDER_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_SALES_ORDER_ENTITY_STORE` (`store_id`),
  KEY `IDX_SALES_ORDER_ENTITY_PARENT` (`parent_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_order_entity`
--

LOCK TABLES `sales_order_entity` WRITE;
/*!40000 ALTER TABLE `sales_order_entity` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_order_entity` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_order_entity_datetime`
--

DROP TABLE IF EXISTS `sales_order_entity_datetime`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_order_entity_datetime` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `UNQ_ENTITY_ATTRIBUTE_TYPE` (`entity_id`,`attribute_id`,`entity_type_id`),
  KEY `FK_SALES_ORDER_ENTITY_DATETIME_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_SALES_ORDER_ENTITY_DATETIME_ATTRIBUTE` (`attribute_id`),
  KEY `FK_SALES_ORDER_ENTITY_DATETIME` (`entity_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_order_entity_datetime`
--

LOCK TABLES `sales_order_entity_datetime` WRITE;
/*!40000 ALTER TABLE `sales_order_entity_datetime` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_order_entity_datetime` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_order_entity_decimal`
--

DROP TABLE IF EXISTS `sales_order_entity_decimal`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_order_entity_decimal` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` decimal(12,4) NOT NULL default '0.0000',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `UNQ_ENTITY_ATTRIBUTE_TYPE` (`entity_id`,`attribute_id`,`entity_type_id`),
  KEY `FK_SALES_ORDER_ENTITY_DECIMAL_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_SALES_ORDER_ENTITY_DECIMAL_ATTRIBUTE` (`attribute_id`),
  KEY `FK_SALES_ORDER_ENTITY_DECIMAL` (`entity_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_order_entity_decimal`
--

LOCK TABLES `sales_order_entity_decimal` WRITE;
/*!40000 ALTER TABLE `sales_order_entity_decimal` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_order_entity_decimal` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_order_entity_int`
--

DROP TABLE IF EXISTS `sales_order_entity_int`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_order_entity_int` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` int(11) NOT NULL default '0',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `UNQ_ENTITY_ATTRIBUTE_TYPE` (`entity_id`,`attribute_id`,`entity_type_id`),
  KEY `FK_SALES_ORDER_ENTITY_INT_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_SALES_ORDER_ENTITY_INT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_SALES_ORDER_ENTITY_INT` (`entity_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_order_entity_int`
--

LOCK TABLES `sales_order_entity_int` WRITE;
/*!40000 ALTER TABLE `sales_order_entity_int` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_order_entity_int` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_order_entity_text`
--

DROP TABLE IF EXISTS `sales_order_entity_text`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_order_entity_text` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` text NOT NULL,
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `UNQ_ENTITY_ATTRIBUTE_TYPE` (`entity_id`,`attribute_id`,`entity_type_id`),
  KEY `FK_SALES_ORDER_ENTITY_TEXT_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_SALES_ORDER_ENTITY_TEXT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_SALES_ORDER_ENTITY_TEXT` (`entity_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_order_entity_text`
--

LOCK TABLES `sales_order_entity_text` WRITE;
/*!40000 ALTER TABLE `sales_order_entity_text` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_order_entity_text` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_order_entity_varchar`
--

DROP TABLE IF EXISTS `sales_order_entity_varchar`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_order_entity_varchar` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `UNQ_ENTITY_ATTRIBUTE_TYPE` (`entity_id`,`attribute_id`,`entity_type_id`),
  KEY `FK_SALES_ORDER_ENTITY_VARCHAR_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_SALES_ORDER_ENTITY_VARCHAR_ATTRIBUTE` (`attribute_id`),
  KEY `FK_SALES_ORDER_ENTITY_VARCHAR` (`entity_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_order_entity_varchar`
--

LOCK TABLES `sales_order_entity_varchar` WRITE;
/*!40000 ALTER TABLE `sales_order_entity_varchar` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_order_entity_varchar` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_order_int`
--

DROP TABLE IF EXISTS `sales_order_int`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_order_int` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` int(11) NOT NULL default '0',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `UNQ_ENTITY_ATTRIBUTE_TYPE` (`entity_id`,`attribute_id`,`entity_type_id`),
  KEY `FK_SALES_ORDER_INT_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_SALES_ORDER_INT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_SALES_ORDER_INT` (`entity_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_order_int`
--

LOCK TABLES `sales_order_int` WRITE;
/*!40000 ALTER TABLE `sales_order_int` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_order_int` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_order_tax`
--

DROP TABLE IF EXISTS `sales_order_tax`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_order_tax` (
  `tax_id` int(10) unsigned NOT NULL auto_increment,
  `order_id` int(10) unsigned NOT NULL,
  `code` varchar(255) NOT NULL,
  `title` varchar(255) NOT NULL,
  `percent` decimal(12,4) NOT NULL,
  `amount` decimal(12,4) NOT NULL,
  `priority` int(11) NOT NULL,
  `position` int(11) NOT NULL,
  `base_amount` decimal(12,4) NOT NULL,
  `process` smallint(6) NOT NULL,
  `base_real_amount` decimal(12,4) NOT NULL,
  `hidden` smallint(5) unsigned NOT NULL default '0',
  PRIMARY KEY  (`tax_id`),
  KEY `IDX_ORDER_TAX` (`order_id`,`priority`,`position`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_order_tax`
--

LOCK TABLES `sales_order_tax` WRITE;
/*!40000 ALTER TABLE `sales_order_tax` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_order_tax` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_order_text`
--

DROP TABLE IF EXISTS `sales_order_text`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_order_text` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` text NOT NULL,
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `UNQ_ENTITY_ATTRIBUTE_TYPE` (`entity_id`,`attribute_id`,`entity_type_id`),
  KEY `FK_SALES_ORDER_TEXT_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_SALES_ORDER_TEXT_ATTRIBUTE` (`attribute_id`),
  KEY `FK_SALES_ORDER_TEXT` (`entity_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_order_text`
--

LOCK TABLES `sales_order_text` WRITE;
/*!40000 ALTER TABLE `sales_order_text` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_order_text` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sales_order_varchar`
--

DROP TABLE IF EXISTS `sales_order_varchar`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sales_order_varchar` (
  `value_id` int(11) NOT NULL auto_increment,
  `entity_type_id` smallint(5) unsigned NOT NULL default '0',
  `attribute_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `value` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`value_id`),
  UNIQUE KEY `UNQ_ENTITY_ATTRIBUTE_TYPE` (`entity_id`,`attribute_id`,`entity_type_id`),
  KEY `FK_SALES_ORDER_VARCHAR_ENTITY_TYPE` (`entity_type_id`),
  KEY `FK_SALES_ORDER_VARCHAR_ATTRIBUTE` (`attribute_id`),
  KEY `FK_SALES_ORDER_VARCHAR` (`entity_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sales_order_varchar`
--

LOCK TABLES `sales_order_varchar` WRITE;
/*!40000 ALTER TABLE `sales_order_varchar` DISABLE KEYS */;
/*!40000 ALTER TABLE `sales_order_varchar` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `salesrule`
--

DROP TABLE IF EXISTS `salesrule`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `salesrule` (
  `rule_id` int(10) unsigned NOT NULL auto_increment,
  `name` varchar(255) NOT NULL default '',
  `description` text NOT NULL,
  `from_date` date default '0000-00-00',
  `to_date` date default '0000-00-00',
  `coupon_code` varchar(255) default NULL,
  `uses_per_coupon` int(11) NOT NULL default '0',
  `uses_per_customer` int(11) NOT NULL default '0',
  `customer_group_ids` varchar(255) NOT NULL default '',
  `is_active` tinyint(1) NOT NULL default '0',
  `conditions_serialized` mediumtext NOT NULL,
  `actions_serialized` mediumtext NOT NULL,
  `stop_rules_processing` tinyint(1) NOT NULL default '1',
  `is_advanced` tinyint(3) unsigned NOT NULL default '1',
  `product_ids` text,
  `sort_order` int(10) unsigned NOT NULL default '0',
  `simple_action` varchar(32) NOT NULL default '',
  `discount_amount` decimal(12,4) NOT NULL default '0.0000',
  `discount_qty` decimal(12,4) unsigned default NULL,
  `discount_step` int(10) unsigned NOT NULL,
  `simple_free_shipping` tinyint(1) unsigned NOT NULL default '0',
  `times_used` int(11) unsigned NOT NULL default '0',
  `is_rss` tinyint(4) NOT NULL default '0',
  `website_ids` text,
  PRIMARY KEY  (`rule_id`),
  KEY `sort_order` (`is_active`,`sort_order`,`to_date`,`from_date`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `salesrule`
--

LOCK TABLES `salesrule` WRITE;
/*!40000 ALTER TABLE `salesrule` DISABLE KEYS */;
/*!40000 ALTER TABLE `salesrule` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `salesrule_customer`
--

DROP TABLE IF EXISTS `salesrule_customer`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `salesrule_customer` (
  `rule_customer_id` int(10) unsigned NOT NULL auto_increment,
  `rule_id` int(10) unsigned NOT NULL default '0',
  `customer_id` int(10) unsigned NOT NULL default '0',
  `times_used` smallint(11) unsigned NOT NULL default '0',
  PRIMARY KEY  (`rule_customer_id`),
  KEY `rule_id` (`rule_id`,`customer_id`),
  KEY `customer_id` (`customer_id`,`rule_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `salesrule_customer`
--

LOCK TABLES `salesrule_customer` WRITE;
/*!40000 ALTER TABLE `salesrule_customer` DISABLE KEYS */;
/*!40000 ALTER TABLE `salesrule_customer` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sendfriend_log`
--

DROP TABLE IF EXISTS `sendfriend_log`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sendfriend_log` (
  `log_id` int(11) NOT NULL auto_increment,
  `ip` int(11) unsigned NOT NULL default '0',
  `time` int(11) NOT NULL default '0',
  PRIMARY KEY  (`log_id`),
  KEY `ip` (`ip`),
  KEY `time` (`time`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT='Send to friend function log storage table';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sendfriend_log`
--

LOCK TABLES `sendfriend_log` WRITE;
/*!40000 ALTER TABLE `sendfriend_log` DISABLE KEYS */;
/*!40000 ALTER TABLE `sendfriend_log` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `shipping_tablerate`
--

DROP TABLE IF EXISTS `shipping_tablerate`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `shipping_tablerate` (
  `pk` int(10) unsigned NOT NULL auto_increment,
  `website_id` int(11) NOT NULL default '0',
  `dest_country_id` varchar(4) NOT NULL default '0',
  `dest_region_id` int(10) NOT NULL default '0',
  `dest_zip` varchar(10) NOT NULL default '',
  `condition_name` varchar(20) NOT NULL default '',
  `condition_value` decimal(12,4) NOT NULL default '0.0000',
  `price` decimal(12,4) NOT NULL default '0.0000',
  `cost` decimal(12,4) NOT NULL default '0.0000',
  PRIMARY KEY  (`pk`),
  UNIQUE KEY `dest_country` (`website_id`,`dest_country_id`,`dest_region_id`,`dest_zip`,`condition_name`,`condition_value`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `shipping_tablerate`
--

LOCK TABLES `shipping_tablerate` WRITE;
/*!40000 ALTER TABLE `shipping_tablerate` DISABLE KEYS */;
/*!40000 ALTER TABLE `shipping_tablerate` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `sitemap`
--

DROP TABLE IF EXISTS `sitemap`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `sitemap` (
  `sitemap_id` int(11) NOT NULL auto_increment,
  `sitemap_type` varchar(32) default NULL,
  `sitemap_filename` varchar(32) default NULL,
  `sitemap_path` tinytext,
  `sitemap_time` timestamp NULL default NULL,
  `store_id` smallint(5) unsigned NOT NULL,
  PRIMARY KEY  (`sitemap_id`),
  KEY `FK_SITEMAP_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `sitemap`
--

LOCK TABLES `sitemap` WRITE;
/*!40000 ALTER TABLE `sitemap` DISABLE KEYS */;
/*!40000 ALTER TABLE `sitemap` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `tag`
--

DROP TABLE IF EXISTS `tag`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `tag` (
  `tag_id` int(11) unsigned NOT NULL auto_increment,
  `name` varchar(255) NOT NULL default '',
  `status` smallint(6) NOT NULL default '0',
  PRIMARY KEY  (`tag_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `tag`
--

LOCK TABLES `tag` WRITE;
/*!40000 ALTER TABLE `tag` DISABLE KEYS */;
/*!40000 ALTER TABLE `tag` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `tag_relation`
--

DROP TABLE IF EXISTS `tag_relation`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `tag_relation` (
  `tag_relation_id` int(11) unsigned NOT NULL auto_increment,
  `tag_id` int(11) unsigned NOT NULL default '0',
  `customer_id` int(10) unsigned NOT NULL default '0',
  `product_id` int(11) unsigned NOT NULL default '0',
  `store_id` smallint(6) unsigned NOT NULL default '1',
  `active` tinyint(1) unsigned NOT NULL default '1',
  `created_at` datetime default NULL,
  PRIMARY KEY  (`tag_relation_id`),
  KEY `IDX_PRODUCT` (`product_id`),
  KEY `IDX_TAG` (`tag_id`),
  KEY `IDX_CUSTOMER` (`customer_id`),
  KEY `IDX_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `tag_relation`
--

LOCK TABLES `tag_relation` WRITE;
/*!40000 ALTER TABLE `tag_relation` DISABLE KEYS */;
/*!40000 ALTER TABLE `tag_relation` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `tag_summary`
--

DROP TABLE IF EXISTS `tag_summary`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `tag_summary` (
  `tag_id` int(11) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL default '0',
  `customers` int(11) unsigned NOT NULL default '0',
  `products` int(11) unsigned NOT NULL default '0',
  `uses` int(11) unsigned NOT NULL default '0',
  `historical_uses` int(11) unsigned NOT NULL default '0',
  `popularity` int(11) unsigned NOT NULL default '0',
  PRIMARY KEY  (`tag_id`,`store_id`),
  KEY `FK_TAG_SUMMARY_STORE` (`store_id`),
  KEY `IDX_TAG` (`tag_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `tag_summary`
--

LOCK TABLES `tag_summary` WRITE;
/*!40000 ALTER TABLE `tag_summary` DISABLE KEYS */;
/*!40000 ALTER TABLE `tag_summary` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `tax_calculation`
--

DROP TABLE IF EXISTS `tax_calculation`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `tax_calculation` (
  `tax_calculation_rate_id` int(11) NOT NULL,
  `tax_calculation_rule_id` int(11) NOT NULL,
  `customer_tax_class_id` smallint(6) NOT NULL,
  `product_tax_class_id` smallint(6) NOT NULL,
  KEY `FK_TAX_CALCULATION_RULE` (`tax_calculation_rule_id`),
  KEY `FK_TAX_CALCULATION_RATE` (`tax_calculation_rate_id`),
  KEY `FK_TAX_CALCULATION_CTC` (`customer_tax_class_id`),
  KEY `FK_TAX_CALCULATION_PTC` (`product_tax_class_id`),
  KEY `IDX_TAX_CALCULATION` (`tax_calculation_rate_id`,`customer_tax_class_id`,`product_tax_class_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `tax_calculation`
--

LOCK TABLES `tax_calculation` WRITE;
/*!40000 ALTER TABLE `tax_calculation` DISABLE KEYS */;
INSERT INTO `tax_calculation` VALUES (1,1,3,2),(3,3,3,6),(2,2,3,5);
/*!40000 ALTER TABLE `tax_calculation` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `tax_calculation_rate`
--

DROP TABLE IF EXISTS `tax_calculation_rate`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `tax_calculation_rate` (
  `tax_calculation_rate_id` int(11) NOT NULL auto_increment,
  `tax_country_id` char(2) NOT NULL,
  `tax_region_id` mediumint(9) NOT NULL,
  `tax_postcode` varchar(12) NOT NULL,
  `code` varchar(255) NOT NULL,
  `rate` decimal(12,4) NOT NULL,
  PRIMARY KEY  (`tax_calculation_rate_id`),
  KEY `IDX_TAX_CALCULATION_RATE` (`tax_country_id`,`tax_region_id`,`tax_postcode`),
  KEY `IDX_TAX_CALCULATION_RATE_CODE` (`code`)
) ENGINE=InnoDB AUTO_INCREMENT=4 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `tax_calculation_rate`
--

LOCK TABLES `tax_calculation_rate` WRITE;
/*!40000 ALTER TABLE `tax_calculation_rate` DISABLE KEYS */;
INSERT INTO `tax_calculation_rate` VALUES (1,'ES',0,'*','IVA General','16.0000'),(2,'ES',0,'*','IVA Reducido','7.0000'),(3,'ES',0,'*','IVA Superreducido','4.0000');
/*!40000 ALTER TABLE `tax_calculation_rate` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `tax_calculation_rate_title`
--

DROP TABLE IF EXISTS `tax_calculation_rate_title`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `tax_calculation_rate_title` (
  `tax_calculation_rate_title_id` int(11) NOT NULL auto_increment,
  `tax_calculation_rate_id` int(11) NOT NULL,
  `store_id` smallint(5) unsigned NOT NULL,
  `value` varchar(255) NOT NULL,
  PRIMARY KEY  (`tax_calculation_rate_title_id`),
  KEY `IDX_TAX_CALCULATION_RATE_TITLE` (`tax_calculation_rate_id`,`store_id`),
  KEY `FK_TAX_CALCULATION_RATE_TITLE_RATE` (`tax_calculation_rate_id`),
  KEY `FK_TAX_CALCULATION_RATE_TITLE_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `tax_calculation_rate_title`
--

LOCK TABLES `tax_calculation_rate_title` WRITE;
/*!40000 ALTER TABLE `tax_calculation_rate_title` DISABLE KEYS */;
/*!40000 ALTER TABLE `tax_calculation_rate_title` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `tax_calculation_rule`
--

DROP TABLE IF EXISTS `tax_calculation_rule`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `tax_calculation_rule` (
  `tax_calculation_rule_id` int(11) NOT NULL auto_increment,
  `code` varchar(255) NOT NULL,
  `priority` mediumint(9) NOT NULL,
  `position` mediumint(9) NOT NULL,
  PRIMARY KEY  (`tax_calculation_rule_id`),
  KEY `IDX_TAX_CALCULATION_RULE` (`priority`,`position`,`tax_calculation_rule_id`),
  KEY `IDX_TAX_CALCULATION_RULE_CODE` (`code`)
) ENGINE=InnoDB AUTO_INCREMENT=4 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `tax_calculation_rule`
--

LOCK TABLES `tax_calculation_rule` WRITE;
/*!40000 ALTER TABLE `tax_calculation_rule` DISABLE KEYS */;
INSERT INTO `tax_calculation_rule` VALUES (1,'IVA General',1,1),(2,'IVA Reducido',2,2),(3,'IVA Superreducido',3,3);
/*!40000 ALTER TABLE `tax_calculation_rule` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `tax_class`
--

DROP TABLE IF EXISTS `tax_class`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `tax_class` (
  `class_id` smallint(6) NOT NULL auto_increment,
  `class_name` varchar(255) NOT NULL default '',
  `class_type` enum('CUSTOMER','PRODUCT') NOT NULL default 'CUSTOMER',
  PRIMARY KEY  (`class_id`)
) ENGINE=InnoDB AUTO_INCREMENT=7 DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `tax_class`
--

LOCK TABLES `tax_class` WRITE;
/*!40000 ALTER TABLE `tax_class` DISABLE KEYS */;
INSERT INTO `tax_class` VALUES (2,'IVA General','PRODUCT'),(3,'Clientes Finales','CUSTOMER'),(5,'IVA Reducido','PRODUCT'),(6,'IVA Superreducido','PRODUCT');
/*!40000 ALTER TABLE `tax_class` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `weee_discount`
--

DROP TABLE IF EXISTS `weee_discount`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `weee_discount` (
  `entity_id` int(10) unsigned NOT NULL default '0',
  `website_id` smallint(5) unsigned NOT NULL default '0',
  `customer_group_id` smallint(5) unsigned NOT NULL,
  `value` decimal(12,4) NOT NULL default '0.0000',
  KEY `FK_CATALOG_PRODUCT_ENTITY_WEEE_DISCOUNT_WEBSITE` (`website_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_WEEE_DISCOUNT_PRODUCT_ENTITY` (`entity_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_WEEE_DISCOUNT_GROUP` (`customer_group_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `weee_discount`
--

LOCK TABLES `weee_discount` WRITE;
/*!40000 ALTER TABLE `weee_discount` DISABLE KEYS */;
/*!40000 ALTER TABLE `weee_discount` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `weee_tax`
--

DROP TABLE IF EXISTS `weee_tax`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `weee_tax` (
  `value_id` int(11) NOT NULL auto_increment,
  `website_id` smallint(5) unsigned NOT NULL default '0',
  `entity_id` int(10) unsigned NOT NULL default '0',
  `country` varchar(2) NOT NULL default '',
  `value` decimal(12,4) NOT NULL default '0.0000',
  `state` varchar(255) NOT NULL default '*',
  `attribute_id` smallint(5) unsigned NOT NULL,
  `entity_type_id` smallint(5) unsigned NOT NULL,
  PRIMARY KEY  (`value_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_WEEE_TAX_WEBSITE` (`website_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_WEEE_TAX_PRODUCT_ENTITY` (`entity_id`),
  KEY `FK_CATALOG_PRODUCT_ENTITY_WEEE_TAX_COUNTRY` (`country`),
  KEY `FK_WEEE_TAX_ATTRIBUTE_ID` (`attribute_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `weee_tax`
--

LOCK TABLES `weee_tax` WRITE;
/*!40000 ALTER TABLE `weee_tax` DISABLE KEYS */;
/*!40000 ALTER TABLE `weee_tax` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `wishlist`
--

DROP TABLE IF EXISTS `wishlist`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `wishlist` (
  `wishlist_id` int(10) unsigned NOT NULL auto_increment,
  `customer_id` int(10) unsigned NOT NULL default '0',
  `shared` tinyint(1) unsigned default '0',
  `sharing_code` varchar(32) character set latin1 collate latin1_general_ci NOT NULL default '',
  PRIMARY KEY  (`wishlist_id`),
  UNIQUE KEY `FK_CUSTOMER` (`customer_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC COMMENT='Wishlist main';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `wishlist`
--

LOCK TABLES `wishlist` WRITE;
/*!40000 ALTER TABLE `wishlist` DISABLE KEYS */;
/*!40000 ALTER TABLE `wishlist` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `wishlist_item`
--

DROP TABLE IF EXISTS `wishlist_item`;
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;
CREATE TABLE `wishlist_item` (
  `wishlist_item_id` int(10) unsigned NOT NULL auto_increment,
  `wishlist_id` int(10) unsigned NOT NULL default '0',
  `product_id` int(10) unsigned NOT NULL default '0',
  `store_id` smallint(5) unsigned NOT NULL,
  `added_at` datetime default NULL,
  `description` text,
  PRIMARY KEY  (`wishlist_item_id`),
  KEY `FK_ITEM_WISHLIST` (`wishlist_id`),
  KEY `FK_WISHLIST_PRODUCT` (`product_id`),
  KEY `FK_WISHLIST_STORE` (`store_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Wishlist items';
SET character_set_client = @saved_cs_client;

--
-- Dumping data for table `wishlist_item`
--

LOCK TABLES `wishlist_item` WRITE;
/*!40000 ALTER TABLE `wishlist_item` DISABLE KEYS */;
/*!40000 ALTER TABLE `wishlist_item` ENABLE KEYS */;
UNLOCK TABLES;
/*!40103 SET TIME_ZONE=@OLD_TIME_ZONE */;

/*!40101 SET SQL_MODE=@OLD_SQL_MODE */;
/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;
/*!40014 SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS */;
/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;
/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;
/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;
/*!40111 SET SQL_NOTES=@OLD_SQL_NOTES */;

-- Dump completed on 2009-07-02 22:39:59
